// XCamera.h: interface for the XCamera class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_)
#define AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define		_C_MAX_ROTATION_SPEED		5.0f
#define		_C_MAX_ZOOMING_SPEED		20.0f
#define		_C_MAX_SCROLLING_SPEED		20.0f
#define		_C_MAX_ZANGLE_SPEED			2.0f

#define		MAX_ZOOM					400
#define		MIN_ZOOM					5

#define		Z_ANGLE						D3DX_PI / 2

#define		MAX_Z_ANGLE					D3DX_PI / 2
#define		MIN_Z_ANGLE					D3DX_PI / 7 

const float NEAR_CLIP = 0.1f;
const float FAR_CLIP = 500.0f;
const float FOV = D3DX_PI / 4.0f;
const float ASPECT = 4.0f / 3.0f;


class XCamera : public ICamera
{
	// D3D device object...
	LPDIRECT3DDEVICE8	m_lpDevice;
	I3DFont				*m_lpCameraInfo;
	//
	// Camera bounds
	int					m_iMinPos;
	int					m_iMaxPos; 
	//
	// Velocities in all directions
	float				m_fScrlVelX;	
	float				m_fScrlVelY;	
	float				m_fRotVel;
	float				m_fZoomVel;
	float				m_fZAngleVel;
	//
	// Camera position
	D3DXVECTOR3			m_vLookAtPoint;
	D3DXVECTOR3			m_vEyePt;
	float				m_fZoomDistance;
	float				m_fZAngle;		
	float				m_fViewAngle;
	float				m_fCameraHeight;
	//
	// Camera matrices
	D3DXMATRIX			m_matBillboard;		
	D3DXMATRIX			m_matProjection;
	D3DXMATRIX			m_matView;	

	DWORD	m_dwRef;

public:
	//
	// Inicialization
	virtual HRESULT				InitCamera(int iMinBound, int iMaxBound);
	// Update
	virtual HRESULT				ProcessCamera(float	fElapsedTime);	
	// Call after device reset
	virtual	HRESULT				RestoreCamera();
	// Camera motion
	virtual void				ZoomIn() {m_fZoomVel = -(_C_MAX_ZOOMING_SPEED);}
	virtual void				ZoomOut() {m_fZoomVel = _C_MAX_ZOOMING_SPEED;}
	virtual void				RotateLeft() {m_fRotVel = _C_MAX_ROTATION_SPEED;}
	virtual void				RotateRight() {m_fRotVel = -(_C_MAX_ROTATION_SPEED);}
	virtual void				MoveLeft() {m_fScrlVelX = _C_MAX_SCROLLING_SPEED;}
	virtual void				MoveRight() {m_fScrlVelX = -(_C_MAX_SCROLLING_SPEED);}
	virtual void				MoveUp() {m_fScrlVelY = -(_C_MAX_SCROLLING_SPEED);}
	virtual void				MoveDown() {m_fScrlVelY = _C_MAX_SCROLLING_SPEED;}
	virtual void				IncreaseZAngle() {m_fZAngleVel =   _C_MAX_ZANGLE_SPEED;}
	virtual void				DecreaseZAngle() {m_fZAngleVel = -(_C_MAX_ZANGLE_SPEED);}
	virtual void				SetCameraHeight(float fHeight) {m_fCameraHeight = fHeight;}			
	//
	// Get methods
	virtual float				GetViewAngle() {return m_fViewAngle;}
	virtual float				GetZoom() {	return m_fZoomDistance;}
	virtual float				GetZAngle() {return m_fZAngle;}
	virtual D3DXVECTOR3*		GetCameraPos() {return &m_vLookAtPoint;}
	virtual D3DXVECTOR3*		GetEyePoint() {return &m_vEyePt;}
	virtual D3DXMATRIX*			GetBillboardMatrix() {return &m_matBillboard;}
	virtual void				GetWorldViewProjMatrix(D3DXMATRIX * matWVP) {;}

	virtual HRESULT		AddRef();
	virtual HRESULT		Release();

	XCamera();
	~XCamera();
};

#endif // !defined(AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_)
