#include "StdAfx.h"


#define		DISPLAY_API	__declspec(dllexport)

#include "../common/common.h"
#include "interfaces.h"
#include "manager.h"
#include "xcamera.h"
#include "xtexture.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"

#include "xdisplay.h"

XDisplay::XDisplay(void)
{
	m_lpD3DObject = NULL;
	m_lpD3DDevice = NULL;
	m_lpCamera = NULL;
	m_lpResManager = NULL;
	m_fStopTime = cmnGetTime(TIMER_GETAPPTIME);
	m_Frames = 0.0f;
	m_dwRef = 0;
	ZeroMemory(m_szFPSString, sizeof(char) * 50);
	strcpy(m_szFPSString, "Invalid FPS");
	m_wFlags = 0;
}

XDisplay::~XDisplay(void)
{
	Clean();
}

HRESULT		XDisplay::AddRef()
{
	return m_dwRef++;
}

HRESULT		XDisplay::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XDisplay object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

void XDisplay::Clean(void)
{
	if(m_lpD3DDevice) {
 
		m_lpD3DDevice->SetTexture(0, NULL);
	
		SAFE_RELEASE(m_lpD3DDevice);
		SAFE_RELEASE(m_lpD3DObject);
		SAFE_RELEASE(m_lpCamera);
		SAFE_RELEASE(m_lpResManager);
		SAFE_RELEASE(m_lpFPSFont);
		SAFE_RELEASE(m_lpInfoFont);
	}
}

HRESULT XDisplay::Init(HWND hWnd)
{
	TRACE("Initializing Direct3D system...");	
	HRESULT dwRet;
	DWORD BehaviorFlags;	
	//
	// Already initialized?
	if(m_lpD3DDevice) {
		XException exp("Direct3D system is already initialized.");
		THROW(exp);	
	}
	// Invalid handle
	if(!hWnd)
	{
		XException exp("Window handle is NULL.");
		THROW(exp);	
	}
	//	
	// As first, create one Direct3D object	
	m_lpD3DObject = Direct3DCreate8(D3D_SDK_VERSION);
	if(!m_lpD3DObject) {		
		cmnMessageBoxA("Please, install DirectX 8.1.", MSG_ERROR);
		XException exp("Cannot create object of Direct3D.");
		THROW(exp);			
	}
	else
	{
		TRACE("Direct3D object created.");
	}
	//
	//
	// Read desired display format from ini file
	//
	// Get resolution values from setup.ini file	
	m_sRes.x = (float)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIDTH);	
	m_sRes.y = (float)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_HEIGHT);	
	//
	// Get screen depth from ini file (may be 16 or 32)	
	m_formatScreenDepth  = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FORMAT);		
	//
	// If system fullsreen or windowed
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WINDOWED) != 0) {
		m_wFlags |= FLAG_WINDOWED;
	}
	//
	// Is FPS enabled?	Default is disabled.
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FPS)) {
		m_wFlags |= FLAG_FPS;
	}
	//
	// Get desired FPS from ini. Default is off.
	// If m_iDesiredFPS == 0, then FPS limiter is off.
	m_iDesiredFPS = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DESIREDFPS);	
	//
	// Read type of vertex processing
	BehaviorFlags = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_BEHAVIOR);		
	//
	// Read texture format
	m_formatTextureDepth = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_TEXDEPTH);	
	//
	// Set size of back buffer from window position
	if(m_wFlags & FLAG_WINDOWED) {	
		D3DDISPLAYMODE displaymode;
		RECT rcWindowClient;
		//
		// Get window position
		GetClientRect(hWnd, &rcWindowClient);
		// Get current display mode
		dwRet = m_lpD3DObject->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displaymode);
		if(dwRet != D3D_OK) {
			XException exp("Cannot get display format in window mode.", dwRet);
			THROW(exp);				
		}
		//
		// Set size of back buffer
        m_sRes.x  = (float)rcWindowClient.right - rcWindowClient.left;
        m_sRes.y  =  (float)rcWindowClient.bottom - rcWindowClient.top;
		// Set display mode to current
        m_formatScreenDepth = displaymode.Format;
	}
	//
	// Read ordinal number of used adapter
	m_iAdapterOrdinal = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_ADAPTER);
	// Read type of device on this adapter
	m_typeDeviceType = (D3DDEVTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DEVICE);	
	m_tftMagTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MAGTEXTUREFILTER);	
	m_tftMinTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MINTEXTUREFILTER);	
	m_tftMipTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MIPTEXTUREFILTER);	
	// 
	// Set properties of device	
	ZeroMemory(&m_d3dDeviceParam, sizeof(m_d3dDeviceParam));
	m_d3dDeviceParam.Windowed							= (m_wFlags & FLAG_WINDOWED) ? TRUE : FALSE;
	m_d3dDeviceParam.SwapEffect							= D3DSWAPEFFECT_DISCARD;
    m_d3dDeviceParam.BackBufferWidth					= (int)m_sRes.x;
    m_d3dDeviceParam.BackBufferHeight					= (int)m_sRes.y;
    m_d3dDeviceParam.BackBufferFormat					= m_formatScreenDepth;
	m_d3dDeviceParam.BackBufferCount	 				= 1;	
	m_d3dDeviceParam.hDeviceWindow						= hWnd;
	m_d3dDeviceParam.Flags								= 0;
	//
	// -> Fullscreen AA
	m_d3dDeviceParam.MultiSampleType = (D3DMULTISAMPLE_TYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FSAA);
	m_d3dDeviceParam.EnableAutoDepthStencil				= TRUE;
	m_d3dDeviceParam.AutoDepthStencilFormat				= D3DFMT_D16;
	//
	// Set some settings for fullscreen mode
	// -> PresentationInterval
	// -> RefreshRate
	if(!(m_wFlags & FLAG_WINDOWED)) {
		//
		// -> PresentationInterval
		if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_VSYNC) == 0) {
			m_d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		}
		else {
			m_d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;
		}
		//
		// -> RefreshRate
		m_d3dDeviceParam.FullScreen_RefreshRateInHz	= cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_REFRESHRATE);		
	}	
	// Now we can create D3D device
	dwRet = m_lpD3DObject->CreateDevice(m_iAdapterOrdinal, m_typeDeviceType, hWnd,
										 BehaviorFlags,	&m_d3dDeviceParam, &m_lpD3DDevice);
	if(dwRet != D3D_OK) {
		cmnMessageBoxA("Please, run Setup to set display settings.", MSG_ERROR);
		Clean();
		XException exp("Cannot create D3D device.", dwRet);
		THROW(exp);			
	}
	else
	{
		TRACE("Direct3D Device created.");
	}
	// format info string 
	if(!m_d3dDeviceParam.Windowed)
	{
		sprintf(m_szInfoString, "Ad:%d Format:%2.0lfx%2.0lfx%d Dev:%s\n%s", m_iAdapterOrdinal, m_sRes.x, m_sRes.y, m_formatScreenDepth, (m_typeDeviceType == D3DDEVTYPE_HAL) ? "HAL" : ((m_typeDeviceType == D3DDEVTYPE_SW) ? "SW" : "REF"), 
			(BehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING) 
				? "Hardware Vertex Processing" 
				: (BehaviorFlags & D3DCREATE_PUREDEVICE) 
					? "Pure device" 
					: (BehaviorFlags & D3DCREATE_SOFTWARE_VERTEXPROCESSING) 
						? "Software Vertex Processing" 
						: "Unknown vertex processing");
	}
	else
	{
		sprintf(m_szInfoString, "Ad:%d Dev:%s\n%s", m_iAdapterOrdinal, (m_typeDeviceType == D3DDEVTYPE_HAL) ? "HAL" : ((m_typeDeviceType == D3DDEVTYPE_SW) ? "SW" : "REF"), (BehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING) 
				? "Hardware Vertex Processing" 
				: (BehaviorFlags & D3DCREATE_PUREDEVICE) 
					? "Pure device" 
					: (BehaviorFlags & D3DCREATE_SOFTWARE_VERTEXPROCESSING) 
						? "Software Vertex Processing" 
						: "Unknown vertex processing");
	}

	D3DCAPS8 d3dc;
	// get max active lights
	if(S_OK == m_lpD3DDevice->GetDeviceCaps(&d3dc))
	{
		m_iMaxLights = d3dc.MaxActiveLights;
		if(m_iMaxLights == -1)
		{
			m_iMaxLights = 65535;
		}
	}
	else
	{
		// light support off
		m_iMaxLights = 0;
	}

	m_dwBackgroundColor = 0;

	/* create internal objects */
	CreateDisplayObject(DISIID_ICamera, (void**)&m_lpCamera);
	CreateDisplayObject(DISIID_IResourceManager, (void**)&m_lpResManager);
	CreateDisplayObject(DISIID_I3DFont, (void**)&m_lpFPSFont);
	CreateDisplayObject(DISIID_I3DFont, (void**)&m_lpInfoFont);
	m_lpFPSFont->CreateFont(15, 8, "Arial", TRUE, D3DCOLOR_ARGB(255, 255, 255, 255));
	m_lpInfoFont->CreateFont(12, 5, "Arial", TRUE, D3DCOLOR_ARGB(255, 255, 255, 0));

	StartupSettings();

	return S_OK;
}

void		XDisplay::StartupSettings()
{	
	//
	// Enable Z-buffer
	m_lpD3DDevice->SetRenderState(D3DRS_ZENABLE, TRUE);		
	//
	// Set wireframe, if user wants to render wireframe model
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIREFRAME)) {
		m_lpD3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		m_lpD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	}	
	// set standard material
	D3DMATERIAL8 mtrl;
	ZeroMemory( &mtrl, sizeof(D3DMATERIAL8) );
	mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
	mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
	mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
	mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
	m_lpD3DDevice->SetMaterial(&mtrl);
	// turn on lighting
	m_lpD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	//
	// Set texture filter for first stage
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, m_tftMipTextureFilter);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, m_tftMagTextureFilter);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, m_tftMinTextureFilter);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);		
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);		
}

HRESULT			XDisplay::UpdateBackground()
{	
	if(m_lpD3DDevice) {
		//
		// Clear the back buffer to a blue color
		m_lpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET| D3DCLEAR_ZBUFFER, m_dwBackgroundColor, 1.0f, 0);					
		return 0;
	}	
	return 1;
}

HRESULT			XDisplay::Present()
{
	DWORD dwRet;
	if(m_lpD3DDevice) {
		// Get time
		m_fTime = cmnGetTime(TIMER_GETELAPSEDTIME);
		m_fTotalTime += m_fTime;

		// Test the cooperative level to see if it's okay to render
		dwRet = m_lpD3DDevice->TestCooperativeLevel();
		if(dwRet != D3D_OK)
		{
			// If the device was lost, do not render until we get it back
			if( D3DERR_DEVICELOST == dwRet ) {
				TRACE("Device is lost.");
				//return (int)dwRet;
			}
			// Check if the device needs to be resized.
			if( D3DERR_DEVICENOTRESET == dwRet ) {				
				RestoreDisplay();					
			}						
			return (int)dwRet;
		}	
		
		if(m_wFlags & FLAG_FPS) {
			UpdateFPS();
		}
		// Set limitations
		if(m_iDesiredFPS != 0) {
			LimitFPS();
		}	
		
		m_lpD3DDevice->Present(NULL, NULL, NULL, NULL);				
	}
	return 0;
}

int		XDisplay::UpdateFPS()
{	
	//
	// Get time of system start
	m_fStartTime = cmnGetTime(TIMER_GETAPPTIME);		
	// 
	// Increment frames counter	
	m_Frames++;							
	//
	// Compute number of frames per one second
	if((m_fStartTime - m_fStopTime) >= FPS_REFRESH_RATE) {	
		
		m_fCurrentFPS = m_Frames / (m_fStartTime - m_fStopTime);
		m_Frames = 0;
		//
		// Save old-new time
		m_fStopTime = m_fStartTime;		
		//
		// create FPS text
		sprintf(m_szFPSString, "%2.1lf FPS ", m_fCurrentFPS);		
		TRACE("%4.1f FPS - ElapsedT: %4.8f", m_fCurrentFPS, m_fTime);
	}	 
	//		
	// Draw FPS on the screen
	m_lpD3DDevice->BeginScene();
	m_lpFPSFont->Draw(m_szFPSString, 0, 0);
	m_lpInfoFont->Draw(m_szInfoString, 0, 15);	
	m_lpD3DDevice->EndScene();	
	return S_OK;
}

void	XDisplay::LimitFPS()
{
	static float last = 0;
    do
    {
        float now = cmnGetTime(TIMER_GETAPPTIME);
        float passed = now - last;
        m_fFrameRate = float(1.0 / float(passed));
    }
    while(m_fFrameRate > m_iDesiredFPS);
    last = cmnGetTime(TIMER_GETAPPTIME); 
}

HRESULT	XDisplay::RestoreDisplay()
{
	TRACE("Restoring device...");
	//
	// Release all resources
	if(!m_lpResManager->ReleaseAll()) 
	{
		TRACE("Resources released.");
	}
	//	
	// Reset the device
	if(D3D_OK != m_lpD3DDevice->Reset(&m_d3dDeviceParam))
	{
		TRACE("Fatal error: Cannot RESET device object.");

	}
	else {
		TRACE("Device is reset.");
	}
	// Wait until device is reset
	Sleep(150);

	// restore some render settings
	StartupSettings();	
	m_lpCamera->RestoreCamera();
	
	// reinit resources
	m_lpResManager->RestoreAll();
			
	return 0;
}

HRESULT	XDisplay::EnableLight(int iIndex, BOOL bEnable)
{
	if(m_lpD3DDevice && (iIndex >= 0 && iIndex < m_iMaxLights))
	{
		return m_lpD3DDevice->LightEnable(iIndex, bEnable);
	}
	return -1;
}

HRESULT	XDisplay::SetLight(int iIndex, D3DLIGHT8 * pLight)
{
	if(m_lpD3DDevice && pLight && (iIndex >= 0 && iIndex < m_iMaxLights))
	{
		return m_lpD3DDevice->SetLight(iIndex, pLight);
	}
	return -1;
}
