#pragma once

//
#define		_S_SECTION_DISPLAY			_T("Display")
// Graphics section
#define		_S_KEY_ADAPTER				_T("AdapterOrdinal")
#define		_S_KEY_DEVICE				_T("DeviceType")
#define		_S_KEY_WIDTH				_T("ScreenWidth")
#define		_S_KEY_HEIGHT				_T("ScreenHeight")
#define		_S_KEY_FORMAT				_T("ScreenFormat")
#define		_S_KEY_REFRESHRATE			_T("RefreshRate")
#define		_S_KEY_BEHAVIOR				_T("DeviceBehavior")
#define		_S_KEY_FSAA					_T("FSAA")
#define		_S_KEY_MAGTEXTUREFILTER		_T("MagTextureFilter")	
#define		_S_KEY_MINTEXTUREFILTER		_T("MinTextureFilter")	
#define		_S_KEY_MIPTEXTUREFILTER		_T("MipTextureFilter")	
#define		_S_KEY_TEXDEPTH				_T("TextureFormat")
#define		_S_KEY_FPS					_T("FPS")
#define		_S_KEY_DESIREDFPS			_T("DesiredFPS")
#define		_S_KEY_WIREFRAME			_T("WireFrame")
#define		_S_KEY_VSYNC				_T("VSync")
#define		_S_KEY_WINDOWED				_T("Windowed")

#define		_S_KEY_SCROLLINGSPEED		_T("ScrollingSpeed")

#define		FLAG_FPS				0x0001
#define		FLAG_WINDOWED			0x0002
#define		FLAG_VERTEXSHADER		0x0004

#define		FPS_REFRESH_RATE 0.5f

struct VERTEX 
{
	D3DXVECTOR3		vecPos;
	D3DXVECTOR3		vecNormal;
	DWORD			dwDiffuse;		
	float			tu1, tv1;		
}; 

#define		VERTEXFORMAT			D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1|D3DFVF_DIFFUSE 


class XDisplay : public IDisplay
{
	// D3D objects
	IDirect3D8	*			m_lpD3DObject;
	IDirect3DDevice8*		m_lpD3DDevice;
	D3DPRESENT_PARAMETERS	m_d3dDeviceParam;	
	DWORD					m_dwRef;
	//
	// Display settings
	WORD					m_wFlags;
	// Display parameters
	D3DXVECTOR2				m_sRes;
	UINT					m_iDepth;
	D3DDEVTYPE				m_typeDeviceType;
	int						m_iAdapterOrdinal;
	D3DFORMAT				m_formatScreenDepth;
	D3DFORMAT				m_formatTextureDepth;
	D3DTEXTUREFILTERTYPE	m_tftMagTextureFilter;
	D3DTEXTUREFILTERTYPE	m_tftMinTextureFilter;
	D3DTEXTUREFILTERTYPE	m_tftMipTextureFilter;
	//
	// FPS atributes
	float				m_fStartTime;
	float				m_fStopTime;
	FLOAT				m_Frames;
	float				m_fCurrentFPS;
	float				m_fFrameRate;
	int					m_iDesiredFPS;
	char				m_szFPSString[50];
	char				m_szInfoString[512];
	//
	// Camera
	ICamera	*			m_lpCamera;
	I3DFont	*			m_lpFPSFont;
	I3DFont	*			m_lpInfoFont;
	IResourceManager* 	m_lpResManager;
	//
	// Time between two frames
	float				m_fTime;
	// Time from app start
	float				m_fTotalTime;
	// Background color
	D3DCOLOR			m_dwBackgroundColor;
	int					m_iMaxLights;
	
private:
	int			UpdateFPS();
	void		LimitFPS();	
	void		Clean(void);
	void		StartupSettings();
    
public:
	virtual HRESULT			Init(HWND hWnd);
	virtual HRESULT			UpdateBackground();	
	virtual HRESULT			Present();
	virtual HRESULT			RestoreDisplay();
	virtual D3DXVECTOR2*	GetResolution() { return &m_sRes;} 
	virtual ICamera *			GetCamera() {return m_lpCamera;}
	virtual IResourceManager*	GetResourceManager() {return m_lpResManager; }
	virtual LPDIRECT3DDEVICE8	GetDevice() {return m_lpD3DDevice;}	
	virtual D3DFORMAT			GetTextureFormat() {return m_formatTextureDepth;}
	virtual HRESULT			EnableLight(int iIndex, BOOL bEnable = TRUE);
	virtual HRESULT			SetLight(int iIndex, D3DLIGHT8 * pLight);
	
public:
	virtual HRESULT		AddRef();
	virtual HRESULT		Release();

	XDisplay(void);
	~XDisplay(void);		
};
