#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xindexbuffer.h"

XIndexBuffer::XIndexBuffer(void)
{
	m_lpDevice = NULL;
	m_pIndexBuffer = NULL;
	m_dwRef = 0;
}

XIndexBuffer::~XIndexBuffer(void)
{
	ReleaseResource();
	m_lpDevice = NULL;
}

HRESULT		XIndexBuffer::AddRef()
{
	return m_dwRef++;
}

HRESULT		XIndexBuffer::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XIndexBuffer object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

int		XIndexBuffer::ReleaseResource()
{
	SAFE_RELEASE(m_pIndexBuffer);
	return S_OK;
}


int		XIndexBuffer::Create(UINT uiLength, DWORD dwUsage, D3DFORMAT dwFormat, D3DPOOL dwPool)
{		
	IDisplay *dis = NULL;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();
	
	if(!m_pIndexBuffer) 
	{		
		m_uiLength = uiLength;
		m_dwUsage = dwUsage;
		m_dwFormat = dwFormat;
		m_dwPool = dwPool;

		DWORD dwRet = m_lpDevice->CreateIndexBuffer(uiLength, dwUsage, dwFormat, dwPool, &m_pIndexBuffer);
		if(dwRet != S_OK) {
			XException exp("Cannot create index buffer.", dwRet);
			THROW(exp);				
		}
		dis->GetResourceManager()->AddIndexBuffer(this);
		SAFE_RELEASE(dis);
		return 0;
	}
	SAFE_RELEASE(dis);
	return -1;
}

int			XIndexBuffer::Restore()
{
	if(m_lpDevice && !m_pIndexBuffer)
	{
		DWORD dwRet = m_lpDevice->CreateIndexBuffer(m_uiLength, m_dwUsage, m_dwFormat, m_dwPool, &m_pIndexBuffer);
		if(dwRet != S_OK) {
			XException exp("Cannot create index buffer.", dwRet);
			THROW(exp);	
		}
		return 0;
	}
	return -1;
}
