#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "xtexture.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"

XResourceManager::XResourceManager(void)
{
	m_dwRef = 0;
}

XResourceManager::~XResourceManager(void)
{
	m_arTextures.clear();
}

HRESULT		XResourceManager::AddRef()
{
	return m_dwRef++;
}

HRESULT		XResourceManager::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XResourceManager object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

HRESULT	XResourceManager::AddFont(I3DFont * pFontToAdd)
{
	if(pFontToAdd) {
		m_arFonts.push_back(pFontToAdd);
		return 0;
	}
	return -1;
}

HRESULT	XResourceManager::AddSprite(ID3DXSprite * pSpriteToAdd)
{
	if(pSpriteToAdd) {
		m_arSprites.push_back(pSpriteToAdd);
		return 0;
	}
	return -1;
}


HRESULT		XResourceManager::AddTexture(ITexture * pTextureToAdd)
{
	if(pTextureToAdd) {
		m_arTextures.push_back(pTextureToAdd);
		return 0;
	}
	return -1;
}

HRESULT		XResourceManager::AddMesh(IMesh * pMeshToAdd)
{
	if(pMeshToAdd) {
		m_arMeshes.push_back(pMeshToAdd);
		return 0;
	}
	return -1;
}

HRESULT		XResourceManager::AddIndexBuffer(IIndexBuffer * pBufferToAdd)
{
	if(pBufferToAdd) {
		m_arIB.push_back(pBufferToAdd);
		return 0;
	}
	return -1;
}

HRESULT		XResourceManager::AddVertexBuffer(IVertexBuffer * pBufferToAdd)
{
	if(pBufferToAdd) {
		m_arVB.push_back(pBufferToAdd);
		return 0;
	}
	return -1;
}


HRESULT		XResourceManager::ReleaseAll()
{
	// release textures
	for(int i = 0; i < (int)m_arTextures.size(); i++) {
		m_arTextures[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arMeshes.size(); i++) {
		m_arMeshes[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arVB.size(); i++) {
		m_arVB[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arIB.size(); i++) {
		m_arIB[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arSprites.size(); i++) {
		m_arSprites[i]->OnLostDevice();
	}
	for(i = 0; i < (int)m_arFonts.size(); i++) {
		m_arFonts[i]->ReleaseResource();
	}
	return 0;
}


HRESULT		XResourceManager::RestoreAll()
{
	// restore textures
	for(int i = 0; i < (int)m_arTextures.size(); i++) {
		m_arTextures[i]->Restore();		
	}
	for(i = 0; i < (int)m_arMeshes.size(); i++) {
		m_arMeshes[i]->Restore();		
	}
	for(i = 0; i < (int)m_arVB.size(); i++) {
		m_arVB[i]->Restore();		
	}
	for(i = 0; i < (int)m_arIB.size(); i++) {
		m_arIB[i]->Restore();		
	}
	for(i = 0; i < (int)m_arSprites.size(); i++) {
		m_arSprites[i]->OnResetDevice();
	}
	for(i = 0; i < (int)m_arFonts.size(); i++) {
		m_arFonts[i]->Restore();
	}
	return 0;
}

	