#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xtexture.h"

XTexture::XTexture(void)
{
	m_lpTexture = NULL;
	m_lpDevice = NULL;
	strcpy(m_szFilePath, _S_EMPTY);
	m_dwRef = 0;
}

XTexture::~XTexture(void)
{		
	ReleaseResource();
	m_lpDevice = NULL;
}

HRESULT		XTexture::AddRef()
{
	return m_dwRef++;
}

HRESULT		XTexture::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;		
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XTexture object destroyed.");	
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

int		XTexture::ReleaseResource()
{
	TRACE("Texture %s released.", m_szFilePath);
	SAFE_RELEASE(m_lpTexture); 
	return S_OK;
}

int		XTexture::LoadTextureFromFile(LPCSTR szFileName)
{	
	IDisplay *dis = NULL;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();
	
	if(!m_lpTexture) 
	{		
		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}		
		if(D3D_OK == D3DXCreateTextureFromFile(m_lpDevice,  m_szFilePath, &m_lpTexture))
		{
			TRACE("Texture '%s' was loaded.", szFileName);
		}
		else 
		{
			TRACE("Texture '%s' wasn't loaded.", szFileName);
			return S_FALSE;
		}
		// save information to restore		
		m_iMipLevels = 1;
		// Get some info about texture
		D3DSURFACE_DESC	texdesc;
		m_lpTexture->GetLevelDesc(0, &texdesc);
		m_iWidth = texdesc.Width;
		m_iHeight = texdesc.Height;
		// Add this texture to the manager
		dis->GetResourceManager()->AddTexture(this);
		SAFE_RELEASE(dis);
		return 0;
	}
	SAFE_RELEASE(dis);
	return -1;		
}

int		XTexture::LoadTextureFromFileEx(LPCSTR szFileName, int iMipLevels, D3DFORMAT formatTextureFormat)
{	
	IDisplay *dis = NULL;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();
	
	if(!m_lpTexture) 
	{		
		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}		
		if(D3D_OK == D3DXCreateTextureFromFileEx(m_lpDevice,  m_szFilePath, 0, 0, iMipLevels, 
												D3DUSAGE_RENDERTARGET, formatTextureFormat, D3DPOOL_DEFAULT, 
												D3DX_DEFAULT , D3DX_DEFAULT, 0, NULL, NULL, &m_lpTexture))
		{
			TRACE("Texture '%s' was loaded.", szFileName);
		}
		else 
		{
			TRACE("Texture '%s' wasn't loaded.", szFileName);	
			return S_FALSE;
		}
		// save information to restore		
		m_iMipLevels = iMipLevels;
		m_formatTextureFormat = formatTextureFormat;
		// Get some info about texture
		D3DSURFACE_DESC	texdesc;
		m_lpTexture->GetLevelDesc(0, &texdesc);
		m_iWidth = texdesc.Width;
		m_iHeight = texdesc.Height;
		// Add this texture to the manager
		dis->GetResourceManager()->AddTexture(this);
		SAFE_RELEASE(dis);
		return 0;
	}
	SAFE_RELEASE(dis);
	return -1;	
}


int		XTexture::Restore()
{
	// reinit texture
	if(m_lpDevice) 
	{
		if(m_iMipLevels == 1) 
		{
			// try to create texture	
			if(D3D_OK == D3DXCreateTextureFromFile(m_lpDevice, m_szFilePath, &m_lpTexture)) 
			{
				TRACE("Texture '%s' was reloaded.", m_szFilePath);
			}
			else 
			{
				TRACE("Texture '%s' wasn't reloaded.", m_szFilePath);			
			}
		}
		else
		{
			if(D3D_OK == D3DXCreateTextureFromFileEx(m_lpDevice,  m_szFilePath, 0, 0, m_iMipLevels, 
													D3DUSAGE_RENDERTARGET, m_formatTextureFormat, D3DPOOL_DEFAULT, 
													D3DX_DEFAULT , D3DX_DEFAULT, 0, NULL, NULL, &m_lpTexture))
			{
				TRACE("Texture '%s' was reloaded.", m_szFilePath);
			}
			else 
			{
				TRACE("Texture '%s' wasn't reloaded.", m_szFilePath);			
			}
		}
	}
	return -1;
}
