// Cursor.cpp: implementation of the XCursor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#define		INPUT_API	__declspec(dllexport)

#include "..\Common\Common.h"
#include "..\Display\Common.h"

#include "interfaces.h"
#include "Cursor.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XCursor::XCursor()
{
	m_bInit = FALSE;
	//
	// Indicates static cursor
	m_nPhases = 0;
	m_lpTexture = NULL;
	m_dwRef = 0;
}

XCursor::~XCursor()
{
	m_bInit = FALSE;
	SAFE_RELEASE(m_lpTexture);
	SAFE_RELEASE(m_d3dCursor);
}

HRESULT		XCursor::AddRef()
{
	return m_dwRef++;
}

HRESULT		XCursor::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;		
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XCursor object destroyed.");	
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

/*
 *	First version of creation function
 *	Create only static cursor from bitmap
 */
HRESULT	XCursor::Create(LPCSTR csIntPath, int cx, int cy)
{
	DWORD dwRet = 1;
	IDisplay *dis = NULL;	
	LPDIRECT3DDEVICE8	lpDevice;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	lpDevice = dis->GetDevice();	
	SAFE_RELEASE(dis);

	if(!m_bInit && lpDevice) {
		
		m_iWidth = cx;
		m_iHeight = cy;
		m_lpDevice = lpDevice;

		//
		// Try to create the sprite
		dwRet = D3DXCreateSprite(m_lpDevice, &m_d3dCursor);
		if(dwRet != ERROR_SUCCESS) {
			TRACE("Cannot create the sprite due", dwRet);			
			return dwRet;
		}
		// 
		TRACE(csIntPath);
		// Create texture
		CreateDisplayObject(DISIID_ITexture, (void**)&m_lpTexture);		
		dwRet = m_lpTexture->LoadTextureFromFile(csIntPath);		
		if(dwRet != ERROR_SUCCESS) {
			ERR("Cannot create surface for cursor due", dwRet);
			return dwRet;
		}
		m_bInit = true;
		dwRet = ERROR_SUCCESS;
	}
	return dwRet;
}

/*
 *  Second version of creation function
 *  Create animate cursor
 */
HRESULT	XCursor::Create(LPCSTR csIntPath, int cx, int cy, int iSpeed, int iPhases)
{	
	DWORD dwRet = S_FALSE;
	if(!m_bInit) {
		//
		// Call standard creation
		dwRet = Create(csIntPath, cx, cy);
		if(dwRet == ERROR_SUCCESS) {
			m_iPhase = 0;			
			m_iSpeed  = iSpeed;
			m_nPhases = iPhases;		
			//
			// Ok
			m_bInit = true;
		}		
	}
	return dwRet;
}

/*
 *		Draw cursor
 */
HRESULT		XCursor::Update(D3DXVECTOR2* ptCursor)
{
	RECT rcSrc;
	DWORD dwRet = 1, newTime;
	int iHeight;
	D3DXVECTOR2	vecScaling(1.0f, 1.0f);
	if(m_bInit) {		
		//
		// For static cursors
		if(m_nPhases == 0) {
			//
			// One static place
			rcSrc.top = 0;
			rcSrc.left = 0;
			rcSrc.right = m_iWidth;
			rcSrc.bottom = m_iHeight;
		}
		//
		// Compute move source rectangle
		else {
			//
			// Compute phase, increment phase if is time
			newTime = GetTickCount();
			//
			// Get time from last update
			if((newTime - m_dwOldTime) > (DWORD)m_iSpeed) {
				//
				// If it right time, increment phase
				m_iPhase++;
				//
				// Check to overflow phases
				if(m_iPhase == m_nPhases) {
					m_iPhase = 0;
				}
				//
				// Save time of last update
				m_dwOldTime = newTime;
			}
			// Compute width and height of each field on the texture
			iHeight = m_lpTexture->Height();
			double fWidth  = double(m_lpTexture->Width()) / double(m_nPhases);
			//
			// Compute rectangle from current phase
			//
			// Static vertical postion
			rcSrc.top = 0;
			rcSrc.bottom = iHeight;
			// Variable horizontal pos.
			rcSrc.left = int(m_iPhase * fWidth);
			rcSrc.right = int(rcSrc.left + fWidth);			
			vecScaling.x = float(double(m_nPhases * m_iWidth) / double(m_lpTexture->Width()));
		}			
		//
		// Blit cursor to back surface	
		//m_d3dCursor->Begin();
		//dwRet = m_d3dCursor->Draw(m_lpTexture->GetTexture(), rcSrc, &vecScaling, NULL, NULL, ptCursor, -1);
		//m_d3dCursor->End();
	}
	return dwRet;
}
