// Tester2.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Tester2.h"

#include "../common/common.h"
#include "../display/common.h"
#include "../input/common.h"

#define		_S_DEBUG			"Common"
#define		_S_TRACETOFILE		"TraceToFile"

#define		_S_TERRAIN			"Terrain"
#define		_S_METHOD			"Method"
#define		_S_ITERCOUNT		"IterCount"
#define		_S_MAXHEIGHT		"Maxheight"
#define		_S_DISPLACEMENT		"Displacement"

// Global Variables:
HINSTANCE hInst;								// current instance

IDisplay	*g_pDisplay = NULL;
IInput		*g_pInput = NULL;
I3DFont		*g_pKeys = NULL;

IVertexBuffer	*g_pTerrainVB = NULL;
IIndexBuffer	*g_pTerrainIB = NULL;
ITexture		*g_pTerrainSurface = NULL;
ITexture		*g_pHeightMap = NULL;

D3DLIGHT8		g_theSun;

DWORD			g_dwTerrainVBSize = 0;
DWORD			g_dwTerrainIBSize = 0;
DWORD			g_dwVerticesCount = 0;
DWORD			g_dwIndicesCount = 0;
int				g_iTerrainTilesX = 0;
int				g_iTerrainTilesY = 0;
int				g_iTerrainVerticesX = 0;
int				g_iTerrainVerticesY = 0;

enum TERRAIN_METHOD
{
	TM_HEIGHTMAP = 0,
	TM_CIRCLE
};

struct	TEXTURE_PIXEL
{
	BYTE	b;
	BYTE	g;
	BYTE	r;
	BYTE	a;
};

TERRAIN_METHOD			g_iTerrainMethod = TM_HEIGHTMAP;
// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int, HWND *hWnd);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
void				Exit();
void				UpdateFrame();
void				RestoreUserObjects();
void				FillTerrainBuffers();
void				CircleAlgorithm(VERTEX **arVertices, int dimX, int dimY);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	DWORD dwRet = S_OK;
	srand(GetTickCount());

	MyRegisterClass(hInstance);

	HWND hWnd = NULL;
	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow, &hWnd)) 
	{
		return FALSE;
	}
	// inicializace logovaciho souboru, pokud je zapnuta funkce Tracetofile
	if(cmnReadSetupInt(_S_DEBUG, _S_TRACETOFILE) == 1) {
		cmnInitLog("log.txt", FALSE);
	}
	// read terrain method from ini file	 
	g_iTerrainMethod = (TERRAIN_METHOD)cmnReadSetupInt(_S_TERRAIN, _S_METHOD);	
	//
	// try to create IDisplay object	
	if(S_OK == (dwRet = CreateDisplayObject(DISIID_IDisplay, (void**) &g_pDisplay)))
	{
		try {
			g_pDisplay->Init(hWnd);	
			if(S_OK == (dwRet = CreateDisplayObject(DISIID_I3DFont, (void**) &g_pKeys)))
			{
				g_pKeys->CreateFont(15, 8, "Arial", TRUE, D3DCOLOR_ARGB(255,255,200,200));
			}
			else
			{
				TRACE("Cannot create font: %d", dwRet);
				Exit();
				return dwRet;
			}
			dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&g_pTerrainSurface);
			if(dwRet == S_OK)
			{
				dwRet = g_pTerrainSurface->LoadTextureFromFileEx("grass64.bmp", 6, g_pDisplay->GetTextureFormat());
				if(dwRet != S_OK)
				{
					XException exp("Cannot load texture for terrain surface!", dwRet);
					THROW(exp);
				} 
			}
			// set parameters according terrain method			
			dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&g_pHeightMap);
			if(dwRet == S_OK)
			{
				dwRet = g_pHeightMap->LoadTextureFromFile("default.bmp");
				if(dwRet != S_OK)
				{
					XException exp("Cannot load texture for heightmap!", dwRet);
					THROW(exp);
				}                    
				//set dim of the terrain according dim of the height map
				g_iTerrainTilesX = g_pHeightMap->Width() - 1;
				g_iTerrainTilesY = g_pHeightMap->Height() - 1;
				g_iTerrainVerticesX = g_pHeightMap->Width();
				g_iTerrainVerticesY = g_pHeightMap->Height();
			}				
			if(g_iTerrainTilesX * g_iTerrainTilesY * 2 > 65536)
			{
				XException exp("Invalid dimensions of the heightmap! Bitmap is too large.");
				THROW(exp);
			}
			dwRet = CreateDisplayObject(DISIID_IVertexBuffer, (void**) &g_pTerrainVB);
			if(dwRet == S_OK)
			{				
				g_dwTerrainVBSize = g_iTerrainVerticesX * g_iTerrainVerticesY * sizeof(VERTEX);
				g_dwVerticesCount = g_iTerrainVerticesX * g_iTerrainVerticesY;
				dwRet = g_pTerrainVB->Create(g_dwTerrainVBSize, D3DUSAGE_WRITEONLY, VERTEXFORMAT, D3DPOOL_DEFAULT);
				if(dwRet != S_OK)
				{
					XException exp("Cannot create VB for terrain!", dwRet);
					THROW(exp);
				}								
			}
			dwRet = CreateDisplayObject(DISIID_IIndexBuffer, (void**) &g_pTerrainIB);
			if(dwRet == S_OK)
			{	
				g_dwTerrainIBSize = g_iTerrainTilesX * g_iTerrainTilesY * 6 * sizeof(WORD);
				g_dwIndicesCount = g_iTerrainTilesX * g_iTerrainTilesY * 6;
				dwRet = g_pTerrainIB->Create(g_dwTerrainIBSize, D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_DEFAULT);
				if(dwRet != S_OK)
				{
					XException exp("Cannot create IB for terrain!", dwRet);
					THROW(exp);
				}
			}
			g_pDisplay->GetCamera()->InitCamera(0, g_iTerrainTilesX);
			FillTerrainBuffers();

			ZeroMemory( &g_theSun, sizeof(D3DLIGHT8) );
			g_theSun.Type = D3DLIGHT_DIRECTIONAL;
			g_theSun.Direction.x = -1.0f;
			g_theSun.Direction.y = -1.0f;
			g_theSun.Direction.z = -5.0f;
			g_theSun.Diffuse.r  = 1.0f;
			g_theSun.Diffuse.g  = 1.0f;
			g_theSun.Diffuse.b  = 1.0f;
			g_theSun.Ambient.r  = 0.3f;
			g_theSun.Ambient.g  = 0.3f;
			g_theSun.Ambient.b  = 0.3f;			
			g_pDisplay->SetLight(0, &g_theSun);
			g_pDisplay->EnableLight(0, TRUE);

		}
		catch(XException exp)
		{			
			exp.msgbox();			
			Exit();
			return exp.code();
		}	
		
	}
	else
	{
		TRACE("Fatal error: Cannot create IDisplay object: %d",dwRet);
		return dwRet;
	}

	/* Init DI system */
	if(S_OK == (dwRet = CreateInputObject(INPIID_IInput, (void**) &g_pInput)))
	{
		try {
			g_pInput->Init(hInstance, hWnd, g_pDisplay->GetResolution());
		}
		catch(XException exp)
		{
			exp.msgbox();
			Exit();
			return exp.code();
		}	
	}
	else
	{
		TRACE("Fatal error: Cannot create IInput object: %d",dwRet);
		return dwRet;
	}

	// Main message loop:
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit				
				break;                
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			// Obnova snimku
			UpdateFrame();			
		}
    }	
	Exit();
	return (int) msg.wParam;
}

void	Exit()
{
	SAFE_RELEASE(g_pHeightMap);
	SAFE_RELEASE(g_pTerrainSurface);
	SAFE_RELEASE(g_pTerrainIB);
	SAFE_RELEASE(g_pTerrainVB);
	SAFE_RELEASE(g_pDisplay);
	SAFE_RELEASE(g_pInput);
	SAFE_RELEASE(g_pKeys);
}


//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_TESTER2);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "D3DEngine Window Class";
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hWnd)
{
   hInst = hInstance; // Store instance handle in our global variable

   *hWnd = CreateWindow("D3DEngine Window Class", "D3DEngine 2", WS_OVERLAPPEDWINDOW ,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!*hWnd)
   {
      return FALSE;
   }

   ShowWindow(*hWnd, nCmdShow);
   UpdateWindow(*hWnd);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{		
	switch (message) 
	{
	case WM_KEYDOWN:    
        switch( wParam )
        {
		case VK_F1:
			g_pDisplay->RestoreDisplay();
			RestoreUserObjects();
			break;
        case VK_ESCAPE:
			PostQuitMessage(1);
			break;		
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

void	UpdateFrame()
{
	if(g_pDisplay && g_pInput)
	{
		float fFactor = cmnGetTime(TIMER_GETELAPSEDTIME1);

		g_pDisplay->UpdateBackground();
		g_pInput->ProcessInput();

		if(g_pInput->IsKeyDown(DIK_ADD , FALSE)) {
		g_pDisplay->GetCamera()->ZoomIn();
		}
		if(g_pInput->IsKeyDown(DIK_SUBTRACT , FALSE)) {
			g_pDisplay->GetCamera()->ZoomOut();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD1  , FALSE)) {
			g_pDisplay->GetCamera()->RotateLeft();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD3  , FALSE)) {
			g_pDisplay->GetCamera()->RotateRight();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD8  , FALSE)) {
			g_pDisplay->GetCamera()->IncreaseZAngle();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD2  , FALSE)) {
			g_pDisplay->GetCamera()->DecreaseZAngle();
		}

		int iMouseZone = int(g_pDisplay->GetResolution()->x * 5 / 1024); 
		if(g_pInput->IsKeyDown(DIK_UP, FALSE)) {
			g_pDisplay->GetCamera()->MoveUp();
		} else {		
		if(g_pInput->GetCursor()->y < iMouseZone) {
			g_pDisplay->GetCamera()->MoveUp();			
		}}
		if(g_pInput->IsKeyDown(DIK_LEFT  , FALSE)) {
			g_pDisplay->GetCamera()->MoveLeft();
		} else {		
		if(g_pInput->GetCursor()->x < iMouseZone) {
			g_pDisplay->GetCamera()->MoveLeft();		
		}}
		iMouseZone += int( g_pDisplay->GetResolution()->x * 32 / 1024); 
		if(g_pInput->IsKeyDown(DIK_DOWN  , FALSE)) {
			g_pDisplay->GetCamera()->MoveDown();		
		} else {
		if(g_pInput->GetCursor()->y >=  g_pDisplay->GetResolution()->y-iMouseZone) {
			g_pDisplay->GetCamera()->MoveDown();		
		}}
		if(g_pInput->IsKeyDown(DIK_RIGHT  , FALSE)) {
			g_pDisplay->GetCamera()->MoveRight();	
		} else {
		if(g_pInput->GetCursor()->x >= g_pDisplay->GetResolution()->x-iMouseZone) {
			g_pDisplay->GetCamera()->MoveRight();		
		}}
	
		g_pDisplay->GetDevice()->BeginScene();		
		g_pDisplay->GetDevice()->SetVertexShader(VERTEXFORMAT);			
		g_pDisplay->GetDevice()->SetStreamSource(0, g_pTerrainVB->GetBuffer(), sizeof(VERTEX));
		g_pDisplay->GetDevice()->SetIndices(g_pTerrainIB->GetBuffer(), 0);
		g_pDisplay->GetDevice()->SetTexture(0, g_pTerrainSurface->GetTexture());

		g_pDisplay->GetDevice()->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, g_dwVerticesCount, 0, g_dwIndicesCount / 3);
		
		g_pDisplay->GetCamera()->ProcessCamera(fFactor);
		g_pKeys->Draw("Arrows - CameraMovement\n+ ZoomIn\n- ZoomOut\nNumpad5 - CameraUp\nNumpad2 - CameraDown\nNumpad1 - RotateLeft\nNumpad3 - RotateRight\nF1 - device reset", 0, 100);
		
		g_pInput->UpdateCursor();		
		g_pDisplay->GetDevice()->EndScene();
		if(D3DERR_DEVICENOTRESET == g_pDisplay->Present())
		{
			RestoreUserObjects();		
		}
	}	
}

void RestoreUserObjects()
{
	FillTerrainBuffers();
	g_pDisplay->SetLight(0, &g_theSun);
	g_pDisplay->EnableLight(0, TRUE);
}

void	FillTerrainBuffers()
{
	VERTEX	*pVertices;
	DWORD dwRet;
	int v = 0, i = 0, x, y;
	float z1, z2;
	D3DXVECTOR3	v1, v2, v0;
	D3DXVECTOR3	s1, s2, n0;

	VERTEX	**arTerrain;
	arTerrain = new VERTEX*[g_iTerrainVerticesX];
	for(i = 0; i < g_iTerrainVerticesX; i++)
	{
		arTerrain[i] = new VERTEX[g_iTerrainVerticesY];
	}

	D3DLOCKED_RECT lr;
	g_pHeightMap->GetTexture()->LockRect(0, &lr, NULL, D3DLOCK_READONLY);
	TEXTURE_PIXEL * data = (TEXTURE_PIXEL*)lr.pBits;
	i = 0;	
	for(y = 0; y < g_iTerrainVerticesY; y++)
	{
		for(x = 0; x < g_iTerrainVerticesX; x++)
		{				
			arTerrain[x][y].vecPos = D3DXVECTOR3(float(x), float(y), float(data[i].a)/255.0f*10.0f);
						
			arTerrain[x][y].dwDiffuse = D3DCOLOR_ARGB(128,data[i].r,data[i].g,data[i].b);
			arTerrain[x][y].tu1 = (x % 2) ? 1.0f : 0.0f;
			arTerrain[x][y].tv1 = (y % 2) ? 1.0f : 0.0f;	
			arTerrain[x][y].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
			i++;
		}
	}

	g_pHeightMap->GetTexture()->UnlockRect(0);

	if(g_iTerrainMethod == TM_CIRCLE)
	{
		CircleAlgorithm(arTerrain, g_iTerrainVerticesX, g_iTerrainVerticesY);
	}

	for(y = 0; y < g_iTerrainVerticesY; y++)
	{
		for(x = 0; x < g_iTerrainVerticesX; x++)
		{
			v0 = arTerrain[x][y].vecPos;
			if(x < g_iTerrainTilesX)
			{
				z1 = arTerrain[x+1][y].vecPos.z;				
			}
			else
			{
				z1 = arTerrain[x-1][y].vecPos.z;				
			}
			
			if(y < g_iTerrainTilesY)
			{
				z2 = arTerrain[x][y+1].vecPos.z;				
			}
			else
			{
				z2 = arTerrain[x][y-1].vecPos.z;				
			}
			v1 = D3DXVECTOR3(float(x+1), float(y), z1);
			v2 = D3DXVECTOR3(float(x), float(y+1), z2);
			s1 = v0 - v1;
			s2 = v0 - v2;		
			
			// create normal vector
			D3DXVec3Cross(&n0, &s1, &s2);
			D3DXVec3Normalize(&n0, &n0);
			arTerrain[x][y].vecNormal = n0;
		}
	}	

	dwRet = g_pTerrainVB->GetBuffer()->Lock(0, 0, (BYTE**)&pVertices, 0);
	v = 0;
	for(y = 0; y < g_iTerrainVerticesY; y++)
	{
		for( x = 0; x < g_iTerrainVerticesX; x++)
		{
			pVertices[v] = arTerrain[x][y];
			v++;
		}
	}
	dwRet = g_pTerrainVB->GetBuffer()->Unlock();		

	WORD *pIndices;
	dwRet = g_pTerrainIB->GetBuffer()->Lock(0, 0, (BYTE**)&pIndices, 0);				
	i = 0;	
	for(y = 0; y < g_iTerrainTilesY; y++)
	{
		for(x = 0; x < g_iTerrainTilesX; x++)
		{
			pIndices[i + 0] = x + y * g_iTerrainVerticesX;
			pIndices[i + 1] = (x+1) + y * g_iTerrainVerticesX;
			pIndices[i + 2] = x + (y+1) * g_iTerrainVerticesX;			
			i += 3;
			pIndices[i + 0] = (x+1) + y * g_iTerrainVerticesX;
			pIndices[i + 1] = (x+1) + (y+1) * g_iTerrainVerticesX;
			pIndices[i + 2] = x + (y+1) * g_iTerrainVerticesX;			
			i += 3;				
		}
	}
	dwRet = g_pTerrainIB->GetBuffer()->Unlock();
	
	for(i = 0; i < g_iTerrainVerticesX; i++)
	{
		SAFE_DELETE_ARRAY(arTerrain[i]);
	}
	SAFE_DELETE_ARRAY(arTerrain);
}

void	CircleAlgorithm(VERTEX **arVertices, int dimX, int dimY)
{
	int iCircleX, iCircleY;
	char per[50];
	double pd, circlesize, distance, disp, fPer = 0.0f;
	int	iIterCount = cmnReadSetupInt(_S_TERRAIN, _S_ITERCOUNT);
	int iDisplacement = cmnReadSetupInt(_S_TERRAIN, _S_DISPLACEMENT);
	int iMaxHeight = cmnReadSetupInt(_S_TERRAIN, _S_MAXHEIGHT) * 10;
	// reset terrain
	for(int y = 0; y < dimY; y++)
	{
		for(int x = 0; x < dimX; x++)
		{
			arVertices[x][y].vecPos.z = -10.0f;
		}
	}
	for(int i = 0; i < iIterCount; i++)
	{
		// choose random circle center
		iCircleX = rand() % dimX;
		iCircleY = rand() % dimY;

		circlesize = (rand() % iMaxHeight) / 10.0f + 10.0f;
		disp = (rand() % iDisplacement) / 10.0f;

		// apply displacement
		for(int y = 0; y < dimY; y++)
		{
			for(int x = 0; x < dimX; x++)
			{
				// compute distance current point and center point
				distance = sqrt(pow(arVertices[iCircleX][iCircleY].vecPos.x - arVertices[x][y].vecPos.x, 2) + pow(arVertices[iCircleX][iCircleY].vecPos.y - arVertices[x][y].vecPos.y, 2)); 

				pd = distance * 2 / circlesize;

				if(fabs(pd) <= 1.0f)
				{
					arVertices[x][y].vecPos.z += float(disp / 2 + cos(pd * D3DX_PI) * disp / 2);
				}
			}
		}
		fPer += (100.0f/iIterCount);
		g_pDisplay->UpdateBackground();
		g_pDisplay->GetDevice()->BeginScene();
		sprintf(per, "Generating terrain: %3.1lf%%", fPer);
		g_pKeys->Draw(per, 0, 50, D3DCOLOR_ARGB(255,255,255,0));
		g_pDisplay->GetDevice()->EndScene();
		g_pDisplay->Present();
	}
}
