{ ##
  @PROJECT_NAME             Message Dialog Components
  @PROJECT_DESC             Components that provide customisable message dialog
                            boxes that wrap the Windows MessageBoxIndirect API
                            call and the Delphi VCL CreateMessageDialog
                            function.
  @FILE                     PJMessageDialogDsgn.pas
  @COMMENTS                 Design time code for Message Dialog Components that
                            provides a multi line string property editor.
  @LEGAL_NOTICE             These components and source code are placed in the
                            public domain. They may be freely copied and
                            circulated on a not-for-profit basis providing that
                            the code is unmodified and this notice and
                            information about the author and his copyright
                            remains attached to the source code.
  @AUTHOR                   Peter D Johnson, LLANARTH, Ceredigion, Wales, UK.
  @OWNER                    delphiDabbler
  @EMAIL                    peter.johnson@openlink.org
  @WEBSITE                  http://www.delphidabbler.com/
  @COPYRIGHT                 Peter D Johnson, 2001-2003.
  @HISTORY(
    @REVISION(
      @VERSION              1.0
      @DATE                 05/10/2003
      @COMMENTS             Original version
    )
  )
}


unit PJMessageDialogDsgn;


interface


// Determine compiler
{$IFDEF VER140}
  {$DEFINE DELPHI6ANDUP}
{$ENDIF}
{$IFDEF VER150}
  {$DEFINE DELPHI6ANDUP}
  {$DEFINE DELPHI7ANDUP}
{$ENDIF}


uses
  // Delphi
  StdCtrls, ExtCtrls, Classes, Controls, Forms,
  {$IFDEF DELPHI6ANDUP}
    DesignIntf, DesignEditors;
  {$ELSE}
    DsgnIntf;
  {$ENDIF}


type

  {
  TPJMsgDlgVCLTextEdDlg:
    Dialog box used to edit strings for message dialog's message. Called by
    property editor.
  }
  TPJMsgDlgVCLTextEdDlg = class(TForm)
    edText: TMemo;
    pnlButton: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    procedure btnHelpClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  end;

  {
  TPJMsgDlgTextPE:
    Property editor for message dialog box Text property.
  }
  TPJMsgDlgTextPE = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
      {Tell object inspector that editor displays a dlg box in addition to
      standard string editor}
    procedure Edit; override;
      {Display the property editor dialog box to edit the property}
  end;


procedure Register;
  {Registers components and property editor}


implementation


uses
  // Delphi
  Windows,
  // Project
  PJMessageDialog;


{$R *.DFM}


procedure Register;
  {Registers components and property editor}
begin
  // Register components
  RegisterComponents(
    'DelphiDabbler',
    [TPJMessageDialog, TPJWinMsgDlg, TPJVCLMsgDlg]
  );
  // Register Text property editor for each component
  RegisterPropertyEditor(
    TypeInfo(string),             // type information about property we edit
    TPJMessageDialog,             // use for this component
    'Text',                       // use for Text property
    TPJMsgDlgTextPE);             // property editor class
  RegisterPropertyEditor(
    TypeInfo(string),             // type information about property we edit
    TPJVCLMsgDlg,                 // use for this component
    'Text',                       // use for Text property
    TPJMsgDlgTextPE);             // property editor class
  RegisterPropertyEditor(
    TypeInfo(string),             // type information about property we edit
    TPJWinMsgDlg,                 // use for this component
    'Text',                       // use for Text property
    TPJMsgDlgTextPE);             // property editor class
end;


{ TPJMsgDlgTextPE }

procedure TPJMsgDlgTextPE.Edit;
  {Display the property editor dialog box to edit the property}
begin
  with TPJMsgDlgVCLTextEdDlg.Create(Application) do
    try
      edText.Text := GetStrValue;
      if ShowModal = mrOK then
        SetStrValue(edText.Text);
    finally
      Free;
    end;
end;

function TPJMsgDlgTextPE.GetAttributes: TPropertyAttributes;
  {Tell object inspector that editor displays a dlg box in addition to standard
  string editor}
begin
  Result := inherited GetAttributes + [paDialog];
end;


{ TPJMsgDlgVCLTextEdDlg }

procedure TPJMsgDlgVCLTextEdDlg.btnHelpClick(Sender: TObject);
  {Displays help on using the property editor}
const
  // Keyword for help topic about property editor in component help file
  KW: PChar = 'PJMessageDialog Text string property editor';
begin
  Application.HelpCommand(HELP_KEY, DWORD(KW));
end;

procedure TPJMsgDlgVCLTextEdDlg.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
  {Traps F1 key press and displays property editor's help topic}
begin
  if Key = VK_F1 then btnHelp.Click;
end;

end.
