#ifndef LINSPOJSEZNAM_H
  #define LINSPOJSEZNAM_H

  template<class T> class CPrvek {
		private:
			T m_Data;
			CPrvek<T> *m_lpDalsi;
		public:
			CPrvek() : m_lpDalsi(NULL) { }
			CPrvek(const T& _Data) : m_Data(_Data), m_lpDalsi(NULL) { }

			T VratData() const { return m_Data; }
			CPrvek<T>* VratDalsi() const { return m_lpDalsi; }
			void NastavDalsi(CPrvek<T> *_lpDalsi) { m_lpDalsi = _lpDalsi; }
			void NastavData(const T& _Data) { m_Data = _Data; }
  };

  template<class T> class CLinSpojSeznam {
		private:
			CPrvek<T> *m_lpPrvni, *m_lpPosledni;
		public:
			CLinSpojSeznam() : m_lpPrvni(NULL), m_lpPosledni(NULL) { }
			~CLinSpojSeznam() { if(!JePrazdny()) { SmazSeznam(); } }

			bool VytvorPrazdny();

			bool SmazSeznam();

			bool JePrazdny() { return (m_lpPrvni == m_lpPosledni); }

			bool VlozNaKonec(const T& _Data);	// vklada na konec seznamu
			bool VlozNaZacatek(const T& _Data);	// vklada na zacatek seznamu

			CPrvek<T>* Najdi(const T& _Data);	// vrati ukazatel na nalezeny prvek nebo NULL
			CPrvek<T>* NajdiBezZarazky(const T& _Data);
			CPrvek<T>* Najdi(const T& _Data, unsigned int _k);

			bool VlozZa(CPrvek<T>* _lpPrvek, const T& _Data);
			bool VlozPred(CPrvek<T>* _lpPrvek, const T& _Data);
			
			bool SmazZa(CPrvek<T>* _lpPrvek);
			bool SmazPrvek(CPrvek<T>* _lpPrvek);
			bool SmazPrvekNePosledni(CPrvek<T>* _lpPrvek);

			CPrvek<T>* NajdiNejmensi(CPrvek<T>* _lpPrvek);
			void SetridVyberem();
			
			void Vypis();
  };

  template<class T> bool CLinSpojSeznam<T>::VytvorPrazdny()
  {
	  bool bNavrat = false;

	  m_lpPrvni = new CPrvek<T>;	// konstruktor automaticky nastavi m_lpDalsi na NULL, jinak bychom to
									// museli udelat my
	  if(m_lpPrvni)
	  {
		  m_lpPosledni = m_lpPrvni;	// zacatek je i koncem
		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::VlozNaKonec(const T& _Data)
  {
	  bool bNavrat = false;

	  if(m_lpPosledni)	// pokud existuje zarazka
	  {
		  // vytvorime novy konec (prazdny prvek)- musime dynamicky
		  CPrvek<T> *lpNovy = new CPrvek<T>;

		  // ulozime data do posledniho prvku
		  m_lpPosledni->NastavData(_Data);
		  m_lpPosledni->NastavDalsi(lpNovy);

		  m_lpPosledni = lpNovy;	// poslednim prvkem je nove vytvorena zarazka
		  		  		  
		  bNavrat = true;

		  // pozor prvek novy zde nesmime uvolnit !!!
	  }

	  return bNavrat;
  }

  template<class T> void CLinSpojSeznam<T>::Vypis()
  {
	  cout << "Stav seznamu: ";

	  if(!JePrazdny())
	  {
		  CPrvek<T> *lpVypisovany = m_lpPrvni;
		  
		  while(lpVypisovany != m_lpPosledni)	// data zarazky pro nas nejsou dulezita
		  {
			  cout << lpVypisovany->VratData() << " ";

			  lpVypisovany = lpVypisovany->VratDalsi();
		  }
	  }
	  else
	  {
		  cout << " PRAZDNY";
	  }

	  cout << endl;
  }

  template<class T> bool CLinSpojSeznam<T>::SmazSeznam()
  {
	  bool bNavrat = false;

	  if(!JePrazdny())
	  {
		  CPrvek<T> *lpMazany;
		  
		  while(m_lpPrvni)
		  {
			  lpMazany = m_lpPrvni;
			  m_lpPrvni = m_lpPrvni->VratDalsi();	// posun na dalsi prvek
			  delete lpMazany;
		  }

		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> CPrvek<T>* CLinSpojSeznam<T>::Najdi(const T& _Data)
  {
	  CPrvek<T> *lpNavrat = NULL; // implicitne nenalezeno

      if(!JePrazdny())
      {
		  CPrvek<T> *lpZkoumany = m_lpPrvni;

          m_lpPosledni->NastavData(_Data); // ulozime data do zarazky
          while(lpZkoumany->VratData() != _Data)
          {
              lpZkoumany = lpZkoumany->VratDalsi();
          }

          if(lpZkoumany != m_lpPosledni)
          {
              lpNavrat = lpZkoumany; // nalezli jsme prvek
          }
      }

      return lpNavrat;
  }

  template<class T> CPrvek<T>* CLinSpojSeznam<T>::NajdiBezZarazky(const T& _Data)
  {
	  CPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  CPrvek<T> *lpZkoumany = m_lpPrvni;

		  while(lpZkoumany != m_lpPosledni)
		  {
			  if(_Data == lpZkoumany->VratData())
			  {
				  lpNavrat = lpZkoumany;
				  break;
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> CPrvek<T>* CLinSpojSeznam<T>::Najdi(const T& _Data, unsigned int _k)
  {
	  CPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  CPrvek<T> *lpZkoumany = m_lpPrvni;

		  while((lpZkoumany != m_lpPosledni) && (_k != 0))
		  {
			  if(_Data == lpZkoumany->VratData())
			  {
				  if(--_k == 0)
				  { 
					  lpNavrat = lpZkoumany;
					  break;
				  }
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::VlozNaZacatek(const T& _Data)
  {
	  bool bNavrat = false;

	  if(m_lpPrvni)	// pokud je pred co vkladat
	  {
		  CPrvek<T> *lpNovy = new CPrvek<T>(_Data);

		  if(lpNovy)
		  {
			  lpNovy->NastavDalsi(m_lpPrvni);	// novy prvek bude ukazovat na puvodni zacatek

			  m_lpPrvni = lpNovy;	// a presuneme novy zacatek

			  bNavrat = true;
		  }

		  // opet novy prvek nemazeme
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::VlozZa(CPrvek<T>* _lpPrvek, const T& _Data)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  CPrvek<T> *lpNovy = new CPrvek<T>(_Data);

		  if(lpNovy)
		  {
			  lpNovy->NastavDalsi(_lpPrvek->VratDalsi());
			  _lpPrvek->NastavDalsi(lpNovy);

			  bNavrat = true;
		  }

		  // klasicky nesmime prvek smazat
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::VlozPred(CPrvek<T>* _lpPrvek, const T& _Data)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  CPrvek<T> *lpNovy = new CPrvek<T>(_lpPrvek->VratData());

		  if(lpNovy)
		  {
			  _lpPrvek->NastavData(_Data);	// data jsou prohozena
			  lpNovy->NastavDalsi(_lpPrvek->VratDalsi());
			  _lpPrvek->NastavDalsi(lpNovy);

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::SmazZa(CPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && (_lpPrvek != m_lpPosledni) && _lpPrvek)	// nemuzeme mazat zarazku
	  {
		  CPrvek<T> *lpMazany = _lpPrvek->VratDalsi();

		  if(lpMazany)	// radeji zkontrolujeme jestli tam vubec je prvek
		  {
			  _lpPrvek->NastavDalsi(lpMazany->VratDalsi());	// vypustime mazany prvek

			  delete(lpMazany);

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::SmazPrvek(CPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  CPrvek<T> *lpMazany = m_lpPrvni;
		  CPrvek<T> *lpPredchudce = NULL;

		  while(lpMazany != _lpPrvek)
		  {
			  lpPredchudce = lpMazany;

			  lpMazany = lpMazany->VratDalsi();
		  }

		  // nyni mame predchudce, takze smazeme jeho naslednika
		  if(lpPredchudce)
		  {
			  SmazZa(lpPredchudce);

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CLinSpojSeznam<T>::SmazPrvekNePosledni(CPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && (_lpPrvek->VratDalsi() != m_lpPosledni) && _lpPrvek)	// nelze mazat posledni prvek
	  {
		  CPrvek<T>* lpMazany = _lpPrvek->VratDalsi();	// budeme mazat nasledovnika

		  if(lpMazany)
		  {
			  _lpPrvek->NastavData(lpMazany->VratData());	// presuneme data
			  _lpPrvek->NastavDalsi(lpMazany->VratDalsi());	// preskocime nasledovnika prvku
			  
			  delete(lpMazany);

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> CPrvek<T>* CLinSpojSeznam<T>::NajdiNejmensi(CPrvek<T>* _lpPrvek)
  {
	  CPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  if(!_lpPrvek)
		  {
			  _lpPrvek = m_lpPrvni;
		  }

		  CPrvek<T> *lpZkoumany = _lpPrvek;

		  // prvni prvek je nejdriv nejmensi
		  T Data = lpZkoumany->VratData();
		  lpNavrat = lpZkoumany;


		  while(lpZkoumany != m_lpPosledni)
		  {
			  if(Data > lpZkoumany->VratData())	// pokud nalezneme mensi
			  {
				  lpNavrat = lpZkoumany;	// tak jsme prozatim nalezli nejmensi prvek
				  Data = lpZkoumany->VratData();	// a mame novou nejmensi hodnotu
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();	// posun na dalsi
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> void CLinSpojSeznam<T>::SetridVyberem()
  {
	  if(!JePrazdny())
	  {
		  CPrvek<T> *lpPosledni, *lpNejmensi;
		  T Vymena;

		  lpPosledni = m_lpPrvni;

		  while(lpPosledni->VratDalsi() != m_lpPosledni)
		  {
			  lpNejmensi = NajdiNejmensi(lpPosledni->VratDalsi());

			  if(lpNejmensi && (lpNejmensi->VratData() < lpPosledni->VratData()))
			  {
				  Vymena = lpPosledni->VratData();
				  lpPosledni->NastavData(lpNejmensi->VratData());
				  lpNejmensi->NastavData(Vymena);
			  }

			  lpPosledni = lpPosledni->VratDalsi();
		  }
	  }
  }
  
#endif
