(*******************************************
  File Uploader example
  (c)Copyright Delphi Web Development, 2000
  http://delphiwebdeveloper.virtualave.net
  Modified by Serhiy Perevoznyk
********************************************)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, HTTPApp;

type
  TwmUpload = class(TWebModule)
    procedure WebModule1WebActionItem1Action(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  private
    function ReadMultipartRequest(const Boundary: string; ARequest: string;
      var AHeader: TStrings; var Data: string): string;
  public
    { Public declarations }
  end;

var
  wmUpload: TwmUpload;

implementation

{$R *.DFM}

const
  MaxReadBlockSize = 8192;
  UploadPath = 'C:\temp\';

procedure TwmUpload.WebModule1WebActionItem1Action(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  ByteToRead, ReadedBytes, RSize: LongInt;
  AllContent, BufferStr, Boundary, Data: string;
  Header, HList: TStrings;
  Buffer: PChar;
  OutStream: TFileStream;
begin
  if (Pos('multipart/form-data', LowerCase(Request.ContentType)) > 0) and
    (Pos('boundary', LowerCase(Request.ContentType)) > 0) then
  begin
    Header := TStringList.Create;
    try
      ExtractHeaderFields([';'], [' '], PChar(Request.ContentType), Header,
        False, False);
      Boundary := Header.Values['boundary'];
    finally
      Header.Free;
    end;
    AllContent := Request.Content;
    ByteToRead := Request.ContentLength - Length(AllContent);
    try
      while ByteToRead > 0 do
      begin
        RSize := MaxReadBlockSize;
        if RSize > ByteToRead then
          RSize := ByteToRead;
        GetMem(Buffer, RSize);
        try
          ReadedBytes := Request.ReadClient(Buffer^, RSize);
          SetString(BufferStr, Buffer, ReadedBytes);
          AllContent := AllContent + BufferStr;
        finally
          FreeMem(Buffer, RSize);
        end;
        ByteToRead := Request.ContentLength - Length(AllContent);
       end;
      Response.Content := Response.Content + Format('ContentType = %s<br>' +
        'ContentLength = %d<br>Readed Bytes = %d<br>Boundary = %s<hr>',
        [Request.ContentType, Request.ContentLength, Length(AllContent),
        boundary]);

      if Request.ContentLength = Length(AllContent) then
        while Length(AllContent) > Length('--' + Boundary + '--' + #13#10) do
        begin
          Header := TStringList.Create;
          HList := TStringList.Create;
          try
            AllContent := ReadMultipartRequest('--' + Boundary, AllContent,
              Header, Data);
            ExtractHeaderFields([';'], [' '],
              PChar(Header.Values['Content-Disposition']), HList, False, True);
            if (Header.Values['Content-Type'] <> '') and (Data <> '') then
            begin
              OutStream:=TFileStream.Create(UploadPath +
                ExtractFileName(HList.Values['filename']), fmCreate);
              try
                try
                  OutStream.WriteBuffer(Pointer(Data)^, Length(Data));
                  Response.Content := Response.Content +
                    Format('<p>File <b>%s</b> saved.',
                      [ExtractFileName(HList.Values['filename'])]);
                except
                  Response.Content := Response.Content +
                    Format('<p>Unable to save file <b>%s</b>.', [UploadPath +
                    ExtractFileName(HList.Values['filename'])]);
                end;
              finally
                OutStream.Free;
              end
            end
            else
              Response.Content := Response.Content + Format('<p>Field <b>%s</b> = %s',
                [HList.Values['name'], Data]);
          finally
            Header.Free;
            HList.Free;
          end;
        end;
    except
      on E: Exception do
        Response.Content := Response.Content + '<p>' + E.Message;
    end;
  end
  else
    Response.Content := 'Content-type is not "multipart/form-data".';
end;

function TwmUpload.ReadMultipartRequest(const Boundary: string;
  ARequest: string; var AHeader: TStrings; var Data: string): string;
var
  Req, RHead: string;
  i: Integer;
begin
  Result := '';
  AHeader.Clear;
  Data := '';
  if (Pos(Boundary, ARequest) < Pos (Boundary + '--', ARequest))
    and (Pos(Boundary, ARequest) = 1) then
  begin
    Delete(ARequest, 1, Length(Boundary) + 2);
    Req := Copy(ARequest, 1, Pos(Boundary, ARequest) - 3);
    Delete(ARequest, 1, Length(Req) + 2);
    RHead := Copy(Req, 1, Pos(#13#10#13#10, Req)-1);
    Delete(Req, 1, Length(RHead) + 4);
    AHeader.Text := RHead;
    for i := 0 to AHeader.Count - 1 do
      if Pos(':', AHeader.Strings[i]) > 0 then
        AHeader.Strings[i] := Trim(Copy(AHeader.Strings[i], 1,
          Pos(':', AHeader.Strings[i])-1)) + '=' + Trim(Copy(AHeader.Strings[i],
          Pos(':', AHeader.Strings[i])+1, Length(AHeader.Strings[i]) -
          Pos(':', AHeader.Strings[i])));
    Data := Req;
    Result := ARequest;
  end
end;

end.
