{*******************************************************}
{                                                       }
{       psvCGI unit for developing small and fast       }
{             CGI scrits  using Delphi                  }
{                                                       }
{ Author:                                               }
{ Serhiy Perevoznyk                                     }
{ serge_perevoznyk@hotmail.com                          }
{                                                       }
{     Use, modification and distribution is allowed     }
{without limitation, warranty, or liability of any kind.}
{                                                       }
{*******************************************************}

unit psvCGI;

interface

uses Windows, SysUtils, Classes;

const
  CRLF = #13#10;

  //Request fields
  COMSPEC : string = '';
  WINDIR : string = '';
  SYSTEMROOT : string = '';
  SERVER_ADMIN : string = '';
  AUTH_TYPE : string = '';
  CONTENT_LENGTH: integer = 0;
  CONTENT_TYPE: string = '';
  GATEWAY_INTERFACE: string = '';
  PATH_INFO: string = '';
  PATH_TRANSLATED: string = '';
  REMOTE_ADDR: string = '';
  REMOTE_HOST: string = '';
  REMOTE_IDENT: string = '';
  REMOTE_USER: string = '';
  REQUEST_METHOD: string = '';
  SCRIPT_NAME: string = '';
  SERVER_NAME: string = '';
  SERVER_PORT: string = '';
  SERVER_PROTOCOL: string = '';
  SERVER_SOFTWARE: string = '';
  QUERY_STRING: string = '';
  HTTP_ACCEPT: string = '';
  HTTP_USER_AGENT:string = '';
  HTTP_REFERER: string = '';
  HTTP_COOKIE: string = '';
  COOKIES: string = '';
  BODY : string = '';

  //Response
  ReasonString : string = '';
  StatusCode   : integer = 200;
  ContentType  : string = 'text/html';
  Content      : string = '';

procedure InitScript;
procedure DoneScript;

implementation

var
 Environ : TStringList;

function GetEnvVariable(AName : string) : string;
begin
  Result := '';
  if Assigned(Environ) then
   Result := Environ.Values[AName];
    if Result = '' then
     Result := Environ.Values['HTTP_'+AName];
end;

procedure InitScript;
var
  EnvStrings: PChar;
  S: string;
  i : integer;
  Ch : Char;
begin
  try
   Environ := TStringList.Create;
   EnvStrings := GetEnvironmentStrings;
   while EnvStrings[0] <> #0 do
    begin
      S := StrPas(EnvStrings);
      Environ.Add(S);
      Inc(EnvStrings,StrLen(EnvStrings)+1);
    end;

     SYSTEMROOT        := GetEnvVariable('SYSTEMROOT');
     WINDIR            := GetEnvVariable('WINDIR');
     COMSPEC           := GetEnvVariable('COMSPEC');
     SERVER_ADMIN      := GetEnvVariable('SERVER_ADMIN');
     AUTH_TYPE         := GetEnvVariable('AUTH_TYPE');
     CONTENT_TYPE      := GetEnvVariable('CONTENT_TYPE');
     GATEWAY_INTERFACE := GetEnvVariable('GATEWAY_INTERFACE');
     PATH_INFO       := GetEnvVariable('PATH_INFO');
     PATH_TRANSLATED := GetEnvVariable('PATH_TRANSLATED');
     REMOTE_ADDR     := GetEnvVariable('REMOTE_ADDR');
     REMOTE_HOST     := GetEnvVariable('REMOTE_HOST');
     REMOTE_IDENT    := GetEnvVariable('REMOTE_IDENT');
     REMOTE_USER     := GetEnvVariable('REMOTE_USER');
     REQUEST_METHOD  := GetEnvVariable('REQUEST_METHOD');
     SCRIPT_NAME     := GetEnvVariable('SCRIPT_NAME');
     SERVER_NAME     := GetEnvVariable('SERVER_NAME');
     SERVER_PORT     := GetEnvVariable('SERVER_PORT');
     SERVER_PROTOCOL := GetEnvVariable('SERVER_PROTOCOL');
     SERVER_SOFTWARE := GetEnvVariable('SERVER_SOFTWARE');
     HTTP_ACCEPT     := GetEnvVariable('HTTP_ACCEPT');
     HTTP_USER_AGENT := GetEnvVariable('HTTP_USER_AGENT');
     HTTP_REFERER    := GetEnvVariable('HTTP_REFERER');
     HTTP_COOKIE     := GetEnvVariable('HTTP_COOKIE');
     QUERY_STRING    := GetEnvVariable('QUERY_STRING');
     CONTENT_LENGTH  := StrToIntDef(GetEnvVariable('Content_Length'),0);
     if (CompareText(REQUEST_METHOD,'POST') = 0) then
      begin
       	if (CONTENT_LENGTH > 0) then
          begin
            for i := 1 to Content_Length do
              begin
	         Read(ch);
                 Body := Body + ch;
		end;
           end;
      end;
  except
    writeln('500 Internal Server Error');
    Halt;
  end;
end;

procedure DoneScript;
var
  StatusString: string;
  Headers: string;

  procedure AddHeaderItem(const Item, FormatStr: string);
  begin
    if Item <> '' then
      Headers := Headers + Format(FormatStr, [Item]);
  end;

begin
  try
    if Assigned(Environ) then
     Environ.Free;
    if (ReasonString <> '') and (StatusCode > 0) then
      StatusString := Format('%d %s', [StatusCode, ReasonString])
    else StatusString := '200 OK';
    AddHeaderItem(StatusString, 'Status: %s'+CRLF);
    AddHeaderItem(ContentType, 'Content-Type: %s'+CRLF);
    AddHeaderItem('no-cache','Pragma: %s'+CRLF);
    AddHeaderItem('no-cache','Cache-Control: %s'+CRLF);
    Headers := Headers + 'Content:'+CRLF+CRLF;
    Write(Headers);
    Write(Content);
  except
  end;
end;

initialization
  Environ := nil;
end.
