//*****************************************************************************
//
//    System          : jmsAPI
//    Category        : JMS template classes
//    Source          : JMS API documentation 
//
//    Copyright (c), Leonov Alexander
//
//    File version $Revision:1.0$ at $Date:24.11.2003 13:40:34$
//
//    Author : Leonov Alexander
//    web    : http://home.ural.ru/~tigra/
//    e-mail : shura@ol405.paco.net
//
//*****************************************************************************

unit jmsClass;

interface

uses
  SysUtils, Classes, jmsIntf, jmsException;

type
  TQueueRequestor = class (TObject)
  public
    {{
    Constructor for the QueueRequestor class
    
    This implementation assumes the session parameter to be non-transacted, 
    with a delivery mode of either AUTO_ACKNOWLEDGE or DUPS_OK_ACKNOWLEDGE.
    
    Parameters:
    session - the QueueSession the queue belongs to
    queue - the queue to perform the request/reply call on
    Throws:
    JMSException - if the JMS provider fails to create the QueueRequestor due 
    to some internal error.
    InvalidDestinationException - if an invalid queue is specified.
    }
    constructor QueueRequestor(session : IQueueSession; queue : IQueue);
    {{
    Closes the QueueRequestor and its session. 
    Since a provider may allocate some resources on behalf of a QueueRequestor 
    outside the Java virtual machine, clients should close them when they are 
    not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough. 
    
    Note that this method closes the QueueSession object passed to the 
    QueueRequestor constructor.
    
    Throws:
    JMSException - if the JMS provider fails to close the QueueRequestor due to 
    some internal error.
    }
    procedure close;
    {{
    Sends a request and waits for a reply. The temporary queue is used for the 
    JMSReplyTo destination, and only one reply per request is expected.
    Parameters:
    message - the message to send
    Returns:
    the reply message
    Throws:
    JMSException - if the JMS provider fails to complete the request due to 
    some internal error.
    }
    function request(Message : IMessage): IMessage;
  end;
  
  TTopicRequestor = class (TObject)
  public
    {{
    Constructor for the TopicRequestor class. 
    This implementation assumes the session parameter to be non-transacted, 
    with a delivery mode of either AUTO_ACKNOWLEDGE or DUPS_OK_ACKNOWLEDGE.
    
    Parameters:
    session - the TopicSession the topic belongs to
    topic - the topic to perform the request/reply call on
    Throws:
    JMSException - if the JMS provider fails to create the TopicRequestor due 
    to some internal error.
    InvalidDestinationException - if an invalid topic is specified.
    }
    constructor TopicRequestor(session : ITopicSession; topic : ITopic);
    {{
    Closes the TopicRequestor and its session. 
    Since a provider may allocate some resources on behalf of a TopicRequestor 
    outside the Java virtual machine, clients should close them when they are 
    not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough. 
    
    Note that this method closes the TopicSession object passed to the 
    TopicRequestor constructor.
    
    Throws:
    JMSException - if the JMS provider fails to close the TopicRequestor due to 
    some internal error.
    }
    procedure close;
    {{
    Sends a request and waits for a reply. The temporary topic is used for the 
    JMSReplyTo destination; the first reply is returned, and any following 
    replies are discarded.
    Parameters:
    message - the message to send
    Returns:
    the reply message
    Throws:
    JMSException - if the JMS provider fails to complete the request due to 
    some internal error.
    }
    function request(message : IMessage): IMessage;
  end;
  

implementation

{
******************************* TQueueRequestor ********************************
}
constructor TQueueRequestor.QueueRequestor(session : IQueueSession; queue : 
        IQueue);
begin
end;{ TQueueRequestor.QueueRequestor }

procedure TQueueRequestor.close;
begin
end;{ TQueueRequestor.close }

function TQueueRequestor.request(Message : IMessage): IMessage;
begin
end;{ TQueueRequestor.request }

{
******************************* TTopicRequestor ********************************
}
constructor TTopicRequestor.TopicRequestor(session : ITopicSession; topic : 
        ITopic);
begin
end;{ TTopicRequestor.TopicRequestor }

procedure TTopicRequestor.close;
begin
end;{ TTopicRequestor.close }

function TTopicRequestor.request(message : IMessage): IMessage;
begin
end;{ TTopicRequestor.request }


end.
