//*****************************************************************************
//
//    System          : jmsAPI
//    Category        : JMS API interfaces
//    Source          : JMS API documentation 
//
//    Copyright (c), Leonov Alexander
//
//    File version $Revision:1.0$ at $Date:24.11.2003 13:40:34$
//
//    Author : Leonov Alexander
//    web    : http://home.ural.ru/~tigra/
//    e-mail : shura@ol405.paco.net
//
//*****************************************************************************

unit jmsIntf;

interface

uses
  SysUtils, Classes, jmsException;

type
  {{
  A Destination object encapsulates a provider-specific address. The JMS API 
  does not define a standard address syntax. Although a standard address syntax 
  was considered, it was decided that the differences in address semantics 
  between existing message-oriented middleware (MOM) products were too wide to 
  bridge with a single syntax. 
  
  Since Destination is an administered object, it may contain provider-specific 
  configuration information in addition to its address. 
  
  The JMS API also supports a client's use of provider-specific address names. 
  
  Destination objects support concurrent use. 
  
  A Destination object is a JMS administered object. 
  
  JMS administered objects are objects containing configuration information 
  that are created by an administrator and later used by JMS clients. They make 
  it practical to administer the JMS API in the enterprise. 
  
  Although the interfaces for administered objects do not explicitly depend on 
  the Java Naming and Directory Interface (JNDI) API, the JMS API establishes 
  the convention that JMS clients find administered objects by looking them up 
  in a JNDI namespace. 
  
  An administrator can place an administered object anywhere in a namespace. 
  The JMS API does not define a naming policy. 
  
  It is expected that JMS providers will provide the tools an administrator 
  needs to create and configure administered objects in a JNDI namespace. JMS 
  provider implementations of administered objects should implement the 
  javax.naming.Referenceable and java.io.Serializable interfaces so that they 
  can be stored in all JNDI naming contexts. In addition, it is recommended 
  that these implementations follow the JavaBeansTM design patterns. 
  
  This strategy provides several benefits: 
  
  It hides provider-specific details from JMS clients. 
  It abstracts JMS administrative information into objects in the Java 
  programming language ("Java objects") that are easily organized and 
  administered from a common management console. 
  Since there will be JNDI providers for all popular naming services, JMS 
  providers can deliver one implementation of administered objects that will 
  run everywhere. 
  An administered object should not hold on to any remote resources. Its lookup 
  should not use remote resources other than those used by the JNDI API itself. 
  
  Clients should think of administered objects as local Java objects. Looking 
  them up should not have any hidden side effects or use surprising amounts of 
  local resources. 
  
  
  See Also: 
  Queue, Topic
  }
  IDestination = interface (IUnknown)
    ['{EF272364-BD7A-47E1-8D81-C35DCB1D1E2D}']
  end;
  
  IMessage = interface (IUnknown)
    ['{FBC796C7-487E-4B1B-BEA3-4F1BD7590ABB}']
    procedure acknowledge;
    procedure clearBody;
    procedure clearProperties;
    function getBooleanProperty(name : string): Boolean;
    function getByteProperty(name : string): Byte;
    function getDEFAULT_DELIVERY_MODE: LongInt;
    function getDEFAULT_PRIORITY: LongInt;
    function getDEFAULT_TIME_TO_LIVE: Int64;
    function getDoubleProperty(name : string): Double;
    function getFloatProperty(name : string): Single;
    function getIntProperty(name : string): LongInt;
    function getJMSCorrelationID: string;
    function getJMSCorrelationIDAsBytes: TByteArray;
    function getJMSDeliveryMode: LongInt;
    function getJMSDestination: IDestination;
    function getJMSExpiration: Int64;
    function getJMSMessageID: string;
    function getJMSPriority: LongInt;
    function getJMSRedelivered: Boolean;
    function getJMSReplyTo: IDestination;
    function getJMSTimestamp: Int64;
    function getJMSType: string;
    function getLongProperty(name : string): Int64;
    function getObjectProperty(name : string): IUnknown;
    function getPropertyNames: TStrings;
    function getShortProperty(name : string): SmallInt;
    function getStringProperty(name : string): string;
    function propertyExists(name : string): Boolean;
    procedure setBooleanProperty(name : string; Value: Boolean);
    procedure setByteProperty(name : string; Value: Byte);
    procedure setDoubleProperty(name : string; Value: Double);
    procedure setFloatProperty(name : string; Value: Single);
    procedure setIntProperty(name : string; Value: LongInt);
    procedure setJMSCorrelationID(correlationID: string);
    procedure setJMSCorrelationIDAsBytes(correlationID: TByteArray);
    procedure setJMSDeliveryMode(deliveryMode: LongInt);
    procedure setJMSDestination(Value: IDestination);
    procedure setJMSExpiration(expiration: Int64);
    procedure setJMSMessageID(const id: string);
    procedure setJMSPriority(priority: LongInt);
    procedure setJMSRedelivered(Redelivered: Boolean);
    procedure setJMSReplyTo(ReplyTo: IDestination);
    procedure setJMSTimestamp(timestamp: Int64);
    procedure setJMSType(_type: string);
    procedure setLongProperty(name : string; Value: Int64);
    procedure setObjectProperty(name : string; Value: IUnknown);
    procedure setShortProperty(name : string; Value: SmallInt);
    procedure setStringProperty(name : string; const Value: string);
    property BooleanProperty[name : string]: Boolean read getBooleanProperty 
            write setBooleanProperty;
    property ByteProperty[name : string]: Byte read getByteProperty write 
            setByteProperty;
    property DEFAULT_DELIVERY_MODE: LongInt read getDEFAULT_DELIVERY_MODE;
    property DEFAULT_PRIORITY: LongInt read getDEFAULT_PRIORITY;
    property DEFAULT_TIME_TO_LIVE: Int64 read getDEFAULT_TIME_TO_LIVE;
    property DoubleProperty[name : string]: Double read getDoubleProperty write 
            setDoubleProperty;
    property FloatProperty[name : string]: Single read getFloatProperty write 
            setFloatProperty;
    property IntProperty[name : string]: LongInt read getIntProperty write 
            setIntProperty;
    property JMSCorrelationID: string read getJMSCorrelationID write 
            setJMSCorrelationID;
    property JMSCorrelationIDAsBytes: TByteArray read 
            getJMSCorrelationIDAsBytes write setJMSCorrelationIDAsBytes;
    property JMSDeliveryMode: LongInt read getJMSDeliveryMode write 
            setJMSDeliveryMode;
    property JMSDestination: IDestination read getJMSDestination write 
            setJMSDestination;
    property JMSExpiration: Int64 read getJMSExpiration write setJMSExpiration;
    property JMSMessageID: string read getJMSMessageID write setJMSMessageID;
    property JMSPriority: LongInt read getJMSPriority write setJMSPriority;
    property JMSRedelivered: Boolean read getJMSRedelivered write 
            setJMSRedelivered;
    property JMSReplyTo: IDestination read getJMSReplyTo write setJMSReplyTo;
    property JMSTimestamp: Int64 read getJMSTimestamp write setJMSTimestamp;
    property JMSType: string read getJMSType write setJMSType;
    property LongProperty[name : string]: Int64 read getLongProperty write 
            setLongProperty;
    property ObjectProperty[name : string]: IUnknown read getObjectProperty 
            write setObjectProperty;
    property ShortProperty[name : string]: SmallInt read getShortProperty write 
            setShortProperty;
    property StringProperty[name : string]: string read getStringProperty write 
            setStringProperty;
  end;
  
  ITextMessage = interface (IMessage)
    ['{7EC548AE-5D44-4587-92B6-F2054CD03A5F}']
    function getText: string;
    procedure setText(const Value: string);
    property Text: string read getText write setText;
  end;
  
  IBytesMessage = interface (IMessage)
    ['{011A8376-1441-495E-8920-1D0DAFEC9033}']
    function readBoolean: Boolean;
    function readByte: ShortInt;
    function readBytes(var value : TByteArray): LongInt; overload;
    function readBytes(var value : TByteArray; length : LongInt): LongInt; 
            overload;
    function readChar: Char;
    function readDouble: Double;
    function readFloat: Single;
    function readInt: LongInt;
    function readLong: Int64;
    function readShort: SmallInt;
    function readUnsignedByte: Byte;
    function readUnsignedShort: Word;
    function readUTF: string;
    procedure reset;
    procedure writeBoolean(value : boolean);
    procedure writeByte(value : byte);
    procedure writeBytes(value : TByteArray); overload;
    procedure writeBytes(value : TByteArray; offset : longint; length : 
            longint); overload;
    procedure writeChar(vale : Char);
    procedure writeDouble(value : double);
    procedure writeFloat(value : single);
    procedure writeInt(value : longint);
    procedure writeLong(value : int64);
    procedure writeObject(value : IUnknown);
    procedure writeShort(value : Smallint);
    procedure writeUTF(value : string);
  end;
  
  {{
  A MapMessage object is used to send a set of name-value pairs. The names are 
  String objects, and the values are primitive data types in the Java 
  programming language. The entries can be accessed sequentially or randomly by 
  name. The order of the entries is undefined. MapMessage inherits from the 
  Message interface and adds a message body that contains a Map. 
  
  The primitive types can be read or written explicitly using methods for each 
  type. They may also be read or written generically as objects. For instance, 
  a call to MapMessage.setInt("foo", 6) is equivalent to MapMessage.setObject(
  "foo", new Integer(6)). Both forms are provided, because the explicit form is 
  convenient for static programming, and the object form is needed when types 
  are not known at compile time. 
  
  When a client receives a MapMessage, it is in read-only mode. If a client 
  attempts to write to the message at this point, a 
  MessageNotWriteableException is thrown. If clearBody is called, the message 
  can now be both read from and written to. 
  
  MapMessage objects support the following conversion table. The marked cases 
  must be supported. The unmarked cases must throw a JMSException. The 
  String-to-primitive conversions may throw a runtime exception if the 
  primitive's valueOf() method does not accept it as a valid String 
  representation of the primitive. 
  
  A value written as the row type can be read as the column type. 
  
  |        | boolean byte short char int long float double String byte[]
  |----------------------------------------------------------------------
  |boolean |    X                                            X
  |byte    |          X     X         X   X                  X
  |short   |                X         X   X                  X
  |char    |                     X                           X
  |int     |                          X   X                  X
  |long    |                              X                  X
  |float   |                                    X     X      X
  |double  |                                          X      X
  |String  |    X     X     X         X   X     X     X      X
  |byte[]  |                                                        X
  |----------------------------------------------------------------------
  
  Attempting to read a null value as a primitive type must be treated as 
  calling the primitive's corresponding valueOf(String) conversion method with 
  a null value. Since char does not support a String conversion, attempting to 
  read a null value as a char must throw a NullPointerException. 
  
  
  See Also: 
  Session.createMapMessage(), BytesMessage, Message, ObjectMessage, 
  StreamMessage, TextMessage
  }
  IMapMessage = interface (IMessage)
    ['{B10914CE-7A8E-48BC-AF8F-BEAD31ABC697}']
  end;
  
  {{
  A StreamMessage object is used to send a stream of primitive types in the 
  Java programming language. It is filled and read sequentially. It inherits 
  from the Message interface and adds a stream message body. Its methods are 
  based largely on those found in java.io.DataInputStream and 
  java.io.DataOutputStream. 
  
  The primitive types can be read or written explicitly using methods for each 
  type. They may also be read or written generically as objects. For instance, 
  a call to StreamMessage.writeInt(6) is equivalent to 
  StreamMessage.writeObject(new Integer(6)). Both forms are provided, because 
  the explicit form is convenient for static programming, and the object form 
  is needed when types are not known at compile time. 
  
  When the message is first created, and when clearBody is called, the body of 
  the message is in write-only mode. After the first call to reset has been 
  made, the message body is in read-only mode. After a message has been sent, 
  the client that sent it can retain and modify it without affecting the 
  message that has been sent. The same message object can be sent multiple 
  times. When a message has been received, the provider has called reset so 
  that the message body is in read-only mode for the client. 
  
  If clearBody is called on a message in read-only mode, the message body is 
  cleared and the message body is in write-only mode. 
  
  If a client attempts to read a message in write-only mode, a 
  MessageNotReadableException is thrown. 
  
  If a client attempts to write a message in read-only mode, a 
  MessageNotWriteableException is thrown. 
  
  StreamMessage objects support the following conversion table. The marked 
  cases must be supported. The unmarked cases must throw a JMSException. The 
  String-to-primitive conversions may throw a runtime exception if the 
  primitive's valueOf() method does not accept it as a valid String 
  representation of the primitive. 
  
  A value written as the row type can be read as the column type. 
  
  |        | boolean byte short char int long float double String byte[]
  |----------------------------------------------------------------------
  |boolean |    X                                            X
  |byte    |          X     X         X   X                  X   
  |short   |                X         X   X                  X   
  |char    |                     X                           X
  |int     |                          X   X                  X   
  |long    |                              X                  X   
  |float   |                                    X     X      X   
  |double  |                                          X      X   
  |String  |    X     X     X         X   X     X     X      X   
  |byte[]  |                                                        X
  |----------------------------------------------------------------------
  
  Attempting to read a null value as a primitive type must be treated as 
  calling the primitive's corresponding valueOf(String) conversion method with 
  a null value. Since char does not support a String conversion, attempting to 
  read a null value as a char must throw a NullPointerException. 
  
  
  See Also: 
  Session.createStreamMessage(), BytesMessage, MapMessage, Message, 
  ObjectMessage, TextMessage
  }
  IStreamMessage = interface (IMessage)
    ['{728A915B-BCEA-4F11-ABEA-6583DF75C934}']
    {{
    Reads a boolean from the stream message.
    Returns:
    the boolean value read
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readBoolean: Boolean;
    {{
    Reads a byte value from the stream message.
    Returns:
    the next byte from the stream message as a 8-bit byte
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readByte: ShortInt;
    {{
    Reads a byte array field from the stream message into the specified byte[] 
    object (the read buffer). 
    To read the field value, readBytes should be successively called until it 
    returns a value less than the length of the read buffer. The value of the 
    bytes in the buffer following the last byte read is undefined. 
    
    If readBytes returns a value equal to the length of the buffer, a 
    subsequent readBytes call must be made. If there are no more bytes to be 
    read, this call returns -1. 
    
    If the byte array field value is null, readBytes returns -1. 
    
    If the byte array field value is empty, readBytes returns 0. 
    
    Once the first readBytes call on a byte[] field value has been made, the 
    full value of the field must be read before it is valid to read the next 
    field. An attempt to read the next field before that has been done will 
    throw a MessageFormatException. 
    
    To read the byte field value into a new byte[] object, use the readObject 
    method.
    
    Parameters:
    value - the buffer into which the data is read
    Returns:
    the total number of bytes read into the buffer, or -1 if there is no more 
    data because the end of the byte field has been reached
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    See Also: 
    readObject()
    }
    function readBytes(var value : TByteArray): LongInt; overload;
    {{
    Reads a Unicode character value from the stream message.
    Returns:
    a Unicode character from the stream message
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readChar: Char;
    {{
    Reads a double from the stream message.
    Returns:
    a double value from the stream message
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readDouble: Double;
    {{
    Reads a float from the stream message.
    Returns:
    a float value from the stream message
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readFloat: Single;
    {{
    Reads a 32-bit integer from the stream message.
    Returns:
    a 32-bit integer value from the stream message, interpreted as an int
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readInt: LongInt;
    {{
    Reads a 64-bit integer from the stream message.
    Returns:
    a 64-bit integer value from the stream message, interpreted as a long
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readLong: Int64;
    {{
    Reads an object from the stream message. 
    This method can be used to return, in objectified format, an object in the 
    Java programming language ("Java object") that has been written to the 
    stream with the equivalent writeObject method call, or its equivalent 
    primitive writetype method. 
    
    Note that byte values are returned as byte[], not Byte[]. 
    
    An attempt to call readObject to read a byte field value into a new byte[] 
    object before the full value of the byte field has been read will throw a 
    MessageFormatException.
    
    Returns:
    a Java object from the stream message, in objectified format (for example, 
    if the object was written as an int, an Integer is returned)
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    See Also: 
    readBytes(byte[] value)
    }
    function readObject: TObject;
    {{
    Reads a 16-bit integer from the stream message.
    Returns:
    a 16-bit integer from the stream message
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readShort: SmallInt;
    {{
    Reads a String from the stream message.
    Returns:
    a Unicode string from the stream message
    Throws:
    JMSException - if the JMS provider fails to read the message due to some 
    internal error.
    MessageEOFException - if unexpected end of message stream has been reached.
    MessageFormatException - if this type conversion is invalid.
    MessageNotReadableException - if the message is in write-only mode.
    }
    function readString: string;
    {{
    Puts the message body in read-only mode and repositions the stream to the 
    beginning.
    Throws:
    JMSException - if the JMS provider fails to reset the message due to some 
    internal error.
    MessageFormatException - if the message has an invalid format.
    }
    procedure reset;
    {{
    Writes a boolean to the stream message. The value true is written as the 
    value (byte)1; the value false is written as the value (byte)0.
    Parameters:
    value - the boolean value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeBoolean(value : boolean);
    {{
    Writes a byte to the stream message.
    Parameters:
    value - the byte value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeByte(value : byte);
    {{
    Writes a byte array field to the stream message. 
    The byte array value is written to the message as a byte array field. 
    Consecutively written byte array fields are treated as two distinct fields 
    when the fields are read.
    
    Parameters:
    value - the byte array value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeBytes(value : TByteArray); overload;
    {{
    Writes a portion of a byte array as a byte array field to the stream 
    message. 
    The a portion of the byte array value is written to the message as a byte 
    array field. Consecutively written byte array fields are treated as two 
    distinct fields when the fields are read.
    
    Parameters:
    value - the byte array value to be written
    offset - the initial offset within the byte array
    length - the number of bytes to use
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeBytes(value : TByteArray; offset : longint; length : 
            longint); overload;
    {{
    Writes a char to the stream message.
    Parameters:
    value - the char value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeChar(vale : Char);
    {{
    Writes a double to the stream message.
    Parameters:
    value - the double value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeDouble(value : double);
    {{
    Writes a float to the stream message.
    Parameters:
    value - the float value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeFloat(value : single);
    {{
    Writes an int to the stream message.
    Parameters:
    value - the int value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeInt(value : longint);
    {{
    Writes a long to the stream message.
    Parameters:
    value - the long value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeLong(value : int64);
    procedure writeObject(value : IUnknown); overload;
    {{
    Writes an object to the stream message. 
    This method works only for the objectified primitive object types (Integer, 
    Double, Long ...), String objects, and byte arrays.
    
    Parameters:
    value - the Java object to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageFormatException - if the object is invalid.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeObject(value : TObject); overload;
    {{
    Writes a short to the stream message.
    Parameters:
    value - the short value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeShort(value : Smallint);
    {{
    Writes a String to the stream message.
    Parameters:
    value - the String value to be written
    Throws:
    JMSException - if the JMS provider fails to write the message due to some 
    internal error.
    MessageNotWriteableException - if the message is in read-only mode.
    }
    procedure writeString(value : string);
  end;
  
  {{
  An ObjectMessage object is used to send a message that contains a 
  serializable object in the Java programming language ("Java object"). It 
  inherits from the Message interface and adds a body containing a single 
  reference to an object. Only Serializable Java objects can be used. 
  
  If a collection of Java objects must be sent, one of the Collection classes 
  provided since JDK 1.2 can be used. 
  
  When a client receives an ObjectMessage, it is in read-only mode. If a client 
  attempts to write to the message at this point, a 
  MessageNotWriteableException is thrown. If clearBody is called, the message 
  can now be both read from and written to. 
  
  
  See Also: 
  Session.createObjectMessage(), Session.createObjectMessage(Serializable), 
  BytesMessage, MapMessage, Message, StreamMessage, TextMessage
  }
  IObjectMessage = interface (IMessage)
    ['{9B2393BB-F06A-43CC-BFAC-AB90DA2F393F}']
    function getObject: TObject;
    procedure setObject(_object: TObject);
    property _Object: TObject read getObject write setObject;
  end;
  
  {{
  A MessageListener object is used to receive asynchronously delivered 
  messages. 
  
  Each session must insure that it passes messages serially to the listener. 
  This means that a listener assigned to one or more consumers of the same 
  session can assume that the onMessage method is not called with the next 
  message until the session has completed the last call. 
  }
  IMessageListener = interface (IUnknown)
    ['{16CC7E4F-3C92-4157-AA29-17DCA686BE5B}']
    {{
    Passes a message to the listener.
    Parameters:
    message - the message passed to the listener
    }
    procedure onMessage(message : IMessage);
  end;
  
  {{
  If a JMS provider detects a serious problem with a Connection object, it 
  informs the Connection object's ExceptionListener, if one has been 
  registered. It does this by calling the listener's onException method, 
  passing it a JMSException argument describing the problem. 
  
  An exception listener allows a client to be notified of a problem 
  asynchronously. Some connections only consume messages, so they would have no 
  other way to learn that their connection has failed. 
  
  A JMS provider should attempt to resolve connection problems itself before it 
  notifies the client of them. 
  
  
  See Also: 
  Connection.setExceptionListener(ExceptionListener)
  }
  IExceptionListener = interface (IUnknown)
    ['{3E1821F2-6F49-41B5-9235-C025672233E7}']
    {{
    Notifies user of a JMS exception.
    Parameters:
    exception - the JMS exception
    }
    procedure onException(exception : EJMSException);
  end;
  
  {{
  A ConnectionMetaData object provides information describing the Connection 
  object. 
  }
  IConnectionMetaData = interface (IUnknown)
    ['{B5972954-6DC6-488D-8799-DA8952756F05}']
    {{
    Gets the JMS major version number.
    Returns:
    the JMS API major version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getJMSMajorVersion: LongInt;
    {{
    Gets the JMS minor version number.
    Returns:
    the JMS API minor version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getJMSMinorVersion: LongInt;
    {{
    Gets the JMS provider name.
    Returns:
    the JMS provider name
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getJMSProviderName: string;
    {{
    Gets the JMS API version.
    Returns:
    the JMS API version
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getJMSVersion: string;
    {{
    Gets an enumeration of the JMSX property names.
    Returns:
    an Enumeration of JMSX property names
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getJMSXPropertyNames: TStrings;
    {{
    Gets the JMS provider major version number.
    Returns:
    the JMS provider major version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getProviderMajorVersion: LongInt;
    {{
    Gets the JMS provider minor version number.
    Returns:
    the JMS provider minor version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getProviderMinorVersion: LongInt;
    {{
    Gets the JMS provider version.
    Returns:
    the JMS provider version
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    function getProviderVersion: string;
    {{
    Gets the JMS major version number.
    Returns:
    the JMS API major version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property JMSMajorVersion: LongInt read getJMSMajorVersion;
    {{
    Gets the JMS minor version number.
    Returns:
    the JMS API minor version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property JMSMinorVersion: LongInt read getJMSMinorVersion;
    {{
    Gets the JMS provider name.
    Returns:
    the JMS provider name
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property JMSProviderName: string read getJMSProviderName;
    {{
    Gets the JMS API version.
    Returns:
    the JMS API version
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property JMSVersion: string read getJMSVersion;
    {{
    Gets an enumeration of the JMSX property names.
    Returns:
    an Enumeration of JMSX property names
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property JMSXPropertyNames: TStrings read getJMSXPropertyNames;
    {{
    Gets the JMS provider major version number.
    Returns:
    the JMS provider major version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property ProviderMajorVersion: LongInt read getProviderMajorVersion;
    {{
    Gets the JMS provider minor version number.
    Returns:
    the JMS provider minor version number
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property ProviderMinorVersion: LongInt read getProviderMinorVersion;
    {{
    Gets the JMS provider version.
    Returns:
    the JMS provider version
    Throws:
    JMSException - if the JMS provider fails to retrieve the metadata due to 
    some internal error.
    }
    property ProviderVersion: string read getProviderVersion;
  end;
  
  {{
  A ServerSession object is an application server object that is used by a 
  server to associate a thread with a JMS session (optional). 
  
  A ServerSession implements two methods: 
  
  getSession - returns the ServerSession's JMS session. 
  start - starts the execution of the ServerSession thread and results in the 
  execution of the JMS session's run method. 
  A ConnectionConsumer implemented by a JMS provider uses a ServerSession to 
  process one or more messages that have arrived. It does this by getting a 
  ServerSession from the ConnectionConsumer's ServerSessionPool; getting the 
  ServerSession's JMS session; loading it with the messages; and then starting 
  the ServerSession. 
  
  In most cases the ServerSession will register some object it provides as the 
  ServerSession's thread run object. The ServerSession's start method will call 
  the thread's start method, which will start the new thread, and from it, call 
  the run method of the ServerSession's run object. This object will do some 
  housekeeping and then call the Session's run method. When run returns, the 
  ServerSession's run object can return the ServerSession to the 
  ServerSessionPool, and the cycle starts again. 
  
  Note that the JMS API does not architect how the ConnectionConsumer loads the 
  Session with messages. Since both the ConnectionConsumer and Session are 
  implemented by the same JMS provider, they can accomplish the load using a 
  private mechanism. 
  
  
  See Also: 
  ServerSessionPool, ConnectionConsumer
  }
  IServerSession = interface (IUnknown)
    ['{EADF2029-83A3-4A7A-ABBD-D9CE450A3767}']
    {{
    Return the ServerSession's Session. This must be a Session created by the 
    same Connection that will be dispatching messages to it. The provider will 
    assign one or more messages to the Session and then call start on the 
    ServerSession.
    Returns:
    the server session's session
    Throws:
    JMSException - if the JMS provider fails to get the associated session for 
    this ServerSession due to some internal error.
    }
    function getSession: Integer;
    {{
    Cause the Session's run method to be called to process messages that were 
    just assigned to it.
    Throws:
    JMSException - if the JMS provider fails to start the server session to 
    process messages due to some internal error.
    }
    procedure start;
    {{
    Return the ServerSession's Session. This must be a Session created by the 
    same Connection that will be dispatching messages to it. The provider will 
    assign one or more messages to the Session and then call start on the 
    ServerSession.
    Returns:
    the server session's session
    Throws:
    JMSException - if the JMS provider fails to get the associated session for 
    this ServerSession due to some internal error.
    }
    property Session: Integer read getSession;
  end;
  
  {{
  A ServerSessionPool object is an object implemented by an application server 
  to provide a pool of ServerSession objects for processing the messages of a 
  ConnectionConsumer (optional). 
  
  Its only method is getServerSession. The JMS API does not architect how the 
  pool is implemented. It could be a static pool of ServerSession objects, or 
  it could use a sophisticated algorithm to dynamically create ServerSession 
  objects as needed. 
  
  If the ServerSessionPool is out of ServerSession objects, the 
  getServerSession call may block. If a ConnectionConsumer is blocked, it 
  cannot deliver new messages until a ServerSession is eventually returned. 
  
  
  See Also: 
  ServerSession
  }
  IServerSessionPool = interface (IUnknown)
    ['{06927D4E-D056-40EA-BB15-4A25ED711BA5}']
    {{
    Return a server session from the pool.
    Returns:
    a server session from the pool
    Throws:
    JMSException - if an application server fails to return a ServerSession out 
    of its server session pool.
    }
    function getServerSession: IServerSession;
    {{
    Return a server session from the pool.
    Returns:
    a server session from the pool
    Throws:
    JMSException - if an application server fails to return a ServerSession out 
    of its server session pool.
    }
    property ServerSession: IServerSession read getServerSession;
  end;
  
  {{
  The delivery modes supported by the JMS API are PERSISTENT and 
  NON_PERSISTENT. 
  
  A client marks a message as persistent if it feels that the application will 
  have problems if the message is lost in transit. A client marks a message as 
  non-persistent if an occasional lost message is tolerable. Clients use 
  delivery mode to tell a JMS provider how to balance message transport 
  reliability with throughput. 
  
  Delivery mode covers only the transport of the message to its destination. 
  Retention of a message at the destination until its receipt is acknowledged 
  is not guaranteed by a PERSISTENT delivery mode. Clients should assume that 
  message retention policies are set administratively. Message retention policy 
  governs the reliability of message delivery from destination to message 
  consumer. For example, if a client's message storage space is exhausted, some 
  messages may be dropped in accordance with a site-specific message retention 
  policy. 
  
  A message is guaranteed to be delivered once and only once by a JMS provider 
  if the delivery mode of the message is PERSISTENT and if the destination has 
  a sufficient message retention policy. 
  }
  IDeliveryMode = interface (IUnknown)
    ['{B9EDE7D7-7B70-4B47-ADCC-67728BE30405}']
    {{
    This is the lowest-overhead delivery mode because it does not require that 
    the message be logged to stable storage. The level of JMS provider failure 
    that causes a NON_PERSISTENT message to be lost is not defined. 
    A JMS provider must deliver a NON_PERSISTENT message with an at-most-once 
    guarantee. This means that it may lose the message, but it must not deliver 
    it twice.
    }
    function getNON_PERSISTENT: LongInt;
    {{
    This delivery mode instructs the JMS provider to log the message to stable 
    storage as part of the client's send operation. Only a hard media failure 
    should cause a PERSISTENT message to be lost.
    }
    function getPERSISTENT: Integer;
    {{
    This is the lowest-overhead delivery mode because it does not require that 
    the message be logged to stable storage. The level of JMS provider failure 
    that causes a NON_PERSISTENT message to be lost is not defined. 
    A JMS provider must deliver a NON_PERSISTENT message with an at-most-once 
    guarantee. This means that it may lose the message, but it must not deliver 
    it twice.
    }
    property NON_PERSISTENT: LongInt read getNON_PERSISTENT;
    {{
    This delivery mode instructs the JMS provider to log the message to stable 
    storage as part of the client's send operation. Only a hard media failure 
    should cause a PERSISTENT message to be lost.
    }
    property PERSISTENT: Integer read getPERSISTENT;
  end;
  
  {{
  A Topic object encapsulates a provider-specific topic name. It is the way a 
  client specifies the identity of a topic to JMS API methods. 
  
  Many publish/subscribe (pub/sub) providers group topics into hierarchies and 
  provide various options for subscribing to parts of the hierarchy. The JMS 
  API places no restriction on what a Topic object represents. It may be a leaf 
  in a topic hierarchy, or it may be a larger part of the hierarchy. 
  
  The organization of topics and the granularity of subscriptions to them is an 
  important part of a pub/sub application's architecture. The JMS API does not 
  specify a policy for how this should be done. If an application takes 
  advantage of a provider-specific topic-grouping mechanism, it should document 
  this. If the application is installed using a different provider, it is the 
  job of the administrator to construct an equivalent topic architecture and 
  create equivalent Topic objects. 
  
  
  See Also: 
  TopicSession.createTopic(String)
  }
  ITopic = interface (IDestination)
    ['{602DA7E5-8417-4D4A-B415-CCA80E5D57BF}']
    {{
    Gets the name of this topic. 
    Clients that depend upon the name are not portable.
    
    Returns:
    the topic name
    Throws:
    JMSException - if the JMS provider implementation of Topic fails to return 
    the topic name due to some internal error.
    }
    function getTopicName: string;
    {{
    Returns a string representation of this object.
    Overrides:
    toString in class java.lang.Object
    Returns:
    the provider-specific identity values for this topic
    }
    function toString: string;
    {{
    Gets the name of this topic. 
    Clients that depend upon the name are not portable.
    
    Returns:
    the topic name
    Throws:
    JMSException - if the JMS provider implementation of Topic fails to return 
    the topic name due to some internal error.
    }
    property TopicName: string read getTopicName;
  end;
  
  {{
  A Queue object encapsulates a provider-specific queue name. It is the way a 
  client specifies the identity of a queue to JMS API methods. 
  
  The actual length of time messages are held by a queue and the consequences 
  of resource overflow are not defined by the JMS API. 
  
  
  See Also: 
  QueueSession.createQueue(String)
  }
  IQueue = interface (IDestination)
    ['{9B1F4D79-0B7C-4C22-8DFB-24A9A0821961}']
    {{
    Gets the name of this queue. 
    Clients that depend upon the name are not portable.
    
    Returns:
    the queue name
    Throws:
    JMSException - if the JMS provider implementation of Queue fails to return 
    the queue name due to some internal error.
    }
    function getQueueName: string;
    {{
    Returns a string representation of this object.
    Overrides:
    toString in class java.lang.Object
    Returns:
    the provider-specific identity values for this queue
    }
    function toString: string;
    {{
    Gets the name of this queue. 
    Clients that depend upon the name are not portable.
    
    Returns:
    the queue name
    Throws:
    JMSException - if the JMS provider implementation of Queue fails to return 
    the queue name due to some internal error.
    }
    property QueueName: string read getQueueName;
  end;
  
  {{
  A TemporaryQueue object is a unique Queue object created for the duration of 
  a QueueConnection. It is a system-defined queue that can be consumed only by 
  the QueueConnection that created it. 
  
  
  See Also: 
  QueueSession.createTemporaryQueue()
  }
  ITemporaryQueue = interface (IQueue)
    ['{6BFE97E6-7EF0-4C40-8AD1-B747AFAFA339}']
    {{
    Deletes this temporary queue. If there are existing receivers still using 
    it, a JMSException will be thrown.
    Throws:
    JMSException - if the JMS provider fails to delete the temporary queue due 
    to some internal error.
    }
    procedure delete;
  end;
  
  {{
  A Session object is a single-threaded context for producing and consuming 
  messages. Although it may allocate provider resources outside the Java 
  virtual machine (JVM), it is considered a lightweight JMS object. 
  
  A session serves several purposes: 
  
  It is a factory for its message producers and consumers. 
  It supplies provider-optimized message factories. 
  It supports a single series of transactions that combine work spanning its 
  producers and consumers into atomic units. 
  It defines a serial order for the messages it consumes and the messages it 
  produces. 
  It retains messages it consumes until they have been acknowledged. 
  It serializes execution of message listeners registered with its message 
  consumers. 
  A session can create and service multiple message producers and consumers. 
  
  One typical use is to have a thread block on a synchronous MessageConsumer 
  until a message arrives. The thread may then use one or more of the Session's 
  MessageProducers. 
  
  If a client desires to have one thread produce messages while others consume 
  them, the client should use a separate session for its producing thread. 
  
  Once a connection has been started, any session with one or more registered 
  message listeners is dedicated to the thread of control that delivers 
  messages to it. It is erroneous for client code to use this session or any of 
  its constituent objects from another thread of control. The only exception to 
  this rule is the use of the session or connection close method. 
  
  It should be easy for most clients to partition their work naturally into 
  sessions. This model allows clients to start simply and incrementally add 
  message processing complexity as their need for concurrency grows. 
  
  The close method is the only session method that can be called while some 
  other session method is being executed in another thread. 
  
  A session may be specified as transacted. Each transacted session supports a 
  single series of transactions. Each transaction groups a set of message sends 
  and a set of message receives into an atomic unit of work. In effect, 
  transactions organize a session's input message stream and output message 
  stream into series of atomic units. When a transaction commits, its atomic 
  unit of input is acknowledged and its associated atomic unit of output is 
  sent. If a transaction rollback is done, the transaction's sent messages are 
  destroyed and the session's input is automatically recovered. 
  
  The content of a transaction's input and output units is simply those 
  messages that have been produced and consumed within the session's current 
  transaction. 
  
  A transaction is completed using either its session's commit method or its 
  session's rollback method. The completion of a session's current transaction 
  automatically begins the next. The result is that a transacted session always 
  has a current transaction within which its work is done. 
  
  The Java Transaction Service (JTS) or some other transaction monitor may be 
  used to combine a session's transaction with transactions on other resources (
  databases, other JMS sessions, etc.). Since Java distributed transactions are 
  controlled via the Java Transaction API (JTA), use of the session's commit 
  and rollback methods in this context is prohibited. 
  
  The JMS API does not require support for JTA; however, it does define how a 
  provider supplies this support. 
  
  Although it is also possible for a JMS client to handle distributed 
  transactions directly, it is unlikely that many JMS clients will do this. 
  Support for JTA in the JMS API is targeted at systems vendors who will be 
  integrating the JMS API into their application server products. 
  
  
  See Also: 
  QueueSession, TopicSession, XASession
  }
  ISession = interface (IUnknown)
    ['{59281847-987A-43FE-83E9-9CCABF144DDD}']
    {{
    Closes the session. 
    Since a provider may allocate some resources on behalf of a session outside 
    the JVM, clients should close the resources when they are not needed. 
    Relying on garbage collection to eventually reclaim these resources may not 
    be timely enough. 
    
    There is no need to close the producers and consumers of a closed session. 
    
    This call will block until a receive call or message listener in progress 
    has completed. A blocked message consumer receive call returns null when 
    this session is closed. 
    
    Closing a transacted session must roll back the transaction in progress. 
    
    This method is the only Session method that can be called concurrently. 
    
    Invoking any other Session method on a closed session must throw a 
    JMSException.IllegalStateException. Closing a closed session must not throw 
    an exception.
    
    Throws:
    JMSException - if the JMS provider fails to close the session due to some 
    internal error.
    }
    procedure close;
    {{
    Commits all messages done in this transaction and releases any locks 
    currently held.
    Throws:
    JMSException - if the JMS provider fails to commit the transaction due to 
    some internal error.
    TransactionRolledBackException - if the transaction is rolled back due to 
    some internal error during commit.
    IllegalStateException - if the method is not called by a transacted session.
    }
    procedure commit;
    {{
    Creates a BytesMessage object. A BytesMessage object is used to send a 
    message containing a stream of uninterpreted bytes.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createBytesMessage: IBytesMessage;
    {{
    Creates a MapMessage object. A MapMessage object is used to send a 
    self-defining set of name-value pairs, where names are String objects and 
    values are primitive values in the Java programming language.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createMapMessage: IMapMessage;
    {{
    Creates a Message object. The Message interface is the root interface of 
    all JMS messages. A Message object holds all the standard message header 
    information. It can be sent when a message containing only header 
    information is sufficient.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createMessage: IMessage;
    {{
    Creates an ObjectMessage object. An ObjectMessage object is used to send a 
    message that contains a serializable Java object.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createObjectMessage: IObjectMessage; overload;
    {{
    Creates an initialized ObjectMessage object. An ObjectMessage object is 
    used to send a message that contains a serializable Java object.
    Parameters:
    object - the object to use to initialize this message
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createObjectMessage(_object : TObject): IObjectMessage; overload;
    {{
    Creates a StreamMessage object. A StreamMessage object is used to send a 
    self-defining stream of primitive values in the Java programming language.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createStreamMessage: IStreamMessage;
    {{
    Creates a TextMessage object. A TextMessage object is used to send a 
    message containing a String object.
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createTextMessage: ITextMessage; overload;
    {{
    Creates an initialized TextMessage object. A TextMessage object is used to 
    send a message containing a String.
    Parameters:
    text - the string used to initialize this message
    Throws:
    JMSException - if the JMS provider fails to create this message due to some 
    internal error.
    }
    function createTextMessage(text : string): ITextMessage; overload;
    {{
    With this acknowledgment mode, the session automatically acknowledges a 
    client's receipt of a message either when the session has successfully 
    returned from a call to receive or when the message listener the session 
    has called to process the message successfully returns.
    }
    function getAUTO_ACKNOWLEDGE: LongInt;
    {{
    With this acknowledgment mode, the client acknowledges a consumed message 
    by calling the message's acknowledge method. Acknowledging a consumed 
    message acknowledges all messages that the session has consumed. 
    When client acknowledgment mode is used, a client may build up a large 
    number of unacknowledged messages while attempting to process them. A JMS 
    provider should provide administrators with a way to limit client overrun 
    so that clients are not driven to resource exhaustion and ensuing failure 
    when some resource they are using is temporarily blocked.
    
    See Also: 
    Message.acknowledge()
    }
    function getCLIENT_ACKNOWLEDGE: LongInt;
    {{
    This acknowledgment mode instructs the session to lazily acknowledge the 
    delivery of messages. This is likely to result in the delivery of some 
    duplicate messages if the JMS provider fails, so it should only be used by 
    consumers that can tolerate duplicate messages. Use of this mode can reduce 
    session overhead by minimizing the work the session does to prevent 
    duplicates.
    }
    function getDUPS_OK_ACKNOWLEDGE: LongInt;
    {{
    Returns the session's distinguished message listener (optional).
    Returns:
    the message listener associated with this session
    Throws:
    JMSException - if the JMS provider fails to get the message listener due to 
    an internal error.
    See Also: 
    setMessageListener(javax.jms.MessageListener), ServerSessionPool, 
    ServerSession
    }
    function getMessageListener: IMessageListener;
    {{
    Indicates whether the session is in transacted mode.
    Returns:
    true if the session is in transacted mode
    Throws:
    JMSException - if the JMS provider fails to return the transaction mode due 
    to some internal error.
    }
    function getTransacted: Boolean;
    {{
    Stops message delivery in this session, and restarts message delivery with 
    the oldest unacknowledged message. 
    All consumers deliver messages in a serial order. Acknowledging a received 
    message automatically acknowledges all messages that have been delivered to 
    the client. 
    
    Restarting a session causes it to take the following actions: 
    
    Stop message delivery 
    Mark all messages that might have been delivered but not acknowledged as 
    "redelivered" 
    Restart the delivery sequence including all unacknowledged messages that 
    had been previously delivered. Redelivered messages do not have to be 
    delivered in exactly their original delivery order. 
    Throws:
    JMSException - if the JMS provider fails to stop and restart message 
    delivery due to some internal error.
    IllegalStateException - if the method is called by a transacted session.
    }
    procedure recover;
    {{
    Rolls back any messages done in this transaction and releases any locks 
    currently held.
    Throws:
    JMSException - if the JMS provider fails to roll back the transaction due 
    to some internal error.
    IllegalStateException - if the method is not called by a transacted session.
    }
    procedure rollback;
    {{
    Optional operation, intended to be used only by Application Servers, not by 
    ordinary JMS clients.
    Specified by: 
    run in interface java.lang.Runnable
    See Also: 
    ServerSession
    }
    procedure run;
    {{
    Sets the session's distinguished message listener (optional). 
    When the distinguished message listener is set, no other form of message 
    receipt in the session can be used; however, all forms of sending messages 
    are still supported. 
    
    This is an expert facility not used by regular JMS clients.
    
    Parameters:
    listener - the message listener to associate with this session
    Throws:
    JMSException - if the JMS provider fails to set the message listener due to 
    an internal error.
    See Also: 
    getMessageListener(), ServerSessionPool, ServerSession
    }
    procedure setMessageListener(listener: IMessageListener);
    {{
    With this acknowledgment mode, the session automatically acknowledges a 
    client's receipt of a message either when the session has successfully 
    returned from a call to receive or when the message listener the session 
    has called to process the message successfully returns.
    }
    property AUTO_ACKNOWLEDGE: LongInt read getAUTO_ACKNOWLEDGE;
    {{
    With this acknowledgment mode, the client acknowledges a consumed message 
    by calling the message's acknowledge method. Acknowledging a consumed 
    message acknowledges all messages that the session has consumed. 
    When client acknowledgment mode is used, a client may build up a large 
    number of unacknowledged messages while attempting to process them. A JMS 
    provider should provide administrators with a way to limit client overrun 
    so that clients are not driven to resource exhaustion and ensuing failure 
    when some resource they are using is temporarily blocked.
    
    See Also: 
    Message.acknowledge()
    }
    property CLIENT_ACKNOWLEDGE: LongInt read getCLIENT_ACKNOWLEDGE;
    {{
    This acknowledgment mode instructs the session to lazily acknowledge the 
    delivery of messages. This is likely to result in the delivery of some 
    duplicate messages if the JMS provider fails, so it should only be used by 
    consumers that can tolerate duplicate messages. Use of this mode can reduce 
    session overhead by minimizing the work the session does to prevent 
    duplicates.
    }
    property DUPS_OK_ACKNOWLEDGE: LongInt read getDUPS_OK_ACKNOWLEDGE;
    property MessageListener: IMessageListener read getMessageListener write 
            setMessageListener;
    {{
    Indicates whether the session is in transacted mode.
    Returns:
    true if the session is in transacted mode
    Throws:
    JMSException - if the JMS provider fails to return the transaction mode due 
    to some internal error.
    }
    property Transacted: Boolean read getTransacted;
  end;
  
  {{
  A Connection object is a client's active connection to its JMS provider. It 
  typically allocates provider resources outside the Java virtual machine (
  JVM). 
  
  Connections support concurrent use. 
  
  A connection serves several purposes: 
  
  It encapsulates an open connection with a JMS provider. It typically 
  represents an open TCP/IP socket between a client and a provider service 
  daemon. 
  Its creation is where client authentication takes place. 
  It can specify a unique client identifier. 
  It provides a ConnectionMetaData object. 
  It supports an optional ExceptionListener object. 
  Because the creation of a connection involves setting up authentication and 
  communication, a connection is a relatively heavyweight object. Most clients 
  will do all their messaging with a single connection. Other more advanced 
  applications may use several connections. The JMS API does not architect a 
  reason for using multiple connections; however, there may be operational 
  reasons for doing so. 
  
  A JMS client typically creates a connection, one or more sessions, and a 
  number of message producers and consumers. When a connection is created, it 
  is in stopped mode. That means that no messages are being delivered. 
  
  It is typical to leave the connection in stopped mode until setup is complete 
  (that is, until all message consumers have been created). At that point, the 
  client calls the connection's start method, and messages begin arriving at 
  the connection's consumers. This setup convention minimizes any client 
  confusion that may result from asynchronous message delivery while the client 
  is still in the process of setting itself up. 
  
  A connection can be started immediately, and the setup can be done 
  afterwards. Clients that do this must be prepared to handle asynchronous 
  message delivery while they are still in the process of setting up. 
  
  A message producer can send messages while a connection is stopped. 
  }
  IConnection = interface (IUnknown)
    ['{C83A8886-2C10-4268-A91B-3DF8EC156FF9}']
    {{
    Closes the connection. 
    Since a provider typically allocates significant resources outside the JVM 
    on behalf of a connection, clients should close these resources when they 
    are not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough. 
    
    There is no need to close the sessions, producers, and consumers of a 
    closed connection. 
    
    Closing a connection causes all temporary destinations to be deleted. 
    
    When this method is invoked, it should not return until message processing 
    has been shut down in an orderly fashion. This means that all message 
    listeners that may have been running have returned, and that all pending 
    receives have returned. A close terminates all pending message receives on 
    the connection's sessions' consumers. The receives may return with a 
    message or with null, depending on whether there was a message available at 
    the time of the close. If one or more of the connection's sessions' message 
    listeners is processing a message at the time when connection close is 
    invoked, all the facilities of the connection and its sessions must remain 
    available to those listeners until they return control to the JMS provider. 
    
    Closing a connection causes any of its sessions' transactions in progress 
    to be rolled back. In the case where a session's work is coordinated by an 
    external transaction manager, a session's commit and rollback methods are 
    not used and the result of a closed session's work is determined later by 
    the transaction manager. Closing a connection does NOT force an 
    acknowledgment of client-acknowledged sessions. 
    
    Invoking the acknowledge method of a received message from a closed 
    connection's session must throw an IllegalStateException. Closing a closed 
    connection must NOT throw an exception.
    
    Throws:
    JMSException - if the JMS provider fails to close the connection due to 
    some internal error. For example, a failure to release resources or to 
    close a socket connection can cause this exception to be thrown.
    }
    procedure close;
    {{
    Gets the client identifier for this connection. 
    This value is specific to the JMS provider. It is either preconfigured by 
    an administrator in a ConnectionFactory object or assigned dynamically by 
    the application by calling the setClientID method.
    
    Returns:
    the unique client identifier
    Throws:
    JMSException - if the JMS provider fails to return the client ID for this 
    connection due to some internal error.
    }
    function getClientID: string;
    {{
    Gets the ExceptionListener object for this connection.
    Returns:
    the ExceptionListener for this connection
    Throws:
    JMSException - if the JMS provider fails to get the ExceptionListener for 
    this connection.
    }
    function getExceptionListener: IExceptionListener;
    {{
    Gets the metadata for this connection.
    Returns:
    the connection metadata
    Throws:
    JMSException - if the JMS provider fails to get the connection metadata for 
    this connection.
    See Also: 
    ConnectionMetaData
    }
    function getMetaData: IConnectionMetaData;
    {{
    Sets the client identifier for this connection. 
    The preferred way to assign a JMS client's client identifier is for it to 
    be configured in a client-specific ConnectionFactory object and 
    transparently assigned to the Connection object it creates. 
    
    Alternatively, a client can set a connection's client identifier using a 
    provider-specific value. The facility to set a connection's client 
    identifier explicitly is not a mechanism for overriding the identifier that 
    has been administratively configured. It is provided for the case where no 
    administratively specified identifier exists. If one does exist, an attempt 
    to change it by setting it must throw an IllegalStateException. If a client 
    sets the client identifier explicitly, it must do so immediately after it 
    creates the connection and before any other action on the connection is 
    taken. After this point, setting the client identifier is a programming 
    error that should throw an IllegalStateException. 
    
    The purpose of the client identifier is to associate a connection and its 
    objects with a state maintained on behalf of the client by a provider. The 
    only such state identified by the JMS API is that required to support 
    durable subscriptions. 
    
    If another connection with the same clientID is already running when this 
    method is called, the JMS provider should detect the duplicate ID and throw 
    an InvalidClientIDException.
    
    Parameters:
    clientID - the unique client identifier
    Throws:
    JMSException - if the JMS provider fails to set the client ID for this 
    connection due to some internal error.
    InvalidClientIDException - if the JMS client specifies an invalid or 
    duplicate client ID.
    IllegalStateException - if the JMS client attempts to set a connection's 
    client ID at the wrong time or when it has been administratively configured.
    }
    procedure setClientID(const clientID: string);
    {{
    Sets an exception listener for this connection. 
    If a JMS provider detects a serious problem with a connection, it informs 
    the connection's ExceptionListener, if one has been registered. It does 
    this by calling the listener's onException method, passing it a 
    JMSException object describing the problem. 
    
    An exception listener allows a client to be notified of a problem 
    asynchronously. Some connections only consume messages, so they would have 
    no other way to learn their connection has failed. 
    
    A connection serializes execution of its ExceptionListener. 
    
    A JMS provider should attempt to resolve connection problems itself before 
    it notifies the client of them.
    
    Parameters:
    listener - the exception listener
    Throws:
    JMSException - if the JMS provider fails to set the exception listener for 
    this connection.
    }
    procedure setExceptionListener(listener: IExceptionListener);
    {{
    Starts (or restarts) a connection's delivery of incoming messages. A call 
    to start on a connection that has already been started is ignored.
    Throws:
    JMSException - if the JMS provider fails to start message delivery due to 
    some internal error.
    See Also: 
    stop()
    }
    procedure start;
    {{
    Temporarily stops a connection's delivery of incoming messages. Delivery 
    can be restarted using the connection's start method. When the connection 
    is stopped, delivery to all the connection's message consumers is 
    inhibited: synchronous receives block, and messages are not delivered to 
    message listeners. 
    This call blocks until receives and/or message listeners in progress have 
    completed. 
    
    Stopping a connection has no effect on its ability to send messages. A call 
    to stop on a connection that has already been stopped is ignored. 
    
    A call to stop must not return until delivery of messages has paused. This 
    means that a client can rely on the fact that none of its message listeners 
    will be called and that all threads of control waiting for receive calls to 
    return will not return with a message until the connection is restarted. 
    The receive timers for a stopped connection continue to advance, so 
    receives may time out while the connection is stopped. 
    
    If message listeners are running when stop is invoked, the stop call must 
    wait until all of them have returned before it may return. While these 
    message listeners are completing, they must have the full services of the 
    connection available to them.
    
    Throws:
    JMSException - if the JMS provider fails to stop message delivery due to 
    some internal error.
    See Also: 
    start()
    }
    procedure stop;
    property ClientID: string read getClientID write setClientID;
    property ExceptionListener: IExceptionListener read getExceptionListener 
            write setExceptionListener;
    property MetaData: IConnectionMetaData read getMetaData;
  end;
  
  {{
  A client uses a MessageConsumer object to receive messages from a 
  destination. A MessageConsumer object is created by passing a Destination 
  object to a message-consumer creation method supplied by a session. 
  
  MessageConsumer is the parent interface for all message consumers. 
  
  A message consumer can be created with a message selector. A message selector 
  allows the client to restrict the messages delivered to the message consumer 
  to those that match the selector. 
  
  A client may either synchronously receive a message consumer's messages or 
  have the consumer asynchronously deliver them as they arrive. 
  
  For synchronous receipt, a client can request the next message from a message 
  consumer using one of its receive methods. There are several variations of 
  receive that allow a client to poll or wait for the next message. 
  
  For asynchronous delivery, a client can register a MessageListener object 
  with a message consumer. As messages arrive at the message consumer, it 
  delivers them by calling the MessageListener's onMessage method. 
  
  It is a client programming error for a MessageListener to throw an exception. 
  
  
  See Also: 
  QueueReceiver, TopicSubscriber, Session
  }
  IMessageConsumer = interface (IUnknown)
    ['{648FACCC-7362-4974-98E4-248BF9BF38B3}']
    {{
    Closes the message consumer. 
    Since a provider may allocate some resources on behalf of a MessageConsumer 
    outside the Java virtual machine, clients should close them when they are 
    not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough. 
    
    This call blocks until a receive or message listener in progress has 
    completed. A blocked message consumer receive call returns null when this 
    message consumer is closed.
    
    Throws:
    JMSException - if the JMS provider fails to close the consumer due to some 
    internal error.
    }
    procedure close;
    {{
    Gets the message consumer's MessageListener.
    Returns:
    the listener for the message consumer, or null if no listener is set
    Throws:
    JMSException - if the JMS provider fails to get the message listener due to 
    some internal error.
    See Also: 
    setMessageListener(javax.jms.MessageListener)
    }
    function getMessageListener: IMessageListener;
    {{
    Gets this message consumer's message selector expression.
    Returns:
    this message consumer's message selector, or null if no message selector 
    exists for the message consumer (that is, if the message selector was not 
    set or was set to null or the empty string)
    Throws:
    JMSException - if the JMS provider fails to get the message selector due to 
    some internal error.
    }
    function getMessageSelector: string;
    {{
    Receives the next message produced for this message consumer. 
    This call blocks indefinitely until a message is produced or until this 
    message consumer is closed. 
    
    If this receive is done within a transaction, the consumer retains the 
    message until the transaction commits.
    
    Returns:
    the next message produced for this message consumer, or null if this 
    message consumer is concurrently closed
    Throws:
    JMSException - if the JMS provider fails to receive the next message due to 
    some internal error.
    }
    function receive: IMessage; overload;
    {{
    Receives the next message that arrives within the specified timeout 
    interval. 
    This call blocks until a message arrives, the timeout expires, or this 
    message consumer is closed. A timeout of zero never expires, and the call 
    blocks indefinitely.
    
    Parameters:
    timeout - the timeout value (in milliseconds)
    Returns:
    the next message produced for this message consumer, or null if the timeout 
    expires or this message consumer is concurrently closed
    Throws:
    JMSException - if the JMS provider fails to receive the next message due to 
    some internal error.
    }
    function receive(timeout : int64): IMessage; overload;
    {{
    Receives the next message if one is immediately available.
    Returns:
    the next message produced for this message consumer, or null if one is not 
    available
    Throws:
    JMSException - if the JMS provider fails to receive the next message due to 
    some internal error.
    }
    function receiveNoWait: IMessage;
    {{
    Sets the message consumer's MessageListener. 
    Setting the message listener to null is the equivalent of unsetting the 
    message listener for the message consumer. 
    
    The effect of calling MessageConsumer.setMessageListener while messages are 
    being consumed by an existing listener or the consumer is being used to 
    consume messages synchronously is undefined.
    
    Parameters:
    listener - the listener to which the messages are to be delivered
    Throws:
    JMSException - if the JMS provider fails to set the message listener due to 
    some internal error.
    See Also: 
    getMessageListener()
    }
    procedure setMessageListener(Value: IMessageListener);
    property MessageListener: IMessageListener read getMessageListener write 
            setMessageListener;
    {{
    Gets this message consumer's message selector expression.
    Returns:
    this message consumer's message selector, or null if no message selector 
    exists for the message consumer (that is, if the message selector was not 
    set or was set to null or the empty string)
    Throws:
    JMSException - if the JMS provider fails to get the message selector due to 
    some internal error.
    }
    property MessageSelector: string read getMessageSelector;
  end;
  
  {{
  A client uses a MessageProducer object to send messages to a destination. A 
  MessageProducer object is created by passing a Destination object to a 
  message-producer creation method supplied by a session. 
  
  MessageProducer is the parent interface for all message producers. 
  
  A client also has the option of creating a message producer without supplying 
  a destination. In this case, a destination must be provided with every send 
  operation. A typical use for this kind of message producer is to send replies 
  to requests using the request's JMSReplyTo destination. 
  
  A client can specify a default delivery mode, priority, and time to live for 
  messages sent by a message producer. It can also specify the delivery mode, 
  priority, and time to live for an individual message. 
  
  A client can specify a time-to-live value in milliseconds for each message it 
  sends. This value defines a message expiration time that is the sum of the 
  message's time-to-live and the GMT when it is sent (for transacted sends, 
  this is the time the client sends the message, not the time the transaction 
  is committed). 
  
  A JMS provider should do its best to expire messages accurately; however, the 
  JMS API does not define the accuracy provided. 
  
  
  See Also: 
  TopicPublisher, QueueSender, Session
  }
  IMessageProducer = interface (IUnknown)
    ['{E705F951-4C54-484A-BED4-D203795252C8}']
    {{
    Closes the message producer. 
    Since a provider may allocate some resources on behalf of a MessageProducer 
    outside the Java virtual machine, clients should close them when they are 
    not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough.
    
    Throws:
    JMSException - if the JMS provider fails to close the producer due to some 
    internal error.
    }
    procedure close;
    {{
    Gets the producer's default delivery mode.
    Returns:
    the message delivery mode for this message producer
    Throws:
    JMSException - if the JMS provider fails to get the delivery mode due to 
    some internal error.
    See Also: 
    setDeliveryMode(int)
    }
    function getDeliveryMode: LongInt;
    {{
    Gets an indication of whether message IDs are disabled.
    Returns:
    an indication of whether message IDs are disabled
    Throws:
    JMSException - if the JMS provider fails to determine if message IDs are 
    disabled due to some internal error.
    }
    function getDisableMessageID: Boolean;
    {{
    Gets an indication of whether message timestamps are disabled.
    Returns:
    an indication of whether message timestamps are disabled
    Throws:
    JMSException - if the JMS provider fails to determine if timestamps are 
    disabled due to some internal error.
    }
    function getDisableMessageTimestamp: Boolean;
    {{
    Gets the producer's default priority.
    Returns:
    the message priority for this message producer
    Throws:
    JMSException - if the JMS provider fails to get the priority due to some 
    internal error.
    See Also: 
    setPriority(int)
    }
    function getPriority: LongInt;
    {{
    Gets the default length of time in milliseconds from its dispatch time that 
    a produced message should be retained by the message system.
    Returns:
    the message time to live in milliseconds; zero is unlimited
    Throws:
    JMSException - if the JMS provider fails to get the time to live due to 
    some internal error.
    See Also: 
    setTimeToLive(long)
    }
    function getTimeToLive: Int64;
    {{
    Sets the producer's default delivery mode. 
    Delivery mode is set to PERSISTENT by default.
    
    Parameters:
    deliveryMode - the message delivery mode for this message producer; legal 
    values are DeliveryMode.NON_PERSISTENT and DeliveryMode.PERSISTENT
    Throws:
    JMSException - if the JMS provider fails to set the delivery mode due to 
    some internal error.
    See Also: 
    getDeliveryMode(), DeliveryMode.NON_PERSISTENT, DeliveryMode.PERSISTENT, 
    Message.DEFAULT_DELIVERY_MODE
    }
    procedure setDeliveryMode(deliveryMode: LongInt);
    {{
    Sets whether message IDs are disabled. 
    Since message IDs take some effort to create and increase a message's size, 
    some JMS providers may be able to optimize message overhead if they are 
    given a hint that the message ID is not used by an application. By calling 
    the setDisableMessageID method on this message producer, a JMS client 
    enables this potential optimization for all messages sent by this message 
    producer. If the JMS provider accepts this hint, these messages must have 
    the message ID set to null; if the provider ignores the hint, the message 
    ID must be set to its normal unique value. 
    
    Message IDs are enabled by default.
    
    Parameters:
    value - indicates if message IDs are disabled
    Throws:
    JMSException - if the JMS provider fails to set message ID to disabled due 
    to some internal error.
    }
    procedure setDisableMessageID(Value: Boolean);
    {{
    Sets whether message timestamps are disabled. 
    Since timestamps take some effort to create and increase a message's size, 
    some JMS providers may be able to optimize message overhead if they are 
    given a hint that the timestamp is not used by an application. By calling 
    the setDisableMessageTimestamp method on this message producer, a JMS 
    client enables this potential optimization for all messages sent by this 
    message producer. If the JMS provider accepts this hint, these messages 
    must have the timestamp set to zero; if the provider ignores the hint, the 
    timestamp must be set to its normal value. 
    
    Message timestamps are enabled by default.
    
    Parameters:
    value - indicates if message timestamps are disabled
    Throws:
    JMSException - if the JMS provider fails to set timestamps to disabled due 
    to some internal error.
    }
    procedure setDisableMessageTimestamp(Value: Boolean);
    {{
    Sets the producer's default priority. 
    The JMS API defines ten levels of priority value, with 0 as the lowest 
    priority and 9 as the highest. Clients should consider priorities 0-4 as 
    gradations of normal priority and priorities 5-9 as gradations of expedited 
    priority. Priority is set to 4 by default.
    
    Parameters:
    defaultPriority - the message priority for this message producer; must be a 
    value between 0 and 9
    Throws:
    JMSException - if the JMS provider fails to set the priority due to some 
    internal error.
    See Also: 
    getPriority(), Message.DEFAULT_PRIORITY
    }
    procedure setPriority(defaultPriority: LongInt);
    {{
    Sets the default length of time in milliseconds from its dispatch time that 
    a produced message should be retained by the message system. 
    Time to live is set to zero by default.
    
    Parameters:
    timeToLive - the message time to live in milliseconds; zero is unlimited
    Throws:
    JMSException - if the JMS provider fails to set the time to live due to 
    some internal error.
    See Also: 
    getTimeToLive(), Message.DEFAULT_TIME_TO_LIVE
    }
    procedure setTimeToLive(timeToLive: Int64);
    property DeliveryMode: LongInt read getDeliveryMode write setDeliveryMode;
    property DisableMessageID: Boolean read getDisableMessageID write 
            setDisableMessageID;
    property DisableMessageTimestamp: Boolean read getDisableMessageTimestamp 
            write setDisableMessageTimestamp;
    property Priority: LongInt read getPriority write setPriority;
    property TimeToLive: Int64 read getTimeToLive write setTimeToLive;
  end;
  
  {{
  A client uses a TopicPublisher object to publish messages on a topic. A 
  TopicPublisher object is the publish-subscribe form of a message producer. 
  
  Normally, the Topic is specified when a TopicPublisher is created. In this 
  case, an attempt to use the publish methods for an unidentified 
  TopicPublisher will throw a java.lang.UnsupportedOperationException. 
  
  If the TopicPublisher is created with an unidentified Topic, an attempt to 
  use the publish methods that assume that the Topic has been identified will 
  throw a java.lang.UnsupportedOperationException. 
  
  During the execution of its publish method, a message must not be changed by 
  other threads within the client. If the message is modified, the result of 
  the publish is undefined. 
  
  After publishing a message, a client may retain and modify it without 
  affecting the message that has been published. The same message object may be 
  published multiple times. 
  
  The following message headers are set as part of publishing a message: 
  JMSDestination, JMSDeliveryMode, JMSExpiration, JMSPriority, JMSMessageID and 
  JMSTimeStamp. When the message is published, the values of these headers are 
  ignored. After completion of the publish, the headers hold the values 
  specified by the method publishing the message. It is possible for the 
  publish method not to set JMSMessageID and JMSTimeStamp if the setting of 
  these headers is explicitly disabled by the 
  MessageProducer.setDisableMessageID or 
  MessageProducer.setDisableMessageTimestamp method. 
  
  
  See Also: 
  TopicSession.createPublisher(Topic)
  }
  ITopicPublisher = interface (IMessageProducer)
    ['{192CCD52-AF5D-4F9C-ACF2-2ED0749E9943}']
    {{
    Gets the topic associated with this TopicPublisher.
    Returns:
    this publisher's topic
    Throws:
    JMSException - if the JMS provider fails to get the topic for this 
    TopicPublisher due to some internal error.
    }
    function getTopic: ITopic;
    {{
    Publishes a message to the topic. Uses the TopicPublisher's default 
    delivery mode, priority, and time to live.
    Parameters:
    message - the message to publish
    Throws:
    JMSException - if the JMS provider fails to publish the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with a 
    TopicPublisher with an invalid topic.
    java.lang.UnsupportedOperationException - if a client uses this method with 
    a TopicPublisher that did not specify a topic at creation time.
    See Also: 
    MessageProducer.getDeliveryMode(), MessageProducer.getTimeToLive(), 
    MessageProducer.getPriority()
    }
    procedure publish(message : IMessage); overload;
    {{
    Publishes a message to the topic, specifying delivery mode, priority, and 
    time to live.
    Parameters:
    message - the message to publish
    deliveryMode - the delivery mode to use
    priority - the priority for this message
    timeToLive - the message's lifetime (in milliseconds)
    Throws:
    JMSException - if the JMS provider fails to publish the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with a 
    TopicPublisher with an invalid topic.
    java.lang.UnsupportedOperationException - if a client uses this method with 
    a TopicPublisher that did not specify a topic at creation time.
    }
    procedure publish(message : IMessage; deliveryMode  :LongInt; priority : 
            LongInt; timeToLive : int64); overload;
    {{
    Publishes a message to a topic for an unidentified message producer. Uses 
    the TopicPublisher's default delivery mode, priority, and time to live. 
    Typically, a message producer is assigned a topic at creation time; however,
    the JMS API also supports unidentified message producers, which require 
    that the topic be supplied every time a message is published.
    
    Parameters:
    topic - the topic to publish this message to
    message - the message to publish
    Throws:
    JMSException - if the JMS provider fails to publish the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with an invalid 
    topic.
    See Also: 
    MessageProducer.getDeliveryMode(), MessageProducer.getTimeToLive(), 
    MessageProducer.getPriority()
    }
    procedure publish(topic : ITopic; message : IMessage); overload;
    {{
    Publishes a message to a topic for an unidentified message producer, 
    specifying delivery mode, priority and time to live. 
    Typically, a message producer is assigned a topic at creation time; however,
    the JMS API also supports unidentified message producers, which require 
    that the topic be supplied every time a message is published.
    
    Parameters:
    topic - the topic to publish this message to
    message - the message to publish
    deliveryMode - the delivery mode to use
    priority - the priority for this message
    timeToLive - the message's lifetime (in milliseconds)
    Throws:
    JMSException - if the JMS provider fails to publish the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with an invalid 
    topic.
    }
    procedure publish(topic : ITopic; message : IMessage; deliveryMode :LongInt;
            priority : LongInt; timeToLive : int64); overload;
    {{
    Gets the topic associated with this TopicPublisher.
    Returns:
    this publisher's topic
    Throws:
    JMSException - if the JMS provider fails to get the topic for this 
    TopicPublisher due to some internal error.
    }
    property Topic: ITopic read getTopic;
  end;
  
  {{
  A TemporaryTopic object is a unique Topic object created for the duration of 
  a TopicConnection. It is a system-defined topic that can be consumed only by 
  the TopicConnection that created it. 
  
  
  See Also: 
  TopicSession.createTemporaryTopic()
  }
  ITemporaryTopic = interface (ITopic)
    ['{2F54E1C0-AD84-4073-8818-C2A83F079AA2}']
    {{
    Deletes this temporary topic. If there are existing subscribers still using 
    it, a JMSException will be thrown.
    Throws:
    JMSException - if the JMS provider fails to delete the temporary topic due 
    to some internal error.
    }
    procedure delete;
  end;
  
  {{
  A client uses a QueueReceiver object to receive messages that have been 
  delivered to a queue. 
  
  Although it is possible to have multiple QueueReceivers for the same queue, 
  the JMS API does not define how messages are distributed between the 
  QueueReceivers. 
  
  If a QueueReceiver specifies a message selector, the messages that are not 
  selected remain on the queue. By definition, a message selector allows a 
  QueueReceiver to skip messages. This means that when the skipped messages are 
  eventually read, the total ordering of the reads does not retain the partial 
  order defined by each message producer. Only QueueReceivers without a message 
  selector will read messages in message producer order. 
  
  
  See Also: 
  QueueSession.createReceiver(Queue, String), QueueSession.createReceiver(
  Queue), MessageConsumer
  }
  IQueueReceiver = interface (IMessageConsumer)
    ['{2AF03CFD-C3CA-4EBF-9128-A9BB98AF83A1}']
    {{
    Gets the Queue associated with this queue receiver.
    Returns:
    this receiver's Queue
    Throws:
    JMSException - if the JMS provider fails to get the queue for this queue 
    receiver due to some internal error.
    }
    function getQueue: IQueue;
    {{
    Gets the Queue associated with this queue receiver.
    Returns:
    this receiver's Queue
    Throws:
    JMSException - if the JMS provider fails to get the queue for this queue 
    receiver due to some internal error.
    }
    property Queue: IQueue read getQueue;
  end;
  
  {{
  A ConnectionFactory object encapsulates a set of connection configuration 
  parameters that has been defined by an administrator. A client uses it to 
  create a connection with a JMS provider. 
  
  ConnectionFactory objects support concurrent use. 
  
  A ConnectionFactory object is a JMS administered object. 
  
  JMS administered objects are objects containing configuration information 
  that are created by an administrator and later used by JMS clients. They make 
  it practical to administer the JMS API in the enterprise. 
  
  Although the interfaces for administered objects do not explicitly depend on 
  the Java Naming and Directory Interface (JNDI) API, the JMS API establishes 
  the convention that JMS clients find administered objects by looking them up 
  in a JNDI namespace. 
  
  An administrator can place an administered object anywhere in a namespace. 
  The JMS API does not define a naming policy. 
  
  It is expected that JMS providers will provide the tools an administrator 
  needs to create and configure administered objects in a JNDI namespace. JMS 
  provider implementations of administered objects should be both 
  javax.jndi.Referenceable and java.io.Serializable so that they can be stored 
  in all JNDI naming contexts. In addition, it is recommended that these 
  implementations follow the JavaBeansTM design patterns. 
  
  This strategy provides several benefits: 
  
  It hides provider-specific details from JMS clients. 
  It abstracts administrative information into objects in the Java programming 
  language ("Java objects") that are easily organized and administered from a 
  common management console. 
  Since there will be JNDI providers for all popular naming services, this 
  means that JMS providers can deliver one implementation of administered 
  objects that will run everywhere. 
  An administered object should not hold on to any remote resources. Its lookup 
  should not use remote resources other than those used by the JNDI API itself. 
  
  Clients should think of administered objects as local Java objects. Looking 
  them up should not have any hidden side effects or use surprising amounts of 
  local resources. 
  
  
  See Also: 
  QueueConnectionFactory, TopicConnectionFactory
  }
  IConnectionFactory = interface (IUnknown)
    ['{5EEB9231-AF54-485F-A346-4584956D7678}']
  end;
  
  {{
  For application servers, Connection objects provide a special facility for 
  creating a ConnectionConsumer (optional). The messages it is to consume are 
  specified by a Destination and a message selector. In addition, a 
  ConnectionConsumer must be given a ServerSessionPool to use for processing 
  its messages. 
  
  Normally, when traffic is light, a ConnectionConsumer gets a ServerSession 
  from its pool, loads it with a single message, and starts it. As traffic 
  picks up, messages can back up. If this happens, a ConnectionConsumer can 
  load each ServerSession with more than one message. This reduces the thread 
  context switches and minimizes resource use at the expense of some 
  serialization of message processing. 
  }
  IConnectionConsumer = interface (IUnknown)
    ['{A64131C5-8EA1-48A8-8312-CF94F4813788}']
    {{
    Closes the connection consumer. 
    Since a provider may allocate some resources on behalf of a connection 
    consumer outside the Java virtual machine, clients should close these 
    resources when they are not needed. Relying on garbage collection to 
    eventually reclaim these resources may not be timely enough.
    
    Throws:
    JMSException - if the JMS provider fails to release resources on behalf of 
    the connection consumer or fails to close the connection consumer.
    }
    procedure close;
    {{
    Gets the server session pool associated with this connection consumer.
    Returns:
    the server session pool used by this connection consumer
    Throws:
    JMSException - if the JMS provider fails to get the server session pool 
    associated with this consumer due to some internal error.
    }
    function getServerSessionPool: IServerSessionPool;
    property ServerSessionPool: IServerSessionPool read getServerSessionPool;
  end;
  
  {{
  A client uses a TopicSubscriber object to receive messages that have been 
  published to a topic. A TopicSubscriber object is the publish/subscribe form 
  of a message consumer. 
  
  A TopicSession allows the creation of multiple TopicSubscriber objects per 
  topic. It will deliver each message for a topic to each subscriber eligible 
  to receive it. Each copy of the message is treated as a completely separate 
  message. Work done on one copy has no effect on the others; acknowledging one 
  does not acknowledge the others; one message may be delivered immediately, 
  while another waits for its subscriber to process messages ahead of it. 
  
  Regular TopicSubscriber objects are not durable. They receive only messages 
  that are published while they are active. 
  
  Messages filtered out by a subscriber's message selector will never be 
  delivered to the subscriber. From the subscriber's perspective, they do not 
  exist. 
  
  In some cases, a connection may both publish and subscribe to a topic. The 
  subscriber NoLocal attribute allows a subscriber to inhibit the delivery of 
  messages published by its own connection. 
  
  If a client needs to receive all the messages published on a topic, including 
  the ones published while the subscriber is inactive, it uses a durable 
  TopicSubscriber. The JMS provider retains a record of this durable 
  subscription and insures that all messages from the topic's publishers are 
  retained until they are acknowledged by this durable subscriber or they have 
  expired. 
  
  Sessions with durable subscribers must always provide the same client 
  identifier. In addition, each client must specify a name that uniquely 
  identifies (within client identifier) each durable subscription it creates. 
  Only one session at a time can have a TopicSubscriber for a particular 
  durable subscription. 
  
  A client can change an existing durable subscription by creating a durable 
  TopicSubscriber with the same name and a new topic and/or message selector. 
  Changing a durable subscription is equivalent to unsubscribing (deleting) the 
  old one and creating a new one. 
  
  TopicSessions provide the unsubscribe method for deleting a durable 
  subscription created by their client. This method deletes the state being 
  maintained on behalf of the subscriber by its provider. 
  
  
  See Also: 
  TopicSession, TopicSession.createSubscriber(Topic), 
  TopicSession.createSubscriber(Topic, String, boolean), 
  TopicSession.createDurableSubscriber(Topic, String), 
  TopicSession.createDurableSubscriber(Topic, String, String, boolean), 
  MessageConsumer
  }
  ITopicSubscriber = interface (IMessageConsumer)
    ['{FC507E10-55D6-46B4-913D-1F4D0617D4DB}']
    {{
    Gets the NoLocal attribute for this subscriber. The default value for this 
    attribute is false.
    Returns:
    true if locally published messages are being inhibited
    Throws:
    JMSException - if the JMS provider fails to get the NoLocal attribute for 
    this topic subscriber due to some internal error.
    }
    function getNoLocal: Boolean;
    {{
    Gets the Topic associated with this subscriber.
    Returns:
    this subscriber's Topic
    Throws:
    JMSException - if the JMS provider fails to get the topic for this topic 
    subscriber due to some internal error.
    }
    function getTopic: ITopic;
    {{
    Gets the NoLocal attribute for this subscriber. The default value for this 
    attribute is false.
    Returns:
    true if locally published messages are being inhibited
    Throws:
    JMSException - if the JMS provider fails to get the NoLocal attribute for 
    this topic subscriber due to some internal error.
    }
    property NoLocal: Boolean read getNoLocal;
    {{
    Gets the Topic associated with this subscriber.
    Returns:
    this subscriber's Topic
    Throws:
    JMSException - if the JMS provider fails to get the topic for this topic 
    subscriber due to some internal error.
    }
    property Topic: ITopic read getTopic;
  end;
  
  {{
  A TopicSession object provides methods for creating TopicPublisher, 
  TopicSubscriber, and TemporaryTopic objects. It also provides a method for 
  deleting its client's durable subscribers. 
  
  
  See Also: 
  Session, TopicConnection.createTopicSession(boolean, int), 
  XATopicSession.getTopicSession()
  }
  ITopicSession = interface (ISession)
    ['{7875A1CE-7B77-4D2E-A5B0-1CBF13A9D7C7}']
    {{
    Creates a durable subscriber to the specified topic. 
    If a client needs to receive all the messages published on a topic, 
    including the ones published while the subscriber is inactive, it uses a 
    durable TopicSubscriber. The JMS provider retains a record of this durable 
    subscription and insures that all messages from the topic's publishers are 
    retained until they are acknowledged by this durable subscriber or they 
    have expired. 
    
    Sessions with durable subscribers must always provide the same client 
    identifier. In addition, each client must specify a name that uniquely 
    identifies (within client identifier) each durable subscription it creates. 
    Only one session at a time can have a TopicSubscriber for a particular 
    durable subscription. 
    
    A client can change an existing durable subscription by creating a durable 
    TopicSubscriber with the same name and a new topic and/or message selector. 
    Changing a durable subscriber is equivalent to unsubscribing (deleting) the 
    old one and creating a new one. 
    
    In some cases, a connection may both publish and subscribe to a topic. The 
    subscriber NoLocal attribute allows a subscriber to inhibit the delivery of 
    messages published by its own connection. The default value for this 
    attribute is false.
    
    Parameters:
    topic - the non-temporary Topic to subscribe to
    name - the name used to identify this subscription
    Throws:
    JMSException - if the session fails to create a subscriber due to some 
    internal error.
    InvalidDestinationException - if an invalid topic is specified.
    }
    function createDurableSubscriber(topic : ITopic; name : string): 
            ITopicSubscriber; overload;
    {{
    Creates a durable subscriber to the specified topic, using a message 
    selector or specifying whether messages published by its own connection 
    should be delivered to it. 
    If a client needs to receive all the messages published on a topic, 
    including the ones published while the subscriber is inactive, it uses a 
    durable TopicSubscriber. The JMS provider retains a record of this durable 
    subscription and insures that all messages from the topic's publishers are 
    retained until they are acknowledged by this durable subscriber or they 
    have expired. 
    
    Sessions with durable subscribers must always provide the same client 
    identifier. In addition, each client must specify a name which uniquely 
    identifies (within client identifier) each durable subscription it creates. 
    Only one session at a time can have a TopicSubscriber for a particular 
    durable subscription. An inactive durable subscriber is one that exists but 
    does not currently have a message consumer associated with it. 
    
    A client can change an existing durable subscription by creating a durable 
    TopicSubscriber with the same name and a new topic and/or message selector. 
    Changing a durable subscriber is equivalent to unsubscribing (deleting) the 
    old one and creating a new one.
    
    Parameters:
    topic - the non-temporary Topic to subscribe to
    name - the name used to identify this subscription
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    noLocal - if set, inhibits the delivery of messages published by its own 
    connection
    Throws:
    JMSException - if the session fails to create a subscriber due to some 
    internal error.
    InvalidDestinationException - if an invalid topic is specified.
    InvalidSelectorException - if the message selector is invalid.
    }
    function createDurableSubscriber(topic : ITopic; name : string; 
            messageSelector : string; noLocal :boolean): ITopicSubscriber; 
            overload;
    {{
    Creates a publisher for the specified topic. 
    A client uses a TopicPublisher object to publish messages on a topic. Each 
    time a client creates a TopicPublisher on a topic, it defines a new 
    sequence of messages that have no ordering relationship with the messages 
    it has previously sent.
    
    Parameters:
    topic - the Topic to publish to, or null if this is an unidentified producer
    Throws:
    JMSException - if the session fails to create a publisher due to some 
    internal error.
    InvalidDestinationException - if an invalid topic is specified.
    }
    function createPublisher(topic : ITopic): ITopicPublisher;
    {{
    Creates a nondurable subscriber to the specified topic. 
    A client uses a TopicSubscriber object to receive messages that have been 
    published to a topic. 
    
    Regular TopicSubscriber objects are not durable. They receive only messages 
    that are published while they are active. 
    
    In some cases, a connection may both publish and subscribe to a topic. The 
    subscriber NoLocal attribute allows a subscriber to inhibit the delivery of 
    messages published by its own connection. The default value for this 
    attribute is false.
    
    Parameters:
    topic - the Topic to subscribe to
    Throws:
    JMSException - if the session fails to create a subscriber due to some 
    internal error.
    InvalidDestinationException - if an invalid topic is specified.
    }
    function createSubscriber(topic : ITopic): ITopicSubscriber; overload;
    {{
    Creates a nondurable subscriber to the specified topic, using a message 
    selector or specifying whether messages published by its own connection 
    should be delivered to it. 
    A client uses a TopicSubscriber object to receive messages that have been 
    published to a topic. 
    
    Regular TopicSubscriber objects are not durable. They receive only messages 
    that are published while they are active. 
    
    Messages filtered out by a subscriber's message selector will never be 
    delivered to the subscriber. From the subscriber's perspective, they do not 
    exist. 
    
    In some cases, a connection may both publish and subscribe to a topic. The 
    subscriber NoLocal attribute allows a subscriber to inhibit the delivery of 
    messages published by its own connection. The default value for this 
    attribute is false.
    
    Parameters:
    topic - the Topic to subscribe to
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    noLocal - if set, inhibits the delivery of messages published by its own 
    connection
    Throws:
    JMSException - if the session fails to create a subscriber due to some 
    internal error.
    InvalidDestinationException - if an invalid topic is specified.
    InvalidSelectorException - if the message selector is invalid.
    }
    function createSubscriber(topic : ITopic; messageSelector : string; noLocal 
            :boolean): ITopicSubscriber; overload;
    {{
    Creates a TemporaryTopic object. Its lifetime will be that of the 
    TopicConnection unless it is deleted earlier.
    Returns:
    a temporary topic identity
    Throws:
    JMSException - if the session fails to create a temporary topic due to some 
    internal error.
    }
    function createTemporaryTopic: ITemporaryTopic;
    {{
    Creates a topic identity given a Topic name. 
    This facility is provided for the rare cases where clients need to 
    dynamically manipulate topic identity. This allows the creation of a topic 
    identity with a provider-specific name. Clients that depend on this ability 
    are not portable. 
    
    Note that this method is not for creating the physical topic. The physical 
    creation of topics is an administrative task and is not to be initiated by 
    the JMS API. The one exception is the creation of temporary topics, which 
    is accomplished with the createTemporaryTopic method.
    
    Parameters:
    topicName - the name of this Topic
    Returns:
    a Topic with the given name
    Throws:
    JMSException - if the session fails to create a topic due to some internal 
    error.
    }
    function createTopic(topicName : string): ITopic;
    {{
    Unsubscribes a durable subscription that has been created by a client. 
    This method deletes the state being maintained on behalf of the subscriber 
    by its provider. 
    
    It is erroneous for a client to delete a durable subscription while there 
    is an active TopicSubscriber for the subscription, or while a consumed 
    message is part of a pending transaction or has not been acknowledged in 
    the session.
    
    Parameters:
    name - the name used to identify this subscription
    Throws:
    JMSException - if the session fails to unsubscribe to the durable 
    subscription due to some internal error.
    InvalidDestinationException - if an invalid subscription name is specified.
    }
    procedure unsubscribe(name : string);
  end;
  
  {{
  A TopicConnection object is an active connection to a publish/subscribe JMS 
  provider. A client uses a TopicConnection object to create one or more 
  TopicSession objects for producing and consuming messages. 
  }
  ITopicConnection = interface (IConnection)
    ['{C2D47BC8-5D5F-4F4B-8775-13E2DDACE4FE}']
    {{
    Creates a connection consumer for this connection (optional operation). 
    This is an expert facility not used by regular JMS clients.
    Parameters:
    topic - the topic to access
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    sessionPool - the server session pool to associate with this connection 
    consumer
    maxMessages - the maximum number of messages that can be assigned to a 
    server session at one time
    Returns:
    the connection consumer
    Throws:
    JMSException - if the TopicConnection object fails to create a connection 
    consumer due to some internal error or invalid arguments for sessionPool 
    and messageSelector.
    InvalidDestinationException - if an invalid topic is specified.
    InvalidSelectorException - if the message selector is invalid.
    See Also: 
    ConnectionConsumer
    }
    function createConnectionConsumer(topic : ITopic; messageSelector : string; 
            sessionPool : IServerSessionPool; maxMessages : longint): 
            IConnectionConsumer;
    {{
    Create a durable connection consumer for this connection (optional 
    operation). This is an expert facility not used by regular JMS clients.
    Parameters:
    topic - the topic to access
    subscriptionName - durable subscription name
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    sessionPool - the server session pool to associate with this durable 
    connection consumer
    maxMessages - the maximum number of messages that can be assigned to a 
    server session at one time
    Returns:
    the durable connection consumer
    Throws:
    JMSException - if the TopicConnection object fails to create a connection 
    consumer due to some internal error or invalid arguments for sessionPool 
    and messageSelector.
    InvalidDestinationException - if an invalid topic is specified.
    InvalidSelectorException - if the message selector is invalid.
    See Also: 
    ConnectionConsumer
    }
    function createDurableConnectionConsumer(topic : ITopic; subscriptionName : 
            string; messageSelector : string; sessionPool : IServerSessionPool; 
            maxMessages : longint): IConnectionConsumer;
    {{
    Creates a TopicSession object.
    Parameters:
    transacted - indicates whether the session is transacted
    acknowledgeMode - indicates whether the consumer or the client will 
    acknowledge any messages it receives; ignored if the session is transacted. 
    Legal values are Session.AUTO_ACKNOWLEDGE, Session.CLIENT_ACKNOWLEDGE, and 
    Session.DUPS_OK_ACKNOWLEDGE.
    Returns:
    a newly created topic session
    Throws:
    JMSException - if the TopicConnection object fails to create a session due 
    to some internal error or lack of support for the specific transaction and 
    acknowledgement mode.
    See Also: 
    Session.AUTO_ACKNOWLEDGE, Session.CLIENT_ACKNOWLEDGE, 
    Session.DUPS_OK_ACKNOWLEDGE
    }
    function createTopicSession(transacted : boolean; acknowledgeMode : 
            longint): ITopicSession;
  end;
  
  {{
  A client uses a TopicConnectionFactory object to create TopicConnection 
  objects with a publish/subscribe JMS provider. 
  }
  ITopicConnectionFactory = interface (IConnectionFactory)
    ['{AE51E793-D3F1-45C5-9060-F89516709BB0}']
    {{
    Creates a topic connection with the default user identity. The connection 
    is created in stopped mode. No messages will be delivered until the 
    Connection.start method is explicitly called.
    Returns:
    a newly created topic connection
    Throws:
    JMSException - if the JMS provider fails to create a topic connection due 
    to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createTopicConnection: ITopicConnection; overload;
    {{
    Creates a topic connection with the specified user identity. The connection 
    is created in stopped mode. No messages will be delivered until the 
    Connection.start method is explicitly called.
    Parameters:
    userName - the caller's user name
    password - the caller's password
    Returns:
    a newly created topic connection
    Throws:
    JMSException - if the JMS provider fails to create a topic connection due 
    to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createTopicConnection(userName : string; password : string): 
            ITopicConnection; overload;
  end;
  
  {{
  A client uses a QueueSender object to send messages to a queue. 
  
  Normally, the Queue is specified when a QueueSender is created. In this case, 
  an attempt to use the send methods for an unidentified QueueSender will throw 
  a java.lang.UnsupportedOperationException. 
  
  If the QueueSender is created with an unidentified Queue, an attempt to use 
  the send methods that assume that the Queue has been identified will throw a 
  java.lang.UnsupportedOperationException. 
  
  During the execution of its send method, a message must not be changed by 
  other threads within the client. If the message is modified, the result of 
  the send is undefined. 
  
  After sending a message, a client may retain and modify it without affecting 
  the message that has been sent. The same message object may be sent multiple 
  times. 
  
  The following message headers are set as part of sending a message: 
  JMSDestination, JMSDeliveryMode, JMSExpiration, JMSPriority, JMSMessageID and 
  JMSTimeStamp. When the message is sent, the values of these headers are 
  ignored. After the completion of the send, the headers hold the values 
  specified by the method sending the message. It is possible for the send 
  method not to set JMSMessageID and JMSTimeStamp if the setting of these 
  headers is explicitly disabled by the MessageProducer.setDisableMessageID or 
  MessageProducer.setDisableMessageTimestamp method. 
  
  
  See Also: 
  MessageProducer, QueueSession.createSender(Queue)
  }
  IQueueSender = interface (IMessageProducer)
    ['{A3110EF6-27D2-4E32-9C81-1B8B3C8D7175}']
    {{
    Gets the queue associated with this QueueSender.
    Returns:
    this sender's queue
    Throws:
    JMSException - if the JMS provider fails to get the queue for this 
    QueueSender due to some internal error.
    }
    function getQueue: IQueue;
    {{
    Sends a message to the queue. Uses the QueueSender's default delivery mode, 
    priority, and time to live.
    Parameters:
    message - the message to send
    Throws:
    JMSException - if the JMS provider fails to send the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with a 
    QueueSender with an invalid queue.
    java.lang.UnsupportedOperationException - if a client uses this method with 
    a QueueSender that did not specify a queue at creation time.
    See Also: 
    MessageProducer.getDeliveryMode(), MessageProducer.getTimeToLive(), 
    MessageProducer.getPriority()
    }
    procedure send(message : IMessage); overload;
    {{
    Sends a message to the queue, specifying delivery mode, priority, and time 
    to live.
    Parameters:
    message - the message to send
    deliveryMode - the delivery mode to use
    priority - the priority for this message
    timeToLive - the message's lifetime (in milliseconds)
    Throws:
    JMSException - if the JMS provider fails to send the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with a 
    QueueSender with an invalid queue.
    java.lang.UnsupportedOperationException - if a client uses this method with 
    a QueueSender that did not specify a queue at creation time.
    }
    procedure send(message : IMessage; deliveryMode : LongInt; priority : 
            LongInt; timeToLive : int64); overload;
    {{
    Sends a message to a queue for an unidentified message producer. Uses the 
    QueueSender's default delivery mode, priority, and time to live. 
    Typically, a message producer is assigned a queue at creation time; however,
    the JMS API also supports unidentified message producers, which require 
    that the queue be supplied every time a message is sent.
    
    Parameters:
    queue - the queue to send this message to
    message - the message to send
    Throws:
    JMSException - if the JMS provider fails to send the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with an invalid 
    queue.
    See Also: 
    MessageProducer.getDeliveryMode(), MessageProducer.getTimeToLive(), 
    MessageProducer.getPriority()
    }
    procedure send(queue : IQueue; message : IMessage); overload;
    {{
    Sends a message to a queue for an unidentified message producer, specifying 
    delivery mode, priority and time to live. 
    Typically, a message producer is assigned a queue at creation time; however,
    the JMS API also supports unidentified message producers, which require 
    that the queue be supplied every time a message is sent.
    
    Parameters:
    queue - the queue to send this message to
    message - the message to send
    deliveryMode - the delivery mode to use
    priority - the priority for this message
    timeToLive - the message's lifetime (in milliseconds)
    Throws:
    JMSException - if the JMS provider fails to send the message due to some 
    internal error.
    MessageFormatException - if an invalid message is specified.
    InvalidDestinationException - if a client uses this method with an invalid 
    queue.
    }
    procedure send(queue : IQueue; message : IMessage; deliveryMode  :LongInt; 
            priority : LongInt; timeToLive : int64); overload;
    {{
    Gets the queue associated with this QueueSender.
    Returns:
    this sender's queue
    Throws:
    JMSException - if the JMS provider fails to get the queue for this 
    QueueSender due to some internal error.
    }
    property Queue: IQueue read getQueue;
  end;
  
  {{
  A client uses a QueueBrowser object to look at messages on a queue without 
  removing them. 
  
  The getEnumeration method returns a java.util.Enumeration that is used to 
  scan the queue's messages. It may be an enumeration of the entire content of 
  a queue, or it may contain only the messages matching a message selector. 
  
  Messages may be arriving and expiring while the scan is done. The JMS API 
  does not require the content of an enumeration to be a static snapshot of 
  queue content. Whether these changes are visible or not depends on the JMS 
  provider. 
  
  
  See Also: 
  QueueSession.createBrowser(Queue), QueueSession.createBrowser(Queue, String), 
  QueueReceiver
  }
  IQueueBrowser = interface (IUnknown)
    ['{FF80B097-3750-473C-B49F-5259E1014FD6}']
    {{
    Closes the QueueBrowser. 
    Since a provider may allocate some resources on behalf of a QueueBrowser 
    outside the Java virtual machine, clients should close them when they are 
    not needed. Relying on garbage collection to eventually reclaim these 
    resources may not be timely enough.
    
    Throws:
    JMSException - if the JMS provider fails to close this browser due to some 
    internal error.
    }
    procedure close;
    {{
    Gets an enumeration for browsing the current queue messages in the order 
    they would be received.
    Returns:
    an enumeration for browsing the messages
    Throws:
    JMSException - if the JMS provider fails to get the enumeration for this 
    browser due to some internal error.
    }
    function getEnumeration: TList;
    {{
    Gets this queue browser's message selector expression.
    Returns:
    this queue browser's message selector, or null if no message selector 
    exists for the message consumer (that is, if the message selector was not 
    set or was set to null or the empty string)
    Throws:
    JMSException - if the JMS provider fails to get the message selector for 
    this browser due to some internal error.
    }
    function getMessageSelector: string;
    {{
    Gets the queue associated with this queue browser.
    Returns:
    the queue
    Throws:
    JMSException - if the JMS provider fails to get the queue associated with 
    this browser due to some internal error.
    }
    function getQueue: IQueue;
    {{
    Gets an enumeration for browsing the current queue messages in the order 
    they would be received.
    Returns:
    an enumeration for browsing the messages
    Throws:
    JMSException - if the JMS provider fails to get the enumeration for this 
    browser due to some internal error.
    }
    property Enumeration: TList read getEnumeration;
    {{
    Gets this queue browser's message selector expression.
    Returns:
    this queue browser's message selector, or null if no message selector 
    exists for the message consumer (that is, if the message selector was not 
    set or was set to null or the empty string)
    Throws:
    JMSException - if the JMS provider fails to get the message selector for 
    this browser due to some internal error.
    }
    property MessageSelector: string read getMessageSelector;
    {{
    Gets the queue associated with this queue browser.
    Returns:
    the queue
    Throws:
    JMSException - if the JMS provider fails to get the queue associated with 
    this browser due to some internal error.
    }
    property Queue: IQueue read getQueue;
  end;
  
  {{
  A QueueSession object provides methods for creating QueueReceiver, 
  QueueSender, QueueBrowser, and TemporaryQueue objects. 
  
  If there are messages that have been received but not acknowledged when a 
  QueueSession terminates, these messages will be retained and redelivered when 
  a consumer next accesses the queue. 
  
  
  See Also: 
  Session, QueueConnection.createQueueSession(boolean, int), 
  XAQueueSession.getQueueSession()
  }
  IQueueSession = interface (ISession)
    ['{0992E7B1-9F33-4067-B619-38BC94BF89EF}']
    {{
    Creates a QueueBrowser object to peek at the messages on the specified 
    queue.
    Parameters:
    queue - the Queue to access
    Throws:
    JMSException - if the session fails to create a browser due to some 
    internal error.
    InvalidDestinationException - if an invalid queue is specified.
    }
    function createBrowser(queue : IQueue): IQueueBrowser; overload;
    {{
    Creates a QueueBrowser object to peek at the messages on the specified 
    queue using a message selector.
    Parameters:
    queue - the Queue to access
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    Throws:
    JMSException - if the session fails to create a browser due to some 
    internal error.
    InvalidDestinationException - if an invalid queue is specified.
    InvalidSelectorException - if the message selector is invalid.
    }
    function createBrowser(queue : IQueue; messageSelector : string): 
            IQueueBrowser; overload;
    {{
    Creates a queue identity given a Queue name. 
    This facility is provided for the rare cases where clients need to 
    dynamically manipulate queue identity. It allows the creation of a queue 
    identity with a provider-specific name. Clients that depend on this ability 
    are not portable. 
    
    Note that this method is not for creating the physical queue. The physical 
    creation of queues is an administrative task and is not to be initiated by 
    the JMS API. The one exception is the creation of temporary queues, which 
    is accomplished with the createTemporaryQueue method.
    
    Parameters:
    queueName - the name of this Queue
    Returns:
    a Queue with the given name
    Throws:
    JMSException - if the session fails to create a queue due to some internal 
    error.
    }
    function createQueue(queueName : string): IQueue;
    {{
    Creates a QueueReceiver object to receive messages from the specified queue.
    Parameters:
    queue - the Queue to access
    Throws:
    JMSException - if the session fails to create a receiver due to some 
    internal error.
    InvalidDestinationException - if an invalid queue is specified.
    }
    function createReceiver(queue : IQueue): IQueueReceiver; overload;
    {{
    Creates a QueueReceiver object to receive messages from the specified queue 
    using a message selector.
    Parameters:
    queue - the Queue to access
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    Throws:
    JMSException - if the session fails to create a receiver due to some 
    internal error.
    InvalidDestinationException - if an invalid queue is specified.
    InvalidSelectorException - if the message selector is invalid.
    }
    function createReceiver(queue : IQueue; messageSelector : string): 
            IQueueReceiver; overload;
    {{
    Creates a QueueSender object to send messages to the specified queue.
    Parameters:
    queue - the Queue to access, or null if this is an unidentified producer
    Throws:
    JMSException - if the session fails to create a sender due to some internal 
    error.
    InvalidDestinationException - if an invalid queue is specified.
    }
    function createSender(queue : IQueue): IQueueSender;
    {{
    Creates a TemporaryQueue object. Its lifetime will be that of the 
    QueueConnection unless it is deleted earlier.
    Returns:
    a temporary queue identity
    Throws:
    JMSException - if the session fails to create a temporary queue due to some 
    internal error.
    }
    function createTemporaryQueue: ITemporaryQueue;
  end;
  
  {{
  A QueueConnection object is an active connection to a point-to-point JMS 
  provider. A client uses a QueueConnection object to create one or more 
  QueueSession objects for producing and consuming messages. 
  }
  IQueueConnection = interface (IConnection)
    ['{A113B0C4-E073-4CA9-84AB-F934F76A6EEB}']
    {{
    Creates a connection consumer for this connection (optional operation). 
    This is an expert facility not used by regular JMS clients.
    Parameters:
    queue - the queue to access
    messageSelector - only messages with properties matching the message 
    selector expression are delivered. A value of null or an empty string 
    indicates that there is no message selector for the message consumer.
    sessionPool - the server session pool to associate with this connection 
    consumer
    maxMessages - the maximum number of messages that can be assigned to a 
    server session at one time
    Returns:
    the connection consumer
    Throws:
    JMSException - if the QueueConnection object fails to create a connection 
    consumer due to some internal error or invalid arguments for sessionPool 
    and messageSelector.
    InvalidDestinationException - if an invalid queue is specified.
    InvalidSelectorException - if the message selector is invalid.
    See Also: 
    ConnectionConsumer
    }
    function createConnectionConsumer(queue : IQueue; messageSelector : string; 
            sessionPool : IServerSessionPool; maxMessages : longint): 
            IConnectionConsumer;
    {{
    Creates a QueueSession object.
    Parameters:
    transacted - indicates whether the session is transacted
    acknowledgeMode - indicates whether the consumer or the client will 
    acknowledge any messages it receives; ignored if the session is transacted. 
    Legal values are Session.AUTO_ACKNOWLEDGE, Session.CLIENT_ACKNOWLEDGE, and 
    Session.DUPS_OK_ACKNOWLEDGE.
    Returns:
    a newly created queue session
    Throws:
    JMSException - if the QueueConnection object fails to create a session due 
    to some internal error or lack of support for the specific transaction and 
    acknowledgement mode.
    See Also: 
    Session.AUTO_ACKNOWLEDGE, Session.CLIENT_ACKNOWLEDGE, 
    Session.DUPS_OK_ACKNOWLEDGE
    }
    function createQueueSession(transacted : boolean; acknowledgeMode : 
            longint): IQueueSession;
  end;
  
  {{
  A client uses a QueueConnectionFactory object to create QueueConnection 
  objects with a point-to-point JMS provider. 
  }
  IQueueConnectionFactory = interface (IConnectionFactory)
    ['{06E6C8F4-B460-4A90-BE88-5DD90E494D9B}']
    {{
    Creates a queue connection with the default user identity. The connection 
    is created in stopped mode. No messages will be delivered until the 
    Connection.start method is explicitly called.
    Returns:
    a newly created queue connection
    Throws:
    JMSException - if the JMS provider fails to create the queue connection due 
    to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createQueueConnection: IQueueConnection; overload;
    {{
    Creates a queue connection with the specified user identity. The connection 
    is created in stopped mode. No messages will be delivered until the 
    Connection.start method is explicitly called.
    Parameters:
    userName - the caller's user name
    password - the caller's password
    Returns:
    a newly created queue connection
    Throws:
    JMSException - if the JMS provider fails to create the queue connection due 
    to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createQueueConnection(userName : string; password : string): 
            IQueueConnection; overload;
  end;
  
  IXASession = interface (ISession)
    ['{E0674023-B92B-43EE-ACA3-442DEDFFB532}']
  end;
  
  IXATopicSession = interface (IXASession)
    ['{2E599001-D384-4677-939A-7DA0B979E5B2}']
  end;
  
  IXAQueueSession = interface (IXASession)
    ['{7C74396B-7C43-4903-9519-44817B6DF2A2}']
  end;
  
  IXAConnection = interface (IUnknown)
    ['{C01E077B-2501-4722-A303-9141B205BEC0}']
  end;
  
  IXATopicConnection = interface (ITopicConnection)
    ['{EB1F1A03-D046-4368-86E1-E954F23ED9DD}']
  end;
  
  IXAQueueConnection = interface (IQueueConnection)
    ['{38F75FE5-DFFC-4403-A6E1-BC62AF88EAFF}']
  end;
  
  IXAConnectionFactory = interface (IUnknown)
    ['{30E7EA87-EE1D-4BE1-8E8E-52E3FA76442F}']
  end;
  
  {{
  An XATopicConnectionFactory provides the same create options as a 
  TopicConnectionFactory (optional). 
  }
  IXATopicConnectionFactory = interface (ITopicConnectionFactory)
    ['{1913B203-79EF-47E9-B12A-162E6B34C0F3}']
    {{
    Creates an XA topic connection with the default user identity. The 
    connection is created in stopped mode. No messages will be delivered until 
    the Connection.start method is explicitly called.
    Returns:
    a newly created XA topic connection
    Throws:
    JMSException - if the JMS provider fails to create an XA topic connection 
    due to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createXATopicConnection: IXATopicConnection; overload;
    {{
    Creates an XA topic connection with the specified user identity. The 
    connection is created in stopped mode. No messages will be delivered until 
    the Connection.start method is explicitly called.
    Parameters:
    userName - the caller's user name
    password - the caller's password
    Returns:
    a newly created XA topic connection
    Throws:
    JMSException - if the JMS provider fails to create an XA topic connection 
    due to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createXATopicConnection(userName : string; password : string): 
            IXATopicConnection; overload;
  end;
  
  {{
  An XAQueueConnectionFactory provides the same create options as a 
  QueueConnectionFactory (optional). 
  }
  IXAQueueConnectionFactory = interface (IQueueConnectionFactory)
    ['{E7217436-5699-4ADB-A89E-54EDAFD5E128}']
    {{
    Creates an XA queue connection with the default user identity. The 
    connection is created in stopped mode. No messages will be delivered until 
    the Connection.start method is explicitly called.
    Returns:
    a newly created XA queue connection
    Throws:
    JMSException - if the JMS provider fails to create an XA queue connection 
    due to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createXAQueueConnection: IXAQueueConnection; overload;
    {{
    Creates an XA queue connection with the specified user identity. The 
    connection is created in stopped mode. No messages will be delivered until 
    the Connection.start method is explicitly called.
    Parameters:
    userName - the caller's user name
    password - the caller's password
    Returns:
    a newly created XA queue connection
    Throws:
    JMSException - if the JMS provider fails to create an XA queue connection 
    due to some internal error.
    JMSSecurityException - if client authentication fails due to an invalid 
    user name or password.
    }
    function createXAQueueConnection(userName : string; password : string): 
            IXAQueueConnection; overload;
  end;
  

implementation


end.
