//*****************************************************************************
//
//    System          : jmsSMQ 
//    Category        : JMS API realize for SonicMQ using SonicMQ C Client
//    Source          : none 
//
//    Copyright (c), Leonov Alexander
//
//    File version $Revision:1.0$ at $Date:24.11.2003 13:40:38$
//
//    Author : Leonov Alexander
//    web    : http://home.ural.ru/~tigra/
//    e-mail : shura@ol405.paco.net
//
//*****************************************************************************

unit jmsSonicMQ;

interface

uses
  SysUtils, Classes, jmsException, jmsIntf,
  apiSonicMQ;

type

  TSMQString = class;

  ISMQObject = interface (IUnknown)
    ['{16A96BF2-F2A6-45DA-B76B-EB340B91D8DE}']
    function getObj: HOBJ;
    procedure setObj(Value: HOBJ);
    property Obj: HOBJ read getObj write setObj;
  end;
  
  TSMQObject = class (TInterfacedObject, ISMQObject)
  protected
    FObj: HOBJ;
    function booleanToJBoolean(value : boolean): jboolean;
    procedure CheckSuccess(iRes : TRetVal);
    procedure DestroyHandle;
    function getObj: HOBJ; virtual;
    procedure setObj(Value: HOBJ); virtual;
    function strObjToString(strObj : HOBJ): string;
    procedure UnsupportedMethod(method : string);
  public
    constructor Create(AObj : HOBJ); overload; virtual;
    destructor Destroy; override;
    function equals(objInst : TSMQObject): Boolean;
    function toString: string;
    property Obj: HOBJ read getObj write setObj;
  end;
  
  TSMQString = class (TSMQObject)
  protected
    function getText: string; virtual;
    procedure setText(Value: string); virtual;
  public
    constructor CreateStr(value : string); virtual;
    property Text: string read getText write setText;
  end;
  
  TSMQMessage = class (TSMQObject, IMessage)
  protected
    procedure acknowledge;
    procedure clearBody;
    procedure clearProperties;
    function getBooleanProperty(name : string): Boolean;
    function getByteProperty(name : string): Byte;
    function getDEFAULT_DELIVERY_MODE: LongInt;
    function getDEFAULT_PRIORITY: LongInt;
    function getDEFAULT_TIME_TO_LIVE: Int64;
    function getDoubleProperty(name : string): Double;
    function getFloatProperty(name : string): Single;
    function getIntProperty(name : string): LongInt;
    function getJMSCorrelationID: string;
    function getJMSCorrelationIDAsBytes: TByteArray;
    function getJMSDeliveryMode: LongInt;
    function getJMSDestination: IDestination;
    function getJMSExpiration: Int64;
    function getJMSMessageID: string;
    function getJMSPriority: LongInt;
    function getJMSRedelivered: Boolean;
    function getJMSReplyTo: IDestination;
    function getJMSTimestamp: Int64;
    function getJMSType: string;
    function getLongProperty(name : string): Int64;
    function getObjectProperty(name : string): IUnknown;
    function getPropertyNames: TStrings;
    function getShortProperty(name : string): SmallInt;
    function getStringProperty(name : string): string;
    function propertyExists(name : string): Boolean;
    procedure setBooleanProperty(name : string; Value: Boolean);
    procedure setByteProperty(name : string; Value: Byte);
    procedure setDoubleProperty(name : string; Value: Double);
    procedure setFloatProperty(name : string; Value: Single);
    procedure setIntProperty(name : string; Value: LongInt);
    procedure setJMSCorrelationID(correlationID: string);
    procedure setJMSCorrelationIDAsBytes(correlationID: TByteArray);
    procedure setJMSDeliveryMode(deliveryMode: LongInt);
    procedure setJMSDestination(Value: IDestination);
    procedure setJMSExpiration(expiration: Int64);
    procedure setJMSMessageID(const id: string);
    procedure setJMSPriority(priority: LongInt);
    procedure setJMSRedelivered(Redelivered: Boolean);
    procedure setJMSReplyTo(ReplyTo: IDestination);
    procedure setJMSTimestamp(timestamp: Int64);
    procedure setJMSType(_type: string);
    procedure setLongProperty(name : string; Value: Int64);
    procedure setObjectProperty(name : string; Value: IUnknown);
    procedure setShortProperty(name : string; Value: SmallInt);
    procedure setStringProperty(name : string; const Value: string);
  end;
  
  TSMQObjectMessage = class (TSMQMessage, IObjectMessage)
  protected
    function getObject: TObject;
    procedure setObject(_object : TObject);
  end;
  
  TSMQTextMessage = class (TSMQMessage, ITextMessage)
  protected
    function getText: string;
    procedure setText(const Value: string);
  end;
  
  TSMQStreamMessage = class (TSMQMessage, IStreamMessage)
  protected
    function readBoolean: Boolean;
    function readByte: ShortInt;
    function readBytes(var value : TByteArray): LongInt; overload;
    function readChar: Char;
    function readDouble: Double;
    function readFloat: Single;
    function readInt: LongInt;
    function readLong: Int64;
    function readObject: TObject;
    function readShort: SmallInt;
    function readString: string;
    procedure reset;
    procedure writeBoolean(value : boolean);
    procedure writeByte(value : byte);
    procedure writeBytes(value : TByteArray); overload;
    procedure writeBytes(value : TByteArray; offset : longint; length : 
            longint); overload;
    procedure writeChar(vale : Char);
    procedure writeDouble(value : double);
    procedure writeFloat(value : single);
    procedure writeInt(value : longint);
    procedure writeLong(value : int64);
    procedure writeObject(value : IUnknown); overload;
    procedure writeObject(value : TObject); overload;
    procedure writeShort(value : Smallint);
    procedure writeString(value : string);
  end;
  
  TSMQBytesMessage = class (TSMQMessage, IBytesMessage)
  protected
    function readBoolean: Boolean;
    function readByte: ShortInt;
    function readBytes(var value : TByteArray): LongInt; overload;
    function readBytes(var value : TByteArray; length : LongInt): LongInt; 
            overload;
    function readChar: Char;
    function readDouble: Double;
    function readFloat: Single;
    function readInt: LongInt;
    function readLong: Int64;
    function readShort: SmallInt;
    function readUnsignedByte: Byte;
    function readUnsignedShort: Word;
    function readUTF: string;
    procedure reset;
    procedure writeBoolean(value : boolean);
    procedure writeByte(value : byte);
    procedure writeBytes(value : TByteArray); overload;
    procedure writeBytes(value : TByteArray; offset : longint; length : 
            longint); overload;
    procedure writeChar(vale : Char);
    procedure writeDouble(value : double);
    procedure writeFloat(value : single);
    procedure writeInt(value : longint);
    procedure writeLong(value : int64);
    procedure writeObject(value : IUnknown);
    procedure writeShort(value : Smallint);
    procedure writeUTF(value : string);
  end;
  
  TSMQMapMessage = class (TSMQMessage, IMapMessage)
  end;
  
  TSMQConnectionMetaData = class (TSMQObject, IConnectionMetaData)
  protected
    function getJMSMajorVersion: LongInt;
    function getJMSMinorVersion: LongInt;
    function getJMSProviderName: string;
    function getJMSVersion: string;
    function getJMSXPropertyNames: TStrings;
    function getProviderMajorVersion: LongInt;
    function getProviderMinorVersion: LongInt;
    function getProviderVersion: string;
  end;
  
  TSMQDeliveryMode = class (TSMQObject, IDeliveryMode)
  protected
    function getNON_PERSISTENT: LongInt;
    function getPERSISTENT: Integer;
  end;
  
  TSMQDestination = class (TSMQObject, IDestination)
  end;
  
  TSMQQueue = class (TSMQDestination, IQueue)
  protected
    function getQueueName: string;
  end;
  
  TSMQTemporaryQueue = class (TSMQQueue, ITemporaryQueue)
  protected
    procedure delete;
  end;
  
  TSMQTopic = class (TSMQDestination, ITopic)
  protected
    function getTopicName: string;
  end;
  
  TSMQTemporaryTopic = class (TSMQTopic, ITemporaryTopic)
  protected
    procedure delete;
  end;
  
  TSMQSession = class (TSMQObject, ISession)
  protected
    procedure close;
    procedure commit;
    function createBytesMessage: IBytesMessage;
    function createMapMessage: IMapMessage;
    function createMessage: IMessage;
    function createObjectMessage: IObjectMessage; overload;
    function createObjectMessage(_object : TObject): IObjectMessage; overload;
    function createStreamMessage: IStreamMessage;
    function createTextMessage: ITextMessage; overload;
    function createTextMessage(text : string): ITextMessage; overload;
    function getAUTO_ACKNOWLEDGE: LongInt;
    function getCLIENT_ACKNOWLEDGE: LongInt;
    function getDUPS_OK_ACKNOWLEDGE: LongInt;
    function getMessageListener: IMessageListener;
    function getTransacted: Boolean;
    procedure recover;
    procedure rollback;
    procedure run;
    procedure setMessageListener(listener: IMessageListener);
  end;
  
  TSMQQueueSession = class (TSMQSession, IQueueSession)
  protected
    function createBrowser(queue : IQueue): IQueueBrowser; overload;
    function createBrowser(queue : IQueue; messageSelector : string): 
            IQueueBrowser; overload;
    function createQueue(queueName : string): IQueue;
    function createReceiver(queue : IQueue): IQueueReceiver; overload;
    function createReceiver(queue : IQueue; messageSelector : string): 
            IQueueReceiver; overload;
    function createSender(queue : IQueue): IQueueSender;
    function createTemporaryQueue: ITemporaryQueue;
  end;
  
  TSMQTopicSession = class (TSMQSession, ITopicSession)
  protected
    function createDurableSubscriber(topic : ITopic; name : string): 
            ITopicSubscriber; overload;
    function createDurableSubscriber(topic : ITopic; name : string; 
            messageSelector : string; noLocal :boolean): ITopicSubscriber; 
            overload;
    function createPublisher(topic : ITopic): ITopicPublisher;
    function createSubscriber(topic : ITopic): ITopicSubscriber; overload;
    function createSubscriber(topic : ITopic; messageSelector : string; noLocal 
            :boolean): ITopicSubscriber; overload;
    function createTemporaryTopic: ITemporaryTopic;
    function createTopic(topicName : string): ITopic;
    procedure unsubscribe(name : string);
  end;
  
  TSMQMessageConsumer = class (TSMQObject, IMessageConsumer)
  protected
    procedure close;
    function CreateMessageByObj(msgObj : HOBJ): Imessage;
    function getMessageListener: IMessageListener;
    function getMessageSelector: string;
    function receive: IMessage; overload;
    function receive(timeout : int64): IMessage; overload;
    function receiveNoWait: IMessage;
    procedure setMessageListener(Value: IMessageListener);
  end;
  
  TSMQTopicSubscriber = class (TSMQMessageConsumer, ITopicSubscriber)
  protected
    function getNoLocal: Boolean;
    function getTopic: ITopic;
  end;
  
  TSMQQueueReceiver = class (TSMQMessageConsumer, IQueueReceiver)
  protected
    function getQueue: IQueue;
  end;
  
  TSMQMessageProducer = class (TSMQObject, IMessageProducer)
  protected
    procedure close;
    function getDeliveryMode: LongInt;
    function getDisableMessageID: Boolean;
    function getDisableMessageTimestamp: Boolean;
    function getPriority: LongInt;
    function getTimeToLive: Int64;
    procedure setDeliveryMode(deliveryMode: LongInt);
    procedure setDisableMessageID(Value: Boolean);
    procedure setDisableMessageTimestamp(Value: Boolean);
    procedure setPriority(defaultPriority: LongInt);
    procedure setTimeToLive(timeToLive: Int64);
  end;
  
  TSMQQueueSender = class (TSMQMessageProducer, IQueueSender)
  protected
    function getQueue: IQueue;
    procedure send(message : IMessage); overload;
    procedure send(message : IMessage; deliveryMode : LongInt; priority : 
            LongInt; timeToLive : int64); overload;
    procedure send(queue : IQueue; message : IMessage); overload;
    procedure send(queue : IQueue; message : IMessage; deliveryMode  :LongInt; 
            priority : LongInt; timeToLive : int64); overload;
  end;
  
  TSMQTopicPublisher = class (TSMQMessageProducer, ITopicPublisher)
  protected
    function getTopic: ITopic;
    procedure publish(message : IMessage); overload;
    procedure publish(message : IMessage; deliveryMode  :LongInt; priority : 
            LongInt; timeToLive : int64); overload;
    procedure publish(topic : ITopic; message : IMessage); overload;
    procedure publish(topic : ITopic; message : IMessage; deliveryMode :LongInt;
            priority : LongInt; timeToLive : int64); overload;
  end;
  
  TSMQConnection = class (TSMQObject, IConnection)
  protected
    procedure close;
    function getClientID: string;
    function getExceptionListener: IExceptionListener;
    function getMetaData: IConnectionMetaData;
    procedure setClientID(const clientID: string);
    procedure setExceptionListener(listener: IExceptionListener);
    procedure start;
    procedure stop;
  end;
  
  TSMQQueueConnection = class (TSMQConnection, IQueueConnection)
  protected
    function createConnectionConsumer(queue : IQueue; messageSelector : string; 
            sessionPool : IServerSessionPool; maxMessages : longint): 
            IConnectionConsumer;
    function createQueueSession(transacted : boolean; acknowledgeMode : 
            longint): IQueueSession;
  end;
  
  TSMQTopicConnection = class (TSMQConnection, ITopicConnection)
  protected
    function createConnectionConsumer(topic : ITopic; messageSelector : string; 
            sessionPool : IServerSessionPool; maxMessages : longint): 
            IConnectionConsumer;
    function createDurableConnectionConsumer(topic : ITopic; subscriptionName : 
            string; messageSelector : string; sessionPool : IServerSessionPool; 
            maxMessages : longint): IConnectionConsumer;
    function createTopicSession(transacted : boolean; acknowledgeMode : 
            longint): ITopicSession;
  end;
  
  TSMQConnectionConsumer = class (TSMQObject, IConnectionConsumer)
  protected
    procedure close;
    function getServerSessionPool: IServerSessionPool;
  end;
  
  TSMQConnectionFactory = class (TSMQObject, IConnectionFactory)
  end;
  
  TSMQQueueConnectionFactory = class (TSMQConnectionFactory, 
          IQueueConnectionFactory)
  protected
    function createQueueConnection: IQueueConnection; overload;
    function createQueueConnection(userName : string; password : string): 
            IQueueConnection; overload;
    function getBrokerHostName: string;
    function getBrokerPort: LongInt;
    function getBrokerProtocol: string;
    function getBrokerURL: string;
    function getClientID: string;
    function getConnectID: string;
    function getConnectionURLs: string;
    function getDefaultPassword: string;
    function getDefaultUser: string;
    function getLoadBalancing: Boolean;
    function getSequential: Boolean;
    procedure setBrokerHostName(const Value: string);
    procedure setBrokerPort(Value: LongInt);
    procedure setBrokerProtocol(Value: string);
    procedure setBrokerURL(Value: string);
    procedure setClientID(Value: string);
    procedure setConnectID(Value: string);
    procedure setConnectionURLs(Value: string);
    procedure setLoadBalancing(Value: Boolean);
    procedure setSequential(Value: Boolean);
  public
    constructor Create(brokerURL : string); overload; virtual;
    constructor Create(brokerURL : string; connectID : string); overload; 
            virtual;
    constructor Create(brokerHostName : string; brokerPort : LongInt; 
            brokerProtocol : string; defaultUserName : string; defaultPassword 
            : string); overload; virtual;
    constructor Create(brokerHostName : string; brokerPort : LongInt; 
            brokerProtocol : string; connectID : string; defaultUserName : 
            string; defaultPassword : string); overload; virtual;
    constructor Create(brokerURL : string; defaultUserName : string; 
            defaultPassword : string); overload; virtual;
    constructor Create(brokerURL : string; connectID : string; defaultUserName 
            : string; defaultPassword : string); overload; virtual;
    property BrokerHostName: string read getBrokerHostName write 
            setBrokerHostName;
    property BrokerPort: LongInt read getBrokerPort write setBrokerPort;
    property BrokerProtocol: string read getBrokerProtocol write 
            setBrokerProtocol;
    property BrokerURL: string read getBrokerURL write setBrokerURL;
    property ClientID: string read getClientID write setClientID;
    property ConnectID: string read getConnectID write setConnectID;
    property ConnectionURLs: string read getConnectionURLs write 
            setConnectionURLs;
    property DefaultPassword: string read getDefaultPassword;
    property DefaultUser: string read getDefaultUser;
    property LoadBalancing: Boolean read getLoadBalancing write 
            setLoadBalancing;
    property Sequential: Boolean read getSequential write setSequential;
  end;
  
  TSMQTopicConnectionFactory = class (TSMQConnectionFactory, 
          ITopicConnectionFactory)
  protected
    function createTopicConnection: ITopicConnection; overload;
    function createTopicConnection(userName : string; password : string): 
            ITopicConnection; overload;
    function getBrokerHostName: string;
    function getBrokerPort: LongInt;
    function getBrokerProtocol: string;
    function getBrokerURL: string;
    function getClientID: string;
    function getConnectID: string;
    function getConnectionURLs: string;
    function getDefaultPassword: string;
    function getDefaultUser: string;
    function getLoadBalancing: Boolean;
    function getSequential: Boolean;
    procedure setBrokerHostName(const Value: string);
    procedure setBrokerPort(Value: LongInt);
    procedure setBrokerProtocol(Value: string);
    procedure setBrokerURL(Value: string);
    procedure setClientID(Value: string);
    procedure setConnectID(Value: string);
    procedure setConnectionURLs(Value: string);
    procedure setLoadBalancing(Value: Boolean);
    procedure setSequential(Value: Boolean);
  public
    constructor Create(brokerURL : string); overload; virtual;
    constructor Create(brokerURL : string; connectID : string); overload; 
            virtual;
    constructor Create(brokerHostName : string; brokerPort : LongInt; 
            brokerProtocol : string; defaultUserName : string; defaultPassword 
            : string); overload; virtual;
    constructor Create(brokerHostName : string; brokerPort : LongInt; 
            brokerProtocol : string; connectID : string; defaultUserName : 
            string; defaultPassword : string); overload; virtual;
    constructor Create(brokerURL : string; defaultUserName : string; 
            defaultPassword : string); overload; virtual;
    constructor Create(brokerURL : string; connectID : string; defaultUserName 
            : string; defaultPassword : string); overload; virtual;
    property BrokerHostName: string read getBrokerHostName write 
            setBrokerHostName;
    property BrokerPort: LongInt read getBrokerPort write setBrokerPort;
    property BrokerProtocol: string read getBrokerProtocol write 
            setBrokerProtocol;
    property BrokerURL: string read getBrokerURL write setBrokerURL;
    property ClientID: string read getClientID write setClientID;
    property ConnectID: string read getConnectID write setConnectID;
    property ConnectionURLs: string read getConnectionURLs write 
            setConnectionURLs;
    property DefaultPassword: string read getDefaultPassword;
    property DefaultUser: string read getDefaultUser;
    property LoadBalancing: Boolean read getLoadBalancing write 
            setLoadBalancing;
    property Sequential: Boolean read getSequential write setSequential;
  end;
  
  TSMQQueueBrowser = class (TSMQObject, IQueueBrowser)
  protected
    procedure close;
    function getEnumeration: TList;
    function getMessageSelector: string;
    function getQueue: IQueue;
  end;
  

implementation

{
********************************** TSMQObject **********************************
}
constructor TSMQObject.Create(AObj : HOBJ);
begin
  FObj := AObj;
end;{ TSMQObject.Create }

destructor TSMQObject.Destroy;
begin
  inherited;
  
  DestroyHandle;
end;{ TSMQObject.Destroy }

function TSMQObject.booleanToJBoolean(value : boolean): jboolean;
begin
  if Value then
    Result := jtrue
  else
    Result := jfalse;
end;{ TSMQObject.booleanToJBoolean }

procedure TSMQObject.CheckSuccess(iRes : TRetVal);
var
  lastError: jint;
  errorStr: TSMQString;
begin
  if iRes <> SMQ_SUCCESS then
  begin
    lastError := SMQ_getLastError;
    if lastError <> SMQ_SUCCESS then
    begin
      errorStr := TSMQString.Create (SMQ_getLastErrorText);
      try
        raise EJMSException.Create (errorStr.Text, IntToStr (lastError));
      finally
        FreeAndNil (errorStr);
      end;
    end;
  end;
end;{ TSMQObject.CheckSuccess }

procedure TSMQObject.DestroyHandle;
var
  ref_count: jlong;
begin
  if FObj <> nil then
  begin
    Object_release (FObj, ref_count);
    FObj := nil;
  end;
end;{ TSMQObject.DestroyHandle }

function TSMQObject.equals(objInst : TSMQObject): Boolean;
begin
  Result := objInst = self;
end;{ TSMQObject.equals }

function TSMQObject.getObj: HOBJ;
begin
  Result := FObj;
end;{ TSMQObject.getObj }

procedure TSMQObject.setObj(Value: HOBJ);
begin
  DestroyHandle;
  
  FObj := value;
end;{ TSMQObject.setObj }

function TSMQObject.strObjToString(strObj : HOBJ): string;
var
  str: TSMQString;
begin
  str := TSMQString.Create (strObj);
  try
    Result := str.Text;
  finally
    FreeAndNil (str);
  end;
end;{ TSMQObject.strObjToString }

function TSMQObject.toString: string;
begin
end;{ TSMQObject.toString }

procedure TSMQObject.UnsupportedMethod(method : string);
begin
  raise Exception.CreateFmt ('Unsupported method %s->%s', [ClassName, method]);
end;{ TSMQObject.UnsupportedMethod }

{
********************************** TSMQString **********************************
}
constructor TSMQString.CreateStr(value : string);
begin
  Text := value;
end;{ TSMQString.CreateStr }

function TSMQString.getText: string;
var
  astring: HOBJ;
  len: jint;
  ref_count: jlong;
  str_data: PChar;
begin
  Result := '';
  if FObj <> nil then
    if String_toAscii (FObj, astring) = 0 then
    begin
      if AString_length (astring, len) = 0 then
      begin
        if len <> 0 then
        begin
          if AString_getValue (astring, str_data) = 0 then
          begin
            SetLength (Result, len);
            Move (str_data^, Result [1], len);
          end;
        end;
      end;
      Object_release (astring, ref_count);
    end;
end;{ TSMQString.getText }

procedure TSMQString.setText(Value: string);
begin
  DestroyHandle;
  
  String_create2 (PChar (Value), FObj);
end;{ TSMQString.setText }

{
********************************* TSMQMessage **********************************
}
procedure TSMQMessage.acknowledge;
begin
  CheckSuccess (Message_acknowledge (Obj));
end;{ TSMQMessage.acknowledge }

procedure TSMQMessage.clearBody;
begin
  CheckSuccess (Message_clearBody (Obj));
end;{ TSMQMessage.clearBody }

procedure TSMQMessage.clearProperties;
begin
  CheckSuccess (Message_clearProperties (Obj));
end;{ TSMQMessage.clearProperties }

function TSMQMessage.getBooleanProperty(name : string): Boolean;
var
  nameStr: TSMQString;
  ret: jboolean;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getBooleanProperty (Obj, nameStr.Obj, ret));
    Result := ret = jtrue;
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getBooleanProperty }

function TSMQMessage.getByteProperty(name : string): Byte;
var
  nameStr: TSMQString;
  retVal: jbyte;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getByteProperty (Obj, nameStr.Obj, retVal));
    Result := retVal;
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getByteProperty }

function TSMQMessage.getDEFAULT_DELIVERY_MODE: LongInt;
begin
  Result := apiSonicMQ.Message_DEFAULT_DELIVERY_MODE;
end;{ TSMQMessage.getDEFAULT_DELIVERY_MODE }

function TSMQMessage.getDEFAULT_PRIORITY: LongInt;
begin
  Result := apiSonicMQ.Message_DEFAULT_PRIORITY;
end;{ TSMQMessage.getDEFAULT_PRIORITY }

function TSMQMessage.getDEFAULT_TIME_TO_LIVE: Int64;
begin
  Result := apiSonicMQ.Message_DEFAULT_TIME_TO_LIVE;
end;{ TSMQMessage.getDEFAULT_TIME_TO_LIVE }

function TSMQMessage.getDoubleProperty(name : string): Double;
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getDoubleProperty (Obj, nameStr.Obj, Result));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getDoubleProperty }

function TSMQMessage.getFloatProperty(name : string): Single;
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getFloatProperty (Obj, nameStr.Obj, Result));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getFloatProperty }

function TSMQMessage.getIntProperty(name : string): LongInt;
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getIntProperty (Obj, nameStr.Obj, Result));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getIntProperty }

function TSMQMessage.getJMSCorrelationID: string;
begin
  UnsupportedMethod ('getJMSCorrelationID');
end;{ TSMQMessage.getJMSCorrelationID }

function TSMQMessage.getJMSCorrelationIDAsBytes: TByteArray;
begin
  UnsupportedMethod ('getJMSCorrelationIDAsBytes');
end;{ TSMQMessage.getJMSCorrelationIDAsBytes }

function TSMQMessage.getJMSDeliveryMode: LongInt;
begin
  CheckSuccess (Message_getJMSDeliveryMode (Obj, Result));
end;{ TSMQMessage.getJMSDeliveryMode }

function TSMQMessage.getJMSDestination: IDestination;
begin
  UnsupportedMethod ('getJMSDestination');
end;{ TSMQMessage.getJMSDestination }

function TSMQMessage.getJMSExpiration: Int64;
begin
  CheckSuccess (Message_getJMSExpiration (Obj, Result));
end;{ TSMQMessage.getJMSExpiration }

function TSMQMessage.getJMSMessageID: string;
begin
  UnsupportedMethod ('getJMSMessageID');
end;{ TSMQMessage.getJMSMessageID }

function TSMQMessage.getJMSPriority: LongInt;
begin
  CheckSuccess (Message_getJMSPriority (Obj, Result));
end;{ TSMQMessage.getJMSPriority }

function TSMQMessage.getJMSRedelivered: Boolean;
var
  ret: jboolean;
begin
  CheckSuccess (Message_getJMSRedelivered (Obj, ret));
  Result := ret = jtrue;
end;{ TSMQMessage.getJMSRedelivered }

function TSMQMessage.getJMSReplyTo: IDestination;
begin
  UnsupportedMethod ('getJMSReplyTo');
end;{ TSMQMessage.getJMSReplyTo }

function TSMQMessage.getJMSTimestamp: Int64;
begin
  CheckSuccess (Message_getJMSTimestamp (Obj, Result));
end;{ TSMQMessage.getJMSTimestamp }

function TSMQMessage.getJMSType: string;
begin
  UnsupportedMethod ('getJMSType');
end;{ TSMQMessage.getJMSType }

function TSMQMessage.getLongProperty(name : string): Int64;
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getLongProperty (Obj, nameStr.Obj, Result));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getLongProperty }

function TSMQMessage.getObjectProperty(name : string): IUnknown;
begin
  UnsupportedMethod ('getObjectProperty');
end;{ TSMQMessage.getObjectProperty }

function TSMQMessage.getPropertyNames: TStrings;
begin
  UnsupportedMethod ('getPropertyNames');
  
  Result := nil;
end;{ TSMQMessage.getPropertyNames }

function TSMQMessage.getShortProperty(name : string): SmallInt;
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getShortProperty (Obj, nameStr.Obj, Result));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getShortProperty }

function TSMQMessage.getStringProperty(name : string): string;
var
  nameStr: TSMQString;
  retStrObj: HOBJ;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess (Message_getStringProperty (Obj, nameStr.Obj, retStrObj));
    Result := strObjToString (retStrObj);
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.getStringProperty }

function TSMQMessage.propertyExists(name : string): Boolean;
var
  nameStr: TSMQString;
  exist: jboolean;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_propertyExists (Obj, nameStr.Obj, exist));
    Result := exist = jtrue;
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.propertyExists }

procedure TSMQMessage.setBooleanProperty(name : string; Value: Boolean);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setBooleanProperty (Obj, nameStr.Obj, booleanToJBoolean (Value)));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setBooleanProperty }

procedure TSMQMessage.setByteProperty(name : string; Value: Byte);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setByteProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setByteProperty }

procedure TSMQMessage.setDoubleProperty(name : string; Value: Double);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setDoubleProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setDoubleProperty }

procedure TSMQMessage.setFloatProperty(name : string; Value: Single);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setFloatProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setFloatProperty }

procedure TSMQMessage.setIntProperty(name : string; Value: LongInt);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setIntProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setIntProperty }

procedure TSMQMessage.setJMSCorrelationID(correlationID: string);
begin
  UnsupportedMethod ('setJMSCorrelationID');
end;{ TSMQMessage.setJMSCorrelationID }

procedure TSMQMessage.setJMSCorrelationIDAsBytes(correlationID: TByteArray);
begin
  UnsupportedMethod ('setJMSCorrelationIDAsBytes');
end;{ TSMQMessage.setJMSCorrelationIDAsBytes }

procedure TSMQMessage.setJMSDeliveryMode(deliveryMode: LongInt);
begin
  CheckSuccess (Message_setJMSDeliveryMode (Obj, deliveryMode));
end;{ TSMQMessage.setJMSDeliveryMode }

procedure TSMQMessage.setJMSDestination(Value: IDestination);
begin
  UnsupportedMethod ('setJMSDestination');
end;{ TSMQMessage.setJMSDestination }

procedure TSMQMessage.setJMSExpiration(expiration: Int64);
begin
  CheckSuccess (Message_setJMSExpiration (Obj, expiration));
end;{ TSMQMessage.setJMSExpiration }

procedure TSMQMessage.setJMSMessageID(const id: string);
begin
  UnsupportedMethod ('setJMSMessageID');
end;{ TSMQMessage.setJMSMessageID }

procedure TSMQMessage.setJMSPriority(priority: LongInt);
begin
  CheckSuccess (Message_setJMSPriority (Obj, priority));
end;{ TSMQMessage.setJMSPriority }

procedure TSMQMessage.setJMSRedelivered(Redelivered: Boolean);
begin
  CheckSuccess (Message_setJMSRedelivered (Obj, booleanToJBoolean(Redelivered)));
end;{ TSMQMessage.setJMSRedelivered }

procedure TSMQMessage.setJMSReplyTo(ReplyTo: IDestination);
begin
  UnsupportedMethod ('setJMSReplyTo');
end;{ TSMQMessage.setJMSReplyTo }

procedure TSMQMessage.setJMSTimestamp(timestamp: Int64);
begin
  CheckSuccess (Message_setJMSTimestamp (Obj, timestamp));
end;{ TSMQMessage.setJMSTimestamp }

procedure TSMQMessage.setJMSType(_type: string);
begin
  UnsupportedMethod ('setJMSType');
end;{ TSMQMessage.setJMSType }

procedure TSMQMessage.setLongProperty(name : string; Value: Int64);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setLongProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setLongProperty }

procedure TSMQMessage.setObjectProperty(name : string; Value: IUnknown);
begin
  UnsupportedMethod ('setObjectProperty');
end;{ TSMQMessage.setObjectProperty }

procedure TSMQMessage.setShortProperty(name : string; Value: SmallInt);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    CheckSuccess(Message_setShortProperty (Obj, nameStr.Obj, Value));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setShortProperty }

procedure TSMQMessage.setStringProperty(name : string; const Value: string);
var
  nameStr: TSMQString;
  valStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr(name);
  try
    valStr := TSMQString.CreateStr(Value);
    try
      CheckSuccess(Message_setStringProperty (Obj, nameStr.Obj, valStr.Obj));
    finally
      FreeAndNil (valStr);
    end;
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQMessage.setStringProperty }

{
****************************** TSMQObjectMessage *******************************
}
function TSMQObjectMessage.getObject: TObject;
begin
  UnsupportedMethod ('getObject');
  
  Result := nil;
end;{ TSMQObjectMessage.getObject }

procedure TSMQObjectMessage.setObject(_object : TObject);
begin
  UnsupportedMethod ('setObject');
end;{ TSMQObjectMessage.setObject }

{
******************************* TSMQTextMessage ********************************
}
function TSMQTextMessage.getText: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (TextMessage_getText (Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQTextMessage.getText }

procedure TSMQTextMessage.setText(const Value: string);
var
  valStr: TSMQString;
begin
  valStr := TSMQString.CreateStr(Value);
  try
    CheckSuccess(TextMessage_setText (Obj, valStr.Obj));
  finally
    FreeAndNil (valStr);
  end;
end;{ TSMQTextMessage.setText }

{
****************************** TSMQStreamMessage *******************************
}
function TSMQStreamMessage.readBoolean: Boolean;
begin
  UnsupportedMethod ('readBoolean');
  
  Result := false;
end;{ TSMQStreamMessage.readBoolean }

function TSMQStreamMessage.readByte: ShortInt;
begin
  UnsupportedMethod ('readByte');
  
  Result := 0;
end;{ TSMQStreamMessage.readByte }

function TSMQStreamMessage.readBytes(var value : TByteArray): LongInt;
begin
  UnsupportedMethod ('readBytes(var value : TByteArray)');
  
  Result := 0;
end;{ TSMQStreamMessage.readBytes }

function TSMQStreamMessage.readChar: Char;
begin
  UnsupportedMethod ('readChar');
  
  Result := #0;
end;{ TSMQStreamMessage.readChar }

function TSMQStreamMessage.readDouble: Double;
begin
  UnsupportedMethod ('readDouble');
  
  Result := 0;
end;{ TSMQStreamMessage.readDouble }

function TSMQStreamMessage.readFloat: Single;
begin
  UnsupportedMethod ('readFloat');
  
  Result := 0;
end;{ TSMQStreamMessage.readFloat }

function TSMQStreamMessage.readInt: LongInt;
begin
  UnsupportedMethod ('readInt');
  
  Result := 0;
end;{ TSMQStreamMessage.readInt }

function TSMQStreamMessage.readLong: Int64;
begin
  UnsupportedMethod ('readLong');
  
  Result := 0;
end;{ TSMQStreamMessage.readLong }

function TSMQStreamMessage.readObject: TObject;
begin
  UnsupportedMethod ('readObject:TObject');
  
  Result := nil;
end;{ TSMQStreamMessage.readObject }

function TSMQStreamMessage.readShort: SmallInt;
begin
  UnsupportedMethod ('readShort');
  
  Result := 0;
end;{ TSMQStreamMessage.readShort }

function TSMQStreamMessage.readString: string;
begin
  UnsupportedMethod ('readString');
  
  Result := '';
end;{ TSMQStreamMessage.readString }

procedure TSMQStreamMessage.reset;
begin
  UnsupportedMethod ('reset');
end;{ TSMQStreamMessage.reset }

procedure TSMQStreamMessage.writeBoolean(value : boolean);
begin
  UnsupportedMethod ('writeBoolean');
end;{ TSMQStreamMessage.writeBoolean }

procedure TSMQStreamMessage.writeByte(value : byte);
begin
  UnsupportedMethod ('writeByte');
end;{ TSMQStreamMessage.writeByte }

procedure TSMQStreamMessage.writeBytes(value : TByteArray);
begin
  UnsupportedMethod ('writeBytes(value : TByteArray)');
end;{ TSMQStreamMessage.writeBytes }

procedure TSMQStreamMessage.writeBytes(value : TByteArray; offset : longint; 
        length : longint);
begin
  UnsupportedMethod ('writeBytes(value : TByteArray; offset : longint; length : longint)');
end;{ TSMQStreamMessage.writeBytes }

procedure TSMQStreamMessage.writeChar(vale : Char);
begin
  UnsupportedMethod ('writeChar');
end;{ TSMQStreamMessage.writeChar }

procedure TSMQStreamMessage.writeDouble(value : double);
begin
  UnsupportedMethod ('writeDouble');
end;{ TSMQStreamMessage.writeDouble }

procedure TSMQStreamMessage.writeFloat(value : single);
begin
  UnsupportedMethod ('writeFloat');
end;{ TSMQStreamMessage.writeFloat }

procedure TSMQStreamMessage.writeInt(value : longint);
begin
  UnsupportedMethod ('writeInt');
end;{ TSMQStreamMessage.writeInt }

procedure TSMQStreamMessage.writeLong(value : int64);
begin
  UnsupportedMethod ('writeLong');
end;{ TSMQStreamMessage.writeLong }

procedure TSMQStreamMessage.writeObject(value : IUnknown);
begin
  UnsupportedMethod ('writeObject(IUnknown)');
end;{ TSMQStreamMessage.writeObject }

procedure TSMQStreamMessage.writeObject(value : TObject);
begin
  UnsupportedMethod ('writeObject(TObject)');
end;{ TSMQStreamMessage.writeObject }

procedure TSMQStreamMessage.writeShort(value : Smallint);
begin
  UnsupportedMethod ('writeShort');
end;{ TSMQStreamMessage.writeShort }

procedure TSMQStreamMessage.writeString(value : string);
begin
  UnsupportedMethod ('writeString');
end;{ TSMQStreamMessage.writeString }

{
******************************* TSMQBytesMessage *******************************
}
function TSMQBytesMessage.readBoolean: Boolean;
begin
  UnsupportedMethod ('readBoolean');
  
  Result := false;
end;{ TSMQBytesMessage.readBoolean }

function TSMQBytesMessage.readByte: ShortInt;
begin
  UnsupportedMethod ('readByte');
  
  Result := 0;
end;{ TSMQBytesMessage.readByte }

function TSMQBytesMessage.readBytes(var value : TByteArray): LongInt;
begin
  UnsupportedMethod ('readBytes(var value : TByteArray)');
  
  Result := 0;
end;{ TSMQBytesMessage.readBytes }

function TSMQBytesMessage.readBytes(var value : TByteArray; length : LongInt): 
        LongInt;
begin
  UnsupportedMethod ('readBytes(var value : TByteArray; length : LongInt)');
  
  Result := 0;
end;{ TSMQBytesMessage.readBytes }

function TSMQBytesMessage.readChar: Char;
begin
  UnsupportedMethod ('readChar');
  
  Result := #0;
end;{ TSMQBytesMessage.readChar }

function TSMQBytesMessage.readDouble: Double;
begin
  UnsupportedMethod ('readDouble');
  
  Result := 0;
end;{ TSMQBytesMessage.readDouble }

function TSMQBytesMessage.readFloat: Single;
begin
  UnsupportedMethod ('readFloat');
  
  Result := 0;
end;{ TSMQBytesMessage.readFloat }

function TSMQBytesMessage.readInt: LongInt;
begin
  UnsupportedMethod ('readInt');
  
  Result := 0;
end;{ TSMQBytesMessage.readInt }

function TSMQBytesMessage.readLong: Int64;
begin
  UnsupportedMethod ('readLong');
  
  Result := 0;
end;{ TSMQBytesMessage.readLong }

function TSMQBytesMessage.readShort: SmallInt;
begin
  UnsupportedMethod ('readShort');
  
  Result := 0;
end;{ TSMQBytesMessage.readShort }

function TSMQBytesMessage.readUnsignedByte: Byte;
begin
  UnsupportedMethod ('readUnsignedByte');
  
  Result := 0;
end;{ TSMQBytesMessage.readUnsignedByte }

function TSMQBytesMessage.readUnsignedShort: Word;
begin
  UnsupportedMethod ('readUnsignedShort');
  
  Result := 0;
end;{ TSMQBytesMessage.readUnsignedShort }

function TSMQBytesMessage.readUTF: string;
begin
  UnsupportedMethod ('readUTF');
  
  Result := '';
end;{ TSMQBytesMessage.readUTF }

procedure TSMQBytesMessage.reset;
begin
  UnsupportedMethod ('reset');
end;{ TSMQBytesMessage.reset }

procedure TSMQBytesMessage.writeBoolean(value : boolean);
begin
  UnsupportedMethod ('writeBoolean');
end;{ TSMQBytesMessage.writeBoolean }

procedure TSMQBytesMessage.writeByte(value : byte);
begin
  UnsupportedMethod ('writeByte');
end;{ TSMQBytesMessage.writeByte }

procedure TSMQBytesMessage.writeBytes(value : TByteArray);
begin
  UnsupportedMethod ('writeBytes(value : TByteArray)');
end;{ TSMQBytesMessage.writeBytes }

procedure TSMQBytesMessage.writeBytes(value : TByteArray; offset : longint; 
        length : longint);
begin
  UnsupportedMethod ('writeBytes(value : TByteArray; offset : longint; length : longint)');
end;{ TSMQBytesMessage.writeBytes }

procedure TSMQBytesMessage.writeChar(vale : Char);
begin
  UnsupportedMethod ('writeChar');
end;{ TSMQBytesMessage.writeChar }

procedure TSMQBytesMessage.writeDouble(value : double);
begin
  UnsupportedMethod ('writeDouble');
end;{ TSMQBytesMessage.writeDouble }

procedure TSMQBytesMessage.writeFloat(value : single);
begin
  UnsupportedMethod ('writeFloat');
end;{ TSMQBytesMessage.writeFloat }

procedure TSMQBytesMessage.writeInt(value : longint);
begin
  UnsupportedMethod ('writeInt');
end;{ TSMQBytesMessage.writeInt }

procedure TSMQBytesMessage.writeLong(value : int64);
begin
  UnsupportedMethod ('writeLong');
end;{ TSMQBytesMessage.writeLong }

procedure TSMQBytesMessage.writeObject(value : IUnknown);
begin
  UnsupportedMethod ('writeObject');
end;{ TSMQBytesMessage.writeObject }

procedure TSMQBytesMessage.writeShort(value : Smallint);
begin
  UnsupportedMethod ('writeShort');
end;{ TSMQBytesMessage.writeShort }

procedure TSMQBytesMessage.writeUTF(value : string);
begin
  UnsupportedMethod ('writeUTF');
end;{ TSMQBytesMessage.writeUTF }

{
**************************** TSMQConnectionMetaData ****************************
}
function TSMQConnectionMetaData.getJMSMajorVersion: LongInt;
begin
  CheckSuccess (ConnectionMetaData_getJMSMajorVersion(Obj, Result));
end;{ TSMQConnectionMetaData.getJMSMajorVersion }

function TSMQConnectionMetaData.getJMSMinorVersion: LongInt;
begin
  CheckSuccess (ConnectionMetaData_getJMSMinorVersion(Obj, Result));
end;{ TSMQConnectionMetaData.getJMSMinorVersion }

function TSMQConnectionMetaData.getJMSProviderName: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (ConnectionMetaData_getJMSProviderName(Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQConnectionMetaData.getJMSProviderName }

function TSMQConnectionMetaData.getJMSVersion: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (ConnectionMetaData_getJMSVersion(Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQConnectionMetaData.getJMSVersion }

function TSMQConnectionMetaData.getJMSXPropertyNames: TStrings;
begin
  UnsupportedMethod ('getJMSXPropertyNames');
  
  Result := nil;
end;{ TSMQConnectionMetaData.getJMSXPropertyNames }

function TSMQConnectionMetaData.getProviderMajorVersion: LongInt;
begin
  CheckSuccess (ConnectionMetaData_getProviderMajorVersion(Obj, Result));
end;{ TSMQConnectionMetaData.getProviderMajorVersion }

function TSMQConnectionMetaData.getProviderMinorVersion: LongInt;
begin
  CheckSuccess (ConnectionMetaData_getProviderMinorVersion(Obj, Result));
end;{ TSMQConnectionMetaData.getProviderMinorVersion }

function TSMQConnectionMetaData.getProviderVersion: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (ConnectionMetaData_getProviderVersion(Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQConnectionMetaData.getProviderVersion }

{
******************************* TSMQDeliveryMode *******************************
}
function TSMQDeliveryMode.getNON_PERSISTENT: LongInt;
begin
  Result := apiSonicMQ.DeliveryMode_NON_PERSISTENT;
end;{ TSMQDeliveryMode.getNON_PERSISTENT }

function TSMQDeliveryMode.getPERSISTENT: Integer;
begin
  Result := apiSonicMQ.DeliveryMode_PERSISTENT;
end;{ TSMQDeliveryMode.getPERSISTENT }

{
********************************** TSMQQueue ***********************************
}
function TSMQQueue.getQueueName: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (Queue_getQueueName (Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQQueue.getQueueName }

{
****************************** TSMQTemporaryQueue ******************************
}
procedure TSMQTemporaryQueue.delete;
begin
  CheckSuccess (TemporaryQueue_delete (Obj));
end;{ TSMQTemporaryQueue.delete }

{
********************************** TSMQTopic ***********************************
}
function TSMQTopic.getTopicName: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (Topic_getTopicName (Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQTopic.getTopicName }

{
****************************** TSMQTemporaryTopic ******************************
}
procedure TSMQTemporaryTopic.delete;
begin
  CheckSuccess (TemporaryTopic_delete (Obj));
end;{ TSMQTemporaryTopic.delete }

{
********************************* TSMQSession **********************************
}
procedure TSMQSession.close;
begin
  CheckSuccess (Session_close (Obj));
end;{ TSMQSession.close }

procedure TSMQSession.commit;
begin
  CheckSuccess (Session_commit (Obj));
end;{ TSMQSession.commit }

function TSMQSession.createBytesMessage: IBytesMessage;
var
  msg: HOBJ;
begin
  CheckSuccess (Session_createBytesMessage (Obj, msg));
  Result := TSMQBytesMessage.Create (msg);
end;{ TSMQSession.createBytesMessage }

function TSMQSession.createMapMessage: IMapMessage;
begin
  UnsupportedMethod ('createMapMessage');
end;{ TSMQSession.createMapMessage }

function TSMQSession.createMessage: IMessage;
var
  msg: HOBJ;
begin
  CheckSuccess (Session_createMessage (Obj, msg));
  Result := TSMQMessage.Create (msg);
end;{ TSMQSession.createMessage }

function TSMQSession.createObjectMessage: IObjectMessage;
begin
  UnsupportedMethod ('createObjectMessage');
end;{ TSMQSession.createObjectMessage }

function TSMQSession.createObjectMessage(_object : TObject): IObjectMessage;
begin
  UnsupportedMethod ('createObjectMessage(TObject)');
end;{ TSMQSession.createObjectMessage }

function TSMQSession.createStreamMessage: IStreamMessage;
begin
  UnsupportedMethod ('createStreamMessage');
end;{ TSMQSession.createStreamMessage }

function TSMQSession.createTextMessage: ITextMessage;
var
  msg: HOBJ;
begin
  CheckSuccess (Session_createTextMessage (Obj, msg));
  Result := TSMQTextMessage.Create (msg);
end;{ TSMQSession.createTextMessage }

function TSMQSession.createTextMessage(text : string): ITextMessage;
var
  msg: HOBJ;
  textStr: TSMQString;
begin
  textStr := TSMQString.CreateStr (text);
  try
    CheckSuccess (Session_createTextMessage2 (Obj, textStr.Obj, msg));
  finally
    FreeAndNil (textStr);
  end;
  Result := TSMQTextMessage.Create (msg);
end;{ TSMQSession.createTextMessage }

function TSMQSession.getAUTO_ACKNOWLEDGE: LongInt;
begin
  Result := apiSonicMQ.Session_AUTO_ACKNOWLEDGE;
end;{ TSMQSession.getAUTO_ACKNOWLEDGE }

function TSMQSession.getCLIENT_ACKNOWLEDGE: LongInt;
begin
  Result := apiSonicMQ.Session_CLIENT_ACKNOWLEDGE;
end;{ TSMQSession.getCLIENT_ACKNOWLEDGE }

function TSMQSession.getDUPS_OK_ACKNOWLEDGE: LongInt;
begin
  Result := apiSonicMQ.Session_DUPS_OK_ACKNOWLEDGE;
end;{ TSMQSession.getDUPS_OK_ACKNOWLEDGE }

function TSMQSession.getMessageListener: IMessageListener;
begin
  UnsupportedMethod ('getMessageListener');
end;{ TSMQSession.getMessageListener }

function TSMQSession.getTransacted: Boolean;
var
  ret: jboolean;
begin
  CheckSuccess (Session_getTransacted(Obj, ret));
  Result := ret = jtrue;
end;{ TSMQSession.getTransacted }

procedure TSMQSession.recover;
begin
  CheckSuccess (Session_recover (Obj));
end;{ TSMQSession.recover }

procedure TSMQSession.rollback;
begin
  CheckSuccess (Session_rollback (Obj));
end;{ TSMQSession.rollback }

procedure TSMQSession.run;
begin
  UnsupportedMethod ('run');
end;{ TSMQSession.run }

procedure TSMQSession.setMessageListener(listener: IMessageListener);
begin
  UnsupportedMethod ('setMessageListener');
end;{ TSMQSession.setMessageListener }

{
******************************* TSMQQueueSession *******************************
}
function TSMQQueueSession.createBrowser(queue : IQueue): IQueueBrowser;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
begin
  smqObj := queue as ISMQObject;
  CheckSuccess (QueueSession_createBrowser (Obj, smqObj.Obj, retObj));
  Result := TSMQQueueBrowser.Create (retObj);
end;{ TSMQQueueSession.createBrowser }

function TSMQQueueSession.createBrowser(queue : IQueue; messageSelector : 
        string): IQueueBrowser;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
  msgSelector: TSMQString;
begin
  smqObj := queue as ISMQObject;
  msgSelector := TSMQString.CreateStr (messageSelector);
  try
    CheckSuccess (QueueSession_createBrowser2 (Obj, smqObj.Obj, msgSelector.Obj, retObj));
    Result := TSMQQueueBrowser.Create (retObj);
  finally
    FreeAndNil (msgSelector);
  end;
end;{ TSMQQueueSession.createBrowser }

function TSMQQueueSession.createQueue(queueName : string): IQueue;
var
  nameStr: TSMQString;
  retObj: HOBJ;
begin
  nameStr := TSMQString.CreateStr (queueName);
  try
    CheckSuccess (QueueSession_createQueue (Obj, nameStr.Obj, retObj));
  finally
    FreeAndNil (nameStr);
  end;
  Result := TSMQQueue.Create (retObj);
end;{ TSMQQueueSession.createQueue }

function TSMQQueueSession.createReceiver(queue : IQueue): IQueueReceiver;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
begin
  smqObj := queue as ISMQObject;
  CheckSuccess (QueueSession_createReceiver (Obj, smqObj.Obj, retObj));
  Result := TSMQQueueReceiver.Create (retObj);
end;{ TSMQQueueSession.createReceiver }

function TSMQQueueSession.createReceiver(queue : IQueue; messageSelector : 
        string): IQueueReceiver;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
  msgSelector: TSMQString;
begin
  smqObj := queue as ISMQObject;
  msgSelector := TSMQString.CreateStr (messageSelector);
  try
    CheckSuccess (QueueSession_createReceiver2 (Obj, smqObj.Obj, msgSelector.Obj, retObj));
    Result := TSMQQueueReceiver.Create (retObj);
  finally
    FreeAndNil (msgSelector);
  end;
end;{ TSMQQueueSession.createReceiver }

function TSMQQueueSession.createSender(queue : IQueue): IQueueSender;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
begin
  smqObj := queue as ISMQObject;
  CheckSuccess (QueueSession_createSender (Obj, smqObj.Obj, retObj));
  Result := TSMQQueueSender.Create (retObj);
end;{ TSMQQueueSession.createSender }

function TSMQQueueSession.createTemporaryQueue: ITemporaryQueue;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueSession_createTemporaryQueue (Obj, retObj));
  Result := TSMQTemporaryQueue.Create (retObj);
end;{ TSMQQueueSession.createTemporaryQueue }

{
******************************* TSMQTopicSession *******************************
}
function TSMQTopicSession.createDurableSubscriber(topic : ITopic; name : 
        string): ITopicSubscriber;
var
  retObj: HOBJ;
  nameStr: TSMQString;
  smqObj: ISMQObject;
begin
  smqObj := topic as ISMQObject;
  nameStr := TSMQString.CreateStr (name);
  try
    CheckSuccess (TopicSession_createDurableSubscriber (Obj, smqObj.Obj, nameStr.Obj, retObj));
    Result := TSMQTopicSubscriber.Create (retObj);
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQTopicSession.createDurableSubscriber }

function TSMQTopicSession.createDurableSubscriber(topic : ITopic; name : string;
        messageSelector : string; noLocal :boolean): ITopicSubscriber;
var
  retObj: HOBJ;
  msgSelector: TSMQString;
  nameStr: TSMQString;
  smqObj: ISMQObject;
begin
  smqObj := topic as ISMQObject;
  msgSelector := TSMQString.CreateStr (messageSelector);
  try
    nameStr := TSMQString.CreateStr (name);
    try
      CheckSuccess (TopicSession_createDurableSubscriber2 (Obj, smqObj.Obj,
        nameStr.Obj, msgSelector.Obj, booleanToJBoolean (noLocal), retObj));
      Result := TSMQTopicSubscriber.Create (retObj);
    finally
      FreeAndNil (nameStr);
    end;
  finally
    FreeAndNil (msgSelector);
  end;
end;{ TSMQTopicSession.createDurableSubscriber }

function TSMQTopicSession.createPublisher(topic : ITopic): ITopicPublisher;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
begin
  smqObj := topic as ISMQObject;
  CheckSuccess (TopicSession_createPublisher (Obj, smqObj.Obj, retObj));
  Result := TSMQTopicPublisher.Create (retObj);
end;{ TSMQTopicSession.createPublisher }

function TSMQTopicSession.createSubscriber(topic : ITopic): ITopicSubscriber;
var
  retObj: HOBJ;
  smqObj: ISMQObject;
begin
  smqObj := topic as ISMQObject;
  CheckSuccess (TopicSession_createSubscriber (Obj, smqObj.Obj, retObj));
  Result := TSMQTopicSubscriber.Create (retObj);
end;{ TSMQTopicSession.createSubscriber }

function TSMQTopicSession.createSubscriber(topic : ITopic; messageSelector : 
        string; noLocal :boolean): ITopicSubscriber;
var
  retObj: HOBJ;
  msgSelector: TSMQString;
  smqObj: ISMQObject;
begin
  smqObj := topic as ISMQObject;
  msgSelector := TSMQString.CreateStr (messageSelector);
  try
    CheckSuccess (TopicSession_createSubscriber2 (Obj, smqObj.Obj,
      msgSelector.Obj, booleanToJBoolean (noLocal), retObj));
    Result := TSMQTopicSubscriber.Create (retObj);
  finally
    FreeAndNil (msgSelector);
  end;
end;{ TSMQTopicSession.createSubscriber }

function TSMQTopicSession.createTemporaryTopic: ITemporaryTopic;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicSession_createTemporaryTopic (Obj, retObj));
  Result := TSMQTemporaryTopic.Create (retObj);
end;{ TSMQTopicSession.createTemporaryTopic }

function TSMQTopicSession.createTopic(topicName : string): ITopic;
var
  nameStr: TSMQString;
  retObj: HOBJ;
begin
  nameStr := TSMQString.CreateStr (topicName);
  try
    CheckSuccess (TopicSession_createTopic (Obj, nameStr.Obj, retObj));
    Result := TSMQTopic.Create (retObj);
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQTopicSession.createTopic }

procedure TSMQTopicSession.unsubscribe(name : string);
var
  nameStr: TSMQString;
begin
  nameStr := TSMQString.CreateStr (name);
  try
    CheckSuccess (TopicSession_unsubscribe (Obj, nameStr.Obj));
  finally
    FreeAndNil (nameStr);
  end;
end;{ TSMQTopicSession.unsubscribe }

{
***************************** TSMQMessageConsumer ******************************
}
procedure TSMQMessageConsumer.close;
begin
  UnsupportedMethod ('close');
end;{ TSMQMessageConsumer.close }

function TSMQMessageConsumer.CreateMessageByObj(msgObj : HOBJ): Imessage;
var
  objType: Integer;
begin
  if msgObj <> nil then
  begin
    CheckSuccess (Object_getType (msgObj, objType));
  
    if objType = BytesMessage_type then
      Result := TSMQBytesMessage.Create (msgObj)
    else
    if objType = TextMessage_type then
      Result := TSMQTextMessage.Create (msgObj)
    else
      Result := TSMQMessage.Create (msgObj);
  end
  else
    Result := nil;
end;{ TSMQMessageConsumer.CreateMessageByObj }

function TSMQMessageConsumer.getMessageListener: IMessageListener;
begin
  UnsupportedMethod ('getMessageListener');
end;{ TSMQMessageConsumer.getMessageListener }

function TSMQMessageConsumer.getMessageSelector: string;
var
  strObj: HOBJ;
begin
  CheckSuccess (MessageConsumer_getMessageSelector (Obj, strObj));
  Result := strObjToString (strObj);
end;{ TSMQMessageConsumer.getMessageSelector }

function TSMQMessageConsumer.receive: IMessage;
var
  retObj: HOBJ;
begin
  CheckSuccess (MessageConsumer_receive (Obj, retObj));
  Result := CreateMessageByObj (retObj);
end;{ TSMQMessageConsumer.receive }

function TSMQMessageConsumer.receive(timeout : int64): IMessage;
var
  retObj: HOBJ;
begin
  CheckSuccess (MessageConsumer_receive2 (Obj, timeout, retObj));
  Result := CreateMessageByObj (retObj);
end;{ TSMQMessageConsumer.receive }

function TSMQMessageConsumer.receiveNoWait: IMessage;
var
  retObj: HOBJ;
begin
  CheckSuccess (MessageConsumer_receiveNoWait (Obj, retObj));
  Result := CreateMessageByObj (retObj);
end;{ TSMQMessageConsumer.receiveNoWait }

procedure TSMQMessageConsumer.setMessageListener(Value: IMessageListener);
begin
  UnsupportedMethod ('setMessageListener');
end;{ TSMQMessageConsumer.setMessageListener }

{
***************************** TSMQTopicSubscriber ******************************
}
function TSMQTopicSubscriber.getNoLocal: Boolean;
var
  j_noLocal: jboolean;
begin
  CheckSuccess (TopicSubscriber_getNoLocal (Obj, j_noLocal));
  Result := j_noLocal = jtrue;
end;{ TSMQTopicSubscriber.getNoLocal }

function TSMQTopicSubscriber.getTopic: ITopic;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicSubscriber_getTopic (Obj, retObj));
  Result := TSMQTopic.Create(retObj);
end;{ TSMQTopicSubscriber.getTopic }

{
****************************** TSMQQueueReceiver *******************************
}
function TSMQQueueReceiver.getQueue: IQueue;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueReceiver_getQueue (Obj, retObj));
  Result := TSMQQueue.Create(retObj);
end;{ TSMQQueueReceiver.getQueue }

{
***************************** TSMQMessageProducer ******************************
}
procedure TSMQMessageProducer.close;
begin
  CheckSuccess (MessageProducer_close (Obj));
end;{ TSMQMessageProducer.close }

function TSMQMessageProducer.getDeliveryMode: LongInt;
begin
  CheckSuccess (MessageProducer_getDeliveryMode (Obj, Result));
end;{ TSMQMessageProducer.getDeliveryMode }

function TSMQMessageProducer.getDisableMessageID: Boolean;
var
  j_var: jboolean;
begin
  CheckSuccess (MessageProducer_getDisableMessageID (Obj, j_var));
  Result := j_var = jtrue;
end;{ TSMQMessageProducer.getDisableMessageID }

function TSMQMessageProducer.getDisableMessageTimestamp: Boolean;
var
  j_var: jboolean;
begin
  CheckSuccess (MessageProducer_getDisableMessageTimestamp (Obj, j_var));
  Result := j_var = jtrue;
end;{ TSMQMessageProducer.getDisableMessageTimestamp }

function TSMQMessageProducer.getPriority: LongInt;
begin
  CheckSuccess (MessageProducer_getPriority (Obj, Result));
end;{ TSMQMessageProducer.getPriority }

function TSMQMessageProducer.getTimeToLive: Int64;
begin
  CheckSuccess (MessageProducer_getTimeToLive (Obj, Result));
end;{ TSMQMessageProducer.getTimeToLive }

procedure TSMQMessageProducer.setDeliveryMode(deliveryMode: LongInt);
begin
  CheckSuccess (MessageProducer_setDeliveryMode (Obj, deliveryMode));
end;{ TSMQMessageProducer.setDeliveryMode }

procedure TSMQMessageProducer.setDisableMessageID(Value: Boolean);
begin
  CheckSuccess (MessageProducer_setDisableMessageID (Obj, booleanToJBoolean (Value)));
end;{ TSMQMessageProducer.setDisableMessageID }

procedure TSMQMessageProducer.setDisableMessageTimestamp(Value: Boolean);
begin
  CheckSuccess (MessageProducer_setDisableMessageTimestamp (Obj, booleanToJBoolean (Value)));
end;{ TSMQMessageProducer.setDisableMessageTimestamp }

procedure TSMQMessageProducer.setPriority(defaultPriority: LongInt);
begin
  CheckSuccess (MessageProducer_setPriority (Obj, defaultPriority));
end;{ TSMQMessageProducer.setPriority }

procedure TSMQMessageProducer.setTimeToLive(timeToLive: Int64);
begin
  CheckSuccess (MessageProducer_setTimeToLive (Obj, timeToLive));
end;{ TSMQMessageProducer.setTimeToLive }

{
******************************* TSMQQueueSender ********************************
}
function TSMQQueueSender.getQueue: IQueue;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueSender_getQueue(Obj, retObj));
  Result := TSMQQueue.Create (retObj);
end;{ TSMQQueueSender.getQueue }

procedure TSMQQueueSender.send(message : IMessage);
var
  smqObj: ISMQObject;
begin
  smqObj := message as ISMQObject;
  CheckSuccess (QueueSender_send(Obj, smqObj.Obj));
end;{ TSMQQueueSender.send }

procedure TSMQQueueSender.send(message : IMessage; deliveryMode : LongInt; 
        priority : LongInt; timeToLive : int64);
var
  smqObj: ISMQObject;
begin
  smqObj := message as ISMQObject;
  CheckSuccess (QueueSender_send2(Obj, smqObj.Obj, deliveryMode, priority, timeToLive));
end;{ TSMQQueueSender.send }

procedure TSMQQueueSender.send(queue : IQueue; message : IMessage);
var
  smqObj: ISMQObject;
  smqQueue: ISMQObject;
begin
  smqObj := message as ISMQObject;
  smqQueue := queue as ISMQObject;
  CheckSuccess (QueueSender_send3(Obj, smqQueue.Obj, smqObj.Obj));
end;{ TSMQQueueSender.send }

procedure TSMQQueueSender.send(queue : IQueue; message : IMessage; deliveryMode 
        :LongInt; priority : LongInt; timeToLive : int64);
var
  smqObj: ISMQObject;
  smqQueue: ISMQObject;
begin
  smqObj := message as ISMQObject;
  smqQueue := queue as ISMQObject;
  CheckSuccess (QueueSender_send4(Obj, smqQueue.Obj, smqObj.Obj,
    deliveryMode, priority, timeToLive));
end;{ TSMQQueueSender.send }

{
****************************** TSMQTopicPublisher ******************************
}
function TSMQTopicPublisher.getTopic: ITopic;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicPublisher_getTopic(Obj, retObj));
  Result := TSMQTopic.Create (retObj);
end;{ TSMQTopicPublisher.getTopic }

procedure TSMQTopicPublisher.publish(message : IMessage);
var
  smqObj: ISMQObject;
begin
  smqObj := message as ISMQObject;
  CheckSuccess (TopicPublisher_publish(Obj, smqObj.Obj));
end;{ TSMQTopicPublisher.publish }

procedure TSMQTopicPublisher.publish(message : IMessage; deliveryMode  :LongInt;
        priority : LongInt; timeToLive : int64);
var
  smqObj: ISMQObject;
begin
  smqObj := message as ISMQObject;
  CheckSuccess (TopicPublisher_publish2(Obj, smqObj.Obj, deliveryMode, priority, timeToLive));
end;{ TSMQTopicPublisher.publish }

procedure TSMQTopicPublisher.publish(topic : ITopic; message : IMessage);
var
  smqObj: ISMQObject;
  smqTopic: ISMQObject;
begin
  smqObj := message as ISMQObject;
  smqTopic := topic as ISMQObject;
  CheckSuccess (TopicPublisher_publish3(Obj, smqTopic.Obj, smqObj.Obj));
end;{ TSMQTopicPublisher.publish }

procedure TSMQTopicPublisher.publish(topic : ITopic; message : IMessage; 
        deliveryMode :LongInt; priority : LongInt; timeToLive : int64);
var
  smqObj: ISMQObject;
  smqTopic: ISMQObject;
begin
  smqObj := message as ISMQObject;
  smqTopic := topic as ISMQObject;
  CheckSuccess (TopicPublisher_publish4(Obj, smqTopic.Obj, smqObj.Obj,
    deliveryMode, priority, timeToLive));
end;{ TSMQTopicPublisher.publish }

{
******************************** TSMQConnection ********************************
}
procedure TSMQConnection.close;
begin
  CheckSuccess (Connection_close(Obj));
end;{ TSMQConnection.close }

function TSMQConnection.getClientID: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (Connection_getClientID (Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQConnection.getClientID }

function TSMQConnection.getExceptionListener: IExceptionListener;
begin
  UnsupportedMethod ('getExceptionListener');
end;{ TSMQConnection.getExceptionListener }

function TSMQConnection.getMetaData: IConnectionMetaData;
var
  retObj: HOBJ;
begin
  CheckSuccess (Connection_getMetaData (Obj, retObj));
  Result := TSMQConnectionMetadata.Create (retObj);
end;{ TSMQConnection.getMetaData }

procedure TSMQConnection.setClientID(const clientID: string);
var
  objStr: TSMQString;
begin
  objStr := TSMQString.CreateStr (clientID);
  try
    CheckSuccess (Connection_setClientID (Obj, objStr.Obj));
  finally
    FreeAndNil (objStr);
  end;
end;{ TSMQConnection.setClientID }

procedure TSMQConnection.setExceptionListener(listener: IExceptionListener);
begin
  UnsupportedMethod ('setExceptionListener');
end;{ TSMQConnection.setExceptionListener }

procedure TSMQConnection.start;
begin
  CheckSuccess (Connection_start(Obj));
end;{ TSMQConnection.start }

procedure TSMQConnection.stop;
begin
  CheckSuccess (Connection_stop(Obj));
end;{ TSMQConnection.stop }

{
***************************** TSMQQueueConnection ******************************
}
function TSMQQueueConnection.createConnectionConsumer(queue : IQueue; 
        messageSelector : string; sessionPool : IServerSessionPool; maxMessages 
        : longint): IConnectionConsumer;
begin
  UnsupportedMethod ('createConnectionConsumer');
end;{ TSMQQueueConnection.createConnectionConsumer }

function TSMQQueueConnection.createQueueSession(transacted : boolean; 
        acknowledgeMode : longint): IQueueSession;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnection_createQueueSession (Obj,
    booleanToJBoolean (transacted), acknowledgeMode, retObj));
  Result := TSMQQueueSession.Create (retObj);
end;{ TSMQQueueConnection.createQueueSession }

{
***************************** TSMQTopicConnection ******************************
}
function TSMQTopicConnection.createConnectionConsumer(topic : ITopic; 
        messageSelector : string; sessionPool : IServerSessionPool; maxMessages 
        : longint): IConnectionConsumer;
begin
  UnsupportedMethod ('createConnectionConsumer');
end;{ TSMQTopicConnection.createConnectionConsumer }

function TSMQTopicConnection.createDurableConnectionConsumer(topic : ITopic; 
        subscriptionName : string; messageSelector : string; sessionPool : 
        IServerSessionPool; maxMessages : longint): IConnectionConsumer;
begin
  UnsupportedMethod ('createDurableConnectionConsumer');
end;{ TSMQTopicConnection.createDurableConnectionConsumer }

function TSMQTopicConnection.createTopicSession(transacted : boolean; 
        acknowledgeMode : longint): ITopicSession;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnection_createTopicSession (Obj,
    booleanToJBoolean (transacted), acknowledgeMode, retObj));
  Result := TSMQTopicSession.Create (retObj);
end;{ TSMQTopicConnection.createTopicSession }

{
**************************** TSMQConnectionConsumer ****************************
}
procedure TSMQConnectionConsumer.close;
begin
  UnsupportedMethod ('close');
end;{ TSMQConnectionConsumer.close }

function TSMQConnectionConsumer.getServerSessionPool: IServerSessionPool;
begin
  UnsupportedMethod ('getServerSessionPool');
end;{ TSMQConnectionConsumer.getServerSessionPool }

{
************************** TSMQQueueConnectionFactory **************************
}
constructor TSMQQueueConnectionFactory.Create(brokerURL : string);
var
  strBrokerURL: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerURL);
  try
    CheckSuccess (QueueConnectionFactory_create (
      strBrokerURL.Obj, FObj));
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQQueueConnectionFactory.Create }

constructor TSMQQueueConnectionFactory.Create(brokerURL : string; connectID : 
        string);
var
  strBrokerURL: TSMQString;
  strConnectID: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strConnectID := TSMQString.CreateStr(connectID);
    try
      CheckSuccess (QueueConnectionFactory_create4 (
        strBrokerURL.Obj, strConnectID.Obj, FObj));
    finally
      FreeAndNil (strConnectID);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQQueueConnectionFactory.Create }

constructor TSMQQueueConnectionFactory.Create(brokerHostName : string; 
        brokerPort : LongInt; brokerProtocol : string; defaultUserName : string;
        defaultPassword : string);
var
  strBrokerHostName: TSMQString;
  strBrokerProtocol: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerHostName := TSMQString.CreateStr(brokerHostName);
  try
    strBrokerProtocol := TSMQString.CreateStr(brokerProtocol);
    try
      strDefaultUserName := TSMQString.CreateStr(defaultUserName);
      try
        strDefaultPassword := TSMQString.CreateStr(defaultPassword);
        try
          CheckSuccess (QueueConnectionFactory_create2 (
            strBrokerHostName.Obj, brokerPort, strBrokerProtocol.Obj,
            strDefaultUserName.Obj, strDefaultPassword.Obj,
            FObj));
        finally
          FreeAndNil (strDefaultPassword);
        end;
      finally
        FreeAndNil (strDefaultUserName);
      end;
    finally
      FreeAndNil (strBrokerProtocol);
    end;
  finally
    FreeAndNil (strBrokerHostName);
  end;
end;{ TSMQQueueConnectionFactory.Create }

constructor TSMQQueueConnectionFactory.Create(brokerHostName : string; 
        brokerPort : LongInt; brokerProtocol : string; connectID : string; 
        defaultUserName : string; defaultPassword : string);
var
  strBrokerHostName: TSMQString;
  strBrokerProtocol: TSMQString;
  strConnectID: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerHostName := TSMQString.CreateStr(brokerHostName);
  try
    strBrokerProtocol := TSMQString.CreateStr(brokerProtocol);
    try
      strConnectID := TSMQString.CreateStr(connectID);
      try
        strDefaultUserName := TSMQString.CreateStr(defaultUserName);
        try
          strDefaultPassword := TSMQString.CreateStr(defaultPassword);
          try
            CheckSuccess (QueueConnectionFactory_create3 (
              strBrokerHostName.Obj, brokerPort, strBrokerProtocol.Obj,
              strConnectID.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
              FObj));
          finally
            FreeAndNil (strDefaultPassword);
          end;
        finally
          FreeAndNil (strDefaultUserName);
        end;
      finally
        FreeAndNil (strConnectID);
      end;
    finally
      FreeAndNil (strBrokerProtocol);
    end;
  finally
    FreeAndNil (strBrokerHostName);
  end;
end;{ TSMQQueueConnectionFactory.Create }

constructor TSMQQueueConnectionFactory.Create(brokerURL : string; 
        defaultUserName : string; defaultPassword : string);
var
  strBrokerURL: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strDefaultUserName := TSMQString.CreateStr(defaultUserName);
    try
      strDefaultPassword := TSMQString.CreateStr(defaultPassword);
      try
        CheckSuccess (QueueConnectionFactory_create5 (
          strBrokerURL.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
          FObj));
      finally
        FreeAndNil (strDefaultPassword);
      end;
    finally
      FreeAndNil (strDefaultUserName);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQQueueConnectionFactory.Create }

constructor TSMQQueueConnectionFactory.Create(brokerURL : string; connectID : 
        string; defaultUserName : string; defaultPassword : string);
var
  strBrokerURL: TSMQString;
  strConnectID: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strConnectID := TSMQString.CreateStr(connectID);
    try
      strDefaultUserName := TSMQString.CreateStr(defaultUserName);
      try
        strDefaultPassword := TSMQString.CreateStr(defaultPassword);
        try
          CheckSuccess (QueueConnectionFactory_create6 (
            strBrokerURL.Obj,
            strConnectID.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
            FObj));
        finally
          FreeAndNil (strDefaultPassword);
        end;
      finally
        FreeAndNil (strDefaultUserName);
      end;
    finally
      FreeAndNil (strConnectID);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQQueueConnectionFactory.Create }

function TSMQQueueConnectionFactory.createQueueConnection: IQueueConnection;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_createQueueConnection (Obj, retObj));
  Result := TSMQQueueConnection.Create (retObj);
end;{ TSMQQueueConnectionFactory.createQueueConnection }

function TSMQQueueConnectionFactory.createQueueConnection(userName : string; 
        password : string): IQueueConnection;
var
  retObj: HOBJ;
  strUserName: TSMQString;
  strPassword: TSMQString;
begin
  strUserName := TSMQString.CreateStr(userName);
  try
    strPassword := TSMQString.CreateStr(password);
    try
      CheckSuccess (QueueConnectionFactory_createQueueConnection2 (Obj,
        strUserName.Obj, strPassword.Obj, retObj));
    finally
      FreeAndNil (strPassword);
    end;
  finally
    FreeAndNil (strUserName);
  end;
  Result := TSMQQueueConnection.Create (retObj);
end;{ TSMQQueueConnectionFactory.createQueueConnection }

function TSMQQueueConnectionFactory.getBrokerHostName: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getBrokerHostName(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getBrokerHostName }

function TSMQQueueConnectionFactory.getBrokerPort: LongInt;
begin
  CheckSuccess (QueueConnectionFactory_getBrokerPort(Obj, Result));
end;{ TSMQQueueConnectionFactory.getBrokerPort }

function TSMQQueueConnectionFactory.getBrokerProtocol: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getBrokerProtocol(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getBrokerProtocol }

function TSMQQueueConnectionFactory.getBrokerURL: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getBrokerURL(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getBrokerURL }

function TSMQQueueConnectionFactory.getClientID: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getClientID(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getClientID }

function TSMQQueueConnectionFactory.getConnectID: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getConnectID(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getConnectID }

function TSMQQueueConnectionFactory.getConnectionURLs: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getConnectionURLs(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getConnectionURLs }

function TSMQQueueConnectionFactory.getDefaultPassword: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getDefaultPassword(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getDefaultPassword }

function TSMQQueueConnectionFactory.getDefaultUser: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueConnectionFactory_getDefaultUser(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueConnectionFactory.getDefaultUser }

function TSMQQueueConnectionFactory.getLoadBalancing: Boolean;
var
  retBool: jboolean;
begin
  CheckSuccess (QueueConnectionFactory_getLoadBalancing(Obj, retBool));
  Result := retBool = jtrue;
end;{ TSMQQueueConnectionFactory.getLoadBalancing }

function TSMQQueueConnectionFactory.getSequential: Boolean;
var
  retBool: jboolean;
begin
  CheckSuccess (QueueConnectionFactory_getSequential(Obj, retBool));
  Result := retBool = jtrue;
end;{ TSMQQueueConnectionFactory.getSequential }

procedure TSMQQueueConnectionFactory.setBrokerHostName(const Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setBrokerHostName(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setBrokerHostName }

procedure TSMQQueueConnectionFactory.setBrokerPort(Value: LongInt);
begin
  CheckSuccess (QueueConnectionFactory_setBrokerPort(Obj, Value));
end;{ TSMQQueueConnectionFactory.setBrokerPort }

procedure TSMQQueueConnectionFactory.setBrokerProtocol(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setBrokerProtocol(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setBrokerProtocol }

procedure TSMQQueueConnectionFactory.setBrokerURL(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setBrokerURL(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setBrokerURL }

procedure TSMQQueueConnectionFactory.setClientID(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setClientID(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setClientID }

procedure TSMQQueueConnectionFactory.setConnectID(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setConnectID(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setConnectID }

procedure TSMQQueueConnectionFactory.setConnectionURLs(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (QueueConnectionFactory_setConnectionURLs(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQQueueConnectionFactory.setConnectionURLs }

procedure TSMQQueueConnectionFactory.setLoadBalancing(Value: Boolean);
begin
  CheckSuccess (QueueConnectionFactory_setLoadBalancing(Obj, booleanToJBoolean (Value)));
end;{ TSMQQueueConnectionFactory.setLoadBalancing }

procedure TSMQQueueConnectionFactory.setSequential(Value: Boolean);
begin
  CheckSuccess (QueueConnectionFactory_setSequential(Obj, booleanToJBoolean (Value)));
end;{ TSMQQueueConnectionFactory.setSequential }

{
************************** TSMQTopicConnectionFactory **************************
}
constructor TSMQTopicConnectionFactory.Create(brokerURL : string);
var
  strBrokerURL: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerURL);
  try
    CheckSuccess (TopicConnectionFactory_create (
      strBrokerURL.Obj, FObj));
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQTopicConnectionFactory.Create }

constructor TSMQTopicConnectionFactory.Create(brokerURL : string; connectID : 
        string);
var
  strBrokerURL: TSMQString;
  strConnectID: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strConnectID := TSMQString.CreateStr(connectID);
    try
      CheckSuccess (TopicConnectionFactory_create4 (
        strBrokerURL.Obj, strConnectID.Obj, FObj));
    finally
      FreeAndNil (strConnectID);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQTopicConnectionFactory.Create }

constructor TSMQTopicConnectionFactory.Create(brokerHostName : string; 
        brokerPort : LongInt; brokerProtocol : string; defaultUserName : string;
        defaultPassword : string);
var
  strBrokerHostName: TSMQString;
  strBrokerProtocol: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerHostName := TSMQString.CreateStr(brokerHostName);
  try
    strBrokerProtocol := TSMQString.CreateStr(brokerProtocol);
    try
      strDefaultUserName := TSMQString.CreateStr(defaultUserName);
      try
        strDefaultPassword := TSMQString.CreateStr(defaultPassword);
        try
          CheckSuccess (TopicConnectionFactory_create2 (
            strBrokerHostName.Obj, brokerPort, strBrokerProtocol.Obj,
            strDefaultUserName.Obj, strDefaultPassword.Obj,
            FObj));
        finally
          FreeAndNil (strDefaultPassword);
        end;
      finally
        FreeAndNil (strDefaultUserName);
      end;
    finally
      FreeAndNil (strBrokerProtocol);
    end;
  finally
    FreeAndNil (strBrokerHostName);
  end;
end;{ TSMQTopicConnectionFactory.Create }

constructor TSMQTopicConnectionFactory.Create(brokerHostName : string; 
        brokerPort : LongInt; brokerProtocol : string; connectID : string; 
        defaultUserName : string; defaultPassword : string);
var
  strBrokerHostName: TSMQString;
  strBrokerProtocol: TSMQString;
  strConnectID: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerHostName := TSMQString.CreateStr(brokerHostName);
  try
    strBrokerProtocol := TSMQString.CreateStr(brokerProtocol);
    try
      strConnectID := TSMQString.CreateStr(connectID);
      try
        strDefaultUserName := TSMQString.CreateStr(defaultUserName);
        try
          strDefaultPassword := TSMQString.CreateStr(defaultPassword);
          try
            CheckSuccess (TopicConnectionFactory_create3 (
              strBrokerHostName.Obj, brokerPort, strBrokerProtocol.Obj,
              strConnectID.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
              FObj));
          finally
            FreeAndNil (strDefaultPassword);
          end;
        finally
          FreeAndNil (strDefaultUserName);
        end;
      finally
        FreeAndNil (strConnectID);
      end;
    finally
      FreeAndNil (strBrokerProtocol);
    end;
  finally
    FreeAndNil (strBrokerHostName);
  end;
end;{ TSMQTopicConnectionFactory.Create }

constructor TSMQTopicConnectionFactory.Create(brokerURL : string; 
        defaultUserName : string; defaultPassword : string);
var
  strBrokerURL: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strDefaultUserName := TSMQString.CreateStr(defaultUserName);
    try
      strDefaultPassword := TSMQString.CreateStr(defaultPassword);
      try
        CheckSuccess (TopicConnectionFactory_create5 (
          strBrokerURL.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
          FObj));
      finally
        FreeAndNil (strDefaultPassword);
      end;
    finally
      FreeAndNil (strDefaultUserName);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQTopicConnectionFactory.Create }

constructor TSMQTopicConnectionFactory.Create(brokerURL : string; connectID : 
        string; defaultUserName : string; defaultPassword : string);
var
  strBrokerURL: TSMQString;
  strConnectID: TSMQString;
  strDefaultUserName: TSMQString;
  strDefaultPassword: TSMQString;
begin
  strBrokerURL := TSMQString.CreateStr(brokerHostName);
  try
    strConnectID := TSMQString.CreateStr(connectID);
    try
      strDefaultUserName := TSMQString.CreateStr(defaultUserName);
      try
        strDefaultPassword := TSMQString.CreateStr(defaultPassword);
        try
          CheckSuccess (TopicConnectionFactory_create6 (
            strBrokerURL.Obj,
            strConnectID.Obj, strDefaultUserName.Obj, strDefaultPassword.Obj,
            FObj));
        finally
          FreeAndNil (strDefaultPassword);
        end;
      finally
        FreeAndNil (strDefaultUserName);
      end;
    finally
      FreeAndNil (strConnectID);
    end;
  finally
    FreeAndNil (strBrokerURL);
  end;
end;{ TSMQTopicConnectionFactory.Create }

function TSMQTopicConnectionFactory.createTopicConnection: ITopicConnection;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_createTopicConnection (Obj, retObj));
  Result := TSMQTopicConnection.Create (retObj);
end;{ TSMQTopicConnectionFactory.createTopicConnection }

function TSMQTopicConnectionFactory.createTopicConnection(userName : string; 
        password : string): ITopicConnection;
var
  retObj: HOBJ;
  strUserName: TSMQString;
  strPassword: TSMQString;
begin
  strUserName := TSMQString.CreateStr(userName);
  try
    strPassword := TSMQString.CreateStr(password);
    try
      CheckSuccess (TopicConnectionFactory_createTopicConnection2 (Obj,
        strUserName.Obj, strPassword.Obj, retObj));
    finally
      FreeAndNil (strPassword);
    end;
  finally
    FreeAndNil (strUserName);
  end;
  Result := TSMQTopicConnection.Create (retObj);
end;{ TSMQTopicConnectionFactory.createTopicConnection }

function TSMQTopicConnectionFactory.getBrokerHostName: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getBrokerHostName(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getBrokerHostName }

function TSMQTopicConnectionFactory.getBrokerPort: LongInt;
begin
  CheckSuccess (TopicConnectionFactory_getBrokerPort(Obj, Result));
end;{ TSMQTopicConnectionFactory.getBrokerPort }

function TSMQTopicConnectionFactory.getBrokerProtocol: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getBrokerProtocol(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getBrokerProtocol }

function TSMQTopicConnectionFactory.getBrokerURL: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getBrokerURL(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getBrokerURL }

function TSMQTopicConnectionFactory.getClientID: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getClientID(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getClientID }

function TSMQTopicConnectionFactory.getConnectID: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getConnectID(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getConnectID }

function TSMQTopicConnectionFactory.getConnectionURLs: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getConnectionURLs(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getConnectionURLs }

function TSMQTopicConnectionFactory.getDefaultPassword: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getDefaultPassword(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getDefaultPassword }

function TSMQTopicConnectionFactory.getDefaultUser: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (TopicConnectionFactory_getDefaultUser(Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQTopicConnectionFactory.getDefaultUser }

function TSMQTopicConnectionFactory.getLoadBalancing: Boolean;
var
  retBool: jboolean;
begin
  CheckSuccess (TopicConnectionFactory_getLoadBalancing(Obj, retBool));
  Result := retBool = jtrue;
end;{ TSMQTopicConnectionFactory.getLoadBalancing }

function TSMQTopicConnectionFactory.getSequential: Boolean;
var
  retBool: jboolean;
begin
  CheckSuccess (TopicConnectionFactory_getSequential(Obj, retBool));
  Result := retBool = jtrue;
end;{ TSMQTopicConnectionFactory.getSequential }

procedure TSMQTopicConnectionFactory.setBrokerHostName(const Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setBrokerHostName(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setBrokerHostName }

procedure TSMQTopicConnectionFactory.setBrokerPort(Value: LongInt);
begin
  CheckSuccess (TopicConnectionFactory_setBrokerPort(Obj, Value));
end;{ TSMQTopicConnectionFactory.setBrokerPort }

procedure TSMQTopicConnectionFactory.setBrokerProtocol(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setBrokerProtocol(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setBrokerProtocol }

procedure TSMQTopicConnectionFactory.setBrokerURL(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setBrokerURL(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setBrokerURL }

procedure TSMQTopicConnectionFactory.setClientID(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setClientID(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setClientID }

procedure TSMQTopicConnectionFactory.setConnectID(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setConnectID(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setConnectID }

procedure TSMQTopicConnectionFactory.setConnectionURLs(Value: string);
var
  strVal: TSMQString;
begin
  strVal := TSMQString.CreateStr (Value);
  try
    CheckSuccess (TopicConnectionFactory_setConnectionURLs(Obj, strVal.Obj));
  finally
    FreeAndNil (strVal);
  end;
end;{ TSMQTopicConnectionFactory.setConnectionURLs }

procedure TSMQTopicConnectionFactory.setLoadBalancing(Value: Boolean);
begin
  CheckSuccess (TopicConnectionFactory_setLoadBalancing(Obj, booleanToJBoolean (Value)));
end;{ TSMQTopicConnectionFactory.setLoadBalancing }

procedure TSMQTopicConnectionFactory.setSequential(Value: Boolean);
begin
  CheckSuccess (TopicConnectionFactory_setSequential(Obj, booleanToJBoolean (Value)));
end;{ TSMQTopicConnectionFactory.setSequential }

{
******************************* TSMQQueueBrowser *******************************
}
procedure TSMQQueueBrowser.close;
begin
  CheckSuccess (QueueBrowser_close(Obj));
end;{ TSMQQueueBrowser.close }

function TSMQQueueBrowser.getEnumeration: TList;
begin
  UnsupportedMethod ('getEnumeration');
  
  Result := nil;
end;{ TSMQQueueBrowser.getEnumeration }

function TSMQQueueBrowser.getMessageSelector: string;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueBrowser_getMessageSelector (Obj, retObj));
  Result := strObjToString (retObj);
end;{ TSMQQueueBrowser.getMessageSelector }

function TSMQQueueBrowser.getQueue: IQueue;
var
  retObj: HOBJ;
begin
  CheckSuccess (QueueBrowser_getQueue (Obj, retObj));
  Result := TSMQQueue.Create (retObj);
end;{ TSMQQueueBrowser.getQueue }


end.
