{
  TRichEditPDF

  a VCL component to use TRichEdit with TRolePDF
  author: mohab elsherif
  email: info@roledata.com
  version 1.01
  license: as is, without any kind of warrantee.
}

unit repdf;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, RichEdit, StdCtrls, pdfhdr;

type
  TRichEditPDF = class(TComponent)
  private
    FPageSize: TPageSize;
    FPageWidth: single;
    FPageHeight: single;
    FLeftMargin: single;
    FTopMargin: single;
    FRightMargin: single;
    FBottomMargin: single;
    FAutoLaunch: boolean;
    FCreateOutlines: boolean;
    FForm: TForm; // parent for RichEdit
    FRichEdit: TRichEdit;
    FLicense: TStrings;
    procedure SetPageSize(Value: TPageSize);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveToFile(FileName: string);
  published
    property PageWidth: single read FPageWidth write FPageWidth;
    property PageHeight: single read FPageHeight write FPageHeight;
    property LeftMargin: single read FLeftMargin write FLeftMargin;
    property TopMargin: single read FTopMargin write FTopMargin;
    property RightMargin: single read FRightMargin write FRightMargin;
    property BottomMargin: single read FBottomMargin write FBottomMargin;
    property CreateOutlines: boolean read FCreateOutlines write FCreateOutlines;
    property AutoLaunch: boolean read FAutoLaunch write FAutoLaunch;
    property RichEdit: TRichEdit read FRichEdit write FRichEdit;
    property License: TStrings read FLicense write FLicense;
    property PageSize: TPageSize read FPageSize write SetPageSize;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Roledata', [TRichEditPDF]);
end;

constructor TRichEditPDF.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm := TForm.Create(nil);
  FRichEdit := TRichEdit.Create(nil);
  FRichEdit.Parent := FForm;
  FLicense := TStringList.Create;
  SetPageSize(psA4);
  LeftMargin := 1.0; // inch
  TopMargin := 1.0;
  RightMargin := 1.0;
  BottomMargin := 1.0;
  CreateOutlines := false;
  FAutoLaunch := false;
end;

destructor TRichEditPDF.Destroy;
begin
  FRichEdit.Free;
  FForm.Free;
  FLicense.Free;
  inherited Destroy;
end;

procedure TRichEditPDF.SetPageSize(Value: TPageSize);
begin
  FPageSize := Value;
  case Value of
    psA4: begin PageWidth := 8.26; PageHeight := 11.69; end;
    psLetter: begin PageWidth := 8.5; PageHeight := 11; end;
    psLegal: begin PageWidth := 8.5; PageHeight := 14; end;
  end;
end;

procedure TRichEditPDF.SaveToFile(FileName: string);
var
  PageNo, ParentID: integer;
  fr: TFormatRange;
  AFileName: string;
begin
  if FRichEdit.Lines.Count = 0 then exit;
  AFileName := ChangeFileExt(FileName, '.pdf');
  PageNo := 0;
  ParentID := 0;
  with fr do begin
    with rcPage, rc do begin
      Left := round(LeftMargin * 1440); // inch -> twips
      Top := round(TopMargin * 1440);
      Right := round((FPageWidth - RightMargin) * 1440);
      Bottom := round((FPageHeight - BottomMargin) * 1440);
    end;
    chrg.cpMin := 0;
    chrg.cpMax := FRichEdit.GetTextLen;
  end;
  with TRolePDF.Create(nil) do
  try
    FileName := AFileName;
    AutoLaunch := FAutoLaunch;
    License.Assign(FLicense);
    PageSize := FPageSize;
    if FPageSize = psCustom then begin
      PageWidth := round(FPageWidth * 72);
      PageHeight := round(FPageHeight * 72);
    end;
    BeginDoc;
    if CreateOutlines then
      ParentID := AddOutline('Pages of ' + ExtractFileName(AFileName), -1, 1, 0);
    while fr.chrg.cpMin < fr.chrg.cpMax do begin
      with fr do begin
        hdc := Canvas.Handle;
        hdcTarget := Canvas.Handle;
      end;
      if CreateOutlines then begin
        Inc(PageNo);
        AddOutline('Page ' + IntToStr(PageNo), ParentID, PageNo, 0);
      end;
      fr.chrg.cpMin := SendMessage(FRichEdit.Handle, EM_FORMATRANGE, 1, LPARAM(@fr));
      if fr.chrg.cpMin < fr.chrg.cpMax then NewPage;
    end;
    EndDoc;
  finally
    Free;
  end;
end;

end.
