unit DataManager;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList, ComCtrls, ExtCtrls, Menus, DSpGlobals, DSpPFA, DSpViewer,
  StdCtrls, DSpCoordinateSystem, Grids, DSpRRT, ExtDlgs, DSpDataMgr;

type
  TDataMgmtForm = class(TForm)
    tvDataSets: TTreeView;
    ilDataSets: TImageList;
    StatusBar1: TStatusBar;
    mnuDataMgr: TMainMenu;
    mFile: TMenuItem;
    mFileAddDatasets: TMenuItem;
    mFileCloseDataset: TMenuItem;
    N1: TMenuItem;
    mFileClose: TMenuItem;
    mHelp: TMenuItem;
    Splitter1: TSplitter;
    OpenDialog1: TOpenDialog;
    pmDataSets: TPopupMenu;
    pmDataSetsAdd: TMenuItem;
    N2: TMenuItem;
    pmDataSetsRefresh: TMenuItem;
    pmDataSetsSort: TMenuItem;
    pmDataSetsBar: TMenuItem;
    pmDataSetsClose: TMenuItem;
    pmDataSetsRenderer: TMenuItem;
    pmDataSetsProperties: TMenuItem;
    mView: TMenuItem;
    mViewRefresh: TMenuItem;
    mViewSort: TMenuItem;
    pcInfo: TPageControl;
    tsInfo: TTabSheet;
    tsViewer: TTabSheet;
    mmInfo: TMemo;
    pmRenderer: TPopupMenu;
    pmRendererDelete: TMenuItem;
    N3: TMenuItem;
    pmRendererUp: TMenuItem;
    pmRendererDown: TMenuItem;
    tvRenderers: TTreeView;
    Splitter2: TSplitter;
    Panel1: TPanel;
    gbRenderers: TGroupBox;
    cbRenderers: TComboBox;
    pnlMinMax: TPanel;
    Label1: TLabel;
    edRRTMin: TEdit;
    edRRTMax: TEdit;
    Label2: TLabel;
    pnlRamp: TPanel;
    cmdRendUpdate: TButton;
    imgBackground: TImage;
    Label3: TLabel;
    ColorDialog1: TColorDialog;
    dgRamp: TDrawGrid;
    cmdLoadPalette: TButton;
    cmdSavePalette: TButton;
    pmRamp: TPopupMenu;
    pmRampAbove: TMenuItem;
    pmRampBelow: TMenuItem;
    pmRampDelete: TMenuItem;
    N4: TMenuItem;
    pmRampInvert: TMenuItem;
    imgRamp: TImage;
    N5: TMenuItem;
    pmRendererSaveImage: TMenuItem;
    SavePictureDialog1: TSavePictureDialog;
    pmDataSetsHistogram: TMenuItem;
    procedure AddDataSource(Sender: TObject);
    procedure mFileCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure tvDataSetsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure EditTreeViewLabel(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure tvDataSetsEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure RefreshDataSetTree(Sender: TObject);
    procedure SortTreeView(Sender: TObject);
    procedure AddSourceToViewer(Sender: TObject);
    procedure DeleteRenderer(Sender: TObject);
    procedure pmRendererUpClick(Sender: TObject);
    procedure pmRendererDownClick(Sender: TObject);
    procedure tvRenderersDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure tvRenderersDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tvRenderersMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure tvRenderersEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure cbRenderersChange(Sender: TObject);
    procedure edRRTMaxExit(Sender: TObject);
    procedure edRRTMinExit(Sender: TObject);
    procedure cmdRendUpdateClick(Sender: TObject);
    procedure imgBackgroundClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure dgRampMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure dgRampDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure dgRampGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure dgRampSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure dgRampSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure InsertBreak(Sender: TObject);
    procedure pmRampDeleteClick(Sender: TObject);
    procedure pmRampInvertClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure pmRendererSaveImageClick(Sender: TObject);
    procedure pmDataSetsCloseClick(Sender: TObject);
    procedure pmDataSetsHistogramClick(Sender: TObject);
  private
    FDataMgr  : TDSpDataMgr;
    FViewer   : TDSpViewer;
    FCoordSys : TDSpCoordinateSystem;
    FRampBmp  : TBitmap;                    // bitmap to render ramp
  protected
    procedure  DisplayNodeInfo(node: TTreeNode);
    procedure  ShowRendererPanel(node: TTreeNode);
    procedure  MakeRamp;
    procedure  RefreshRendererTree(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property   DataMgr: TDSpDataMgr write FDataMgr;
    property   Viewer: TDSpViewer write FViewer;
    property   CoordinateSystem: TDSpCoordinateSystem read FCoordSys;
  end;

var
  DataMgmtForm: TDataMgmtForm;

implementation
{$R *.DFM}
uses DSpUtils, DSpDataSet, DSpRaster, DSpRenderer, DSpRasterSource, DSpModuleBase,
     DSpHistogram, HistoForm;

constructor TDataMgmtForm.Create(AOwner: TComponent);
begin
  inherited;
  FRampBmp := TBitmap.Create;
  FRampBmp.PixelFormat := pf24bit;
end;

destructor TDataMgmtForm.Destroy;
begin
  FRampBmp.Free;
  if Assigned(FCoordSys) then FCoordSys.Release;
  inherited;
end;

procedure TDataMgmtForm.AddDataSource(Sender: TObject);
var src: TDSpSourceHandler;
    i, b, num: integer;
    datasetNode, child: TTreeNode;
    ds: TDSpSourceDataSet;
    dst: TDSpPFAFormat;
    raster: TDSpRaster;
begin
  OpenDialog1.Filter := PFA.FilterString(dspReadRaster);
  if OpenDialog1.Execute then begin
    // Open the data source, there is precisely one. This is a reference to an
    // instance owned by the PFA. Call src.Close when done. The src object
    // persists at the level of the PFA until all data sets src supports are
    // released. Since at the application level the src object only needs to
    // open and delete data sets, the src object should always be closed after
    // the necessary data sets are created frm the source.
    src := PFA.Open(OpenDialog1.Files[0]);

    // Hopefully the source has some data...
    num := src.DataSets;
    if num = 0 then begin
      src.Close;
      Exit;
    end;

    // Add data set descriptions to the tvDataSets tree.
    for i := 0 to num - 1 do begin
      ds := src.ReadLock(i);
      try
        datasetNode := tvDataSets.Items.AddChild(tvDataSets.TopItem, ds.Name);
        dst := ds.DataSetType;
      finally
        ds.ReadUnlock;
      end;

      case dst of
        dspPFARaster: begin
          datasetNode.ImageIndex := 5;
          datasetNode.SelectedIndex := 5;
          raster := TDSpRaster(FDataMgr.OpenDataSet(src, i));
          datasetNode.Data := raster;
          if raster.Bands > 0 then
            for b := 0 to raster.Bands - 1 do begin
              if raster.BandType[b] = dspRasterBandSurface then begin
                child := tvDataSets.Items.AddChild(datasetNode, Format('Surface - %d', [b]));
                child.ImageIndex := 3;
                child.SelectedIndex := 3;
              end else begin
                child := tvDataSets.Items.AddChild(datasetNode, Format('Ordinal - %d', [b]));
                child.ImageIndex := 2;
                child.SelectedIndex := 2;
             end;
              child.Data := pointer(b);
            end;
        end;
      end;
    end;
    tvDataSets.TopItem.Expand(false);
    src.Close;   // This only decreases the reference count. The source
  end;           // object persists at the PFA until all data sets are closed.
end;

procedure TDataMgmtForm.mFileCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TDataMgmtForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FViewer.Refresh;
  FViewer := nil;
  FDataMgr := nil;
  Action := caHide;
end;

procedure TDataMgmtForm.FormShow(Sender: TObject);
begin
  Assert(FDataMgr <> nil, 'DataMgmtForm.Show: FDataMgr not initialized');
  RefreshDataSetTree(self);
end;

procedure TDataMgmtForm.tvDataSetsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var node: TTreeNode;
begin
  node := tvDataSets.GetNodeAt(X, Y);

  if Button = mbRight then begin
    if (node = nil) or (node.Level = 0) then begin
      pmDataSetsBar.Visible := false;
      pmDataSetsClose.Visible := false;
      pmDataSetsRenderer.Visible := false;
      pmDataSetsHistogram.Visible := false;
      pmDataSetsProperties.Visible := false;
    end else begin
      tvDataSets.Selected := node;
      pmDataSetsBar.Visible := true;
      pmDataSetsClose.Visible := node.Level = 1;
      pmDataSetsHistogram.Visible := node.Level = 2;
      pmDataSetsRenderer.Visible := node.Level = 2;
      pmDataSetsProperties.Visible := true;
    end;
    pmDataSets.Popup(ClientOrigin.x+X, clientOrigin.y+Y+10);
  end else if Button = mbLeft then begin
    mmInfo.Clear;
    if (node = nil) or (node.Level = 0) then Exit;
    DisplayNodeInfo(node);
    if (TObject(node.Parent.Data) is TDSpRaster) then
      if not Assigned(FCoordSys) or FCoordSys.IsCompatible(TDSpRaster(node.Parent.Data).CoordinateSystem) then
        tvDataSets.BeginDrag(false);
  end;
end;

procedure TDataMgmtForm.EditTreeViewLabel(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  AllowEdit := Node <> (Sender as TTreeView).TopItem;
end;

procedure TDataMgmtForm.tvDataSetsEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
begin
  if TObject(Node.Data) is TDSpDataSet then
    TDSpDataSet(Node.Data).Name := S;
end;

procedure TDataMgmtForm.RefreshDataSetTree(Sender: TObject);
var node, child: TTreeNode;
    i, b: integer;
    ds: TDSpDataSet;
begin
  tvDatasets.Items.Clear;
  node := tvDataSets.Items.Add(nil, 'Data sets');
  node.ImageIndex := 9;
  node.SelectedIndex := 9;
  if FDataMgr.DataSets > 0 then
    for i := 0 to FDataMgr.DataSets - 1 do begin
      ds := FDataMgr.DataSet[i];
      node := tvDataSets.Items.AddChild(tvDataSets.TopItem, ds.Name);
      node.Data := ds;
      if ds is TDSpRaster then begin
        node.ImageIndex := 5;
        node.SelectedIndex := 5;
        if (ds as TDSpRaster).Bands > 0 then
          for b := 0 to (ds as TDSpRaster).Bands - 1 do begin
            if (ds as TDSpRaster).BandType[b] = dspRasterBandSurface then begin
              child := tvDataSets.Items.AddChild(node, Format('Surface - %d', [b]));
              child.ImageIndex := 3;
              child.SelectedIndex := 3;
            end else begin
              child := tvDataSets.Items.AddChild(node, Format('Ordinal - %d', [b]));
              child.ImageIndex := 2;
              child.SelectedIndex := 2;
            end;
            child.Data := pointer(b);
          end;
      end;
    end;
  tvDataSets.TopItem.Expand(false);
  FViewer.Refresh;
end;

procedure TDataMgmtForm.RefreshRendererTree(Sender: TObject);
var node, child: TTreeNode;
    i, b: integer;
    band: TDSpRasterBand;
    renderers: TList;
    rend: TDSpRenderer;
begin
  tvRenderers.Items.Clear;
  node := tvRenderers.Items.Add(nil, 'Renderers');
  node.ImageIndex := 2;
  node.SelectedIndex := 2;

  renderers := FViewer.Renderers;
  if renderers.Count = 0 then Exit;
  for i := 0 to renderers.Count - 1 do begin
    rend := TDSpRenderer(renderers[i]);
    node := tvRenderers.Items.AddChildObject(tvRenderers.TopItem, rend.Name, rend);
    node.ImageIndex := 2;
    node.SelectedIndex := 2;
    if (rend is TDSpRasterRendererSimple) and (TDSpRasterRendererSimple(rend).Bands > 0) then
      for b := 0 to TDSpRasterRendererSimple(rend).Bands - 1 do begin
        band := TDSpRasterRendererSimple(rend).Band[b];
        child := tvRenderers.Items.AddChildObject(node, 'band.X', band);
        child.ImageIndex := 3;
        child.SelectedIndex := 3;
      end;
  end;
  tvRenderers.TopItem.Expand(false);
end;

procedure TDataMgmtForm.SortTreeView(Sender: TObject);
begin
  (Sender as TTreeView).SortType := stText;
end;

procedure TDataMgmtForm.AddSourceToViewer(Sender: TObject);
var node, child, grandchild: TTreeNode;
    rend: TDSpRenderer;
begin
  node := tvDataSets.Selected;
  if not Assigned(FCoordSys) then
    if TObject(node.Parent.Data) is TDSpDataSet then
      FCoordSys := TDSpDataSet(node.Parent.Data).CoordinateSystem.AddRef
    else Assert(false, 'must add some code here');

  if TObject(node.Parent.Data) is TDSpRaster then begin
    rend := TDSpRasterRendererSimple.Create(FViewer, node.Parent.Text);
    (rend as TDSpRasterRendererSimple).AddRasterBand(TDSpRaster(node.Parent.Data).Band[integer(node.Data)]);
    FViewer.AddRenderer(rend);
    child := tvRenderers.Items.AddChildObject(tvRenderers.TopItem, node.Parent.Text, rend);
    child.ImageIndex := 2;
    child.SelectedIndex := 2;
    grandchild := tvRenderers.Items.AddChildObject(child, node.Text, TDSpRaster(node.Parent.Data).Band[integer(node.Data)]);
    grandchild.ImageIndex := 3;
    grandchild.SelectedIndex := 3;
    pnlRamp.Visible := false;
    pnlMinMax.Visible := true;
  end;
end;

procedure TDataMgmtForm.DeleteRenderer(Sender: TObject);
var node: TTreeNode;
begin
  node := tvRenderers.Selected;
  if TObject(node.Data) is TDSpRenderer then begin
    TDSpRenderer(node.Data).DissociateFromViewer;
    TDSpRenderer(node.Data).Free;
    if FViewer.NumRenderers = 0 then begin
      FCoordSys.Release;
      FCoordSys := nil;
    end;
  end else if TObject(node.Data) is TDSpRasterBand then
    TDSpRasterRendererSimple(node.Parent.Data).DeleteRasterBand(TDSpRasterBand(node.Data))  
  else Exit;
  tvRenderers.Items.Delete(node);
  ShowRendererPanel(nil);
end;

procedure TDataMgmtForm.pmDataSetsHistogramClick(Sender: TObject);
var node: TTreeNode;
    raster: TDSpRaster;
    band: integer;
    data: pointer;
    histo: TDSpHisto;
    histFrm: THistoFrm;
    min, max: TDSpFloat64;
begin
  node := tvDataSets.Selected;
  if TObject(node.Parent.Data) is TDSpRaster then begin
    raster := TDSpRaster(node.Parent.Data);
    band := integer(node.Data);
    min := raster.Band[band].Min;
    max := raster.Band[band].Max;
    if min > max then begin
      MessageDlg('Cannot produce histogram from this data: Min and Max are unknown.', mtInformation, [mbOk], 0);
      Exit;
    end;
    Cursor := crHourglass;
    data := DSpReadSingleBandData(raster, band);
    case raster.BandDataType[band] of
      dspUInt8  : histo := TDSpHistoUInt8.Create;
      dspInt8   : histo := TDSpHistoInt8.Create;
      dspUInt16 : histo := TDSpHistoUInt16.Create(Round(min), Round(max));
      dspInt16  : histo := TDSpHistoInt16.Create(Round(min), Round(max));
      dspUInt32 : histo := TDSpHistoUInt32.Create(Round(min), Round(max));
      dspInt32  : histo := TDSpHistoInt32.Create(Round(min), Round(max));
      dspInt64  : histo := TDSpHistoInt64.Create(Round(min), Round(max));
      dspFloat32: histo := TDSpHistoFloat32.Create(min, max);
      dspFloat64: histo := TDSpHistoFloat64.Create(min, max);
    end;
    histo.AddData(data, raster.Rows * raster.Columns);
    FreeMem(data);
    histFrm := THistoFrm.Create(self);
    histFrm.Caption := Format('Histogram for: %s (band %d)', [raster.Name, band]);
    histFrm.SetHistogramData(histo);
    histFrm.Show;
    Cursor := crDefault;
  end;
end;

procedure TDataMgmtForm.pmRendererUpClick(Sender: TObject);
var node, nodeUp: TTreeNode;
    ndx: integer;
begin
  node := tvRenderers.Selected;
  ndx := node.Index;
  nodeUp := node.GetPrevSibling;
  FViewer.SwapRenderers(ndx, ndx-1);
  node.MoveTo(nodeUp, naInsert);
end;

procedure TDataMgmtForm.pmRendererDownClick(Sender: TObject);
var node, nodeDown: TTreeNode;
    ndx: integer;
begin
  node := tvRenderers.Selected;
  ndx := node.Index;
  nodeDown := node.GetNextSibling;
  FViewer.SwapRenderers(ndx, ndx+1);
  nodeDown.MoveTo(node, naInsert);
end;

procedure TDataMgmtForm.tvRenderersDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var node: TTreeNode;
begin
  node := tvRenderers.GetNodeAt(X, Y);
  Accept := (Source = tvDataSets) and                           // so dragged object is renderable
            ((node = nil) or (node = tvRenderers.TopItem) or    // not dropped on an existing renderer
             ((TObject(node.Data) is TDSpRasterRenderer) and (TObject(tvDataSets.Selected.Parent.Data) is TDSpRaster))
            );
end;

procedure TDataMgmtForm.tvRenderersDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var node, child: TTreeNode;
begin
  node := tvRenderers.GetNodeAt(X, Y);
  if (node = nil) or (node = tvRenderers.TopItem) then begin
    AddSourceToViewer(Sender); // create a new renderer, update the viewer
    tvRenderers.TopItem.Expand(true);
  end else begin
    if TObject(node.Data) is TDSpRasterRendererSimple then begin
      TDSpRasterRendererSimple(node.Data).AddRasterBand(TDSpRaster(tvDataSets.Selected.Parent.Data).Band[integer(tvDataSets.Selected.Data)]);
      child := tvRenderers.Items.AddChildObject(node, tvDataSets.Selected.Text, TDSpRaster(tvDataSets.Selected.Parent.Data).Band[integer(tvDataSets.Selected.Data)]);
    end else child := tvRenderers.Items.AddChildObject(node, tvDataSets.Selected.Text, tvDataSets.Selected.Data);
    child.ImageIndex := 3;
    child.SelectedIndex := 3;
    node.Expand(false);
    FViewer.SetRendererMap;
  end;
end;

procedure TDataMgmtForm.ShowRendererPanel(node: TTreeNode);
var rend : TDSpRasterRendererSimple;
begin
  if node = nil then begin
    gbRenderers.Visible := false;
    FViewer.Invalidate;
  end else begin
    tvRenderers.Selected := node;

    if (node.Level = 1) and (TObject(node.Data) is TDSpRasterRendererSimple) then begin
      rend := TDSpRasterRendererSimple(node.Data);
      if rend.Transformation is TDSpRRTHotToCold then cbRenderers.ItemIndex := 0
      else if rend.Transformation is TDSpRRTGrayscale then cbRenderers.ItemIndex := 1
      else begin
        cbRenderers.ItemIndex := 2;
        dgRamp.Tag := longint(rend.Transformation);
        dgRamp.RowCount := TDSpRRTSmoothRamp(rend.Transformation).Intervals + 1;
        dgRamp.Repaint;
      end;
      pnlMinMax.Visible := cbRenderers.ItemIndex < 2;
      pnlRamp.Visible := not pnlMinMax.Visible;
      edRRTMin.Text := FloatToStr(rend.Min);
      edRRTMax.Text := FloatToStr(rend.Max);
      MakeRamp;
      gbRenderers.Visible := true;
    end else gbRenderers.Visible := false;
  end;
end;

procedure TDataMgmtForm.tvRenderersMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var node: TTreeNode;
begin
  node := tvRenderers.GetNodeAt(X, Y);
  ShowRendererPanel(node);
  if node = nil then Exit;

  if Button = mbRight then begin
    if node = tvRenderers.TopItem then Exit;
    pmRendererUp.Enabled := (node.Index > 0) and (node.Level = 1);
    pmRendererDown.Enabled := (node.Index < node.Parent.Count - 1) and (node.Level = 1);
    pmRendererSaveImage.Enabled := node.Level = 1;
    pmRenderer.Popup(tvRenderers.ClientOrigin.x+X, tvRenderers.clientOrigin.y+Y+10);
  end;
end;

procedure TDataMgmtForm.tvRenderersEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
begin
  if TObject(Node.Data) is TDSpRenderer then TDSpRenderer(Node.Data).Name := S;
end;

procedure TDataMgmtForm.cbRenderersChange(Sender: TObject);
var rrt: TDSpRRTSingleBand;
begin
  rrt := nil; // satify the compiler
  case cbRenderers.ItemIndex of
    0: rrt := TDSpRRTHotToCold.Create;
    1: rrt := TDSpRRTGrayScale.Create;
    2: rrt := TDSpRRTSmoothRamp.Create;
  end;
  TDSpRasterRendererSimple(tvRenderers.Selected.Data).Transformation := rrt;
  if cbRenderers.ItemIndex = 2 then begin
    pnlRamp.Visible := true;
    dgRamp.Tag := longint(rrt);
    dgRamp.RowCount := TDSpRRTSmoothRamp(rrt).Intervals + 1;
    dgRamp.Repaint;
  end else begin
    pnlRamp.Visible := false;
    cmdRendUpdate.Enabled := true;
  end;
  pnlMinMax.Visible := not pnlRamp.Visible;
  MakeRamp;
end;

procedure TDataMgmtForm.edRRTMaxExit(Sender: TObject);
begin
  try
    StrToFloat(edRRTMax.Text);
  except
    Beep;
    edRRTMax.SetFocus;
  end;
end;

procedure TDataMgmtForm.edRRTMinExit(Sender: TObject);
begin
  try
    StrToFloat(edRRTMin.Text);
  except
    Beep;
    edRRTMin.SetFocus;
  end;
end;

procedure TDataMgmtForm.DisplayNodeInfo(node: TTreeNode);
var str: string;
    ext: TDSpExtent;
    raster: TDSpRaster;
    band: integer;
begin
  case node.Level of
    1: begin
         if TObject(node.Data) is TDSpRaster then begin
           mmInfo.Lines.Add('RASTER INFORMATION');
           mmInfo.Lines.Add('');
           mmInfo.Lines.Add(Format('Raster:'#9#9'%s (ID %d)', [TDSpRaster(node.Data).Name, TDSpRaster(node.Data).ID]));
           mmInfo.Lines.Add(Format('Bands:'#9#9'%d', [TDSpRaster(node.Data).Bands]));
           mmInfo.Lines.Add('');
           mmInfo.Lines.Add(Format('Rows:'#9#9'%d', [TDSpRaster(node.Data).Rows]));
           mmInfo.Lines.Add(Format('Columns:'#9#9'%d', [TDSpRaster(node.Data).Columns]));
         end;

         mmInfo.Lines.Add('');
         mmInfo.Lines.Add(TDSpDataSet(node.Data).DescribeCS);
         mmInfo.Lines.Add('Extent:');
         ext := TDSpDataSet(node.Data).Extent;
         mmInfo.Lines.Add('  North:'#9 + FloatToStrF(ext.North, ffGeneral, 15, 3) +
                          #9#9'East:'#9 + FloatToStrF(ext.East, ffGeneral, 15, 3));
         mmInfo.Lines.Add('  South:'#9 + FloatToStrF(ext.South, ffGeneral, 15, 3) +
                          #9#9'West:'#9 + FloatToStrF(ext.West, ffGeneral, 15, 3));
       end;
    2: begin
         if TObject(node.Parent.Data) is TDSpRaster then begin
           raster := TDSpRaster(node.Parent.Data);
           band := integer(node.Data);
           mmInfo.Lines.Add('RASTER BAND INFORMATION');
           mmInfo.Lines.Add('');
           mmInfo.Lines.Add(Format('Raster:'#9#9'%s (ID %d)', [raster.Name, raster.ID]));
           mmInfo.Lines.Add(Format('Band:'#9#9'%d', [band]));
           mmInfo.Lines.Add('');
           case raster.BandType[band] of
             dspRasterBandSurface : str := 'Band type:'#9'Continuous (surface)';
             dspRasterBandOrdinal : str := 'Band type:'#9'Categorical (classes)';
           end;
           mmInfo.Lines.Add(str);
           case raster.BandDataType[band] of
             dspUInt8    : str := 'Data type:'#9'unsigned 8-bit integer'#13#10#9#9'range 0 ~ 255';
             dspInt8     : str := 'Data type:'#9'signed 8-bit integer'#13#10#9#9'range -128 ~ 127';
             dspUInt16   : str := 'Data type:'#9'unsigned 16-bit integer'#13#10#9#9'range 0 ~ 65,535';
             dspInt16    : str := 'Data type:'#9'signed 16-bit integer'#13#10#9#9'range -32,768 ~ 32,767';
             dspUInt32   : str := 'Data type:'#9'unsigned 32-bit integer'#13#10#9#9'range 0 ~ 4,294,967,295';
             dspInt32    : str := 'Data type:'#9'signed 32-bit integer'#13#10#9#9'range -2,147,483,648 ~ 2,147,483,647';
             dspInt64    : str := 'Data type:'#9'signed 64-bit integer'#13#10#9#9'range -9,223,372,036,854,775,808 ~ 9,223,372,036,854,775,807';
             dspFloat32  : str := 'Data type:'#9'32-bit floating point'#13#10#9#9'7 significant digits'#13#10#9#9'range '#177' 1.5E-45 ~ 3.4E38';
             dspFloat64  : str := 'Data type:'#9'64-bit floating point'#13#10#9#9'15 significant digits'#13#10#9#9'range '#177' 5.0E-324 ~ 1.7E308';
           end;
           mmInfo.Lines.Add(str);
         end;
    end;
  end;
  mmInfo.SelStart := 0;
end;

procedure TDataMgmtForm.cmdRendUpdateClick(Sender: TObject);
begin
  if tvRenderers.Items.Count > 1 then
    if cbRenderers.ItemIndex < 2 then begin
      TDSpRasterRendererSimple(tvRenderers.Selected.Data).Range(StrToFloat(edRRTMin.Text), StrToFloat(edRRTMax.Text));
      MakeRamp;
    end;
  FViewer.SetRendererMap;
  FViewer.Invalidate;
end;

procedure TDataMgmtForm.imgBackgroundClick(Sender: TObject);
begin
  ColorDialog1.Color := FViewer.Color;
  if ColorDialog1.Execute then begin
    imgBackground.Canvas.Brush.Color := ColorDialog1.Color;
    imgBackground.Canvas.FillRect(imgBackground.ClientRect);
    FViewer.Color := ColorDialog1.Color;
  end;
end;

procedure TDataMgmtForm.FormActivate(Sender: TObject);
begin
  imgBackground.Canvas.Brush.Color := FViewer.Color;
  imgBackground.Canvas.FillRect(imgBackground.ClientRect);
end;

procedure TDataMgmtForm.dgRampMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var xy: TGridCoord;
begin
  xy := dgRamp.MouseCoord(X, Y);
  if xy.Y = 0 then Exit;
  dgRamp.Row := xy.Y;

  if Button = mbLeft then begin
    if xy.X = 0 then begin
      ColorDialog1.Color := TDSpRRTSmoothRamp(dgRamp.Tag).Colour[xy.Y-1];
      if ColorDialog1.Execute then begin
        TDSpRRTSmoothRamp(dgRamp.Tag).Colour[xy.Y-1] := ColorDialog1.Color;
        dgRamp.Repaint;
        MakeRamp;
      end
    end;
  end else begin
    pmRampDelete.Enabled := dgRamp.RowCount > 3;
    pmRamp.Popup(dgRamp.ClientOrigin.X + X, dgRamp.ClientOrigin.Y + Y);
  end;
end;

procedure TDataMgmtForm.dgRampDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  if ARow = 0 then begin
    if ACol = 0 then dgRamp.Canvas.TextRect(Rect, Rect.Left+2, Rect.Top+2, 'Col')
    else dgRamp.Canvas.TextRect(Rect, Rect.Left+2, Rect.Top+2, 'Value')
  end else begin
    if ACol = 0 then begin
      dgRamp.Canvas.Brush.Color := TDSpRRTSmoothRamp(dgRamp.Tag).Colour[ARow-1];
      dgRamp.Canvas.FillRect(Rect);
    end else begin
      dgRamp.Canvas.TextRect(Rect, rect.Left+2, rect.Top+2, FloatToStrF(TDSpRRTSmoothRamp(dgRamp.Tag).Value[ARow-1], ffGeneral, 10, 3));
    end;
  end;
end;

procedure TDataMgmtForm.dgRampGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: String);
begin
  if (ACol = 1) and (ARow > 0) then
    Value := FloatToStrF(TDSpRRTSmoothRamp(dgRamp.Tag).Value[ARow-1], ffGeneral, 10, 3);
end;

procedure TDataMgmtForm.dgRampSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: String);
var ramp: TDSpRRTSmoothRamp;
begin
  Assert(ARow > 0);
  if Value = '' then Exit;
  cmdRendUpdate.Enabled := true;
  try
    ramp := TDSpRRTSmoothRamp(dgRamp.Tag);
    ramp.Value[ARow-1] := StrToFloat(Value);
    if (ARow = 1) or (ARow = dgRamp.RowCount - 1) then
      TDSpRasterRendererSimple(tvRenderers.Selected.Data).Range(ramp.Value[0], ramp.Value[dgRamp.RowCount-2]);
  except
    cmdRendUpdate.Enabled := false;
  end;
end;

procedure TDataMgmtForm.InsertBreak(Sender: TObject);
var ramp: TDSpRRTSmoothRamp;
    ndx: integer;
begin
  ramp := TDSpRRTSmoothRamp(dgRamp.Tag);
  if (Sender as TMenuItem).Name = 'pmRampAbove' then ndx := dgRamp.Row - 1 else ndx := dgRamp.Row;
  ramp.Insert(ndx);
  if (ndx = 0) or (ndx = dgRamp.RowCount - 2) then
    TDSpRasterRendererSimple(tvRenderers.Selected.Data).Range(ramp.Value[0], ramp.Value[dgRamp.RowCount-2]);
  dgRamp.RowCount := ramp.Intervals + 1;
  dgRamp.Repaint;
  MakeRamp;
end;

procedure TDataMgmtForm.dgRampSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  CanSelect := ACol > 0;
end;

procedure TDataMgmtForm.pmRampDeleteClick(Sender: TObject);
begin
  if TDSpRRTSmoothRamp(dgRamp.Tag).Delete(dgRamp.Row - 1) then begin
    dgRamp.RowCount := TDSpRRTSmoothRamp(dgRamp.Tag).Intervals + 1;
    dgRamp.Repaint;
    MakeRamp;
  end;
end;

procedure TDataMgmtForm.pmRampInvertClick(Sender: TObject);
begin
  TDSpRRTSmoothRamp(dgRamp.Tag).Invert;
  dgRamp.Repaint;
  MakeRamp;
end;

procedure TDataMgmtForm.MakeRamp;
begin
  FRampBmp.Width := imgRamp.Width;
  FRampBmp.Height := imgRamp.Height;
  TDSpRRT(TDSpRasterRendererSimple(tvRenderers.Selected.Data).Transformation).Ramp(FRampBmp, dspVert);
  imgRamp.Picture.Bitmap := FRampBmp;
end;

procedure TDataMgmtForm.FormResize(Sender: TObject);
begin
  if (tvRenderers.Selected <> nil) and (TObject(tvRenderers.Selected.Data) is TDSpRasterRendererSimple)
     and Active then MakeRamp;
end;

procedure TDataMgmtForm.pmRendererSaveImageClick(Sender: TObject);
begin
  if SavePictureDialog1.Execute then
    TDSpRenderer(tvRenderers.Selected.Data).SaveImage(SavePictureDialog1.Files[0]);
end;

procedure TDataMgmtForm.pmDataSetsCloseClick(Sender: TObject);
var node, renderTop, renderNode: TTreeNode;
    i: integer;
    ds: TDSpDataSet;
begin
  // Find the data set
  node := tvDataSets.Selected;    // node holds a data set
  Assert(node.Level = 1);
  ds := TDSpDataSet(node.Data);

  // Instruct the renderers to release any of the bands being used
  if (ds is TDSpRaster) and (TDSpRaster(ds).Bands > 0) then begin
    renderTop := tvRenderers.Items[0];
    renderNode := renderTop.GetFirstChild;
    while renderNode <> nil do begin
      if TObject(renderNode.Data) is TDSpRasterRenderer then
        for i := 0 to TDSpRaster(ds).Bands - 1 do
          TDSpRasterRenderer(renderNode.Data).DeleteRasterBand(TDSpRaster(ds).Band[i]);
      renderNode := renderTop.GetNextChild(renderNode);
    end;
  end;
 
  // Delete the data set from the DataMgr and zap the in-memory object
  FDataMgr.CloseDataSet(ds);

  // Synch the tree views with the current data
  RefreshDataSetTree(tvDataSets);
  RefreshRendererTree(tvRenderers);
  mmInfo.Clear;
end;

end.
