{******************************************************************************}
{                                                                              }
{ Project DSpatial - Geospatial analysis for Delphi                            }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.}
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is DSpGraphics.pas. The Original Code is available through }
{ the home page of DSpatial at http://dspatial.sourceforge.net.                }
{                                                                              }
{ The Initial Developer of the Original Code is Patrick Van Laake              }
{ (pvanlaake@users.sourceforge.net). Portions created by the Initial Developer }
{ are Copyright (C) of the Initial Developer. Changes to the Original Code     }
{ available through DSpatial are Copyright (C) of the Contributing Authors.    }
{                                                                              }
{******************************************************************************}
{                                                                              }
{ This unit provides support for saving data as images in standard graphics    }
{ formats. This unit uses the polymorphic TPicture structure, so you can add   }
{ formats to your heart's content. This unit relies on the open source         }
{ PNGImage component.                                                          }
{                                                                              }
{ Unit owner    : Patrick Van Laake                                            }
{ Last modified : 2003 November 22                                             }
{                                                                              }
{ Dependencies  : Graphics, JPEG, PNGImage (SaveAsImage)                       }
{                 SysUtils (ExtractFileExt, Exception)                         }
{                                                                              }
{ Version       : 0.1  03Nov22 Initial coding.                                 }
{                                                                              }
{******************************************************************************}
unit DSpGraphics;
{$I DSpatial.inc}
interface
uses Graphics;

  procedure SaveAsImage(bmp: TBitmap; const fn: string);

implementation
uses SysUtils, JPEG, PNGImage;

procedure SaveAsImage(bmp: TBitmap; const fn: string);
var ext: string;
    png: TPNGObject;
    jpg: TJPEGImage;
begin
  ext := ExtractFileExt(fn);
  if ext = '.png' then begin
    png := TPNGObject.Create;
    try
      png.Assign(bmp);
      png.SaveToFile(fn);
    finally
      png.Free;
    end;
  end else if ext = '.jpg' then begin
    jpg := TJPEGImage.Create;
    try
      jpg.Assign(bmp);
      jpg.CompressionQuality := 80;
      jpg.Compress;
      jpg.SaveToFile(fn);
    finally
      jpg.Free;
    end;
  end else if ext = '.bmp' then bmp.SaveToFile(fn);
end;

end.
