{
   SA Editor Script Enhancements

   Copyright (C) 2001-2002 by S. A. Kryukov
   http://www.SAKryukov.org
}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
//{$D-,O+,Y-,L-}
unit SABinding;
interface

procedure Register;

implementation
//==============================================================================
uses Windows, Classes, SysUtils, ToolsAPI, Menus, Forms, Dialogs, Controls,
     ShortCuts, SAHelp;

type TMethodResult = (cmrNone, cmrNotAMethod, cmrNoName, cmrNoEndFound,
                      cmrAlreadyCommented, cmrCommented, cmrBodyAdded);

resourcestring
   STR_ENHANCEMENT_SAMACRO = 'SA Edit Macro';
   STR_BINDING_NAME = 'Borland.SAEditMacro'; // do not localize
resourcestring // formats:
   FORMAT_METHOD = #13'begin // %s'#13'end {%s};';
   FORMAT_MSGHANDLER = 'procedure %s(var Msg: tMessage); message %s;';
resourcestring // messages:
   MSG_NOBLOCK = ' no block marked';
   MSG_BLOCKCOMMENTED = ' block commented';
   MSG_BLOCKUNCOMMENTED = ' bloc uncommented';
   MSG_BLOCKHINT = ' (use Ctrl+BC to comment Ctrl+BU to uncomment block)';
   MSG_MESSAGEHANDLER = ' message handler created (type message ID (WM_XXX) and use Ctrl+S)';
   MSG_COPY       = ' block is saved in the local clipboard @%d';
   MSG_PASTE = ' block is restored from the local clipboard @%d';
const
   STR_COMMENT = '// ';
   STR_WMPREFIX = 'WM_';
   STR_METHODTYPE: array[0..5] of string = (
      'constructor', // longest comes first, it's a must!
      'destructor',
      'procedure',
      'class procedure',
      'class function',
      'function');
const
   MSG_COMMENTRESULT: array[TMethodResult] of string = ('',
      ' no method ("procedure" "function" "constructor" or "destructor") found',
      ' no method name found',
      ' no ending "end" found',
      ' already commented',
      ' method commented',
      ' method body added');
const
   MASK_COPY  = $0F;
   MASK_PASTE = $F0;

type
cSAMacros = class(TNotifierObject, {IUnknown, IOTANotifier, }IOTAKeyboardBinding)
private // local clipboard
   LocalClipboard: array [0..9] of string;
private // helpers:
   function  FMarkWord(const Context: IOTAKeyContext): IOTAEditBlock;
   procedure FCommentBlock(const Context: IOTAKeyContext; DoComment: boolean);
   function  FCommentMethod(const Context: IOTAKeyContext): TMethodResult;
   function  FLineAtPos(Context: IOTAKeyContext): string;
   function  FExtractIndentAndMethodNameAtPos(Context: IOTAKeyContext; out Name: string; out Indent: integer): TMethodResult;
private // bound procedures:
   procedure Help           (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure AddMethodBody  (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure CommentBlock   (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure UnCommentBlock (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure MessageHandler (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure CommentMethod  (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure CommentMethods (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   procedure LocalCopyPaste (const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
private // interface implementation
   function GetBindingType: TBindingType;
   function GetDisplayName: string;
   function GetName: string;
   procedure BindKeyboard(const BindingServices: IOTAKeyBindingServices);
end {cSAMacros};

procedure Register;
var
   m: cSAMacros;
   serv: IOTAKeyBoardServices;
begin // Register
   m := cSAMacros.Create;
   serv := BorlandIDEServices as IOTAKeyBoardServices;
   serv.AddKeyboardBinding(m);
end {Register};

//-- cSAMacros: ----------------------------------------------------------------
{ cSAMacros }

function cSAMacros.FMarkWord(const Context: IOTAKeyContext): IOTAEditBlock;
var
   EP: IOTAEditPosition;
   EB: IOTAEditBlock;
begin
   EP := Context.EditBuffer.EditPosition;
   EB := Context.EditBuffer.EditBlock;
   if EP.IsWordCharacter then
      EP.MoveCursor(mmSkipLeft or mmSkipWord);
   if EP.IsWhiteSpace then
      EP.MoveCursor(mmSkipRight or mmSkipWhite);
   if not EP.IsWhiteSpace then begin
      if not EP.IsWordCharacter then
         EP.MoveCursor(mmSkipRight or mmSkipNonWord);
      if EP.IsWordCharacter then begin
         EB.Reset;
         EB.Style := btNonInclusive;
         EB.BeginBlock;
         EP.MoveCursor(mmSkipRight or mmSkipWord);
         EB.EndBlock;
      end {ClassComplete};
   end {if};
   Result := EB;
end {cSAMacros.FMarkWord};

function cSAMacros.FLineAtPos(Context: IOTAKeyContext): string;
var
   block: IOTAEditBlock;
   pos: IOTAEditPosition;
begin
   block := Context.EditBuffer.Editblock;
   pos := Context.EditBuffer.EditPosition;
   block.Save;
   pos.Save;
   pos.MoveBOL;
   block.BeginBlock;
   pos.MoveEOL;
   block.EndBlock;
   Result := block.Text;
   pos.Restore;
   block.Restore;
end {cSAMacros.FLineAtPos};

function cSAMacros.FExtractIndentAndMethodNameAtPos(Context: IOTAKeyContext; out Name: string; out Indent: integer): TMethodResult;
var
   trimmed, ss: string;
   jj, typefound, apos, endpos: integer;
   endposcorrected: boolean;
   const stopchar: array[1..5] of char = (#9, ' ', '(', ';', ':');
begin
   Result := cmrNotAMethod;
   Name := '';
   ss := FLineAtPos(Context);
   trimmed := TrimLeft(ss);
   Indent := Length(ss)-Length(trimmed);
   typefound := -1;
   for jj := Low(STR_METHODTYPE) to High(STR_METHODTYPE) do begin
      if system.pos(STR_METHODTYPE[jj], trimmed)=1 then begin
         Delete(trimmed, 1, length(STR_METHODTYPE[jj]));
         trimmed := TrimLeft(trimmed);
         typefound := jj;
         break;
      end {if};
   end {if};
   if typefound<0 then exit;
   Name := trimmed;
   endpos := Length(Name);
   endposcorrected := false;
   for jj := Low(stopchar) to high(stopchar) do begin
      apos := system.pos(stopchar[jj], Name);
      if apos>0 then
         endposcorrected := true;
      if (apos>0) and (apos<endpos) then
         endpos := apos;
   end {loop};
   if endposcorrected then
      SetLength(name, endpos-1);
   if name='' then
      Result := cmrNoName
   else
      Result := cmrNone;
end {cSAMacros.ExtractIndentAndMethodNameAtPos};

procedure cSAMacros.FCommentBlock(const Context: IOTAKeyContext; DoComment: boolean); //SA???
var
   view: IOTAEditView;
   block: IOTAEditblock;
   pos: IOTAEditPosition;
   msg, ss: string;
   jj: integer;
   noBlock: boolean;
begin
   view := Context.EditBuffer.TopView;
   block := Context.EditBuffer.Editblock;
   pos := Context.EditBuffer.EditPosition;
   noBlock := block.Text='';
   if not noBlock then begin
      block.Save;
      pos.Save;
      for jj := block.StartingRow to block.EndingRow do begin
         pos.Gotoline(jj);
         if DoComment then begin
            pos.InsertText(STR_COMMENT);
         end else begin
            ss := pos.Read(Length(STR_COMMENT));
            if ss=STR_COMMENT then begin
               pos.MoveRelative(0, Length(STR_COMMENT));
               pos.BackspaceDelete(Length(STR_COMMENT)); // Delete(x), x>1 does not work!
            end {if};
         end {if};
      end {loop};
      pos.Restore;
      block.Restore;
   end {if};
   if noBlock then begin
      msg := MSG_NOBLOCK;
   end else if DoComment then begin
      msg := MSG_BLOCKCOMMENTED;
   end else begin
      msg := MSG_BLOCKUNCOMMENTED;
   end {if};
   msg := msg + MSG_BLOCKHINT;
   view.SetTempMsg(msg);
end {cSAMacros.FCommentBlock};

procedure cSAMacros.Help(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
begin
   TFormHelp.Help;
   BindingResult := krHandled;
end {cSAMacros.Help};

procedure cSAMacros.CommentBlock(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
begin
   FCommentBlock(Context, true);
   BindingResult := krHandled;
end {cSAMacros.CommentBlock};

procedure cSAMacros.UnCommentBlock(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
begin
   FCommentBlock(Context, false);
   BindingResult := krHandled;
end {cSAMacros.UnCommentBlock};

procedure cSAMacros.MessageHandler(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
   function ProcFromWM(const WM: string): string;
   begin
      Result := WM;
      if UpperCase(system.Copy(WM, 1, Length(STR_WMPREFIX)))=UpperCase(STR_WMPREFIX) then begin
         Result := system.Copy(WM, Length(STR_WMPREFIX)+1, Length(WM));
         Result := LowerCase(Result);
         if Length(Result)>0 then
            Result[1] := UpCase(Result[1]);
         Result := system.Copy(STR_WMPREFIX, 1, Length(STR_WMPREFIX)-1) + Result;
      end {if};
   end {ProcFromWM};
var
   block: IOTAEditblock;
   pos: IOTAEditPosition;
   msgID, procID: string;
begin
   pos := Context.EditBuffer.EditPosition;
   pos.Save;
   pos.MoveBOL;
   block := FMarkWord(Context);
   msgID := block.Text;
   procID := ProcFromWM(msgID);
   pos.InsertText(Format(FORMAT_MSGHANDLER, [procID, msgID]));
   pos.Restore;
   Context.EditBuffer.TopView.SetTempMsg(MSG_MESSAGEHANDLER);
   BindingResult := krHandled;
end {cSAMacros.MessageHandler};

procedure cSAMacros.AddMethodBody(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
var
   block: IOTAEditblock;
   pos: IOTAEditPosition;
   name: string;
   indent: integer;
   saveAutoIndent: boolean;
   res: TMethodResult;
begin
   res := FExtractIndentAndMethodNameAtPos(Context, name, indent);
   if res=cmrNone then res := cmrBodyAdded;
   if name<>'' then begin
      saveAutoIndent := Context.EditBuffer.BufferOptions.AutoIndent;
      Context.EditBuffer.BufferOptions.AutoIndent := true;
      block := Context.EditBuffer.Editblock;
      pos := Context.EditBuffer.EditPosition;
      block.Save;
      pos.Save;
      pos.MoveEOL;
      pos.InsertText(Format(FORMAT_METHOD,[name, name]));
      pos.Restore;
      block.Restore;
      Context.EditBuffer.BufferOptions.AutoIndent := saveAutoIndent;
   end {if};
   Context.EditBuffer.TopView.SetTempMsg(MSG_COMMENTRESULT[res]);
   BindingResult := krHandled;
end {cSAMacros.AddMethodBody};

function cSAMacros.FCommentMethod(const Context: IOTAKeyContext): TMethodResult;
var
   pos: IOTAEditPosition;
   block: IOTAEditblock;
   name, line, trimline: string;
   jj, lastrow, indent: integer;
   beginCommented, endCommented, beginFound, endFound: boolean;
begin
   pos := Context.EditBuffer.EditPosition;
   lastrow := pos.LastRow;
   block := Context.EditBuffer.Editblock;
   Result := FExtractIndentAndMethodNameAtPos(Context, name, indent);
   if Result=cmrNone then begin
      Result := cmrNoEndFound;
      block.Save;
      pos.Save;
      beginCommented := false;
      endCommented := false;
      beginFound := false;
      endFound := false;
      for jj := pos.Row+1 to lastrow do begin
         pos.GotoLine(jj);
         line := FLineAtPos(Context);
         trimline := TrimLeft(line);
         if Length(line)-Length(trimline)=indent then begin
            trimline := TrimRight(trimline);
            if system.pos('end', trimline)=1 then begin
               endFound := true;
               if trimline = 'end;' then begin
                  pos.MoveEOL;
                  pos.BackspaceDelete(1);
                  pos.InsertText(' {'+name+'};');
                  endCommented := true;
                  break;
               end {if must comment};  // else already commented:
               break;
            end else if (not beginCommented) and (system.pos('begin', trimline)=1) then begin
               beginFound := true;
               endFound := true;
               if trimline = 'begin' then begin
                  pos.MoveEOL;
                  pos.InsertText(' // '+name);
                  beginCommented := true;
               end {if};
            end {if};
         end {if right indent};
      end {loop};
      if beginFound or endFound then
         Result := cmrAlreadyCommented;
      if beginCommented or endCommented then
         Result := cmrCommented;
      pos.Restore;
      block.Restore;
   end {if name};
end {cSAMacros.FCommentMethod};

procedure cSAMacros.CommentMethod(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
begin
   Context.EditBuffer.TopView.SetTempMsg(MSG_COMMENTRESULT[FCommentMethod(Context)]);
   BindingResult := krHandled;
end {cSAMacros.CommentMethod};

procedure cSAMacros.CommentMethods(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
var
   block: IOTAEditblock;
   pos: IOTAEditPosition;
   jj: integer;
   res, finalres: TMethodResult;
begin
   block := Context.EditBuffer.Editblock;
   if block.Text='' then begin
      Context.EditBuffer.TopView.SetTempMsg(MSG_NOBLOCK);
   end else begin
      pos := Context.EditBuffer.EditPosition;
      block.Save;
      pos.Save;
      finalres := cmrNotAMethod;
      for jj := block.StartingRow to block.EndingRow do begin
         pos.GoToLine(jj);
         res := FCommentMethod(Context);
         if res>finalres then
            finalres := res;
      end {loop};
      pos.Restore;
      block.Restore;
      Context.EditBuffer.TopView.SetTempMsg(MSG_COMMENTRESULT[finalres]);
   end {if block defined};
   BindingResult := krHandled;
end {cSAMacros.CommentMethods};

procedure cSAMacros.LocalCopyPaste(const Context: IOTAKeyContext; KeyCode: TShortCut; var BindingResult: TKeyBindingResult);
var
   indbyte, index: Cardinal;
   doPaste: boolean;
   block: IOTAEditblock;
   pos: IOTAEditPosition;
   saveAutoIndent: boolean;
begin
   pos := Context.EditBuffer.EditPosition;
   block := Context.EditBuffer.Editblock;
   indbyte := Cardinal(Context.Context);
   index := indbyte and MASK_COPY;
   doPaste := (indbyte and MASK_PASTE)<>0;
   saveAutoIndent := Context.EditBuffer.BufferOptions.AutoIndent;
   Context.EditBuffer.BufferOptions.AutoIndent := false;
   if doPaste then begin
      block.BeginBlock;
      pos.InsertText(LocalClipboard[index]);
      block.EndBlock;
      Context.EditBuffer.TopView.SetTempMsg(Format(MSG_PASTE, [index]));
   end else begin
      LocalClipboard[index] := block.Text;
      Context.EditBuffer.TopView.SetTempMsg(Format(MSG_COPY, [index]));
   end {if};
   Context.EditBuffer.BufferOptions.AutoIndent := saveAutoIndent;
   BindingResult := krHandled;
end {cSAMacros.LocalCopyPaste};

//-- interface implementation: -------------------------------------------------

procedure cSAMacros.BindKeyboard(const BindingServices: IOTAKeyBindingServices);
var
   jj: integer;
begin
   BindingServices.AddKeyBinding([ShortCut(shortHelp, CommonPrefix)], Help, nil);
   BindingServices.AddKeyBinding([ShortCut(shortAddMethodBody, CommonPrefix)], AddMethodBody, nil);
   BindingServices.AddKeyBinding([ShortCut(shortMessageHandler, CommonPrefix)], MessageHandler, nil);
   BindingServices.AddKeyBinding([
                                     ShortCut(shortCommentBlock, CommonPrefix),
                                     ShortCut(shortCommentBlock2, CommonPrefix)
                                  ], CommentBlock, nil);
   BindingServices.AddKeyBinding([
                                     ShortCut(shortUncommentBlock, CommonPrefix),
                                     ShortCut(shortUncommentBlock2, CommonPrefix)
                                  ], UnCommentBlock, nil);
   BindingServices.AddKeyBinding([ShortCut(shortCommentMethod, CommonPrefix)], CommentMethod, nil);
   BindingServices.AddKeyBinding([ShortCut(ShortCommentMethods, CommonPrefix)], CommentMethods, nil);
   // local clipboard:
   for jj := Low(LocalClipboard) to High(LocalClipboard) do begin
      BindingServices.AddKeyBinding(
         [
            ShortCut(shortLocalCopy, CommonPrefix),
            ShortCut(Ord(IntToStr(jj)[1]), CommonPrefix)
         ], LocalCopyPaste, pointer(jj));
      BindingServices.AddKeyBinding(
         [
            ShortCut(shortLocalPaste, CommonPrefix),
            ShortCut(Ord(IntToStr(jj)[1]), CommonPrefix)
         ], LocalCopyPaste, pointer(jj or MASK_PASTE));
   end {loop};
end {cSAMacros.BindKeyboard};

function cSAMacros.GetBindingType: TBindingType;
begin
   Result := btPartial;
end {cSAMacros.GetBindingType};

function cSAMacros.GetDisplayName: string;
begin
   Result := STR_ENHANCEMENT_SAMACRO;
end {cSAMacros.GetDisplayName};

function cSAMacros.GetName: string;
begin
   Result := STR_BINDING_NAME;  //do not localize
end {cSAMacros.GetName};

end.
