{*******************************************************}
{                                                       }
{         Delphi Volume Control Library v.1             }
{                                                       }
{         Copyright (c) 2003 archie@emapa.pl            }
{                                                       }
{*******************************************************}

unit volctrl;

interface

uses
  MMSystem, //whole mm interface
  SysUtils; //exceptions creating

type
  TVolumeInformation = record
    Name: String;
    Desc: String;
    MinValue: Integer;
    MaxValue: Integer;
    Volume: Integer;
  end;
  TVolumeInformations = array of TVolumeInformation;

  function VolumeInformation(NewVolume: Integer = -1): TVolumeInformations;
  function GetVolumeInformation: TVolumeInformation; //returns volume of the first sound card
  procedure SetVolume(Value: Integer); //sets volume of all sound cards

implementation

function VolumeInformation(NewVolume: Integer = -1): TVolumeInformations;
var
  I, M: Integer;
  E: Integer;
  MC: TMixerControl;
  ML: TMixerLine;
  MLC: TMixerLineControls;
  Volume: LongWord;
  MCD: TMixerControlDetails;
  MCD_LT: TMixerControlDetailsListText;
begin
  SetLength(Result, 0);
  //will change volume for all cards in the system
  for I:=0 to mixerGetNumDevs-1 do
  begin
    //here goes callbacks (events) instead 0,0 if u want to monitor the peak
    E := mixerOpen(@M, I, 0, 0, 0);
    if (E = MMSYSERR_NOERROR) then
    begin
      FillChar(ML, SizeOf(ML), 0);
      ML.cbStruct := sizeof(ML);
      ML.dwComponentType := MIXERLINE_COMPONENTTYPE_DST_SPEAKERS; //master volume

      E := mixerGetLineInfo(M, @ML, MIXER_GETLINEINFOF_COMPONENTTYPE);
      if (E = MMSYSERR_NOERROR) then
      begin
        SetLength(Result, I+1);
        Result[I].Name := ML.szShortName;
        Result[I].Desc := ML.szName;
        FillChar(MLC, SizeOf(MLC), 0);
        MLC.cbStruct := sizeof(MLC);
        MLC.dwLineID := ML.dwLineID;
        MLC.dwControlType := MIXERCONTROL_CONTROLTYPE_VOLUME;
        MLC.cControls := 1;

        FillChar(MC, SizeOf(MC), 0);
        MC.cbStruct := sizeof(MC);

        MLC.cbmxctrl := sizeof(MC);
        MLC.pamxctrl := @MC;

        E := mixerGetLineControls(M, @MLC, MIXER_GETLINECONTROLSF_ONEBYTYPE);
        if (E = MMSYSERR_NOERROR) then
        begin
          Result[I].MinValue := MC.Bounds.lMinimum;
          Result[I].MaxValue := MC.Bounds.lMaximum;

          FillChar(MCD, SizeOf(MCD), 0);
          MCD.cbStruct := SizeOf(MCD);
          MCD.dwControlID := MC.dwControlID;

          FillChar(MCD_LT, SizeOf(MCD_LT), 0);

          MCD.cbDetails := SizeOf(MCD_LT);
          MCD.paDetails := @MCD_LT;
          MCD.cChannels := 1;

          mixerGetControlDetails(M, @MCD, MIXER_GETCONTROLDETAILSF_VALUE);

          if NewVolume <> -1 then
          begin
            MCD_LT.dwParam1 := NewVolume;
            mixerSetControlDetails(M, @MCD, MIXER_GETCONTROLDETAILSF_VALUE);
          end;
          Volume := Abs(MCD_LT.dwParam1);
          Result[I].Volume := Volume;
        end
        else
          raise Exception.Create('Error in mixerGetLineControls');
      end
      else
        raise Exception.Create('Error in mixerGetLineInfo');
      mixerClose(M);
    end
    else
      raise Exception.Create('Error in mixerOpen');
  end;
end;

procedure SetVolume(Value: Integer);
begin
  VolumeInformation(Value);
end;

function GetVolumeInformation: TVolumeInformation;
var
  VI: TVolumeInformations;
begin
  VI := VolumeInformation;
  if Length(VI) > 0 then
    Result := VI[0]
  else
    raise Exception.Create('No audio card detected');
end;

end.
