//*****************************************************************************
//
//    System          : jmsSMQ
//    Category        : API for SonicMQ
//    Source          : based on SonicMQ C Client header hand translation
//
//    Copyright (c), Leonov Alexander
//
//    File version $Revision:1.0$ at $Date:24.11.2003 13:40:36$
//
//    Author : Leonov Alexander
//    web    : http://home.ural.ru/~tigra/
//    e-mail : shura@ol405.paco.net
//
//*****************************************************************************

unit apiSonicMQ;

interface

Type
  HOBJ = pointer;
  jboolean = byte;
  jbyte = shortint;
  jchar = word;
  jshort = smallint;
  jfloat = single;
  jdouble = double;
  jint = longint;
  jlong = int64;
  TRetVal = integer;

(*
//---------------------------------
// Signatures for callback functions
//---------------------------------
*)
Type
  pfnCExceptionListener = procedure (hobj : HOBJ); cdecl;
  pfnCMessageListener = procedure (msg : HOBJ); cdecl;

Const
  jfalse = 0;
  jtrue  = 1;

Const
  SMQ_SUCCESS   = 0;
  SMQ_ERROR     = -1;
  
  Message_DEFAULT_DELIVERY_MODE                         = 2;
  Message_DEFAULT_PRIORITY                              = 4;
  Message_DEFAULT_TIME_TO_LIVE                          = 0;
  
  Session_AUTO_ACKNOWLEDGE                              = 1;
  Session_CLIENT_ACKNOWLEDGE                            = 2;
  Session_DUPS_OK_ACKNOWLEDGE                           = 3;
  Session_SINGLE_MESSAGE_ACKNOWLEDGE                    = 1004;

  DeliveryMode_NON_PERSISTENT                           = 1;
  DeliveryMode_PERSISTENT                               = 2;
  DeliveryMode_NON_PERSISTENT_ASYNC                     = 102;
  DeliveryMode_DISCARDABLE                              = 103;

(**
 * Returns the last error that occurred in the current thread.
 *)
function SMQ_getLastError : jint; cdecl;

(**
 * Returns a String object containing the text, if any, of the last
 * error that occurred. The caller is responsible for releasing
 * the String object by calling String_release.
 *)
function SMQ_getLastErrorText : HOBJ; cdecl;

(*
//---------------------------------
// Object
//---------------------------------
*)

(**
 * Returns the int corresponding to the Object type.
 *
 * @return the int corresponding to the Object type
 *)
function Object_type : TRetVal; cdecl;

(**
 * Decrements the reference count of the Object.
 *
 * @param thisObj the object to operate against.
 * @param pret will contain the reference count after decrementing.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Object_release(thisObj : HOBJ; var ret : jlong) : TRetVal; cdecl;

(**
 * Get the int corresponding to the type of a given object.
 *
 * @param thisObj the object to operate against.
 * @param pret will contain the int corresponding to the type.
 *)
function Object_getType(thisObj : HOBJ; var ret : integer) : TRetVal; cdecl;

(**
 * Determines whether the object is an instance of the given type.
 *
 * @param thisObj the object to check.
 * @param classtype the type to check against.
 * @param pret non-zero if the object is an instance of the type.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Object_instanceof(thisObj : HOBJ; classtype : integer; var pret : integer) : TRetVal; cdecl;

(**
 * Determines whether the two Objects have the same content.
 *
 * @param thisObj the first object to compare.
 * @param hobj2 the second object to compare.
 * @param pret true if the Objects have identical content, false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Object_equals(obj1, obj2 : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Get the String representation for this object.
 *
 * @param thisObj the object to operate against.
 * @param pret will contain the String that represents the object.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Object_toString(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new Object that is a copy of the given Object.
 *
 * @param thisObj the object to clone.
 * @param pret will contain the newly created Object.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Object_clone(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// String
//---------------------------------
*)

{(**
 * Determines whether the two String objects refer to the same string.
 *
 * @param stringObj the first String to compare.
 * @param hobj2 the second String to compare.
 * @param pret true if the String objects have identical content, false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
#define String_equals(stringObj, hobj2, pret)          Object_equals(stringObj, hobj2, pret)

(**
 * Get the String representation for the given String.
 *
 * @param stringObj the object to operate against.
 * @param pret will contain the String that represents the String.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
#define String_toString(stringObj, pret)               Object_toString(stringObj, pret)

(*TODO remove unimplemented methods*)

(**
 * Allocates a new string that contains the same
 * sequence of characters as the string argument.
 *
 * @param pString String to clone from
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)

(** 
 * Allocates a new String containing no characters.
 *
 * @param pString the new String (which contains no characters)
 *)
function String_create(HOBJ *pString) : TRetVal; cdecl;}
(**
 * Allocates a new String so that it represents the
 * sequence of ASCII characters that are pointed to by
 * the argument c
 *
 * @param c the incoming char pointer
 * @param pString the new String that was created
 *)
function String_create2 (c : PChar; var pString : HOBJ) : TRetVal; cdecl;

{(**
 * Allocates a new string that contains the same
 * sequence of characters as the string argument.
 *
 * @param inString incoming String to clone.
 * @param pString the new String that was created
 *)
function String_create3(HOBJ inString, HOBJ *pString) : TRetVal; cdecl;

(**
 * Allocates a new String that contains jchar 
 * from a subarray of the jchar array argument.
 * The offset argument is the index of the first
 * jchar of the subarray and the count argument
 * specifies the length of the subarray.
 *
 * @param jcharArr array that is the source of jchar. 
 * @param offset the initial offset. 
 * @param count the length.
 * @param pString the newly created String
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *            StringIndexOutOfBoundsException if the offset and
 *            count arguments index characters outside the bounds
 *            of the array.          
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_create4(HOBJ jcharArr, jint offset, jint count, HOBJ *pString) : TRetVal; cdecl;

(**
* Allocates a new String constructed from a subarray of an array of 8-bit
 * integer values. 
 * The offset argument is the index of the first byte of the subarray, 
 * and the count argument specifies the length of the subarray. 
 * Each byte in the subarray is converted to a char as specified in the 
 * method above.
 *
 * Note: String() is deprecated. This method does not properly convert bytes 
 * into characters. The preferred way to do this is via the 
 * String constructors that take a character-encoding name or that use the
 * platform's default encoding.

 *
 * @param ascii the bytes to be converted to characters. 
 * @param hibyte the top 8 bits of each 16-bit Unicode character.
 * @param offset the initial offset.
 * @param count the length.
 * @param pString the newly created String
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *            StringIndexOutOfBoundsException if the offset and
 *            count arguments index characters outside the bounds
 *            of the array.          
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_create5(HOBJ ascii, jint hibyte, jint offset, jint count, HOBJ *pString) : TRetVal; cdecl;

(**
 * Construct a new String by converting the specified subarray of bytes
 * using the specified character encoding. The length of the new String 
 * is a function of the encoding, and hence may not be equal to the length
 * of the subarray.
 * 
 * @param bytes The bytes to be converted into characters 
 * @param offset Index of the first byte to convert 
 * @param length Number of bytes to convert 
 * @param enc The name of a character encoding 
 * @param pString the newly created String
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *            UnsupportedEncodingException if the named encoding is not supported
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_create6(HOBJ bytes, jint offset, jint length, HOBJ enc, HOBJ *pString) : TRetVal; cdecl;

(**
 * Construct a new String by converting the specified subarray of bytes
 * using the platform's default character encoding. The length of the new
 * String is a function of the encoding, and hence may not be equal to the
 * length of the subarray.
 *
 * @param bytes The bytes to be converted into characters 
 * @param offset Index of the first byte to convert
 * @param length Number of bytes to convert 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_create7(HOBJ bytes, jint offset, jint length, HOBJ *pString) : TRetVal; cdecl;

(**
 * Allocates a new String so that it represents the
 * sequence of wide characters that are pointed to by
 * the argument c 
 *
 * @param c the incoming wchar_t pointer 
 * @param pString the new String that was created
 *)
function String_create8(wchar_t *c, HOBJ *pString) : TRetVal; cdecl;
}
(**
 * Returns the int corresponding to the String type.
 *
 * @return the int corresponding to the String type
 *)
function String_type : TRetVal; cdecl;

{(**
 * Returns the length of this string. The length 
 * is equal to the number of jchar(s) string.
 *
 * @param stringObj the String object to operate against
 * @param pLength the length of the sequence of jchars in the String. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_length(HOBJ stringObj, jint *plength) : TRetVal; cdecl;

(**
 * Returns the jchar at the specified index. An index 
 * ranges from 0 to length() - 1. The first character is at index 0.
 *
 * @param stringObj the String object to operate against
 * @param index the index of the jchar.
 * @param pJchar the jchar at the specified index of this string.
 * 
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_charAt(HOBJ stringObj, jint index, jchar *pJchar) : TRetVal; cdecl;

(**
 * Copies characters from this string into the destination
 * character array. The first character to be copied is at
 * index srcBegin; the last character to be copied is at
 * index srcEnd-1 (thus the total number of characters to
 * be copied is srcEnd-srcBegin). The characters are copied
 * into the subarray of dst starting at index dstBegin and
 * ending at index: 
 *
 *    dstbegin + (srcEnd-srcBegin) - 1
 *
 *
 * @param stringObj the String object to operate against
 * @param srcBegin index of the first character in the string to copy. 
 * @param srcEnd index after the last character in the string to copy. 
 * @param dst the destination array. 
 * @dstBegin the start offset in the destination array. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *            StringIndexOutOfBoundsException if srcBegin or srcEnd is out
 *            of range, or if srcBegin is greater than the srcEnd.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_getChars(HOBJ stringObj, jint srcBegin, jint srcEnd, HOBJ dst, jint dstBegin) : TRetVal; cdecl;

(**
*
 * Copies characters from this string into the destination byte array.
 * Each byte receives the 8 low-order bits of the corresponding
 * character. 
 *
 * The first character to be copied is at index srcBegin;
 * the last character to be copied is at index srcEnd-1.
 * The total number of characters to be copied is srcEnd-srcBegin.
 * The characters, converted to bytes, are copied into the subarray
 * of dst starting at index dstBegin and ending at index: 
* Note: getBytes() is deprecated. This method does not properly
 * convert characters into bytes. The preferred
 * way to do this is via the getBytes(String enc) method,
 * which takes a character-encoding name, or the getBytes() method,
 * which uses the platform's default encoding.
 *
 *   dstbegin + (srcEnd-srcBegin) - 1
 *
 * @param stringObj the String object to operate against
 * @param srcBegin index of the first character in the string to copy. 
 * @param srcEnd index after the last character in the string to copy. 
 * @param dst the destination array.
 * @param dstBegin the start offset in the destination array. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *            StringIndexOutOfBoundsException if srcBegin or srcEnd is out
 *            of range, or if srcBegin is greater than the srcEnd.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_getBytes(HOBJ stringObj, jint srcBegin, jint srcEnd, HOBJ dst, jint dstBegin) : TRetVal; cdecl;

(**
 * Convert this String into bytes according to the specified character
 * encoding, storing the result into a new byte array.
 * 
 * @param enc A character-encoding name 
 * @param pJbyteArr the resultant byte array
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_getBytes2(HOBJ stringObj, HOBJ enc, HOBJ *pJbyteArr) : TRetVal; cdecl;

(** Convert this String into bytes according to the platform's default
 * character encoding, storing the result into a new byte array.
 *
 * @param stringObj the String object to operate against
 * @param pJbytesArr the resultant byte array.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_getBytes3(HOBJ stringObj, HOBJ *pJbytesArr) : TRetVal; cdecl;

(**
 * Determines whether the two String objects refer to the same string ignoring case differences.
 *
 * @param stringObj the first String to compare.
 * @param anotherString the second String to compare.
 * @param pret true if the String objects have identical content, false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_equalsIgnoreCase(HOBJ stringObj, HOBJ anotherString, jboolean *pret) : TRetVal; cdecl;

(**
 * Compares two strings lexicographically. The comparison is based on the 
 * Unicode value of each character in the strings.
 *
 * @param o the String to be compared. 
 *
 * @param pret returns the value 0 if the argument string is equal to this string;
 * a value less than 0 if this string is lexicographically less than 
 * the string argument; and a value greater than 0 if this string is 
 * lexicographically greater than the string argument. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_compareTo(HOBJ stringObj, HOBJ anotherString, jint *pret) : TRetVal; cdecl;

(**
 * Compares two strings lexicographically. The comparison is based on the
 * Unicode value of each character in the strings.
 *
 * @param stringObj the String object to operate against
 * @param anotherString the String to be compared. 
 * @param pret returns the value 0 if the argument string is equal to this string; 
 * a value less than 0 if this string is lexicographically less than the
 * string argument; and a value greater than 0 if this string is 
 * lexicographically greater than the string argument. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_compareToIgnoreCase(HOBJ stringObj, HOBJ str, jint *pret) : TRetVal; cdecl;

function String_regionMatches(HOBJ stringObj, jint toffset, HOBJ other, jint ooffset, jint len, jboolean *pret) : TRetVal; cdecl;

(**
 * Tests if two string regions are equal. If toffset or ooffset is negative,
 * or if toffset+length is greater than the length of this string, or if 
 * ooffset+length is greater than the length of the string argument, 
 * then this method returns false.
 *
 * @param stringObj the String object to operate against
 * @param ignoreCase if true, ignore case when comparing characters. 
 * @param toffset the starting offset of the subregion in this string. 
 * @param other the string argument. 
 * @param ooffset the starting offset of the subregion in the string argument. 
 * @param len the number of characters to compare.
 *
 * @param pret return true if the specified subregion of this string matches the
 * specified subregion of the string argument; false otherwise. Whether
 * the matching is exact or case insensitive depends on the ignoreCase 
 * argument.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_regionMatches2(HOBJ stringObj, jboolean ignoreCase, jint toffset, HOBJ other, jint ooffset, jint len, jboolean *pret) : TRetVal; cdecl;

(**
 * Tests if this string starts with the specified prefix.
 *
 * @param stringObj the String object to operate against
 * @param prefix the prefix. 
 * @param toffset where to begin looking in the string. 
 * @param pStartsWith returns true if the character sequence represented by
 *        the argument is a prefix of the substring of this object starting
 *        at index toffset; false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_startsWith(HOBJ stringObj, HOBJ prefix, jint toffset, jboolean *pStartsWith) : TRetVal; cdecl;

(**
 * Tests if this string starts with the specified prefix.
 *
 * @param stringObj the String object to operate against
 * @param prefix a String containing the prefix. 
 * @param pStartsWith returns true if the character sequence represented by
 *        the argument is a prefix of the substring of this object starting
 *        at index toffset; false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_startsWith2(HOBJ stringObj, HOBJ prefix, jboolean *pret) : TRetVal; cdecl;

(**
 * Tests if this string ends with the specified suffix.
 *
 * @param stringObj the String object to operate against
 * @param suffix a String containing the suffix.
 *
 * @param pret true if the character sequence represented by the argument
 * is a suffix of the character sequence represented by this object;
 * false otherwise.

 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_endsWith(HOBJ stringObj, HOBJ suffix, jboolean *pret) : TRetVal; cdecl;

(**
 * Returns the index within this string of the first occurrence
 * of the specified character.
 * 
 * @param stringObj the String object to operate against
 * @param ch a character. 
 * @param pIndex the index of the first occurrence of the character in
 * the character sequence represented by this object, or -1 if 
 * the character does not occur. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_indexOf(HOBJ stringObj, jint ch, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the first occurrence of 
 * the specified character, starting the search at the specified
 * index.
 *
 * @param stringObj the String object to operate against
 * @param ch a character.
 * @param fromIndex the index to start the search from. 
 * @param pIndex the index of the first occurrence of the character
 * in the character sequence represented by this object that 
 * is greater than or equal to fromIndex, or -1 if the character 
 * does not occur.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_indexOf2(HOBJ stringObj, jint ch, jint fromIndex, jint *pIndex) : TRetVal; cdecl;
(**
 * Returns the index within this string of the first occurrence of
 * the specified substring.
 *
 * @param stringObj the String object to operate against.
 * @param str any String.
 * @param pIndex if the string argument occurs as a substring within
 * this object, then the index of the first character of
 * the first such substring is returned; if it does not occur
 * as a substring, -1 is returned.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_indexOf3(HOBJ stringObj, HOBJ str, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the first occurrence of the
 * specified substring, starting at the specified index.
 *
 * @param stringObj the String object to operate against.
 * @param str the substring to search for. 
 * @param fromIndex  the index to start the search from. 
 *
 * @param pIndex if the string argument occurs as a substring within
 * this object at a starting index no smaller than fromIndex,
 * then the index of the first character of the first such substring
 * is returned. If it does not occur as a substring starting at fromIndex
 * or beyond, -1 is returned.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_indexOf4(HOBJ stringObj, HOBJ str, jint fromIndex, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the last occurrence 
 * of the specified character. The String is searched 
 * backwards starting at the last character.
 *
 * @param stringObj the String object to operate against
 * @param ch a character. 
 * @param pIndex the index of the last occurrence of the 
 * character in the character sequence represented by this 
 * object, or -1 if the character does not occur.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_lastIndexOf(HOBJ stringObj, jint ch, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the last occurrence
 * of the specified character, searching backward starting at the
 * specified index.
 * 
 * @param stringObj the String object to operate against
 * @param ch a character.
 * @param fromIndex the index to start the search from. 
 * @param pIndex the index of the last occurrence of the character 
 * in the character sequence represented by this object that 
 * is less than or equal to fromIndex, or -1 if the character
 * does not occur before that point.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_lastIndexOf2(HOBJ stringObj, jint ch, jint fromIndex, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the rightmost
 * occurrence of the specified substring. The rightmost empty
 * string "" is considered to occur at the index value this.length().
 *
 * @param stringObj the String object to operate against
 * @param str the substring to search for. 
 * @param pIndex if the string argument occurs one or more times
 * as a substring within this object, then the index of the 
 * first character of the last such substring is returned.
 * If it does not occur as a substring, -1 is returned.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_lastIndexOf3(HOBJ stringObj, HOBJ str, jint *pIndex) : TRetVal; cdecl;

(**
 * Returns the index within this string of the last occurrence
 * of the specified substring. The returned index indicates the
 * start of the substring, and it must be equal to or less than
 * fromIndex.
 *
 * @param stringObj the String object to operate against
 * @param str the substring to search for. 
 * @param fromIndex the index to start the search from. 
 *
 * @param pIndex if the string argument occurs one or more times
 * as a substring within this object at a starting index no
 * greater than fromIndex, then the index of the first character
 * of the last such substring is returned. If it does not occur
 * as a substring starting at fromIndex or earlier, -1 is returned. 
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_lastIndexOf4(HOBJ stringObj, HOBJ str, jint fromIndex, jint *pIndex) : TRetVal; cdecl;


(**
 * Returns a new string that is a substring of this string.
 * The substring begins at the specified index and extends
 * to the end of this string.
 * 
 * @param stringObj the String object to operate against
 * @param beginIndex the beginning index, inclusive. 
 * @param pString the specified substring.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *           StringIndexOutOfBoundsException if the beginIndex is out of range.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_substring(HOBJ stringObj, jint beginIndex, HOBJ *pString) : TRetVal; cdecl;

(**
 * Returns a new string that is a substring of this string. 
 * The substring begins at the specified beginIndex and extends
 * to the character at index endIndex - 1.
 *
 * @param stringObj the String object to operate against
 * @param beginIndex the beginning index, inclusive.
 * @param endIndex the ending index, exclusive. 
 * @param pString the specified substring.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *           StringIndexOutOfBoundsException if the beginIndex or endIndex
 *           is out of range.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_substring2(HOBJ stringObj, jint beginIndex, jint endIndex, HOBJ *pString) : TRetVal; cdecl;

(**
 * Concatenates the specified string to the end of this string.
 * If the length of the argument string is 0, then this object is
 * returned.
 *
 * @param stringObj the String object to operate against
 * @param str the String that is concatenated to the end of this String. 
 * @param pString a string that represents the concatenation of this object's 
 * characters followed by the string argument's characters
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_concat(HOBJ stringObj, HOBJ str, HOBJ *pString) : TRetVal; cdecl;

(**
 * Replaces all occurrences of a given character in the String w/ another character.
 *
 * @param stringObj the String object to operate against.
 * @param oldChar the old character.
 * @param newChar the new character.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_replace(HOBJ stringObj, jchar oldChar, jchar newChar) : TRetVal; cdecl;

(**
 * Converts all of the characters in this String to lower case.
 *
 * @param stringObj the String object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_toLowerCase(HOBJ stringObj) : TRetVal; cdecl;

(**
 * Converts all of the characters in this String to upper case.
 *
 * @param stringObj the String object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_toUpperCase(HOBJ stringObj) : TRetVal; cdecl;

(**
 * Removes white space from both ends of this String.
 *
 * @param stringObj the String object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_trim(HOBJ stringObj) : TRetVal; cdecl;

(**
 * Converts this String to a new jcharArray.
 *
 * @param stringObj the String object to operate against.
 * @param pret will contain the new jcharArray.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_toCharArray(HOBJ stringObj, HOBJ *pret) : TRetVal; cdecl;
}
(**
 * Converts this String to an AString (ASCII string).
 *
 * @param stringObj the String object to operate against.
 * @param pret the new AString.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_toAscii(stringObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

{(**
 * Converts this String to a WString (wide character string).
 *
 * @param stringObj the String object to operate against.
 * @param pret the new WString.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_toWide(HOBJ stringObj, HOBJ *pret) : TRetVal; cdecl;

(**
 * Determines if all jchars are in the non-extended char set (0-127).
 *
 * @param stringObj the String object to operate against.
 * @param pret true if all jchars are in the non-extended char set, false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_isAscii(HOBJ stringObj, jboolean *pret) : TRetVal; cdecl;

(**
 * Returns a canonical representation for the string object. 
 * It follows that for any two strings s and t, s.intern() == t.intern() is true if and 
 * only if s.equals(t) is true.
 *
 * @param stringObj the String object to operate against.
 * @param pret the String containing the canonical representation.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_intern(HOBJ stringObj, HOBJ *pret) : TRetVal; cdecl;

(**
 * Assigns the String represented by str to the String represented by stringObj.
 *
 * @param stringObj the String object to assign to.
 * @param str the String to assign from.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_assign(HOBJ stringObj, HOBJ str) : TRetVal; cdecl;

(**
 * Appends a subarray of a jcharArray to the end of this String.
 *
 * @param stringObj the String object to append to.
 * @param str the jcharArray containing the characters to append.
 * @param offset the index of the first character to append.
 * @param len the number of characters to append.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_append(HOBJ stringObj, HOBJ str, jint offset, jint len) : TRetVal; cdecl;

(**
 * Appends a jchar to the end of this String.
 *
 * @param stringObj the String object to append to.
 * @param c the character to append.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_append2(HOBJ stringObj, jchar c) : TRetVal; cdecl;

(**
 * Appends a String to the end of this String.
 *
 * @param stringObj the String object to append to.
 * @param s the String to append.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_append3(HOBJ stringObj, HOBJ s) : TRetVal; cdecl;

(**
 * Returns the string representation of the obj argument.
 *
 * @param obj the object to operate against.
 * @param pret will contain the String representation of the obj.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOf(HOBJ obj, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string corresponding to a subarray of a jcharArray.
 *
 * @param data the jcharArray containing the characters to return as a String.
 * @param offset the index of the first character to be part of the String.
 * @param count the number of characters to be part of the String.
 * @param pret will contain the new String.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOf2(HOBJ data, jint offset, jint count, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jboolean argument.
 *
 * @param b a boolean.
 * @param pret will contain the String representation of b.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfBoolean(jboolean b, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jbyte argument.
 *
 * @param b a jbyte.
 * @param pret will contain the String representation of b.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfByte(jbyte b, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jchar argument.
 *
 * @param c a jchar.
 * @param pret will contain the String representation of c.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfChar(jchar c, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jshort argument.
 *
 * @param s a jshort.
 * @param pret will contain the String representation of s.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfShort(jshort s, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jint argument.
 *
 * @param i a jint.
 * @param pret will contain the String representation of i.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfInt(jint i, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jlong argument.
 *
 * @param l a jlong.
 * @param pret will contain the String representation of l.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfLong(jlong l, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jfloat argument.
 *
 * @param f a jfloat.
 * @param pret will contain the String representation of f.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfFloat(jfloat f, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns the string representation of the jdouble argument.
 *
 * @param d a jdouble.
 * @param pret will contain the String representation of d.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_valueOfDouble(jdouble d, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns a String that is equivalent to the specified character array. It creates a new 
 * array and copies the characters into it.
 *
 * @param data the jcharArray.
 * @param offset initial offset of the subarray.
 * @param count length of the subarray.
 * @param pret will contain a String w/ the specified subarray of the jcharArray.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_copyValueOf(HOBJ data, jint offset, jint count, HOBJ *pret) : TRetVal; cdecl;

(**
 * Returns a String that is equivalent to the specified character array. It creates a new 
 * array and copies the characters into it.
 *
 * @param data the jcharArray.
 * @param pret will contain a String w/ the characters of the jcharArray.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function String_copyValueOf2(HOBJ data, HOBJ *pret) : TRetVal; cdecl;
}

(*
//---------------------------------
// AString
//---------------------------------
*)
{(**
 * Determines whether the two AStrings refer to the same strings.
 *
 * @param thisObj the first object to compare.
 * @param hobj2 the second object to compare.
 * @param pret true if the AStrings refer to identical strings, false otherwise.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
#define AString_equals(thisObj, hobj2, pret)          Object_equals(thisObj, hobj2, pret)

(**
 * Get the String representation for this object.
 *
 * @param thisObj the object to operate against.
 * @param pret will contain the String that represents the object.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
#define AString_toString(thisObj, pret)               Object_toString(thisObj, pret)
}
(**
 * Returns the int corresponding to the AString type.
 *
 * @return the int corresponding to the AString type
 *)
function AString_type : TRetVal; cdecl;

(**
 * Retrieve the const char* corresponding to the value of the AString.
 *
 * @param thisObj the AString object to operate against.
 * @param pret will contain the const char* corresponding to the AString value.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function AString_getValue(thisObj : HOBJ; var pret : PChar) : TRetVal; cdecl;

(**
 * Get the length of the AString.
 *
 * @param thisObj the AString to operate against.
 * @param pret will contain the length of the AString in characters.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function AString_length(thisObj : HOBJ; var pret : jint) : integer; cdecl;

{(**
 * Get the char at a particular location in the AString. Invalid indices will trigger an error.
 *
 * @param thisObj the AString to operate against.
 * @param index the zero-based index of the character to retrieve.
 * @param pret will contain the char at the designated location in the AString.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function AString_at(thisObj : HOBJ; jint index, char *pret) : TRetVal; cdecl;
}

(*
//---------------------------------
// Message
//---------------------------------
*)

(**
 * Returns the int corresponding to the Message type.
 *
 * @return the int corresponding to the Message type
 *)
function Message_type : TRetVal; cdecl;

(** Get the message ID.
  *
  * <P>The messageID header field contains a value that uniquely
  * identifies each message sent by a SonicMQ client.
  *
  * <P>When a message is sent, messageID can be ignored. When
  * the send method returns it contains an assigned value.
  *
  * <P>A JMSMessageID is a String value which should function as a
  * unique key for identifying messages in a historical repository.
  *
  * <P>All JMSMessageID values must start with the prefix 'ID:'.
  * Uniqueness of message ID values across different brokers is
  * not required.
  *
  * <P>Since message ID's take some effort to create and increase a
  * message's size, SonicMQ can optimize message
  * overhead if it is given a hint that message ID is not used by
  * an application. SonicMQ message Producers provide a hint to disable
  * message ID. When a client sets a Producer to disable message ID
  * they are saying that they do not depend on the value of message
  * ID for the messages it produces. These messages must either have
  * message ID set to null or, if the hint is ignored, messageID must
  * be set to its normal unique value.
  *
  * @param msgObj the Message to operate against
  * @param pMsgId the returned message ID as a String
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSMessageID()
  *)
function Message_getJMSMessageID(msgObj : HOBJ; var pmsgId : HOBJ) : TRetVal; cdecl;

(** Set the message ID.
  *
  * <P>The SonicMQ run time sets this field when a message is sent. This operation
  * can be used to change the value of a message that has been received.
  *
  * @param msgObj the Message to operate against
  * @param msgId the String to set the message ID to
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_getJMSMessageID()
  *)
function Message_setJMSMessageID(msgObj : HOBJ; msgId : HOBJ) : TRetVal; cdecl;

(** Get the message timestamp.
  *
  * <P>The JMSTimestamp header field contains the time a message was
  * handed off to the SonicMQW client run time to be sent. It is not the time the
  * message was actually transmitted because the actual send may occur
  * later due to transactions or other client side queueing of messages.
  *
  * <P>When a message is sent, JMSTimestamp is ignored. When the send
  * method returns it contains a a time value somewhere in the interval
  * between the call and the return. It is in the format of a normal
  * Java millis time value.
  *
  * <P>Since timestamps take some effort to create and increase a
  * message's size, SonicMQ can optimize message
  * overhead if it is given a hint that timestamp is not used by an
  * application. SonicMQ message Producers provide a hint to disable
  * timestamps. When a client sets a producer to disable timestamps
  * they are saying that they do not depend on the value of timestamp
  * for the messages it produces. These messages must either have
  * timestamp set to null or, if the hint is ignored, timestamp must
  * be set to its normal value.
  *
  * @param msgObj the Message to operate against
  * @param pTimestamp the message timestamp value
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSTimestamp()
  *)
function Message_getJMSTimestamp(msgObj : HOBJ; var pTimestamp : jlong) : TRetVal; cdecl;

(** Set the message timestamp.
  *
  * <P>SonicMQ sets this field when a message is sent. This operation
  * can be used to change the value of a message that's been received.
  *
  * @param msgObj the Message to operate against
  * @param timestamp the timestamp for this message
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_getJMSTimestamp()
  *)
function Message_setJMSTimestamp(msgObj : HOBJ; timestamp : jlong) : TRetVal; cdecl;

(** Get the correlation ID for the message.
  *
  * <P>This method is used to return correlation id values that are
  * either SonicMQ-specific message ID's or application-specific Strings.
  *
  * @param msgObj the Message to operate against
  * @param pCOrrelationID the correlation ID of a message as a String.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSCorrelationID()
  * @see Message_getJMSCorrelationIDAsBytes()
  * @see Message_setJMSCorrelationIDAsBytes()
  *)
function Message_getJMSCorrelationID(msgObj : HOBJ; var pCorrelationID : HOBJ) : TRetVal; cdecl;

(** Set the correlation ID for the message.
  *
  * <P>A client can use the JMSCorrelationID header field to link one
  * message with another. A typically use is to link a response message
  * with its request message.
  *
  * <P>JMSCorrelationID can hold one of the following:
  *    <UL>
  *      <LI>A SonicMQ-specific message ID
  *      <LI>An application-specific String
  *      <LI>A native jbyteArray value.
  *    </UL>
  *
  * <P>Since each message sent in SonicMQ is assigned a message ID
  * value it is convenient to link messages via message ID. All message ID
  * values must start with the 'ID:' prefix.
  *
  * <P>In some cases, an application (made up of several clients) needs to
  * use an application specific value for linking messages. For instance,
  * an application may use JMSCorrelationID to hold a value referencing
  * some external information. Application specified values must not start
  * with the 'ID:' prefix; this is reserved for SonicMQ-generated message
  * ID values.
  *
  * <P>If a provider supports the native concept of correlation ID, a SonicMQ
  * client may need to assign specific JMSCorrelationID values to match
  * those expected by non-JMS clients. A jbyteArray value is used for this
  * purpose. JMS providers without native correlation ID values are not
  * required to support jbyteArray values. The use of a jbyteArray value for
  * JMSCorrelationID is non-portable.
  *
  * @param msgObj the Message to operate against
  * @param correlationID the message ID of a message being referred to.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_getJMSCorrelationID()
  * @see Message_getJMSCorrelationIDAsBytes()
  * @see Message_setJMSCorrelationIDAsBytes()
  *)
function Message_setJMSCorrelationID(msgObj : HOBJ; correlationID : HOBJ) : TRetVal; cdecl;

(** Get the correlation ID as an array of bytes for the message.
  *
  * <P>The use of a jbyteArray value for JMSCorrelationID is non-portable.
  *
  * @param msgObj the Message to operate against
  * @param pCorrelationID the correlation ID of a message as an array of bytes.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSCorrelationID()
  * @see Message_getJMSCorrelationID()
  *)
function Message_getJMSCorrelationIDAsBytes(msgObj : HOBJ; var pCorrelationID : HOBJ) : TRetVal; cdecl;

(** Set the correlation ID as an array of bytes for the message.
  *
  * <P>If a provider supports the native concept of correlation id, a
  * SonicMQ client may need to assign specific JMSCorrelationID values to
  * match those expected by non-JMS clients. JMS providers without native
  * correlation id values are not required to support this (and the
  * corresponding get) method; their implementation may throw
  * java.lang.UnsupportedOperationException).
  *
  * <P>The use of a jbyteArray value for JMSCorrelationID is non-portable.
  *
  * @param msgObj the Message to operate against
  * @param correlationID the correlation ID value as an array of bytes.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSCorrelationID()
  * @see Message_getJMSCorrelationID()
  * @see Message_getJMSCorrelationIDAsBytes()
  *)
function Message_setJMSCorrelationIDAsBytes(msgObj : HOBJ; value : HOBJ) : TRetVal; cdecl;

(** Get where a reply to this message should be sent.
  *
  * @param msgObj the Message to operate against
  * @param pDestination the destination to send a response to this message
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSReplyTo()
  *)
function Message_getJMSReplyTo(msgObj : HOBJ; var pDestination : HOBJ) : TRetVal; cdecl;

(** Set where a reply to this message should be sent.
  *
  * <P>The replyTo header field contains the destination where a reply
  * to the current message should be sent. If it is null no reply is
  * expected. The destination may be either a Queue or a Topic.
  *
  * <P>Messages with a null replyTo value are called SonicMQ datagrams.
  * Datagrams may be a notification of some change in the sender (i.e.
  * they signal a sender event) or they may just be some data the sender
  * thinks is of interest.
  *
  * Messages with a replyTo value are typically expecting a response.
  * A response may be optional, it is up to the client to decide. These
  * messages are called SonicMQ requests. A message sent in response to a
  * request is called a reply.
  *
  * In some cases a client may wish to match up a request it sent earlier
  * with a reply it has just received. This can be done using the
  * correlationID.
  *
  * @param msgObj the Message to operate against
  * @param replyTo Destination to send a response to this message
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_getJMSReplyTo()
  *)
function Message_setJMSReplyTo(msgObj : HOBJ; replyto : HOBJ) : TRetVal; cdecl;

(** Get the destination for this message.
  *
  * <P>The destination field contains the destination to which the
  * message is being sent.
  *
  * <P>When a message is sent this value is ignored. After completion
  * of the send method it holds the destination specified by the send.
  *
  * <P>When a message is received, its destination value must be
  * equivalent to the value assigned when it was sent.
  *
  * @param msgObj the Message to operate against
  * @param pDestination the Destination of this message.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSDestination()
  *)
function Message_getJMSDestination(msgObj : HOBJ; var pDestination : HOBJ) : TRetVal; cdecl;

(** Set the destination for this message.
  *
  * <P>SonicMQ sets this field when a message is sent. This operation
  * can be used to change the value of a message that's been received.
  *
  * @param msgObj the Message to operate against
  * @param destination the Destination for this message.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_getJMSDestination()
  *)
function Message_setJMSDestination(msgObj : HOBJ; destination : HOBJ) : TRetVal; cdecl;

(** Get the delivery mode for this message.
  *
  * @param msgObj the Message to operate against
  * @param pdeliveryMode the delivery mode of this message.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see Message_setJMSDeliveryMode()
  *)
function Message_getJMSDeliveryMode(msgObj : HOBJ; var pdeliveryMode : jint) : TRetVal; cdecl;

(**
 * Set the delivery mode for this message.
 *
 * <p>Providers set this field when a message is sent. This operation
 * can be used to change the value of a message that's been received.
 *
 * @param msgObj the Message to operate against
 * @param value the delivery mode for this message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see Message_setJMSDeliveryMode()
 *
 * @exception JMSException if JMS fails to set JMS DeliveryMode
 *                         due to some internal JMS error.
 *)
function Message_setJMSDeliveryMode(msgObj : HOBJ; value : jint) : TRetVal; cdecl;

(**
 * Get an indication of whether this message is being redelivered.
 *
 * <p>If a client receives a message with the redelivered indicator set,
 * it is likely, but not guaranteed, that this message was delivered to
 * the client earlier but the client did not acknowledge its receipt at
 * that earlier time.
 *
 * @param msgObj the Message to operate against
 * @param pret set to true if this message is being redelivered.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see Message_setJMSDeliveryMode()
 *
 *
 * @exception JMSException if JMS fails to get JMS Redelivered flag
 *                         due to some internal JMS error.
 *)
function Message_getJMSRedelivered(msgObj : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Set to indicate whether this message is being redelivered.
 *
 * <p>This field is set at the time the message is delivered. This
 * operation can be used to change the value of a message that's
 * been received.
 *
 * @param msgObj the Message to operate against
 * @param value an indication of whether this message is being redelivered.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see Message_setJMSDeliveryMode()
 *
 * @exception JMSException if JMS fails to set JMS Redelivered flag
 *                         due to some internal JMS error.
 *)
function Message_setJMSRedelivered(msgObj : HOBJ; value : jboolean) : TRetVal; cdecl;

(**
 * Get the message type.
 *
 * @param msgObj the Message to operate against
 * @param pret the message type
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to get JMS message type
 *                         due to some internal JMS error.
 *)
function Message_getJMSType(msgObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(** Set the message type.
 *
 * <p>Some JMS providers use a message repository that contains the
 * definition of messages sent by applications. The type header field
 * contains the name of a message's definition.
 *
 * <p>JMS does not define a standard message definition repository nor
 * does it define a naming policy for the definitions it contains. JMS
 * clients should use symbolic values for type that can be configured
 * at installation time to the values defined in the current providers
 * message repository.
 *
 * <p>JMS clients should assign a value to type whether the application
 * makes use of it or not. This insures that it is properly set for
 * those providers that require it.
 *
 * @param msgObj the Message to operate against
 * @param value the class of message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to set JMS message type
 *                         due to some internal JMS error.
 *)
function Message_setJMSType(msgObj : HOBJ; value : HOBJ) : TRetVal; cdecl;

(**
 * Get the message's expiration value.
 *
 * <p>When a message is sent, expiration is left unassigned. After
 * completion of the send method, it holds the expiration time of the
 * message. This is the sum of the time-to-live value specified by the
 * client and the GMT at the time of the send.
 *
 * <p>If the time-to-live is specified as zero, expiration is set to
 * zero which indicates the message does not expire.
 *
 * <p>When a message's expiration time is reached, a provider should
 * discard it. JMS does not define any form of notification of message
 * expiration.
 *
 * <p>Clients should not receive messages that have expired; however,
 * JMS does not guarantee that this will not happen.
 *
 * @param msgObj the Message to operate against
 * @param pret the time the message expires. It is the sum of the
 * time-to-live value specified by the client, and the GMT at the
 * time of the send.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to get JMS message expiration
 *                         due to some internal JMS error.
 *)
function Message_getJMSExpiration(msgObj : HOBJ; var pret : jlong) : TRetVal; cdecl;

(**
 * Set the message's expiration value.
 *
 * <p>Providers set this field when a message is sent. This operation
 * can be used to change the value of a message that's been received.
 *
 * @param msgObj the Message to operate against
 * @param value the message's expiration time
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to set JMS message expiration
 *                         due to some internal JMS error.
 *)
function Message_setJMSExpiration(msgObj : HOBJ; value : jlong) : TRetVal; cdecl;

(**
 * Get the message priority.
 *
 * <p>JMS defines a ten level priority value with 0 as the lowest
 * priority and 9 as the highest. In addition, clients should consider
 * priorities 0-4 as gradations of normal priority and priorities 5-9
 * as gradations of expedited priority.
 *
 * <p>JMS does not require that a provider strictly implement priority
 * ordering of messages; however, it should do its best to deliver
 * expedited messages ahead of normal messages.
 *
 * @param msgObj the Message to operate against
 * @param pret the default message priority
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to get JMS message priority
 *                         due to some internal JMS error.
 *)
function Message_getJMSPriority(msgObj : HOBJ; var pret : jint) : TRetVal; cdecl;

(**
 * Set the priority for this message.
 *
 * <p>Providers set this field when a message is sent. This operation
 * can be used to change the value of a message that's been received.
 *
 * @param msgObj the Message to operate against
 * @param value the priority of this message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to set JMS message priority
 *                         due to some internal JMS error.
 *)
function Message_setJMSPriority(msgObj : HOBJ; value : jint) : TRetVal; cdecl;

(**
 * Clear a message's properties.
 *
 * @param msgObj the Message to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to clear JMS message
 *                         properties due to some internal JMS
 *                         error.
 *)
function Message_clearProperties(msgObj : HOBJ) : TRetVal; cdecl;

(** Check if a property value exists.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the property to test as a String
 * @param pret true if the property does exist.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  check if property
 *                         exists due to some internal JMS
 *                         error.
 *)
function Message_propertyExists(msgObj : HOBJ; name : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Return the boolean property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the boolean property as a String
 * @param pret the boolean property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getBooleanProperty(msgObj : HOBJ; name : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Return the byte property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the byte property as a String
 * @param pret the byte property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getByteProperty(msgObj : HOBJ; name : HOBJ; var pret : jbyte) : TRetVal; cdecl;

(**
 * Return the short property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the short property as a String
 * @param pret the short property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getShortProperty(msgObj : HOBJ; name : HOBJ; var pret : jshort) : TRetVal; cdecl;

(**
 * Return the integer property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the integer property as a String
 * @param pret the integer property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getIntProperty(msgObj : HOBJ; name : HOBJ; var pret : jint) : TRetVal; cdecl;

(**
 * Return the long property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the long property as a String
 * @param pret the long property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getLongProperty(msgObj : HOBJ; name : HOBJ; var pret : jlong) : TRetVal; cdecl;

(**
 * Return the float property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the float property as a String
 * @param pret the float property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getFloatProperty(msgObj : HOBJ; name : HOBJ; var pret : jfloat) : TRetVal; cdecl;

(**
 * Return the double property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the double property as a String
 * @param pret the double property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getDoubleProperty(msgObj : HOBJ; name : HOBJ; var pret : jdouble) : TRetVal; cdecl;

(**
 * Return the String property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the String property as a String
 * @param pret the String property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getStringProperty(msgObj : HOBJ; name : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Return the Object property value with the given name.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the Object property as a String
 * @param pret the Object property value with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property due to
 *                         some internal JMS error.
 * @exception MessageFormatException if this type conversion is invalid.
 * @exception NullPointerException if property does not exist.
 *)
function Message_getObjectProperty(msgObj : HOBJ; name : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Return an Enumeration of all the property names.
 *
 * @param msgObj the Message to operate against
 * @param pret an Enumeration w/ all the names of property values.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  get Property names due to
 *                         some internal JMS error.
 *)
function Message_getPropertyNames(msgObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Set a boolean property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the boolean property as a String
 * @param value the boolean property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setBooleanProperty(msgObj : HOBJ; name : HOBJ; value : jboolean) : TRetVal; cdecl;

(**
 * Set a byte property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the byte property as a String
 * @param value the byte property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setByteProperty(msgObj : HOBJ; name : HOBJ; value : jbyte) : TRetVal; cdecl;

(**
 * Set a short property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the short property as a String
 * @param value the short property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setShortProperty(msgObj : HOBJ; name : HOBJ; value : jshort) : TRetVal; cdecl;

(**
 * Set a integer property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the integer property as a String
 * @param value the integer property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setIntProperty(msgObj : HOBJ; name : HOBJ; value : jint) : TRetVal; cdecl;

(**
 * Set a long property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the long property as a String
 * @param value the long property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setLongProperty(msgObj : HOBJ; name : HOBJ; value : jlong) : TRetVal; cdecl;

(**
 * Set a float property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the float property as a String
 * @param value the float property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setFloatProperty(msgObj : HOBJ; name : HOBJ; value : jfloat) : TRetVal; cdecl;

(**
 * Set a double property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the double property as a String
 * @param value the double property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setDoubleProperty(msgObj : HOBJ; name : HOBJ; value : jdouble) : TRetVal; cdecl;

(**
 * Set a String property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the String property as a String
 * @param value the String property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setStringProperty(msgObj : HOBJ; name : HOBJ; value : HOBJ) : TRetVal; cdecl;

(**
 * Set an Object property value with the given name, into the Message.
 *
 * @param msgObj the Message to operate against
 * @param name the name of the Object property as a String
 * @param value the Object property value to set in the Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to  set Property due to
 *                         some internal JMS error.
 * @exception MessageNotWriteableException if properties are read-only
 *)
function Message_setObjectProperty(msgObj : HOBJ; name : HOBJ; value : HOBJ) : TRetVal; cdecl;

(**
 * Acknowledge this or all previous messages received by the session.
 *
 * <p>All JMS messages support the acknowledge() method for use when a client has specified
 * that a JMS session's messages are to be explicitly acknowledged. A client requests explicit
 * acknowledgement by creating a session with either the standard JMS CLIENT_ACKKNOWLEDGE mode,
 * or the non-JMS acknowledgement mode SINGLE_MESSAGE_ACKNOWLEDGE.
 *
 * <p>In the standard JMS CLIENT_ACKNOWLEDGE mode, all messages previously received for the
 * session are acknowledged. If the session has been created with SINGLE_MESSAGE_ACKNOWLEDGE,
 * only the current message is acknowledged.
 *
 * <p>JMS defaults to implicit message acknowledgement, AUTO_ACKNOWLEDGE. In this mode,
 * calls to acknowledge() are ignored, as JMS automatically acknowledges messages on behalf
 * of the client.
 *
 * <p>Messages that have been received but not acknowledged may be
 * redelivered to the consumer.
 *
 * @param msgObj the Message to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to acknowledge due to some
 *                         internal JMS error.
 *)
function Message_acknowledge(msgObj : HOBJ) : TRetVal; cdecl;

(**
 * Clear out the message body. All other parts of the message are left
 * untouched.
 *
 * @param msgObj the Message to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to due to some internal JMS error.
 *)
function Message_clearBody(msgObj : HOBJ) : TRetVal; cdecl;

(**
 * Get all message properties (Progress-specific message interface extensions) in a
 *  Hashtable object. Note that both the name and the value of a Hashtable entry
 *  will be objects. E.g. a boolean property of "true" w/ the name "isPublic" will
 *  be represented in the Hashtable as an entry w/ the String object "isPublic" as
 *  the key and the Boolean object w/ value "true" as the value. Use the appropriate
 *  API functions to access the values of the objectified values.
 *
 * @param msgObj the Message whose properties to retrieve
 * @param pret a Hashtable object containing the message properties as
 *        key-value pairs (all objects)
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Message_getProperties(msgObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

{(**
 * Create a new message.
 *
 * @param pret the newly created Message.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to due to some internal JMS error.
 *)
function Message_create(HOBJ *pret) : TRetVal; cdecl;
}


(*
//---------------------------------
// TextMessage
//---------------------------------
*)

(**
 * Returns the int corresponding to the TextMessage type.
 *
 * @return the int corresponding to the TextMessage type
 *)
function TextMessage_type : TRetVal; cdecl;

(**
 * Create a new TextMessage
 *
 * @param pret will contain the newly created TextMessage.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TextMessage_create(var pret : HOBJ) : TRetVal; cdecl;

(** Set the string containing this message's data.
 * 
 * @param textMsg the TextMessage to operate against
 * @param string the String containing the message's data
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Likely causes of errors:
 *           MessageNotWriteableException if message in read-only mode.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TextMessage_setText(textMsg : HOBJ; s : HOBJ) : TRetVal; cdecl;

(** Get the string containing this message's data.  The default
 * value is null.
 *
 * @param textMsg the TextMessage to operate against
 * @param pString the String containing the message's data
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TextMessage_getText(textMsg : HOBJ; var pString : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// BytesMessage
//---------------------------------
*)
(**
 * Returns the int corresponding to the BytesMessage type.
 *
 * @return the int corresponding to the BytesMessage type
 *)
function BytesMessage_type : TRetVal; cdecl;

(**
 * Create a new BytesMessage.
 *
 * @param pret the newly created BytesMessage object.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function BytesMessage_create(var pret : HOBJ) : TRetVal; cdecl;

(** Read a <code>jboolean</code> from the stream message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param pBoolean pointer to the <code>jboolean</code> return value
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_readBoolean(bytesMsg : HOBJ; var pBoolean : jboolean) : TRetVal; cdecl;

(** Read a signed 8-bit value from the stream message.
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param pJbyte the next <code>jbyte</code> from the stream message.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_readByte(bytesMsg :HOBJ; var pJbyte : jbyte) : TRetVal; cdecl;

{(** Read an unsigned 8-bit number from the stream message.
  * 
  * @param bytesMsg the BytesMessage to operate against
  * @param pJint the next byte from the stream message, interpreted as a <code>jint</code> (an
  * unsigned 8-bit number).
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_readUnsignedByte(HOBJ bytesMsg, jint* pJint) : TRetVal; cdecl;

(** Read a signed 16-bit number from the stream message.
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param pJshort the next two bytes from the stream message, interpreted as a
  * <code>jshort</code> (signed 16-bit number).
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readShort(HOBJ bytesMsg, jshort* pJshort) : TRetVal; cdecl;

(** Read an unsigned 16-bit number from the stream message.
  *  
  * @param bytesMessage the BytesMessage to operate against
  * @param pJint the next two bytes from the stream message, interpreted as an
  * <code>jint</code> (unsigned 16-bit integer).
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readUnsignedShort(HOBJ bytesMsg, jint* pJint) : TRetVal; cdecl;

(** Read a Unicode character value from the stream message.
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param the next two bytes from the stream message as a <code>jchar</code> (Unicode
  * character).
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readChar(HOBJ bytesMsg, jchar* pJchar) : TRetVal; cdecl;

(** Read a signed 32-bit integer from the stream message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param pJint the next four bytes from the stream message, interpreted as
  * an <code>int</code>.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readInt(HOBJ bytesMsg, jint* pJint) : TRetVal; cdecl;

(** Read a signed 64-bit integer from the stream message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param pJlong the next eight bytes from the stream message, interpreted as
  * a <code>jlong</code>.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readLong(HOBJ bytesMsg, jlong* pJlong) : TRetVal; cdecl;

(** Read a <code>jfloat</code> from the stream message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param pJfloat the next four bytes from the stream message, interpreted as
  * a <code>jfloat</code>.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readFloat(HOBJ bytesMsg, jfloat* pJfloat) : TRetVal; cdecl;

(** Read a <code>jdouble</code> from the stream message
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param pJdouble the next eight bytes from the stream message, interpreted as
  * a <code>jdouble</code>.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readDouble(HOBJ bytesMsg, jdouble* pJdouble) : TRetVal; cdecl;

(** Read in a string that has been encoded using a modified UTF-8
  * format from the stream message.
  *
  * <P>For more information on the UTF-8 format, see "File System Safe
  * UCS Transformation Format (FSS_UTF)", X/Open Preliminary Specification,
  * X/Open Company Ltd., Document Number: P316. This information also
  * appears in ISO/IEC 10646, Annex P.
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param pUTF a Unicode string from the stream message.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readUTF(HOBJ bytesMsg, HOBJ* pUTF) : TRetVal; cdecl;
}
(** Read a byte array from the stream message.
  *
  * @param bytesMessage the BytesMessage to operate against
  * @param value the buffer into which the data is read.
  * @param pNumBytes the total number of bytes read into the buffer, or -1 if
  * there is no more data because the end of the stream has been reached.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotReadable if message is in write-only mode
  *           MessageEOFException if end of message stream
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_readBytes(bytesMsg, value : HOBJ; var pNumBytes : jint) : TRetVal; cdecl;

{(** Read a portion of the bytes message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the buffer into which the data is read.
  * @param length the number of bytes to read.
  *  
  * @param pNumBytes the total number of bytes read into the buffer, or -1 if
  * there is no more data because the end of the stream has been reached.
  *
  * If length is negative, or length is greater than the length of the array 
  * value, then an IndexOutOfBoundsException error is returned. No bytes will be read
  * from the stream for this error case.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           IndexOutOfBoundsException if length is out of bound.
  *           MessageNotReadableException if message in write-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_readBytes2(HOBJ bytesMsg, HOBJ value, jint length, jint* pNumBytes) : TRetVal; cdecl; 

(** Write a <code>jboolean</code> to the stream message as a 1-byte value.
  * The value <code>true</code> is written out as the value 
  * <code>(byte)1</code> the value <code>false</code> is written out as 
  * the value <code>(byte)0</code>.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>jboolean</code> value to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_writeBoolean(HOBJ bytesMsg, jboolean value) : TRetVal; cdecl;
}
(** Write out a <code>byte</code> to the stream message as a 1-byte value.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>byte</code> to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function BytesMessage_writeByte(bytesMsg : HOBJ; value : jbyte) : TRetVal; cdecl;

{(** Write a <code>jshort</code> to the stream message as two bytes, high
  * byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>short</code> to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeShort(HOBJ bytesMsg, jshort value) : TRetVal; cdecl;

(** Write a <code>jchar</code> to the stream message as a 2-byte value, 
  * high byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>jchar</code> value to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeChar(HOBJ bytesMsg, jchar value) : TRetVal; cdecl;

(** Write an <code>jint</code> to the stream message as four bytes, 
  * high byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>jint</code> to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeInt(HOBJ bytesMsg, jint value) : TRetVal; cdecl;

(** Write a <code>jlong</code> to the stream message as eight bytes, 
  * high byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>jlong</code> to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeLong(HOBJ bytesMsg, jlong value) : TRetVal; cdecl;

(** Convert the float argument to an <code>int</code> using the
  * <code>floatToIntBits</code> method in java class <code>Float</code>,
  * and then writes that <code>int</code> value to the stream
  * message as a 4-byte quantity, high byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>float</code> value to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeFloat(HOBJ bytesMsg, jfloat value) : TRetVal; cdecl;

(** Convert the double argument to a <code>long</code> using the
  * <code>doubleToLongBits</code> method in Java class <code>Double</code>,
  * and then writes that <code>long</code> value to the stream
  * message as an 8-byte quantity, high byte first.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>double</code> value to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeDouble(HOBJ bytesMsg, jdouble value) : TRetVal; cdecl;

(** Write a string to the stream message using UTF-8 encoding in a 
  * machine-independent manner.
  *
  * <P>For more information on the UTF-8 format, see "File System Safe
  * UCS Transformation Format (FSS_UFT)", X/Open Preliminary Specification,       * X/Open Company Ltd., Document Number: P316. This information also 
  * appears in ISO/IEC 10646, Annex P. 
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the <code>String</code> value to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeUTF(HOBJ bytesMsg, HOBJ value) : TRetVal; cdecl;

(** Write a byte array to the stream message.
  *
  * @param bytesMsg the BytesMessage to operate against
  * @param value the byte array to be written.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeBytes(HOBJ bytesMsg, HOBJ value) : TRetVal; cdecl;

(** Write a portion of a byte array to the stream message.
  *  
  * @param bytesMsg the BytesMessage to operate against
  * @param value the byte array value to be written.
  * @param offset the initial offset within the byte array.
  * @param length the number of bytes to use.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes for errors:
  *           MessageNotWriteableException if message in read-only mode.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_writeBytes2(HOBJ bytesMsg,
									 HOBJ value,
									 jint offset,
									 jint length) : TRetVal; cdecl;

(** Put the message in read-only mode, and reposition the stream of 
  * bytes to the beginning.
  *  
  * @param bytesMsg the BytesMessage to operate against
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function BytesMessage_reset(HOBJ bytesMsg) : TRetVal; cdecl;

{(*
//---------------------------------
// ConnectionMetaData
//---------------------------------
*)

(**
 * Get the String representation for the given ConnectionMetaData.
 *
 * @param thisObj the object to operate against.
 * @param pret will contain the String that represents the ConnectionMetaData.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
#define ConnectionMetaData_toString(connMeta, pret)               Object_toString(connMeta, pret)
}
(**
 * Returns the int corresponding to the ConnectionMetaData type.
 *
 * @return the int corresponding to the ConnectionMetaData type
 *)
function ConnectionMetaData_type : TRetVal; cdecl;

(**
 * Get the JMS major version number.
 *
 * @param connMeta the object to operate against.
 * @param pMajorVersion the JMS major version number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getJMSMajorVersion(connMeta : HOBJ; var pMajorVersion : jint) : TRetVal; cdecl;

(**
 * Get the JMS minor version number.
 *
 * @param connMeta the object to operate against.
 * @param pMinorVersion the JMS minor version number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getJMSMinorVersion(connMeta : HOBJ; var pMinorVersion : jint) : TRetVal; cdecl;

(**
 * Get the JMS provider name.
 *
 * @param connMeta the object to operate against.
 * @param pProvider a String containing the JMS provider name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getJMSProviderName(connMeta : HOBJ; var pProvider : HOBJ) : TRetVal; cdecl;

(** Get an enumeration of JMSX Property Names.
  *
  * @param connMeta the ConnectionMetaData object to operate against
  * @param pXPropNames an Enumeration of the JMSX PropertyNames.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function ConnectionMetaData_getJMSXPropertyNames(connMeta : HOBJ; var pXPropNames : HOBJ) : TRetVal; cdecl;

(**
 * Get the JMS provider major version number.
 *
 * @param connMeta the object to operate against.
 * @param pMajorVersion the JMS provider major version number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getProviderMajorVersion(connMeta : HOBJ; var pMajorVersion : jint) : TRetVal; cdecl;

(**
 * Get the JMS provider minor version number.
 *
 * @param connMeta the object to operate against.
 * @param pMinorVersion the JMS provider minor version number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getProviderMinorVersion(connMeta : HOBJ; var pMinorVersion : jint) : TRetVal; cdecl;

(**
 * Get the JMS provider version.
 *
 * @param connMeta the object to operate against.
 * @param pVersion a String containing the JMS provider version.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getProviderVersion(connMeta : HOBJ; var pVersion : HOBJ) : TRetVal; cdecl;

(**
 * Get the JMS version.
 *
 * @param connMeta the object to operate against.
 * @param pret a String containing the JMS version.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid protocol).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function ConnectionMetaData_getJMSVersion(connMeta : HOBJ; var pret : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// Queue
//---------------------------------
*)

(**
 * Returns the int corresponding to the Queue type.
 *
 * @return the int corresponding to the Queue type
 *)
function Queue_type : TRetVal; cdecl;

(** Get the name of this queue.
  *
  * <P>Clients that depend upon the name are not portable.
  *  
  * @param qObj the Queue object to operate against
  * @param pNameStr the queue name returned as a String
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function Queue_getQueueName(qObj : HOBJ; var pNameStr : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// TemporaryQueue
//---------------------------------
*)

(**
 * Returns the int corresponding to the TemporaryQueue type.
 *
 * @return the int corresponding to the TemporaryQueue type
 *)
function TemporaryQueue_type : TRetVal; cdecl;

(** Delete this temporary queue. If there are still senders
 * or receivers using it, an error will be returned.
 *
 * @param tmpQ the TemporaryQueue object to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TemporaryQueue_delete(tmpQ : HOBJ) : TRetVal; cdecl;



(*
//---------------------------------
// Topic
//---------------------------------
*)
(**
 * Returns the int corresponding to the Topic type.
 *
 * @return the int corresponding to the Topic type
 *)
function Topic_type : TRetVal; cdecl;

(**
 * Get the name of this topic.
 *  
 * <p>Clients that depend upon the name are not portable.
 *
 * @param topicObj the Topic object to operate against
 * @param pret a String containing the topic name
 *  
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation for Topic fails to
 *                         to return topic name due to some internal error.
 *)
function Topic_getTopicName(topicObj: HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// TemporaryTopic
//---------------------------------
*)
(**
 * Returns the int corresponding to the TemporaryTopic type.
 *
 * @return the int corresponding to the TemporaryTopic type
 *)
function TemporaryTopic_type : TRetVal; cdecl;

(**
 * Delete this temporary topic. If there are still existing publishers
 * or subscribers still using it, then a JMSException will be thrown.
 *
 * @param tmpTopic the TemporaryTopic to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation fails to delete a
 *                         Temporary queue due to some internal error.
 *)
function TemporaryTopic_delete(tmpTopic : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// Session
//---------------------------------
*)

(**
 * Returns the int corresponding to the Session type.
 *
 * @return the int corresponding to the Session type
 *)
function Session_type : TRetVal; cdecl;

(**
 * Create a Message. The Message interface is the root interface of
 * all JMS messages. It holds all the standard message header
 * information. It can be sent when a message containing only header
 * information is sufficient.
 *
 * @param thisObj the Session object to operate against.
 * @param pret will contain the newly created Message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to create this message
 *                         due to some internal error.
 *)
function Session_createMessage(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a BytesMessage. A BytesMessage is used to send a message
 * containing a stream of uninterpreted bytes.
 *
 * @param thisObj the Session object to operate against.
 * @param pret will contain the newly created Message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to create this message
 *                         due to some internal error.
 *)
function Session_createBytesMessage(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a TextMessage. A TextMessage is used to send a message
 * containing a String.
 *
 * @param thisObj the Session object to operate against.
 * @param pret will contain the newly created Message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to create this message
 *                         due to some internal error.
 *)
function Session_createTextMessage(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create an initialized TextMessage. A TextMessage is used to send
 * a message containing a String.
 *
 * @param thisObj the Session object to operate against.
 * @param text a String containing the text used to initialize this message.
 * @param pret will contain the newly created Message
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to create this message
 *                         due to some internal error.
 *)
function Session_createTextMessage2(thisObj : HOBJ; text : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Is the session in transacted mode?
 *
 * @param thisObj the Session object to operate against.
 * @param pret true if in transacted mode
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to return the transaction
 *                         mode due to internal error in JMS Provider.
 *)
function Session_getTransacted(thisObj : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Commit all messages done in this transaction and release any locks
 * currently held.
 *
 * @param thisObj the Session object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation fails to commit the
 *                         the transaction due to some internal error.
 * @exception TransactionRolledBackException  if the transaction
 *                         gets rolled back due to some internal error
 *                         during commit.
 * @exception IllegalStateException if commit() is called on a
 *                         non-transacted session
 *)
function Session_commit(thisObj : HOBJ) : TRetVal; cdecl;

(**
 * Rollback any messages done in this transaction and releases any locks
 * currently held.
 *
 * @param thisObj the Session object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation fails to rollback the
 *                         the transaction due to some internal error.
 * @exception IllegalStateException if rollback() called in a non-transacted
 *                         session
 *)
function Session_rollback(thisObj : HOBJ) : TRetVal; cdecl;

(**
 * Close a session. Clients should close sessions when they are not needed.
 *
 * @param thisObj the Session object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation fails to close a
 *                         Session due to some internal error.
 *)
function Session_close(Session : HOBJ) : TRetVal; cdecl;

(**
 * Stop message delivery in this session, and restart sending messages
 * with the oldest unacknowledged message.
 *
 * <p>All consumers deliver messages in a serial order.
 * Acknowledging a received message automatically acknowledges all
 * messages that have been delivered to the client in the current session.
 *
 * <p>Restarting a session causes it to take the following actions:
 *
 * <ul>
 *   <li>Stop message delivery
 *   <li>Mark all messages that might have been delivered but not
 *       acknowledged as `redelivered'
 *   <li>Restart the delivery sequence including all unacknowledged
 *       messages that had been previously delivered.
 *
 *     <p>Redelivered messages do not have to be delivered in
 *        exactly their original delivery order.
 * </ul>
 *
 * @param thisObj the Session object to operate against.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS implementation fails to stop message
 *                         delivery and restart message send due to
 *                         due to some internal error.
 * @exception IllegalStateException if recover() is called in a transacted
 *                         session
 *)
function Session_recover(thisObj : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// QueueSession
//---------------------------------
*)

(**
 * Returns the int corresponding to the QueueSession type.
 *
 * @return the int corresponding to the QueueSession type
 *)
function QueueSession_type : TRetVal; cdecl;

(**
 * Create a Queue given a Queue name.
 *
 * <p>This facility is provided for the rare cases where clients need to
 * dynamically manipulate queue identity. This allows the creation of a
 * queue with a provider specific name. Clients that depend on this
 * ability are not portable.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param name a String containing the name of the Queue to create
 * @param pret will contain a newly created Queue with the given name.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a queue
 *                         due to some JMS error.
 *)
function QueueSession_createQueue(thisObj, name : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a QueueReceiver to receive messages from the specified queue.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param queue the queue to access
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a receiver
 *                         due to some SonicMQ error.
 * @exception InvalidDestinationException if invalid Queue specified.
 *)
function QueueSession_createReceiver(thisObj, queue : HOBJ; var pReceiver : HOBJ) : TRetVal; cdecl;

(**
 * Create a QueueReceiver to receive messages from the specified queue.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param queue the queue to access
 * @param messageSelector only messages with properties matching the
 * message selector expression are delivered
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a receiver
 *                         due to some SonicMQ error.
 * @exception InvalidDestinationException if invalid Queue specified.
 * @exception InvalidSelectorException if the message selector is invalid.
 *
 *)
function QueueSession_createReceiver2(thisObj : HOBJ; queue : HOBJ; messageSelector : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a QueueSender to send messages to the specified queue.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param queue the Queue to access, or null if this is an unidentifed producer.
 * @param pret will contain the newly created Queue
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a sender due to some JMS error.
 * @exception InvalidDestinationException if invalid Queue specified.
 *)
function QueueSession_createSender(thisObj, queue : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(** Create a QueueBrowser to Browse messages from the specified queue.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param queue the queue to access
 * @param pret will contain the newly created QueueBrowser
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a Browser
 *                         due to some SonicMQ error.
 * @exception InvalidDestinationException if invalid Queue specified.
 *)
function QueueSession_createBrowser(thisObj : HOBJ; queue : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(** Create a QueueBrowser to Browse messages from the specified queue.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param queue the queue to access
 * @param messageSelector only messages with properties matching the 
 *        message selector expression are delivered
 * @param pret will contain the newly created QueueBrowser
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if a session fails to create a Browser
 *               due to some SonicMQ error.
 * @exception InvalidDestinationException if invalid Queue specified.
 * @exception InvalidSelectorException if the message selector is invalid.
 *
 *)
function QueueSession_createBrowser2(thisObj : HOBJ; queue : HOBJ; messageSelector : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(** Create a temporary queue. Its lifetime will be that of the
 * QueueConnection unless deleted earlier.
 *
 * @param thisObj the QueueSession object to operate against.
 * @param pret the newly created TemporaryQueue.
 *
 * @exception JMSException if a session fails to create a Temporary Queue
 *                         due to some JMS error.
 *)
function QueueSession_createTemporaryQueue(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// TopicSession
//---------------------------------
*)

(** Create a Topic given a Topic name.
  *
  * <P>This facility is provided for the rare cases where clients need to
  * dynamically manipulate topic identity. This allows the creation of a
  * topic with a specific name. Clients that depend on this
  * ability are not portable.
  *
  * @param topicSsn the TopicSession to operate against
  * @param topicName the name of this topic
  *
  * @param pTopic pointer to the new Topic
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createTopic(topicSsn : HOBJ; topicName : HOBJ; var pTopic : HOBJ) : TRetVal; cdecl;

(** Create a non-durable Subscriber to the specified topic.
  *
  * <P>A client uses a TopicSubscriber for receiving messages that have
  * been published to a topic.
  *
  * <P>Regular TopicSubscriber's are not durable. They only receive
  * messages that are published while they are active.
  *
  *
  * @param topicSsn the TopicSession to operate against
  * @param topic the topic to subscribe to
  * @param pTopicSubscriber pointer to the new TopicSubscriber
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g.,
  *         InvalidDestinationException due to invalid topic name).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createSubscriber(topicSsn : HOBJ; topic : HOBJ; var pTopicSubscriber : HOBJ) : TRetVal; cdecl;

(** Create a non-durable Subscriber to the specified topic.
  *
  * <P>A client uses a TopicSubscriber for receiving messages that have
  * been published to a topic.
  *  
  * <P>Regular TopicSubscriber's are not durable. They only receive 
  * messages that are published while they are active.
  *
  * <P>Messages filtered out by a subscriber's message selector will 
  * never be delivered to the subscriber. From the subscriber's
  * perspective they simply don't exist.
  *
  * <P>In some cases, a connection may both publish and subscribe to a 
  * topic. The subscriber NoLocal attribute allows a subscriber to 
  * inhibit the delivery of messages published by its own connection.
  *
  * @param topicSsn the TopicSession to operate against
  * @param topic the topic to subscribe to
  * @param messageSelector only messages with properties matching the
  * message selector expression are delivered. This value may be null.
  * @param noLocal if set, inhibits the delivery of messages published
  * by its own connection.
  * @param pTopicSubscriber the new TopicSubscriber
  * 
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g.,
  *         invalid topic name or invalid selector).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createSubscriber2(topicSsn : HOBJ; topic : HOBJ; messageSelector : HOBJ; noLocal : jboolean; var pTopicSubscriber : HOBJ) : TRetVal; cdecl;

(** Create a durable Subscriber to the specified topic.
  *  
  * <P>If a client needs to receive all the messages published on a 
  * topic including the ones published while the subscriber is inactive,
  * it uses a durable TopicSubscriber. The SonicMQ broker retains a record of this 
  * durable subscription and insures that all messages from the topic's 
  * publishers are retained until they are either acknowledged by this 
  * durable subscriber or they have expired.
  *
  * <P>Sessions with durable subscribers must always provide the same 
  * client identifier. In addition, each client must specify a name which 
  * uniquely identifies (within client identifier) each durable 
  * subscription it creates. Only one session at a time can have a 
  * TopicSubscriber for a particular durable subscription.
  *
  * <P>A client can change an existing durable subscription by creating 
  * a durable TopicSubscriber with the same name and a new topic and/or 
  * message selector. Changing a durable subscriber is equivalent to 
  * deleting and recreating it
  *
  * @param topicSsn the TopicSession to operate against
  * @param topic the topic to subscribe to
  * @param name the name used to identify this subscription.
  * @param pTopicSubscriber pointer to the new TopicSubscriber
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g.,
  *         invalid topic name).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *) 
function TopicSession_createDurableSubscriber(topicSsn : HOBJ; topic : HOBJ; name : HOBJ; var pTopicSubscriber : HOBJ) : TRetVal; cdecl;

(** Create a durable Subscriber to the specified topic.
  *
  * <P>If a client needs to receive all the messages published on a
  * topic including the ones published while the subscriber is inactive,
  * it uses a durable TopicSubscriber. The SonicMQ broker retains a record of this
  * durable subscription and insures that all messages from the topic's
  * publishers are retained until they are either acknowledged by this
  * durable subscriber or they have expired.
  *
  * <P>Sessions with durable subscribers must always provide the same
  * client identifier. In addition, each client must specify a name which
  * uniquely identifies (within client identifier) each durable
  * subscription it creates. Only one session at a time can have a
  * TopicSubscriber for a particular durable subscription.
  *
  * <P>A client can change an existing durable subscription by creating
  * a durable TopicSubscriber with the same name and a new topic and/or
  * message selector. Changing a durable subscriber is equivalent to
  * deleting and recreating it
  *
  * @param topicSsn the TopicSession to operate against
  * @param topic the topic to subscribe to
  * @param name the name used to identify this subscription.
  * @param messageSelector only messages with properties matching the
  * message selector expression are delivered. This value may be null.
  * @param noLocal if set, inhibits the delivery of messages published
  * by its own connection.
  * @param pTopicSubscriber pointer to the new TopicSubscriber
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g.,
  *         invalid topic name or invalid selector).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createDurableSubscriber2(topicSsn : HOBJ; topic : HOBJ; name : HOBJ; messageSelector : HOBJ; noLocal : jboolean; var pTopicSubscriber : HOBJ) : TRetVal; cdecl;

(** Create a Publisher for the specified topic.
  *
  * <P>A client uses a TopicPublisher for publishing messages on a topic.
  * Each time a client creates a TopicPublisher on a topic, it defines a
  * new sequence of messages that have no ordering relationship with the
  * messages it has previously sent.
  *
  * @param topicSsn the TopicSession to operate against
  * @param topic the topic to publish to, or null if this is an 
  * unidentifed producer.
  * @param pTopicPublisher pointer to the new TopicPublisher
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g.,
  *         invalid topic name).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createPublisher(topicSsn : HOBJ; topic : HOBJ; var pTopicPublisher : HOBJ) : TRetVal; cdecl;

(** Create a temporary topic. Its lifetime is that of the 
  * TopicConnection.
  *
  * @param topicSsn the TopicSession to operate against
  * @param pTemporaryTopic pointer to the new temporary topic.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSession_createTemporaryTopic(topicSsn : HOBJ; var pTemporaryTopic : HOBJ) : TRetVal; cdecl;

(** Unsubscribe a durable subscription that has been created by a client.
  *  
  * <P>This deletes the state being maintained on behalf of the
  * subscriber by its SonicMQ broker.
  *
  * @param topicSsn the TopicSession to operate against
  * @param name the name used to identify this subscription.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid topic).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *                          
  *)
function TopicSession_unsubscribe(topicSsn : HOBJ; name : HOBJ) : TRetVal; cdecl;

(**
 * Disables or reenables publish flow control
 *
 * @param topicSsn the TopicSession to operate against
 * @param disabled true to disable flow control
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid topic).
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
 *         error details.
 *)
function TopicSession_setFlowControlDisabled(topicSsn : HOBJ; disabled : jboolean) : TRetVal; cdecl;

(**
 * Returns the int corresponding to the TopicSession type.
 *
 * @return the int corresponding to the TopicSession type
 *)
function TopicSession_type : TRetVal; cdecl;

(*
//---------------------------------
// MessageConsumer
//---------------------------------
*)

(**
 * Returns the int corresponding to the MessageConsumer type.
 *
 * @return the int corresponding to the MessageConsumer type
 *)
function MessageConsumer_type : TRetVal; cdecl;

(** Get this message consumer's message selector expression.
  *
  * @param msgCnsmr the MessageConsumer to operate against
  * @param pSelector this message consumer's message selector
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageConsumer_getMessageSelector(msgCnsmr : HOBJ; var pSelector : HOBJ) : TRetVal; cdecl;

(** Get the message consumer's "message listener" (i.e. callback function).
  *
  * @param msgCnsmr the MessageConsumer to operate against
  * @param pListener The listener for the message consumer, or null if there isn't
  *        one set. The proper signature for the message listener function is:
  *        <code>typedef void (*pfnCMessageListener)(HOBJ msg) : TRetVal; cdecl;</code>
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageConsumer_setMessageListener()
  *)
function MessageConsumer_getMessageListener(msgCnsmr : HOBJ; var plistener : pfnCMessageListener) : TRetVal; cdecl;


(** Set the message consumer's "message listener" (i.e. callback function).
  *
  * <p>When a message arrives for this message consumer, the callback function is
  * called and passed an HOBJ for a Message object. More specifically, the Message
  * may be a TextMessage or a BytesMessage.
  *
  * <p>The signature for an appropriate callback function is:
  * <code>typedef void (*pfnCMessageListener)(HOBJ msg) : TRetVal; cdecl;</code>
  *
  * @param msgCnsmr the MessageConsumer to register the message listener for
  * @param messageListener the messages are delivered to this listener
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageConsumer_getMessageListener()
  *)
function MessageConsumer_setMessageListener(msgCnsmr : HOBJ; listener : pfnCMessageListener) : TRetVal; cdecl;

(** Receive the next message produced for this message consumer.
  *
  * <P>This call blocks indefinitely until a message is produced.
  *
  * <P>If this receive is done within a transaction, the message
  * remains on the consumer until the transaction commits.
  *
  * @param msgCnsmr the MessageConsumer to operate against
  * @param pMsg the next message produced for this message consumer,
  *         or null, if the connection closes while waiting
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageConsumer_receive(msgCnsmr : HOBJ; var pMsg : HOBJ) : TRetVal; cdecl;

(** Receive the next message that arrives within the specified
  * timeout interval.
  *
  * <P>This call blocks until either a message arrives or the
  * timeout expires.
  *
  * @param msgCnsmr the MessageConsumer to operate against
  * @param timeout the timeout value (in milliseconds)
  * @param pMsg the next message produced for this message consumer,
  *         or null, if the connection closes while waiting, or if
  *         the wait times out.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageConsumer_receive2(msgCnsmr : HOBJ; timeout : jlong; var pMsg : HOBJ) : TRetVal; cdecl;

(** Receive the next message if one is immediately available.
  *
  * @param msgCnsmr the MessageConsumer to operate against
  * @return the next message produced for this message consumer, or
  * null if one is not available.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageConsumer_receiveNoWait(msgCnsmr : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// TopicSubscriber
//---------------------------------
*)

(** Get the topic associated with this subscriber.
  *
  * @param topicSub the TopicSubscriber to operate against
  * @param pTopic this subscriber's Topic
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid topic).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSubscriber_getTopic(topicSub : HOBJ; var pTopic: HOBJ) : TRetVal; cdecl;

(** Get the NoLocal attribute for this TopicSubscriber.
  * The default value for this attribute is false.
  *
  * @param topicSub the TopicSubscriber to operate against
  * @param pNoLocal set to true if locally published messages are being inhibited.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., invalid topic).
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicSubscriber_getNoLocal(topicSub : HOBJ; var pNoLocal : jboolean) : TRetVal; cdecl;

(**
 * Returns the int corresponding to the TopicSubscriber type.
 *
 * @return the int corresponding to the TopicSubscriber type
 *)
function TopicSubscriber_type : TRetVal; cdecl;

(*
//---------------------------------
// QueueReceiver
//---------------------------------
*)
(**
 * Returns the int corresponding to the QueueReceiver type.
 *
 * @return the int corresponding to the QueueReceiver type
 *)
function QueueReceiver_type : TRetVal; cdecl;

(** Since SonicMQ allocates some resources on behalf of a
 * MessageConsumer inside the SonicMQ client run-time, clients should close them when they
 * are not needed.
 *
 * @param qReceiver the QueueReceiver to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueReceiver_close(qReceiver : HOBJ) : TRetVal; cdecl;

(** Get the queue associated with this queue receiver.
 *
 * @param qReceiver the QueueReceiver to operate against
 * @param pQueue the queue
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueReceiver_getQueue(qReceiver : HOBJ; var pQueue : HOBJ) : TRetVal; cdecl;

(** Get the prefetch count for the QueueReceiver. When this value
 * is greater than one, the broker can send multiple
 * messages as part of a single QueueReceiver request.
 *
 * @param qReceiver the QueueReceiver to operate against
 * @param pCount The number of messages to prefetch.
 *
 * @see QueueReceiver_setPrefetchCount()
 *)
function QueueReceiver_getPrefetchCount(qReceiver : HOBJ; var pCount : jint) : TRetVal; cdecl;

(** Set the prefetch count for the QueueReceiver.  When this value
 * is greater than one, the broker can send multiple
 * messages as part of a single QueueReceiver request. This can
 * improve performance.
 *
 * <p>Note that this is a SonicMQ extension not found in the
 * standard  interface.
 *
 * @param qReceiver the QueueReceiver to operate against
 * @param count The number of messages to prefetch.
 * 
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueReceiver_setPrefetchThreshold(int)
 * @see QueueReceiver_getPrefetchCount()
 *)
function QueueReceiver_setPrefetchCount(qReceiver : HOBJ; count : jint) : TRetVal; cdecl;

(**
 * Get the prefetch threshold for the QueueReceiver.  When the
 * number of messages waiting to be processed by the QueueReceiver
 * falls to, or below, this number, a new batch of messages will be fetched.
 *
 * @param qReceiver the QueueReceiver to operate against
 * @param pCount The threshold value for prefetching messages.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueReceiver_setPrefetchCount()
 *)
function QueueReceiver_getPrefetchThreshold(qReceiver : HOBJ; var pCount : jint) : TRetVal; cdecl;

(**
 * Set the prefetch threshold for the QueueReceiver.  When the
 * number of messages waiting to be processed by the QueueReceiver
 * falls to, or below, this number, a new batch of messages is fetched.
 * This number cannot exceed the prefetch count.
 *
 * <p>Setting this to a value greater than zero allows the QueueReceiver
 * to always have messages available for processing locally without
 * waiting for a broker interaction.  This improves performance.
 *
 * <p>For example, a threshold value of two and a prefetch count of five
 * will cause the QueueReceiver to fetch batches of five messages from the
 * broker whenever the number of messages locally waiting for processing
 * drops below two.
 *
 * <p>Note that this is a SonicMQ extension not found in the
 * standard interface.
 *
 * @param qReceiver the QueueReceiver to operate against
 * @param threshold The threshold value for prefetching messages.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueReceiver_setPrefetchCount(int)
 * @see QueueReceiver_getPrefetchThreshold()
 *)
function QueueReceiver_setPrefetchThreshold(qReceiver : HOBJ; threshold : jint) : TRetVal; cdecl;


(*
//---------------------------------
// MessageProducer
//---------------------------------
*)
(**
 * Returns the int corresponding to the MessageProducer type.
 *
 * @return the int corresponding to the MessageProducer type
 *)
function MessageProducer_type : TRetVal; cdecl;

(** Get an indication of whether message IDs are disabled.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param pDisabled an indication of whether message IDs are disabled.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function MessageProducer_getDisableMessageID(msgPrdcr : HOBJ; var pdisabled : jboolean) : TRetVal; cdecl;

(** Set whether message IDs are disabled.
  *  
  * <P>Since message ID's take some effort to create and increase a
  * message's size, SonicMQ can optimize message
  * overhead if they are given a hint that message ID is not used by
  * an application. SonicMQ message Producers provide a hint to disable
  * message ID. When a client sets a Producer to disable message ID
  * they are saying that they do not depend on the value of message
  * ID for the messages it produces. These messages must either have
  * message ID set to null or, if the hint is ignored, messageID must
  * be set to its normal unique value.
  *
  * <P>Message IDs are enabled by default.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param value indicates if message IDs are disabled.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageProducer_setDisableMessageID(msgPrdcr : HOBJ; value : jboolean) : TRetVal; cdecl;

(** Get an indication of whether message timestamps are disabled.
  *  
  * @param msgPrdcr the MessageProducer to operate against
  * @param pDisabled an indication of whether message IDs are disabled.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageProducer_getDisableMessageTimestamp(msgPrdcr : HOBJ; var pDisabled : jboolean) : TRetVal; cdecl;

(** Set whether message timestamps are disabled.
  *  
  * <P>Since timestamps take some effort to create and increase a
  * message's size, SonicMQ can optimize message 
  * overhead if they are given a hint that timestamp is not used by an 
  * application. SonicMQ message Producers provide a hint to disable 
  * timestamps. When a client sets a producer to disable timestamps
  * they are saying that they do not depend on the value of timestamp 
  * for the messages it produces. These messages must either have 
  * timestamp set to null or, if the hint is ignored, timestamp must 
  * be set to its normal value.
  *  
  * <P>Message timestamps are enabled by default.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param value indicates if message timestamps are disabled.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *) 
function MessageProducer_setDisableMessageTimestamp(msgPrdcr : HOBJ; value : jboolean) : TRetVal; cdecl;

(** Get the producer's default delivery mode.
  *  
  * @param msgPrdcr the MessageProducer to operate against
  * @param pDelMode the message delivery mode for this message producer.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_setDeliveryMode()
  *) 
function MessageProducer_getDeliveryMode(msgPrdcr : HOBJ; var pDelMode : jint) : TRetVal; cdecl;

(** Set the producer's default delivery mode.
  *  
  * <P>Delivery mode is set to NON_PERSISTENT by default.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param deliveryMode the message delivery mode for this message
  * producer.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_getDeliveryMode()
  *) 
function MessageProducer_setDeliveryMode(msgPrdcr : HOBJ; deliveryMode : jint) : TRetVal; cdecl;

(** Get the producer's default priority.
  *  
  * @param msgPrdcr the MessageProducer to operate against
  * @param pPriority the message priority for this message producer.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_setPriority()
  *)
function MessageProducer_getPriority(msgPrdcr : HOBJ; var pPriority : jint) : TRetVal; cdecl;

(** Set the producer's default priority.
  *
  * <P>Priority is set to 4, by default.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param value the message priority for this message producer.
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_getPriority()
  *) 
function MessageProducer_setPriority(msgPrdcr : HOBJ; value : jint) : TRetVal; cdecl;

(** Get the default length of time in milliseconds from its dispatch time
  * that a produced message should be retained by the message system.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param pTTL the message time to live in milliseconds; zero is unlimited
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_setTimeToLive()
  *) 
function MessageProducer_getTimeToLive(msgPrdcr : HOBJ; var pTTL : jlong) : TRetVal; cdecl;

(** Set the default length of time in milliseconds from its dispatch time
  * that a produced message should be retained by the message system.
  *
  * <P>Time to live is set to zero by default.
  *
  * @param msgPrdcr the MessageProducer to operate against
  * @param timeToLive the message time to live in milliseconds; zero is
  * unlimited
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see MessageProducer_getTimeToLive()
  *)
function MessageProducer_setTimeToLive(msgPrdcr : HOBJ; value : jlong) : TRetVal; cdecl;

(** Since SonicMQ allocates some resources on behalf of a
  * MessageProducer, clients should close them when they
  * are not needed.
  *
  * @param msgPrdcr the MessageProducer to operate against
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function MessageProducer_close(msgPrdcr : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// QueueSender
//---------------------------------
*)
(**
 * Returns the int corresponding to the QueueSender type.
 *
 * @return the int corresponding to the QueueSender type
 *)
function QueueSender_type : TRetVal; cdecl;

(**
 * Get the queue associated with this sender.
 *
 * @param thisObj the QueueSender object to operate against.
 * @param pret this sender's Queue object
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to get queue for this queue sender
 *                         due to some internal error.
 *)
function QueueSender_getQueue(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Send a Message to the queue
 * Use the queues default delivery mode, timeToLive and priority.
 *
 * @param thisObj the QueueSender object to operate against.
 * @param message the Message to send
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to send the message
 *                         due to some internal error.
 * @exception MessageFormatException if invalid message specified
 * @exception InvalidDestinationException if a client uses
 *                         this method with a Queue Sender with
 *                         an invalid queue.
 *)
function QueueSender_send(qSender : HOBJ; msg : HOBJ) : TRetVal; cdecl;

(**
 * Send a Message to the queue specifying delivery mode, priority
 * and time to live to the queue.
 *
 * @param thisObj the QueueSender object to operate against.
 * @param message the Message to send
 * @param deliveryMode the delivery mode to use
 * @param priority the priority for this message
 * @param timeToLive the message's lifetime (in milliseconds).
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to send the message
 *                         due to some internal error.
 * @exception MessageFormatException if invalid message specified
 * @exception InvalidDestinationException if a client uses
 *                         this method with a Queue Sender with
 *                         an invalid queue.
 * @exception IllegalStateException if the delivery mode is set
 *                         to DISCARDABLE.
 *)
function QueueSender_send2(qSender, msg : HOBJ; deliveryMode, priority : jint; timeToLive : jlong) : TRetVal; cdecl;

(**
 * Send a Message to a queue for an unidentified message producer.
 * Use the queues default delivery mode, timeToLive and priority.
 *
 * <p>Typically a JMS message producer is assigned a queue at creation
 * time; however, JMS also supports unidentified message producers
 * which require that the queue be supplied on every message send.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param queue the Queue to send this message to
 * @param message the Message to send
 *  
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to send the message
 *                         due to some internal error.
 * @exception MessageFormatException if invalid message specified
 * @exception InvalidDestinationException if a client uses
 *                         this method with an invalid queue.
 *)
function QueueSender_send3(qSender, queue, msg : HOBJ) : TRetVal; cdecl;

(**
 * Send a Message to a queue for an unidentified message producer,
 * specifying delivery mode, priority and time to live.
 *
 * <p>Typically a JMS message producer is assigned a queue at creation
 * time; however, JMS also supports unidentified message producers
 * which require that the queue be supplied on every message send.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param queue the Queue to send this message to
 * @param message the Message to send
 * @param deliveryMode the delivery mode to use
 * @param priority the priority for this message
 * @param timeToLive the message's lifetime (in milliseconds).
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS fails to send the message due to some internal error.
 * @exception MessageFormatException if invalid message specified
 * @exception InvalidDestinationException if a client uses this method with an invalid Queue.
 * @exception IllegalStateException if the delivery mode is set to DISCARDABLE.
 *)
function QueueSender_send4(qSender, queue, msg : HOBJ; deliveryMode, priority : jint; timeToLive : jlong) : TRetVal; cdecl;

(*
//---------------------------------
// TopicPublisher
//---------------------------------
*)
(**
 * Returns the int corresponding to the TopicPublisher type.
 *
 * @return the int corresponding to the TopicPublisher type
 *)
function TopicPublisher_type : TRetVal; cdecl;

(** Get the topic associated with this publisher.
  *
  * @param topicPub the TopicPublisher to operate against
  * @param pTopic this publisher's Topic.  
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicPublisher_getTopic(topicPub : HOBJ; var pTopic : HOBJ) : TRetVal; cdecl;

(** Publish a Message to the topic
  * Uses the topics default delivery mode, timeToLive and priority.
  *
  * @param topicPub the TopicPublisher to operate against
  * @param message the message to publish
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           MessageFormatException if an invalid message is specified.
  *           InvalidDestinationException if invalid topic.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicPublisher_publish(topicPub : HOBJ; msg : HOBJ) : TRetVal; cdecl;

(** Publish a Message to the topic specifying delivery mode, priority 
  * and time to live.
  *
  * @param topicPub the TopicPublisher to operate against
  * @param message the message to publish
  * @param deliveryMode the delivery mode to use
  * @param priority the priority for this message
  * @param timeToLive the message's lifetime (in milliseconds).
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           MessageFormatException if invalid message specified
  *           IllegalStateException if DISCARDABLE is set on a transacted session.
  *           InvalidDestinationException if invalid topic.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicPublisher_publish2(topicPub : HOBJ; msg : HOBJ; deliverymode : jint; priority : jint; timeToLive : jlong) : TRetVal; cdecl;

(** Publish a Message to a topic for an unidentified message producer.	
  * Use the topics default delivery mode, timeToLive and priority.
  *  
  * <P>Typically a SonicMQ message producer is assigned a topic at creation 
  * time; however, SonicMQ also supports unidentified message producers 
  * which require that the topic be supplied on every message publish.
  *
  * @param topicPub the TopicPublisher to operate against
  * @param topic the topic to publish this message to
  * @param message the message to send
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors include:
  *           MessageFormatException if invalid message specified
  *           InvalidDestinationException if a client uses this method
  *             with an invalid topic.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *) 
function TopicPublisher_publish3(topicPub : HOBJ; topic : HOBJ; msg : HOBJ) : TRetVal; cdecl;

(** Publish a Message to a topic for an unidentified message producer,
  * specifying delivery mode, priority and time to live.
  *  
  * <P>Typically a SonicMQ message producer is assigned a topic at creation
  * time; however, SonicMQ also supports unidentified message producers
  * which require that the topic be supplied on every message publish.
  *
  * @param topicPub the TopicPublisher to operate against
  * @param topic the topic to publish this message to
  * @param message the message to send
  * @param deliveryMode the delivery mode to use
  * @param priority the priority for this message
  * @param timeToLive the message's lifetime (in milliseconds).
  *  
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           MessageFormatException if invalid message specified
  *           IllegalStateException if DISCARDABLE is set on a transacted session.
  *           InvalidDestinationException if invalid topic.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate
  *         error details.
  *)
function TopicPublisher_publish4(topicPub : HOBJ; topic : HOBJ; msg : HOBJ; deliverymode : jint; priority : jint; timeToLive : jlong) : TRetVal; cdecl;

(*
//---------------------------------
// Connection
//---------------------------------
*)
(**
 * Returns the int corresponding to the Connection type.
 *
 * @return the int corresponding to the Connection type
 *)
function Connection_type : TRetVal; cdecl;

(**
 * Closes the connection. There is no need to close the sessions, producers, and consumers
 * of a closed connection.
 *
 * @param obj the Connection object to close.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_close(obj : HOBJ) : TRetVal; cdecl;

(**
 * Start (or restart) a Connection's delivery of incoming messages. Starting a started
 * session is ignored.
 *
 * @param obj the Connection object to start.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_start(obj : HOBJ) : TRetVal; cdecl;

(**
 * Used to temporarily stop a Connection's delivery of incoming messages. It can be
 * restarted using its start method. When stopped, delivery to all the Connection's
 * message consumers is inhibited: synchronous receive's block and messages are not
 * delivered to message listeners. 
 *
 * <p>This call blocks until receives and/or message listeners in progress have completed. 
 *
 * <p>Stopping a Session has no affect on its ability to send messages. Stopping a stopped 
 * session is ignored. 
 *
 * <p>A stop method call must not return until delivery of messages has paused. This means 
 * a client can rely on the fact that none of its message listeners will be called and 
 * all threads of control waiting for receive to return will not return with a message 
 * until the connection is restarted. The receive timers for a stopped connection continue 
 * to advance so receives may time out while the connection is stopped. 
 *
 * <p>If MessageListeners are running when stop is invoked, stop must wait until all of them 
 * have returned before it may return. While these MessageListeners are completing, they 
 * must have full services of the connection available to them.
 *
 * @param obj the Connection object to stop.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_stop(connection_obj : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the URL of the broker for this Connection.
 *
 * @param connObj the Connection object to operate against.
 * @param pBrokerURL will contain the broker URL.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_getBrokerURL(connObj : HOBJ; var pBrokerURL : HOBJ) : TRetVal; cdecl;

(** Get the client identifier for this connection.
  *
  * @param connObj the Connection object to operate against
  * @param pClientID the unique client identifier.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function Connection_getClientID(connObj : HOBJ; var pClientID : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the connection ID for this Connection.
 *
 * @param connObj the Connection object to operate against.
 * @param pConnID will contain the connection ID.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_getConnectionID(connObj : HOBJ; var pConnID : jlong) : TRetVal; cdecl;

(**
 * Get the ExceptionListener for this Connection.
 *
 * @param connObj the Connection object to operate against
 * @param p_pfnListener The exception listener (i.e. callback function) for this
 *        Connection. The signature is given by
 *        <code>typedef void (*pfnCExceptionListener)(HOBJ hobjException) : TRetVal; cdecl;</code>
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function Connection_getExceptionListener(connObj : HOBJ; var p_pfnListener : pfnCExceptionListener) : TRetVal; cdecl;

(** Get the meta data for this connection.
  *
  * @param connObj the Connection object to operate against
  * @param pMetaData the ConnectionMetadata object returned
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see ConnectionMetaData_* functions
  *)
function Connection_getMetaData(connObj : HOBJ; var pMetaData : HOBJ) : TRetVal; cdecl;

(** Get the username for this connection.
  *
  * @param connObj the Connection object to operate against
  * @param pUserName returns a String object containing the username.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *
  * @see ConnectionMetaData_* functions
  *)
function Connection_getUsername(connObj : HOBJ; var pUserName : HOBJ) : TRetVal; cdecl;

(** Set the client identifier for this connection.
  *
  * <P>The preferred way to assign a Client's client identifier is for
  * it to be configured in a client-specific ConnectionFactory and
  * transparently assigned to the Connection it creates. Alternatively,
  * a client can set a Connections's client identifier using a
  * specific value.
  *
  * <P>The purpose of client identifier is to associate a session and
  * its objects with a state maintained on behalf of the client by a
  * SonicMQ broker. The only such state identified by SonicMQ is that required
  * to support durable subscriptions
  *
  * @param connObj the Connection object to operate against
  * @param clientID the unique client identifier
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           InvalidClientIDException if SonicMQ client specifies an invalid or duplicate client id.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function Connection_setClientID(connObj : HOBJ; clientID : HOBJ) : TRetVal; cdecl;

(** Set up an exception listener (i.e. a callback function) for this connection.
  *
  * <P>If SonicMQ detects a serious problem with a connection it
  * will inform the connection's "exception listener" if one has been
  * registered. It does this by invoking the callback function set via
  * <code>Connection_setExceptionListener()</code and passing it a JMSException
  * object describing the problem.
  *
  * <P>This allows a client to be asynchronously notified of a problem.
  * Some connections only consume messages so they would have no other
  * way to learn their connection has failed.
  *
  * <P>A Connection serializes execution of its exception listener.
  *
  * <P>A SonicMQ broker attempts to resolve connection problems
  * itself prior to notifying the client of them.
  *
  * @param connObj the Connection object to operate against
  * @param listener the callback function to set as the exception listener--
  *        the signature is <code>typedef void (*pfnCExceptionListener)(HOBJ hobjException) : TRetVal; cdecl;</code>
  *        where hobjException is a JMSException object handle
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           InvalidClientIDException if SonicMQ client specifies an invalid or duplicate client id.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function Connection_setExceptionListener(connObj : HOBJ; listener : pfnCExceptionListener) : TRetVal; cdecl;

(** Enable/disable sending active pings on this connection. This is a non-JMS public method.
  *
  * @param obj the Connection object to operate against
  * @param interval indicates the interval in seconds for sending a ping. Setting
  *        interval to 0 or any negative value effectively disables the ping.
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error.
  *         Likely causes of errors:
  *           InvalidClientIDException if SonicMQ client specifies an invalid or duplicate client id.
  *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
  *)
function Connection_setPingInterval(obj : HOBJ; interval : jlong) : TRetVal; cdecl;



(*
//---------------------------------
// QueueConnection
//---------------------------------
*)
(**
 * Returns the int corresponding to the QueueConnection type.
 *
 * @return the int corresponding to the QueueConnection type
 *)
function QueueConnection_type : TRetVal; cdecl;


(** Construct a QueueConnection to message broker.
 *
 * @param brokerURL the URL of the message broker in the for [protocol://]hostname[:port]
 * @param connectID the client id used for connection.
 * @param username the name of client's user account.
 * @param password the password for client's user account
 * @param pQConn the newly created QueueConnection
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnection_create(brokerURL, connectID, username, password : HOBJ; var pQConn : HOBJ) : TRetVal; cdecl;

(** Create a QueueSession associated with the queue connection.
 *
 * @param qConn the QueueConnection to operate against
 * @param transacted if true, the session is transacted.
 * @param acknowledgeMode indicates whether the consumer or the
 * client will acknowledge any messages it receives. This parameter
 * will be ignored if the session is transacted.
 * @param pQSession the newly created queue session.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnection_createQueueSession(qConn : HOBJ; transacted : jboolean; acknowledgeMode : jint; var pQSession : HOBJ) : TRetVal; cdecl;



(*
//---------------------------------
// TopicConnection
//---------------------------------
*)
(** Creates a TopicConnection to a message broker
  * A TopicConnection is an active connection to a SonicMQ Pub/Sub messaging domain.
  * A client uses a TopicConnection to create one or more TopicSessions
  * for producing and consuming messages.
  *
  * @param brokerURL the URL of the message broker in the for [protocol://]hostname[:port]
  * @param connectID the id used for connection. This may be null.
  * @param username the name of client's user account.
  * @param password the password for client's user account
  * @param pTopicConn pointer to the new topic connection
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error (e.g., connection to message
  *         broker can not be created). Use SMQ_getLastError() and SMQ_getLastErrorText() to
  *         interrogate error details.
  *)
function TopicConnection_create(brokerURL, connectID, username, password : HOBJ; var pTopicConn : HOBJ) : TRetVal; cdecl;

(** Creates a TopicSession associated with the TopicConnection
  *
  * @param topicConn the TopicConnection to operate against
  * @param transacted if true, the session is transacted.
  * @param acknowledgeMode indicates whether the consumer or the
  *        client will acknowledge any messages it receives. This parameter
  *        will be ignored if the session is transacted.
  * @param pTopicSession pointer to the new TopicSession
  *
  * @return SMQ_SUCCESS (0) if successful, non-zero if error. Use SMQ_getLastError() and
  *         SMQ_getLastErrorText() to interrogate error details.
  *)
function TopicConnection_createTopicSession(topicConn : HOBJ; transacted : jboolean; ackMode : jint; var pTopicSession : HOBJ) : TRetVal; cdecl;

(**
 * Returns the int corresponding to the TopicConnection type.
 *
 * @return the int corresponding to the TopicConnection type
 *)
function TopicConnection_type : TRetVal; cdecl;

(*
//---------------------------------
// QueueConnectionFactory
//---------------------------------
*)
(**
 * Create a queue connection with default user identity.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret a newly created QueueConnection.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS Provider fails to create a Queue Connection
 *                         due to some internal error.
 * @exception JMSSecurityException  if client authentication fails due to
 *                         invalid user name or password.
 *)
function QueueConnectionFactory_createQueueConnection(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a queue connection with specified user identity.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param username the caller's user name
 * @param password the caller's password
 * @param pret a newly created QueueConnection.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS Provider fails to create a Queue Connection
 *                         due to some internal error.
 * @exception JMSSecurityException  if client authentication fails due to
 *                         invalid user name or password.
 *)
function QueueConnectionFactory_createQueueConnection2(thisObj : HOBJ; username, password : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker hostname.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret will contain a String w/ the broker hostname.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getBrokerHostName(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker port number.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret will contain the broker port number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getBrokerPort(thisObj : HOBJ; var pret : jint) : TRetVal; cdecl;

(**
 * Retrieve the broker protocol.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret a String containing the broker protocol.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getBrokerProtocol(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker URL.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret the String containing the broker URL.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getBrokerURL(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the client ID.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret the String containing the client ID.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getClientID(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the connect ID.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret the String containing the connect ID.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getConnectID(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Get list of brokers to try to connect to.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret a String containing a comma-separated list of broker URLs..
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getConnectionURLs(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the default password.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret a String containing the default password.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getDefaultPassword(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the default username.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret a String containing the default username.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_getDefaultUser(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Determines whether client-side load balancing is enabled.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret if true, indicates that the client is willing to have a connect request
 *        redirected to another broker within a SonicMQ cluster.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see    QueueConnectionFactory_setLoadBalancing()
 *)
function QueueConnectionFactory_getLoadBalancing(thisObj : HOBJ; var pret : jboolean) : TRetVal;

(**
 * Determines whether the option to start with the first or a random element of the broker list
 * has been selected.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param pret If true, connect attempts start with the first broker in the list.
 *        If false, connect attempts start with a random element in the broker list.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueConnectionFactory_setSequential()
 * @see QueueConnectionFactory_setConnectionURLs()
 * @see QueueConnectionFactory_getConnectionURLs()
 *)
function QueueConnectionFactory_getSequential(thisObj : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Sets the broker hostname.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerHostName a String w/ the new broker hostname to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setBrokerHostName(thisObj : HOBJ; brokerHostName : HOBJ) : TRetVal; cdecl;

(**
 * Sets the broker port number.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerPort the new broker port number to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setBrokerPort(thisObj : HOBJ; brokerPort : jint) : TRetVal; cdecl;

(**
 * Sets the broker protocol.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new broker protocol to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setBrokerProtocol(thisObj : HOBJ; brokerProtocol : HOBJ) : TRetVal; cdecl;

(**
 * Sets the broker URL.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new broker URL to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setBrokerURL(thisObj : HOBJ; brokerURL : HOBJ) : TRetVal; cdecl;

(**
 * Sets the client ID.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new client ID to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setClientID(thisObj : HOBJ; clientID : HOBJ) : TRetVal; cdecl;

(**
 * Sets the connect ID.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new connect ID to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_setConnectID(thisObj : HOBJ; connectID : HOBJ) : TRetVal; cdecl;

(**
 * Configures a list of brokers to try when creating a connection.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param brokerList   a String containing a comma-separated list of broker URLs.
 *                     Allows a client to connect to the first available broker on a list.
 *                     If present, this parameter overrides the brokerURL parameter in the
 *                     QueueConnectionFactory constructor, which specifies a single broker.
 *                     This option can be used independently of any other load balancing options.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueConnectionFactory_getConnectionURLs()
 * @see QueueConnectionFactory_setSequential()
 * @see QueueConnectionFactory_getSequential()
 *)
function QueueConnectionFactory_setConnectionURLs(thisObj : HOBJ; brokerList : HOBJ) : TRetVal; cdecl;

(**
 * Enable or disable client-side load balancing.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param loadBalancing    if true, indicates that the client is willing to have
 *                         a connect request re-directed to another broker within
 *                         a SonicMQ cluster.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueConnectionFactory_getLoadBalancing()
 *)
function QueueConnectionFactory_setLoadBalancing(thisObj : HOBJ; loadBalancing : jboolean) : TRetVal; cdecl;

(**
 * Specifies whether to start with the first broker in the list or a random element.
 *
 * @param thisObj the QueueConnectionFactory object to operate against.
 * @param sequential   If true, starts attempting to connect to the first broker
 *                     in the list; if false, starts attempting to connect to a random element in 
 *                     the list. 
 *                     After that, tries to connect to each broker in sequence 
 *                     until a successful connect occurs, or the list is exhausted.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see QueueConnectionFactory_getSequential()
 * @see QueueConnectionFactory_setConnectionURLs()
 * @see QueueConnectionFactory_getConnectionURLs()
 *)
function QueueConnectionFactory_setSequential(thisObj : HOBJ; sequential : jboolean) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create(brokerURL : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerHostName a String containing the broker hostname.
 * @param brokerPort the broker port number.
 * @param brokerProtocol a String containing the broker protocol.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create2(
	brokerHostName : HOBJ;
	brokerPort : jint;
	brokerProtocol : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerHostName a String containing the broker hostname.
 * @param brokerPort the broker port number.
 * @param brokerProtocol a String containing the broker protocol.
 * @param connectID a String containing the connection ID.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create3(
	brokerHostName : HOBJ;
	brokerPort : jint;
	brokerProtocol : HOBJ;
	connectID : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param connectID a String containing the connection ID.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create4(
	brokerURL : HOBJ;
	connectID : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create5(
	brokerURL : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new QueueConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param connectID a String containing the connection ID.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created QueueConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueConnectionFactory_create6(
	brokerURL : HOBJ;
	connectID : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// TopicConnectionFactory
//---------------------------------
*)
(**
 * Create a queue connection with default user identity.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret a newly created TopicConnection.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS Provider fails to create a Queue Connection
 *                         due to some internal error.
 * @exception JMSSecurityException  if client authentication fails due to
 *                         invalid user name or password.
 *)
function TopicConnectionFactory_createTopicConnection(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a topic connection with specified user identity.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param username the caller's user name
 * @param password the caller's password
 * @param pret a newly created TopicConnection.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @exception JMSException if JMS Provider fails to create a Queue Connection
 *                         due to some internal error.
 * @exception JMSSecurityException  if client authentication fails due to
 *                         invalid user name or password.
 *)
function TopicConnectionFactory_createTopicConnection2(thisObj : HOBJ; username : HOBJ; password : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker hostname.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret will contain a String w/ the broker hostname.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getBrokerHostName(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker port number.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret will contain the broker port number.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getBrokerPort(thisObj : HOBJ; var pret : jint) : TRetVal; cdecl;

(**
 * Retrieve the broker protocol.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret a String containing the broker protocol.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getBrokerProtocol(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the broker URL.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret the String containing the broker URL.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getBrokerURL(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the client ID.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret the String containing the client ID.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getClientID(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the connect ID.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret the String containing the connect ID.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getConnectID(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Get list of brokers to try to connect to.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret a String containing a comma-separated list of broker URLs..
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getConnectionURLs(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the default password.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret a String containing the default password.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getDefaultPassword(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Retrieve the default username.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret a String containing the default username.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_getDefaultUser(thisObj : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Determines whether client-side load balancing is enabled.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret if true, indicates that the client is willing to have a connect request
 *        redirected to another broker within a SonicMQ cluster.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see    TopicConnectionFactory_setLoadBalancing()
 *)
function TopicConnectionFactory_getLoadBalancing(thisObj : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Determines whether the option to start with the first or a random element of the broker list
 * has been selected.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param pret If true, connect attempts start with the first broker in the list.
 *        If false, connect attempts start with a random element in the broker list.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see TopicConnectionFactory_setSequential()
 * @see TopicConnectionFactory_setConnectionURLs()
 * @see TopicConnectionFactory_getConnectionURLs()
 *)
function TopicConnectionFactory_getSequential(thisObj : HOBJ; var pret : jboolean) : TRetVal; cdecl;

(**
 * Sets the broker hostname.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerHostName a String w/ the new broker hostname to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setBrokerHostName(thisObj : HOBJ; brokerHostName : HOBJ) : TRetVal; cdecl;

(**
 * Sets the broker port number.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerPort the new broker port number to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setBrokerPort(thisObj : HOBJ; brokerPort : jint) : TRetVal; cdecl;

(**
 * Sets the broker protocol.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new broker protocol to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setBrokerProtocol(thisObj : HOBJ; brokerProtocol : HOBJ) : TRetVal; cdecl;

(**
 * Sets the broker URL.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new broker URL to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setBrokerURL(thisObj : HOBJ; brokerURL : HOBJ) : TRetVal; cdecl;

(**
 * Sets the client ID.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new client ID to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setClientID(thisObj : HOBJ; clientID : HOBJ) : TRetVal; cdecl;

(**
 * Sets the connect ID.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerPort a String w/ the new connect ID to set.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_setConnectID(thisObj : HOBJ; connectID : HOBJ) : TRetVal; cdecl;

(**
 * Configures a list of brokers to try when creating a connection.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param brokerList   a String containing a comma-separated list of broker URLs.
 *                     Allows a client to connect to the first available broker on a list.
 *                     If present, this parameter overrides the brokerURL parameter in the
 *                     TopicConnectionFactory constructor, which specifies a single broker.
 *                     This option can be used independently of any other load balancing options.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see TopicConnectionFactory_getConnectionURLs()
 * @see TopicConnectionFactory_setSequential()
 * @see TopicConnectionFactory_getSequential()
 *)
function TopicConnectionFactory_setConnectionURLs(thisObj : HOBJ; brokerList : HOBJ) : TRetVal; cdecl;

(**
 * Enable or disable client-side load balancing.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param loadBalancing    if true, indicates that the client is willing to have
 *                         a connect request re-directed to another broker within
 *                         a SonicMQ cluster.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see TopicConnectionFactory_getLoadBalancing()
 *)
function TopicConnectionFactory_setLoadBalancing(thisObj : HOBJ; loadBalancing : jboolean) : TRetVal; cdecl;

(**
 * Specifies whether to start with the first broker in the list or a random element.
 *
 * @param thisObj the TopicConnectionFactory object to operate against.
 * @param sequential   If true, starts attempting to connect to the first broker
 *                     in the list; if false, starts attempting to connect to a random element in 
 *                     the list. 
 *                     After that, tries to connect to each broker in sequence 
 *                     until a successful connect occurs, or the list is exhausted.
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *
 * @see TopicConnectionFactory_getSequential()
 * @see TopicConnectionFactory_setConnectionURLs()
 * @see TopicConnectionFactory_getConnectionURLs()
 *)
function TopicConnectionFactory_setSequential(thisObj : HOBJ; sequential : jboolean) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create(brokerURL : HOBJ; var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerHostName a String containing the broker hostname.
 * @param brokerPort the broker port number.
 * @param brokerProtocol a String containing the broker protocol.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create2(
	brokerHostName : HOBJ;
	brokerPort : jint;
	brokerProtocol : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerHostName a String containing the broker hostname.
 * @param brokerPort the broker port number.
 * @param brokerProtocol a String containing the broker protocol.
 * @param connectID a String containing the connection ID.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create3(
	brokerHostName : HOBJ;
	brokerPort : jint;
	brokerProtocol : HOBJ;
	connectID : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param connectID a String containing the connection ID.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create4(
	brokerURL : HOBJ;
	connectID : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create5(
	brokerURL : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(**
 * Create a new TopicConnectionFactory.
 *
 * @param brokerURL a String containing the broker URL.
 * @param connectID a String containing the connection ID.
 * @param defaultUserName a String containing the default username.
 * @param defaultPassword a String containing the default password.
 * @param pret will contain the newly created TopicConnectionFactory
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function TopicConnectionFactory_create6(
	brokerURL : HOBJ;
	connectID : HOBJ;
	defaultUserName : HOBJ;
	defaultPassword : HOBJ;
	var pret : HOBJ) : TRetVal; cdecl;

(*
//---------------------------------
// QueueBrowser
//---------------------------------
*)
(**
 * Returns the int corresponding to the QueueBrowser type.
 *
 * @return the int corresponding to the QueueBrowser type
 *)
function QueueBrowser_type : TRetVal; cdecl;

(** Since SonicMQ allocates some resources on behalf of a
 * MessageConsumer inside the SonicMQ client run-time, clients should close them when they
 * are not needed.
 *
 * @param qBrowser the QueueBrowser to operate against
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueBrowser_close(qBrowser : HOBJ) : TRetVal; cdecl;

(** Get the queue associated with this queue Browser.
 *
 * @param qBrowser the QueueBrowser to operate against
 * @param pQueue the Queue that the QueueBrowser is associated with
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueBrowser_getQueue(qBrowser : HOBJ; var pQueue : HOBJ) : TRetVal; cdecl;

(** Get this queue browser's message selector expression.
 *
 * @param qBrowser the QueueBrowser to operate against
 * @param pMsgSelector this queue browser's message selector
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueBrowser_getMessageSelector(qBrowser : HOBJ; var pMsgSelector : HOBJ) : TRetVal; cdecl;

(** Get an enumeration for browsing the current queue
 * messages in the order they would be received.
 *
 * @param qBrowser the QueueBrowser to operate against
 * @param pEnum an enumeration for browsing the messages
 *
 * @return SMQ_SUCCESS (0) if successful, non-zero if error.
 *         Use SMQ_getLastError() and SMQ_getLastErrorText() to interrogate error details.
 *)
function QueueBrowser_getEnumeration(qBrowser : HOBJ; var pEnum : HOBJ) : TRetVal; cdecl;


(*
//---------------------------------
// Constants
//---------------------------------
*)
(**  Returns a string representation of the NOTIFY_UNDELIVERED reserved property name used in
 *   dynamic routing.
 *
 * @param pString the NOTIFY_UNDELIVERED string
 *)
function Constants_getNOTIFY_UNDELIVERED(var pString : HOBJ) : TRetVal; cdecl;

(**  Returns a string representation of the PRESERVE_UNDELIVERED reserved property name used in
 *   dynamic routing.
 *
 * @param pString the PRESERVE_UNDELIVERED string
 *)
function Constants_getPRESERVE_UNDELIVERED(var pString : HOBJ) : TRetVal; cdecl;

(**  Returns a string representation of the UNDELIVERED_REASON_CODE reserved property name used in
 *   dynamic routing.
 *
 * @param pString the UNDELIVERED_REASON_CODE string
 *)
function Constants_getUNDELIVERED_REASON_CODE(var pString : HOBJ) : TRetVal; cdecl;

(**  Returns a string representation of the UNDELIVERED_TIMESTAMP reserved property name used in
 *   dynamic routing.
 *
 * @param pString the UNDELIVERED_TIMESTAMP string
 *)
function Constants_getUNDELIVERED_TIMESTAMP(var pString : HOBJ) : TRetVal; cdecl;


implementation

const
  SMQ_ClientDLL = 'libsmq.dll';

function SMQ_getLastError; external SMQ_ClientDLL;
function SMQ_getLastErrorText; external SMQ_ClientDLL;

function Object_release; external SMQ_ClientDLL;
function Object_toString; external SMQ_ClientDLL;
function Object_type; external SMQ_ClientDLL;
function Object_getType; external SMQ_ClientDLL;
function Object_equals; external SMQ_ClientDLL;
function Object_clone; external SMQ_ClientDLL;
function Object_instanceof; external SMQ_ClientDLL;

{function String_append; external SMQ_ClientDLL;
function String_append2; external SMQ_ClientDLL;
function String_append3; external SMQ_ClientDLL;
function String_assign; external SMQ_ClientDLL;
function String_charAt; external SMQ_ClientDLL;
function String_compareTo; external SMQ_ClientDLL;
function String_compareToIgnoreCase; external SMQ_ClientDLL;
function String_concat; external SMQ_ClientDLL;
function String_copyValueOf; external SMQ_ClientDLL;
function String_copyValueOf2; external SMQ_ClientDLL;
function String_create; external SMQ_ClientDLL;}
function String_create2; external SMQ_ClientDLL;
{function String_create3; external SMQ_ClientDLL;
function String_create4; external SMQ_ClientDLL;
function String_create5; external SMQ_ClientDLL;
function String_create6; external SMQ_ClientDLL;
function String_create7; external SMQ_ClientDLL;
function String_create8; external SMQ_ClientDLL;
function String_endsWith; external SMQ_ClientDLL;
function String_equalsIgnoreCase; external SMQ_ClientDLL;
function String_getBytes; external SMQ_ClientDLL;
function String_getBytes2; external SMQ_ClientDLL;
function String_getBytes3; external SMQ_ClientDLL;
function String_getChars; external SMQ_ClientDLL;
function String_indexOf; external SMQ_ClientDLL;
function String_indexOf2; external SMQ_ClientDLL;
function String_indexOf3; external SMQ_ClientDLL;
function String_indexOf4; external SMQ_ClientDLL;
function String_intern; external SMQ_ClientDLL;
function String_isAscii; external SMQ_ClientDLL;
function String_lastIndexOf; external SMQ_ClientDLL;
function String_lastIndexOf2; external SMQ_ClientDLL;
function String_lastIndexOf3; external SMQ_ClientDLL;
function String_lastIndexOf4; external SMQ_ClientDLL;
function String_length; external SMQ_ClientDLL;
function String_regionMatches; external SMQ_ClientDLL;
function String_regionMatches2; external SMQ_ClientDLL;
function String_replace; external SMQ_ClientDLL;
function String_startsWith; external SMQ_ClientDLL;
function String_startsWith2; external SMQ_ClientDLL;
function String_substring; external SMQ_ClientDLL;
function String_substring2; external SMQ_ClientDLL;}
function String_toAscii; external SMQ_ClientDLL;
{function String_toCharArray; external SMQ_ClientDLL;
function String_toLowerCase; external SMQ_ClientDLL;
function String_toUpperCase; external SMQ_ClientDLL;
function String_toWide; external SMQ_ClientDLL;
function String_trim; external SMQ_ClientDLL;}
function String_type; external SMQ_ClientDLL;
{function String_valueOf; external SMQ_ClientDLL;
function String_valueOf2; external SMQ_ClientDLL;
function String_valueOfBoolean; external SMQ_ClientDLL;
function String_valueOfByte; external SMQ_ClientDLL;
function String_valueOfChar; external SMQ_ClientDLL;
function String_valueOfDouble; external SMQ_ClientDLL;
function String_valueOfFloat; external SMQ_ClientDLL;
function String_valueOfInt; external SMQ_ClientDLL;
function String_valueOfLong; external SMQ_ClientDLL;
function String_valueOfShort; external SMQ_ClientDLL;}

//function AString_at; external SMQ_ClientDLL;
function AString_getValue; external SMQ_ClientDLL;
function AString_length; external SMQ_ClientDLL;
function AString_type; external SMQ_ClientDLL;

function Message_acknowledge; external SMQ_ClientDLL;
function Message_clearBody; external SMQ_ClientDLL;
function Message_clearProperties; external SMQ_ClientDLL;
//function Message_create; external SMQ_ClientDLL;
function Message_getBooleanProperty; external SMQ_ClientDLL;
function Message_getByteProperty; external SMQ_ClientDLL;
function Message_getDoubleProperty; external SMQ_ClientDLL;
function Message_getFloatProperty; external SMQ_ClientDLL;
function Message_getIntProperty; external SMQ_ClientDLL;
function Message_getJMSCorrelationID; external SMQ_ClientDLL;
function Message_getJMSCorrelationIDAsBytes; external SMQ_ClientDLL;
function Message_getJMSDeliveryMode; external SMQ_ClientDLL;
function Message_getJMSDestination; external SMQ_ClientDLL;
function Message_getJMSExpiration; external SMQ_ClientDLL;
function Message_getJMSMessageID; external SMQ_ClientDLL;
function Message_getJMSPriority; external SMQ_ClientDLL;
function Message_getJMSRedelivered; external SMQ_ClientDLL;
function Message_getJMSReplyTo; external SMQ_ClientDLL;
function Message_getJMSTimestamp; external SMQ_ClientDLL;
function Message_getJMSType; external SMQ_ClientDLL;
function Message_getLongProperty; external SMQ_ClientDLL;
function Message_getObjectProperty; external SMQ_ClientDLL;
function Message_getProperties; external SMQ_ClientDLL;
function Message_getPropertyNames; external SMQ_ClientDLL;
function Message_getShortProperty; external SMQ_ClientDLL;
function Message_getStringProperty; external SMQ_ClientDLL;
function Message_propertyExists; external SMQ_ClientDLL;
function Message_setBooleanProperty; external SMQ_ClientDLL;
function Message_setByteProperty; external SMQ_ClientDLL;
function Message_setDoubleProperty; external SMQ_ClientDLL;
function Message_setFloatProperty; external SMQ_ClientDLL;
function Message_setIntProperty; external SMQ_ClientDLL;
function Message_setJMSCorrelationID; external SMQ_ClientDLL;
function Message_setJMSCorrelationIDAsBytes; external SMQ_ClientDLL;
function Message_setJMSDeliveryMode; external SMQ_ClientDLL;
function Message_setJMSDestination; external SMQ_ClientDLL;
function Message_setJMSExpiration; external SMQ_ClientDLL;
function Message_setJMSMessageID; external SMQ_ClientDLL;
function Message_setJMSPriority; external SMQ_ClientDLL;
function Message_setJMSRedelivered; external SMQ_ClientDLL;
function Message_setJMSReplyTo; external SMQ_ClientDLL;
function Message_setJMSTimestamp; external SMQ_ClientDLL;
function Message_setJMSType; external SMQ_ClientDLL;
function Message_setLongProperty; external SMQ_ClientDLL;
function Message_setObjectProperty; external SMQ_ClientDLL;
function Message_setShortProperty; external SMQ_ClientDLL;
function Message_setStringProperty; external SMQ_ClientDLL;
function Message_type; external SMQ_ClientDLL;

function TextMessage_create; external SMQ_ClientDLL;
function TextMessage_getText; external SMQ_ClientDLL;
function TextMessage_setText; external SMQ_ClientDLL;
function TextMessage_type; external SMQ_ClientDLL;

function BytesMessage_create; external SMQ_ClientDLL;
function BytesMessage_readBoolean; external SMQ_ClientDLL;
function BytesMessage_readByte; external SMQ_ClientDLL;
function BytesMessage_readBytes; external SMQ_ClientDLL;
{function BytesMessage_readBytes2; external SMQ_ClientDLL;
function BytesMessage_readChar; external SMQ_ClientDLL;
function BytesMessage_readDouble; external SMQ_ClientDLL;
function BytesMessage_readFloat; external SMQ_ClientDLL;
function BytesMessage_readInt; external SMQ_ClientDLL;
function BytesMessage_readLong; external SMQ_ClientDLL;
function BytesMessage_readShort; external SMQ_ClientDLL;
function BytesMessage_readUTF; external SMQ_ClientDLL;
function BytesMessage_readUnsignedByte; external SMQ_ClientDLL;
function BytesMessage_readUnsignedShort; external SMQ_ClientDLL;
function BytesMessage_reset; external SMQ_ClientDLL;}
function BytesMessage_type; external SMQ_ClientDLL;
{function BytesMessage_writeBoolean; external SMQ_ClientDLL;}
function BytesMessage_writeByte; external SMQ_ClientDLL;
{function BytesMessage_writeBytes; external SMQ_ClientDLL;
function BytesMessage_writeBytes2; external SMQ_ClientDLL;
function BytesMessage_writeChar; external SMQ_ClientDLL;
function BytesMessage_writeDouble; external SMQ_ClientDLL;
function BytesMessage_writeFloat; external SMQ_ClientDLL;
function BytesMessage_writeInt; external SMQ_ClientDLL;
function BytesMessage_writeLong; external SMQ_ClientDLL;
function BytesMessage_writeShort; external SMQ_ClientDLL;
function BytesMessage_writeUTF; external SMQ_ClientDLL;}

function ConnectionMetaData_getJMSMajorVersion; external SMQ_ClientDLL;
function ConnectionMetaData_getJMSMinorVersion; external SMQ_ClientDLL;
function ConnectionMetaData_getJMSProviderName; external SMQ_ClientDLL;
function ConnectionMetaData_getJMSVersion; external SMQ_ClientDLL;
function ConnectionMetaData_getJMSXPropertyNames; external SMQ_ClientDLL;
function ConnectionMetaData_getProviderMajorVersion; external SMQ_ClientDLL;
function ConnectionMetaData_getProviderMinorVersion; external SMQ_ClientDLL;
function ConnectionMetaData_getProviderVersion; external SMQ_ClientDLL;
function ConnectionMetaData_type; external SMQ_ClientDLL;

function Queue_getQueueName; external SMQ_ClientDLL;
function Queue_type; external SMQ_ClientDLL;

function TemporaryQueue_delete; external SMQ_ClientDLL;
function TemporaryQueue_type; external SMQ_ClientDLL;

function Topic_getTopicName; external SMQ_ClientDLL;
function Topic_type; external SMQ_ClientDLL;

function TemporaryTopic_delete; external SMQ_ClientDLL;
function TemporaryTopic_type; external SMQ_ClientDLL;

function Session_close; external SMQ_ClientDLL;
function Session_commit; external SMQ_ClientDLL;
function Session_createBytesMessage; external SMQ_ClientDLL;
function Session_createMessage; external SMQ_ClientDLL;
function Session_createTextMessage; external SMQ_ClientDLL;
function Session_createTextMessage2; external SMQ_ClientDLL;
function Session_getTransacted; external SMQ_ClientDLL;
function Session_recover; external SMQ_ClientDLL;
function Session_rollback; external SMQ_ClientDLL;
function Session_type; external SMQ_ClientDLL;

function QueueSession_createBrowser; external SMQ_ClientDLL;
function QueueSession_createBrowser2; external SMQ_ClientDLL;
function QueueSession_createQueue; external SMQ_ClientDLL;
function QueueSession_createReceiver; external SMQ_ClientDLL;
function QueueSession_createReceiver2; external SMQ_ClientDLL;
function QueueSession_createSender; external SMQ_ClientDLL;
function QueueSession_createTemporaryQueue; external SMQ_ClientDLL;
function QueueSession_type; external SMQ_ClientDLL;

function TopicSession_createDurableSubscriber; external SMQ_ClientDLL;
function TopicSession_createDurableSubscriber2; external SMQ_ClientDLL;
function TopicSession_createPublisher; external SMQ_ClientDLL;
function TopicSession_createSubscriber; external SMQ_ClientDLL;
function TopicSession_createSubscriber2; external SMQ_ClientDLL;
function TopicSession_createTemporaryTopic; external SMQ_ClientDLL;
function TopicSession_createTopic; external SMQ_ClientDLL;
function TopicSession_setFlowControlDisabled; external SMQ_ClientDLL;
function TopicSession_type; external SMQ_ClientDLL;
function TopicSession_unsubscribe; external SMQ_ClientDLL;

function MessageConsumer_getMessageListener; external SMQ_ClientDLL;
function MessageConsumer_getMessageSelector; external SMQ_ClientDLL;
function MessageConsumer_receive; external SMQ_ClientDLL;
function MessageConsumer_receive2; external SMQ_ClientDLL;
function MessageConsumer_receiveNoWait; external SMQ_ClientDLL;
function MessageConsumer_setMessageListener; external SMQ_ClientDLL;
function MessageConsumer_type; external SMQ_ClientDLL;

function TopicSubscriber_getNoLocal; external SMQ_ClientDLL name 'ITopicSubscriber_getNoLocal';
function TopicSubscriber_getTopic; external SMQ_ClientDLL name 'ITopicSubscriber_getTopic';
function TopicSubscriber_type; external SMQ_ClientDLL;

function QueueReceiver_close; external SMQ_ClientDLL;
function QueueReceiver_getPrefetchCount; external SMQ_ClientDLL;
function QueueReceiver_getPrefetchThreshold; external SMQ_ClientDLL;
function QueueReceiver_getQueue; external SMQ_ClientDLL;
function QueueReceiver_setPrefetchCount; external SMQ_ClientDLL;
function QueueReceiver_setPrefetchThreshold; external SMQ_ClientDLL;
function QueueReceiver_type; external SMQ_ClientDLL;

function MessageProducer_close; external SMQ_ClientDLL;
function MessageProducer_getDeliveryMode; external SMQ_ClientDLL;
function MessageProducer_getDisableMessageID; external SMQ_ClientDLL;
function MessageProducer_getDisableMessageTimestamp; external SMQ_ClientDLL;
function MessageProducer_getPriority; external SMQ_ClientDLL;
function MessageProducer_getTimeToLive; external SMQ_ClientDLL;
function MessageProducer_setDeliveryMode; external SMQ_ClientDLL;
function MessageProducer_setDisableMessageID; external SMQ_ClientDLL;
function MessageProducer_setDisableMessageTimestamp; external SMQ_ClientDLL;
function MessageProducer_setPriority; external SMQ_ClientDLL;
function MessageProducer_setTimeToLive; external SMQ_ClientDLL;
function MessageProducer_type; external SMQ_ClientDLL;

function QueueSender_getQueue; external SMQ_ClientDLL;
function QueueSender_send; external SMQ_ClientDLL;
function QueueSender_send2; external SMQ_ClientDLL;
function QueueSender_send3; external SMQ_ClientDLL;
function QueueSender_send4; external SMQ_ClientDLL;
function QueueSender_type; external SMQ_ClientDLL;

function TopicPublisher_getTopic; external SMQ_ClientDLL;
function TopicPublisher_publish; external SMQ_ClientDLL;
function TopicPublisher_publish2; external SMQ_ClientDLL;
function TopicPublisher_publish3; external SMQ_ClientDLL;
function TopicPublisher_publish4; external SMQ_ClientDLL;
function TopicPublisher_type; external SMQ_ClientDLL;

function Connection_close; external SMQ_ClientDLL;
function Connection_getBrokerURL; external SMQ_ClientDLL;
function Connection_getClientID; external SMQ_ClientDLL;
function Connection_getConnectionID; external SMQ_ClientDLL;
function Connection_getExceptionListener; external SMQ_ClientDLL;
function Connection_getMetaData; external SMQ_ClientDLL;
function Connection_getUsername; external SMQ_ClientDLL;
function Connection_setClientID; external SMQ_ClientDLL;
function Connection_setExceptionListener; external SMQ_ClientDLL;
function Connection_setPingInterval; external SMQ_ClientDLL;
function Connection_start; external SMQ_ClientDLL;
function Connection_stop; external SMQ_ClientDLL;
function Connection_type; external SMQ_ClientDLL;

function QueueConnection_create; external SMQ_ClientDLL;
function QueueConnection_createQueueSession; external SMQ_ClientDLL;
function QueueConnection_type; external SMQ_ClientDLL;

function TopicConnection_create; external SMQ_ClientDLL;
function TopicConnection_createTopicSession; external SMQ_ClientDLL;
function TopicConnection_type; external SMQ_ClientDLL;

function QueueConnectionFactory_create; external SMQ_ClientDLL;
function QueueConnectionFactory_create2; external SMQ_ClientDLL;
function QueueConnectionFactory_create3; external SMQ_ClientDLL;
function QueueConnectionFactory_create4; external SMQ_ClientDLL;
function QueueConnectionFactory_create5; external SMQ_ClientDLL;
function QueueConnectionFactory_create6; external SMQ_ClientDLL;
function QueueConnectionFactory_createQueueConnection; external SMQ_ClientDLL;
function QueueConnectionFactory_createQueueConnection2; external SMQ_ClientDLL;
function QueueConnectionFactory_getBrokerHostName; external SMQ_ClientDLL;
function QueueConnectionFactory_getBrokerPort; external SMQ_ClientDLL;
function QueueConnectionFactory_getBrokerProtocol; external SMQ_ClientDLL;
function QueueConnectionFactory_getBrokerURL; external SMQ_ClientDLL;
function QueueConnectionFactory_getClientID; external SMQ_ClientDLL;
function QueueConnectionFactory_getConnectID; external SMQ_ClientDLL;
function QueueConnectionFactory_getConnectionURLs; external SMQ_ClientDLL;
function QueueConnectionFactory_getDefaultPassword; external SMQ_ClientDLL;
function QueueConnectionFactory_getDefaultUser; external SMQ_ClientDLL;
function QueueConnectionFactory_getLoadBalancing; external SMQ_ClientDLL;
function QueueConnectionFactory_getSequential; external SMQ_ClientDLL;
function QueueConnectionFactory_setBrokerHostName; external SMQ_ClientDLL;
function QueueConnectionFactory_setBrokerPort; external SMQ_ClientDLL;
function QueueConnectionFactory_setBrokerProtocol; external SMQ_ClientDLL;
function QueueConnectionFactory_setBrokerURL; external SMQ_ClientDLL;
function QueueConnectionFactory_setClientID; external SMQ_ClientDLL;
function QueueConnectionFactory_setConnectID; external SMQ_ClientDLL;
function QueueConnectionFactory_setConnectionURLs; external SMQ_ClientDLL;
function QueueConnectionFactory_setLoadBalancing; external SMQ_ClientDLL;
function QueueConnectionFactory_setSequential; external SMQ_ClientDLL;

function TopicConnectionFactory_create; external SMQ_ClientDLL;
function TopicConnectionFactory_create2; external SMQ_ClientDLL;
function TopicConnectionFactory_create3; external SMQ_ClientDLL;
function TopicConnectionFactory_create4; external SMQ_ClientDLL;
function TopicConnectionFactory_create5; external SMQ_ClientDLL;
function TopicConnectionFactory_create6; external SMQ_ClientDLL;
function TopicConnectionFactory_createTopicConnection; external SMQ_ClientDLL;
function TopicConnectionFactory_createTopicConnection2; external SMQ_ClientDLL;
function TopicConnectionFactory_getBrokerHostName; external SMQ_ClientDLL;
function TopicConnectionFactory_getBrokerPort; external SMQ_ClientDLL;
function TopicConnectionFactory_getBrokerProtocol; external SMQ_ClientDLL;
function TopicConnectionFactory_getBrokerURL; external SMQ_ClientDLL;
function TopicConnectionFactory_getClientID; external SMQ_ClientDLL;
function TopicConnectionFactory_getConnectID; external SMQ_ClientDLL;
function TopicConnectionFactory_getConnectionURLs; external SMQ_ClientDLL;
function TopicConnectionFactory_getDefaultPassword; external SMQ_ClientDLL;
function TopicConnectionFactory_getDefaultUser; external SMQ_ClientDLL;
function TopicConnectionFactory_getLoadBalancing; external SMQ_ClientDLL;
function TopicConnectionFactory_getSequential; external SMQ_ClientDLL;
function TopicConnectionFactory_setBrokerHostName; external SMQ_ClientDLL;
function TopicConnectionFactory_setBrokerPort; external SMQ_ClientDLL;
function TopicConnectionFactory_setBrokerProtocol; external SMQ_ClientDLL;
function TopicConnectionFactory_setBrokerURL; external SMQ_ClientDLL;
function TopicConnectionFactory_setClientID; external SMQ_ClientDLL;
function TopicConnectionFactory_setConnectID; external SMQ_ClientDLL;
function TopicConnectionFactory_setConnectionURLs; external SMQ_ClientDLL;
function TopicConnectionFactory_setLoadBalancing; external SMQ_ClientDLL;
function TopicConnectionFactory_setSequential; external SMQ_ClientDLL;

function QueueBrowser_close; external SMQ_ClientDLL;
function QueueBrowser_getEnumeration; external SMQ_ClientDLL;
function QueueBrowser_getMessageSelector; external SMQ_ClientDLL;
function QueueBrowser_getQueue; external SMQ_ClientDLL;
function QueueBrowser_type; external SMQ_ClientDLL;

function Constants_getNOTIFY_UNDELIVERED; external SMQ_ClientDLL;
function Constants_getPRESERVE_UNDELIVERED; external SMQ_ClientDLL;
function Constants_getUNDELIVERED_REASON_CODE; external SMQ_ClientDLL;
function Constants_getUNDELIVERED_TIMESTAMP; external SMQ_ClientDLL;


end.
